///////////////
// IanniX
///////////////

////////////////////////////
// NxObject.cpp
////////////////////////////

#include "NxObject.h"
#include <iostream>
#include <math.h>

/////////////////////////////////////
// Empty Constructor
/////////////////////////////////////
NxObject::NxObject() 
{}

/////////////////////////////////////
// Default constructor
/////////////////////////////////////
NxObject::NxObject( int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor )
{	
	ID = inID;
	customID = incID ;
	groupID = inGroupID;
	name = inName;
	pos = inPos;
	color = inColor;
	selected = false;
	highlighted = false ;
	active = inActive ;
	layer = inLay ;
}

//////////////////////////////////////
// Abstract Destructor
//////////////////////////////////////
NxObject::~NxObject()
{}

/////////////////////////////
// Bounding Box
/////////////////////////////
void NxObject::updateBB( float )
{}

void NxObject::setBoundingBox( NxPoint inPt1, NxPoint inPt2 )
{
	boundingBox.p1 = inPt1;
	boundingBox.p2 = inPt2;
}
void NxObject::setBox( BBox inBox )
{
	boundingBox = inBox;
}
BBox NxObject::getBoundingBox()
{
	return boundingBox;
}



//////////////////////////////////
// Access Methods
//////////////////////////////////
// SET
void NxObject::setCustomID( int inID )
{
	customID = inID;
}
void NxObject::setGroupID( int inGroupID )
{
	groupID = inGroupID;
}
void NxObject::setName( QString inName )
{
	name = inName;
}
void NxObject::setPosition( NxPoint inPos )
{
	pos = inPos;
}
void NxObject::setColor( Color inColor )
{
	color = inColor;
}
void NxObject::setSelected( bool flag )
{
	selected = flag;
}
void NxObject::setHighlighted( bool flag )
{
	highlighted = flag ;
}
void NxObject::setActive( bool flag )
{
	active = flag ;
}
void NxObject::setLayer( int inLay )
{
	layer = inLay ;
}

// GET
int NxObject::getID()
{
	return ID;
}
int NxObject::getCustomID()
{
	return customID;
}
int NxObject::getGroupID()
{
	return groupID;
}
QString NxObject::getName()
{
	return name;
}
NxPoint	NxObject::getPosition()
{
	return pos;
}
Color NxObject::getColor()
{
	return color;
}
bool NxObject::isSelected()
{
	return selected;
}
bool NxObject::isHighlighted()
{
	return highlighted ;
}
bool NxObject::isActive()
{
	return active ;
}
int NxObject::getLayer()
{
	return layer ;
}
int NxObject::getObjectType()
{
	return objectType;
}
///////////////////////////////////////////////////////////////////////////////
// 					NxCurve
///////////////////////////////////////////////////////////////////////////////
NxCurve::NxCurve() 
{
}

/////////////////////////////////
// Main constructor
/////////////////////////////////
NxCurve::NxCurve(int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor, QVector<NxPoint> inCurves, int inNbPoints, Color inCircleColor, bool inTrans, QString inTexPath, QString inShape, float inRMin, float inRMax, float inScale, float inAngle, int inSpatialPeriod, QString inOscF, QString inHost, int inPort )
	: NxObject( inID, incID, inGroupID, inLay, inName, inActive, inPos, inColor )
{
	splineGenerated = false ;
	
	spatialPeriod = inSpatialPeriod ; ///// in ms (integer)

	curves			= inCurves;
	nbPoints		= inNbPoints;
	circleColor		= inCircleColor;
	transparency	= inTrans;
	texturePath		= inTexPath;
	shape			= inShape;
	objectType		= Curve;
	showPoints		= false;
	realMin			= inRMin;
	realMax			= inRMax;
	xRange			= 0 ;
	yRange			= fabs(realMax - realMin) ;
	oscFormat		= inOscF ;
	host			= inHost ;
	port			= inPort ;
	angle			= inAngle ;
	scale			= inScale ;

	// Offset Variables
	dX = dY = dZ = 0.0;
	
	// Bounding Box
	if (!curves.isEmpty())
	{
		radius = fabs( curves[inNbPoints-1].x - curves[0].x ) / 2 ;
		xRange = fabs( curves[inNbPoints-1].x - curves[0].x ) ;
		center.x = pos.x + radius ;
		center.y = pos.y ;

		boundingBox.p1.x = inPos.x ;
		boundingBox.p1.y = inPos.y + yRange*scale/2 ;
		boundingBox.p1.z = 0.0;
		boundingBox.p2.x = inPos.x + curves[nbPoints-1].x ;
		boundingBox.p2.y = inPos.y - yRange*scale/2 ;
		boundingBox.p2.z = 0.0;
		
		if(drawType == SplineStrip) splineGenerate() ; // regenerate spline id splines
	}
} 

//////////////////////
// Destructor
//////////////////////
NxCurve::~NxCurve() 
{
}
//////////////////////////////
// NxSpline re-Generation
//////////////////////////////
void NxCurve::splineGenerate()
{
	if(splineGenerated)	delete(splineDef);
	splineDef = new NxSpline( pos, curves.data(), curves.count(), realMin, realMax, scale );
	splineDef->Generate();
	splineGenerated = true ;
}
//////////////////////////////
// Bounding Box
//////////////////////////////
void NxCurve::updateBB( float )
{	
	// Calculating offsets
	dX = pos.x - center.x + radius ;
	dY = pos.y - center.y ;
	dZ = pos.z - center.z ;
	
	if(drawType == SplineStrip) splineGenerate() ; // only if spline mode
	
	// Updating Circle
	center.x = pos.x + radius ;
	center.y = pos.y ;
	center.z = pos.z ;
	
	boundingBox.p1.x = pos.x ;
	boundingBox.p1.y = pos.y + yRange*scale/2 ;
	boundingBox.p1.z = 0.0;
	boundingBox.p2.x = pos.x + curves[nbPoints-1].x ;
	boundingBox.p2.y = pos.y - yRange*scale/2 ;
	boundingBox.p2.z = 0.0;
}

/////////////////////////////////
// Access methods
/////////////////////////////////

///////////
// SET
///////////
void NxCurve::setCurve( QVector<NxPoint> inCurve )
{
	curves.clear();
	curves = inCurve;
	
	nbPoints 	= inCurve.count();
	center.x 	= pos.x + ( curves[nbPoints-1].x + curves[0].x ) / 2;
	center.y 	= pos.y ;
	radius 		= fabs( curves[nbPoints-1].x - curves[0].x ) / 2 ;
	xRange		= fabs( curves[nbPoints-1].x - curves[0].x ) ;
	// Re-calculate Bounding Box
	boundingBox.p1.x = center.x - radius;
	boundingBox.p1.y = center.y + yRange*scale/2 ;
	boundingBox.p1.z = 0.0 ;
	boundingBox.p2.x = center.x + radius;
	boundingBox.p2.y = center.y - yRange*scale/2 ;
	boundingBox.p2.z = 0.0 ;
	
	if(drawType == SplineStrip) splineGenerate() ; // only if spline mode
}
bool NxCurve::isInside( float inX, float inY )
{
	bool res = false ;
	if(angle==0) {
		if (	( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && 
				( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) )
			res = true ;
		}
		else {
			float hh = fabs(boundingBox.p2.y - boundingBox.p1.y) ;
			float curvAngle = angle*PI/180.0 ;
			//printf("Angle : %f\n", curvAngle ) ;
			float newInX = inX - center.x ;
			float newInY = inY - center.y ;
			//printf("// center : %f et %f\n", newInX, newInY ) ;
			float aSin = sin(curvAngle) ;
			float aCos = cos(curvAngle) ;
			//printf("sin / cos : %f et %f\n", aSin, aCos ) ;
			float localInX = aCos*newInX + aSin*newInY ;
			float localInY = -aSin*newInX + aCos*newInY ;
			//printf("contours : %f et %f\n", localInX, localInY ) ;
			//printf("local : %f et %f\n", xRange, hh ) ;
			if( (fabs(localInY)<hh/2.0) && (fabs(localInX)<xRange/2.0) ) res = true ;
		}
	return res ;
}
void NxCurve::setCircleColor( Color inColor )
{
	circleColor = inColor;
}
void NxCurve::setCenter( NxPoint inPoint )
{
	center = inPoint;
}
void NxCurve::setRadius( float inF )
{
	if(inF!=0) radius = inF;
}
void NxCurve::setTransparency( bool flag )
{
	transparency = flag;
}
void NxCurve::setTexturePath( QString inPath )
{
	texturePath = inPath;
}
void NxCurve::setShape( QString inShape )
{
	shape = inShape;
}
void NxCurve::setPoints( bool flag )
{
	showPoints = flag;
}
float NxCurve::reEvalRealMin( float inMin )
{	
	float rMin = inMin ;
	for(int i=0; i<size(); i++) {
		if (curves[i].y < rMin) rMin = curves[i].y ;
	}
	return rMin ;
}
float NxCurve::reEvalRealMax( float inMax )
{
	float rMax = inMax ;
	for(int i=0; i<size(); i++) {
		if (curves[i].y > rMax) rMax = curves[i].y ;
	}
	return rMax ;
}
void NxCurve::setRealValues( float inMin, float inMax )
{
	realMin = reEvalRealMin( inMin ) ;
	realMax = reEvalRealMax( inMax ) ;
	if(drawType == SplineStrip)
		splineDef->setRealValues( realMin, realMax ) ;
		
	yRange = fabs(realMax - realMin) ;
	updateBB( 0 ) ;
}
void NxCurve::setRealMin( float inMin )
{
	realMin = reEvalRealMin( inMin ) ;
	if(drawType == SplineStrip) 
		splineDef->setRealMin( realMin ) ;
		
	yRange = fabs(realMax - realMin) ;
	updateBB( 0 ) ;
}
void NxCurve::setRealMax( float inMax )
{
	realMax = reEvalRealMax( inMax ) ;
	if(drawType == SplineStrip) 
		splineDef->setRealMax( realMax ) ;
		
	yRange = fabs(realMax - realMin) ;
	updateBB( 0 ) ;
}
void NxCurve::setSplineMinMax()
{
	splineDef->setRealValues( realMin, realMax ) ;
}
void NxCurve::setAngle( float inAngle )
{
	angle = inAngle;
}
void NxCurve::setScale( float inScale )
{
	scale = inScale;
	if(drawType == SplineStrip) splineGenerate() ; // regenerate spline
}
void NxCurve::setHost( QString inHost )
{
	host = inHost ;
}
void NxCurve::setOscFormat( QString inOscFormat )
{
	oscFormat = inOscFormat ;
}
void NxCurve::setPort( int inPort ) 
{
	port = inPort ;
}
void NxCurve::setSpatialPeriod( int inP )
{
	spatialPeriod = inP ;
}
void NxCurve::setDrawType( int inType )
{
	drawType = inType;
	if (inType == SplineStrip) splineGenerate() ;
}

///////////
// GET 
///////////
QVector<NxPoint> NxCurve::getCurve()
{
	return curves;
}
NxSpline* NxCurve::getSpline()
{
	return splineDef;
}
Color NxCurve::getCircleColor()
{
	return circleColor;
}
NxPoint NxCurve::getCenter()
{
	return center;
}
NxPoint NxCurve::getEnd()
{
	NxPoint res = pos ;
	res.x += 2*radius*cos(angle*PI/180.0) ;
	res.y += 2*radius*sin(angle*PI/180.0) ;
	return res ;
}
float NxCurve::getRadius()
{
	return radius;
}
float NxCurve::getRealMin()
{
	return realMin;
}
float NxCurve::getRealMax()
{
	return realMax;
}
float NxCurve::getXRange()
{
	return xRange;
}
float NxCurve::getYRange()
{
	return yRange;
}
float NxCurve::getYRealRange()
{
	return yRange*scale;
}
float NxCurve::getScale()
{
	return scale ;
}
float NxCurve::getAngle()
{
	return angle;
}
QString NxCurve::getHost()
{
	return host;
}
QString NxCurve::getOscFormat()
{
	return oscFormat;
}
int NxCurve::getPort()
{
	return port;
}
int NxCurve::getSpatialPeriod()
{
	return spatialPeriod ;
}
QString NxCurve::getTexturePath()
{
	return texturePath;
}
QString NxCurve::getShape()
{
	return shape;
}
bool NxCurve::displayPoints()
{
	return showPoints;
}
int NxCurve::size()
{
	return nbPoints;
}
bool NxCurve::isTransparent()
{
	return transparency;
}
int NxCurve::getDrawType()
{
	return drawType;
}
float NxCurve::getLinearValue( float inTime )
{
	float res = 0 ; // initialisation
	int tempID = 0 ;
	float x1, x2, y1, y2 ;
	bool valueFound = false ;
	
	if( (inTime<0.0) || (inTime>curves[size()-1].x) ) {
		valueFound = true ;
		res = 0.0 ;
		}
		
	while( !valueFound ) {
		x1 = curves[tempID].x ;
		x2 = curves[tempID+1].x ;
		if( inTime <= x2 ) {
			valueFound = true ;
			y1 = curves[tempID].y ;
			y2 = curves[tempID+1].y ;
			res = (inTime-x1)*(y2-y1)/(x2-x1) + y1 ;
			}
		else tempID++ ;
		}

	return res ;
}

///////////////////////////////////////////////////////////////////////////////
// 	NxTraject
///////////////////////////////////////////////////////////////////////////////

//////////////////////////
// Constructors
//////////////////////////
NxTraject::NxTraject()
	: NxObject()
{}

/////////////////////////////////
// Main constructor
/////////////////////////////////
NxTraject::NxTraject(int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor,
				int inType, int inNbCursors, QList<int> inCursors )
	: NxObject( inID, incID, inGroupID, inLay, inName, inActive, inPos, inColor ) 
{
	type = inType;
	nbCursors = inNbCursors;
	cursors = inCursors;
	objectType = Trajectory;
	angle = length = 0.0;
}

//////////////////////
// Destructor
//////////////////////
NxTraject::~NxTraject() {}

/////////////////////////////
// Bounding Box
/////////////////////////////
void NxTraject::updateBB( float inSize )
{
	switch (type)
	{
		case 1: // Circles
			boundingBox.p1.x = pos.x - circ.radius;
			boundingBox.p1.y = pos.y + circ.radius;
			boundingBox.p1.z = 0.0;
	
			boundingBox.p2.x = pos.x + circ.radius;
			boundingBox.p2.y = pos.y - circ.radius;
			boundingBox.p2.z = 0.0;
			circ.p = pos;
		break;
		case 2 : // Lines
			float dX = pos.x - initPt.x ;
			float dY = pos.y - initPt.y ;
			
			initPt.x += dX ;
			initPt.y += dY ;
			endPt.x += dX ;
			endPt.y += dY ;
			
			boundingBox.p1.x = initPt.x ;
			boundingBox.p1.y = initPt.y + inSize ;
			boundingBox.p1.z = 0.0;
			
			boundingBox.p2.x = endPt.x ;
			boundingBox.p2.y = endPt.x - inSize ;
			boundingBox.p2.z = 0.0;
		break;
	}
}
bool NxTraject::isInside( float inX, float inY, float inBBsize )
{
	bool res = false ;
	switch( type) {
		case 1 : // Circle
			float distte = sqrt( (pos.x - inX)*(pos.x - inX)+(pos.y - inY)*(pos.y - inY) ) ;
			if( fabs(circ.radius-distte)<=inBBsize ) 
				res = true ;
		break ;
		case 2 : // Line
			NxPoint center ;
			center.ID = 0 ;
			center.z = 0 ;
			center.x = (initPt.x + endPt.x) / 2 ;
			center.y = (initPt.y + endPt.y) / 2 ;
			float trajAngle = angle*PI/180.0 ;
			// we change the reference...
			float newInX = inX - center.x ;
			float newInY = inY - center.y ;
			float aSin = sin(trajAngle) ;
			float aCos = cos(trajAngle) ;
			float localInX = aCos*newInX + aSin*newInY ;
			float localInY = -aSin*newInX + aCos*newInY ;
			if( fabs(localInY)<inBBsize && fabs(localInX)<length/2.0 ) 
				res = true ;
		break ;
	}
	return res ;
}
void NxTraject::updateAngle()
{
	if( type == 2 ) { // LINE
		if(endPt.x!=initPt.x) {
			if(endPt.x>=initPt.x && endPt.y>=initPt.y)
				angle = 180*atan((endPt.y-initPt.y)/(endPt.x-initPt.x))/PI ;
			else
			if(endPt.x>=initPt.x && endPt.y<=initPt.y)
				angle = 360 + 180*atan((endPt.y-initPt.y)/(endPt.x-initPt.x))/PI ;
			else
				angle = 180 + 180*atan((endPt.y-initPt.y)/(endPt.x-initPt.x))/PI ;
		}
		else {
			if(endPt.y>initPt.y)
				angle = 90.0 ;
			else
				angle = 270.0 ;
		}
	}
	else
		angle = 0 ;
}
void NxTraject::updateLength()
{
	if( type == 2 ) // Line
		length = sqrt( pow( endPt.x - initPt.x, 2.0 ) + pow( endPt.y - initPt.y, 2.0 ) );
	else
		length = 2*PI*circ.radius ;
}
///////////
// SET
///////////
void NxTraject::setCircleDef( Circle inC )
{
	circ = inC;
	updateLength() ;
} 
void NxTraject::setRadius( float inRadius )
{
	circ.radius = fabs(inRadius) ;
	updateLength() ;
} 
void NxTraject::setLength( float inLength )
{
	if(inLength!=0) {
		if( type == 2 ) { // Line
			float oldLength = this->getLength() ;
			NxPoint Pt1 = initPt ;
			NxPoint Pt2 = endPt ;
			float dx = Pt2.x - Pt1.x ;
			float dy = Pt2.y - Pt1.y ; 
			float a = oldLength / fabs(inLength) ;
			Pt2.x = Pt1.x + dx / a ;
			Pt2.y = Pt1.y + dy / a ;
			setLineDef( Pt1, Pt2 ) ;
		}
		else { // Circle
			setRadius( fabs(inLength) / (2*PI) ) ;
		}
	}
}
void NxTraject::setLineDef( NxPoint inInit, NxPoint inEnd )
{
	initPt	= inInit;
	endPt	= inEnd;
	
	updateLength() ;
	updateAngle() ;
}
void NxTraject::setInitialPoint( NxPoint inPt )
{
	initPt = inPt;

	updateLength() ;
	updateAngle() ;
}
void NxTraject::setFinalPoint( NxPoint inPt )
{
	endPt = inPt;

	updateLength() ;
	updateAngle() ;
}
void NxTraject::setType (int inType )
{
	type = inType;
}
void NxTraject::setCursorIDs( QList<int> inIDs )
{
	cursors = inIDs;
}
void NxTraject::setOrientation( int inOrient )
{
	orientation = inOrient;
}
void NxTraject::addChild( int ID )
{
	nbCursors++;
	cursors << ID;
}
void NxTraject::removeChild( int ID )
{
	nbCursors--;
	cursors.removeAll(ID);
}


///////////
// GET
///////////
Circle NxTraject::getCircle()
{
	return circ;
}
float NxTraject::getRadius()
{
	return circ.radius;
}
int NxTraject::getType()
{
	return type;
}
QList<int> NxTraject::getCursorIDs()
{
	return cursors;
}
int NxTraject::getNbCursors()
{
	return nbCursors;
}
NxPoint NxTraject::getInitialPoint()
{
	return initPt;
}
NxPoint NxTraject::getFinalPoint()
{
	return endPt;
}
float NxTraject::getLength()
{
	return length ;
}
float NxTraject::getAngle()
{
	return angle ;
}
int NxTraject::getOrientation()
{
	return orientation;
}

///////////////////////////////////////////////////////////////////////////////
//					NxTrigger
///////////////////////////////////////////////////////////////////////////////

/////////////////////////
// Constructor
/////////////////////////
NxTrigger::NxTrigger(int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor, QString inOscF, QString inHost, int inPort, int inIntValue)
	: NxObject( inID, incID, inGroupID, inLay, inName, inActive, inPos, inColor )
{
	color = inColor ;
	oscFormat = inOscF ;
	host = inHost;
	port = inPort;
	value = inIntValue ;
	objectType = Trigger;
}

NxTrigger::~NxTrigger() {}

void NxTrigger::updateBB( float inSize )
{
	boundingBox.p1.x = pos.x - inSize;
	boundingBox.p1.y = pos.y + inSize;
	boundingBox.p1.z = 0.0;
	
	boundingBox.p2.x = pos.x + inSize;
	boundingBox.p2.y = pos.y - inSize;
	boundingBox.p2.z = 0.0;
}
bool NxTrigger::isInside( float inX, float inY )
{
	bool res = false ;
	if ( ( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y ) )
		res = true ;
	return res ;
}
//////////////////////
// SET / GET
//////////////////////
void NxTrigger::setHost( QString inHost )
{
	host = inHost ;
}
void NxTrigger::setOscFormat( QString inOscFormat )
{
	oscFormat = inOscFormat ;
}
void NxTrigger::setPort( int inPort ) 
{
	port = inPort ;
}
void NxTrigger::setValue( int inValue ) 
{
	value = inValue ;
}
QString NxTrigger::getHost()
{
	return host;
}
QString NxTrigger::getOscFormat()
{
	return oscFormat;
}
int NxTrigger::getPort()
{
	return port;
}
int NxTrigger::getValue()
{
	return value;
}


///////////////////////////////////////////////////////////////////////////////
// 					NxCursor
///////////////////////////////////////////////////////////////////////////////
 
/////////////////////////
// Constructor
/////////////////////////
NxCursor::NxCursor( int inID, int incID, int inGroupID, int inLay, NxTraject* inParentTraj, QString inName, bool inActive, NxPoint inPos, Color inColor, bool inPlayF, bool inForward, int inMode, 
				State* inSpeedDef, State* inWidthDef, int inNSpeed, int inNWidth, float inWidth, float inInitOffset, float inFinalOffset)
	: NxObject( inID, incID, inGroupID, inLay, inName, inActive, inPos, inColor )
{
	forwardChanged = false ;
	
	objectType = 2;
	parentTrajectory	= inParentTraj ;
	parentID			= inParentTraj->getID() ;
	type				= inParentTraj->getType() ;
	forward	= inForward ;
	mode	= inMode;
	nbSpeed		= inNSpeed;
	nbWidth		= inNWidth;
	speedDef	= inSpeedDef;
	widthDef	= inWidthDef;
	
	speed = 1.0 ;
	width = inWidth;
	playFinished = inPlayF ;
	
	if(type==2) { // LINE
		customFinalOffset = true ;
		initOffset	= inInitOffset ;
		finalOffset = inFinalOffset ;
		offset		= initOffset ;
		reEvalOffsets() ;
	}
	else { // CIRCLE
		customFinalAngle = true ;
		initAngle	= inInitOffset ;
		finalAngle	= inFinalOffset ;
		angle		= initAngle ;
		reEvalAngles() ;
	}
	
	updateAnglePosition() ;
}

//////////////////////
// Destructor
//////////////////////
NxCursor::~NxCursor()
{
	delete &speedDef;
	delete &widthDef;
	delete &parentID;
}

/////////////////////////////
// Bounding Box
/////////////////////////////
bool NxCursor::isInside( float inX, float inY )
{
	bool res = false ;
	if (	(( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y )) ||
			(( inX >= boundingBox.p1.x && inX <= boundingBox.p2.x ) && ( inY <= boundingBox.p2.y && inY >= boundingBox.p1.y )) ||
			(( inX <= boundingBox.p1.x && inX >= boundingBox.p2.x ) && ( inY >= boundingBox.p2.y && inY <= boundingBox.p1.y )) || 
			(( inX <= boundingBox.p1.x && inX >= boundingBox.p2.x ) && ( inY <= boundingBox.p2.y && inY >= boundingBox.p1.y )) )
		res = true ;

	return res ;
}
void NxCursor::updateBB( float inSize )
{
	boundingBox.p1.x = pos.x - inSize;
	boundingBox.p1.y = pos.y + 2*inSize;
	boundingBox.p1.z = pos.z;
	
	boundingBox.p2.x = pos.x + inSize;
	boundingBox.p2.y = pos.y - 2*inSize;
	boundingBox.p2.z = pos.z;
}
void NxCursor::updateAnglePosition()
{
	if( type==1 ) { // CIRCLE
		pos = parentTrajectory->getCircle().p ;
		pos.x += parentTrajectory->getCircle().radius * cos(PI*angle/180) ;
		pos.y += parentTrajectory->getCircle().radius * sin(PI*angle/180) ;
		pos.z = 0 ;
	}
	else { // LINE
		float parentAngle = parentTrajectory->getAngle() ;
		angle = 90.0 + parentAngle ;
		if(forward) {
			pos.x = parentTrajectory->getInitialPoint().x + offset*cos( PI*parentAngle/180 );
			pos.y = parentTrajectory->getInitialPoint().y + offset*sin( PI*parentAngle/180 );
			pos.z = 0 ;
		}
		else {
			pos.x = parentTrajectory->getFinalPoint().x - offset*cos( PI*parentAngle/180 );
			pos.y = parentTrajectory->getFinalPoint().y - offset*sin( PI*parentAngle/180 );
			pos.z = 0 ;
		}
	}
}
///////////
// SET
///////////
void NxCursor::changeParent( int newID )
{
	parentID = newID;
}
void NxCursor::setSpeedDef( State* inSpeedDef )
{
	speedDef = inSpeedDef;
}
void NxCursor::setSpeed( float inSpeed )
{
	speed = inSpeed;
}
void NxCursor::setWidth( float inWidth )
{
	width = inWidth;
}
void NxCursor::setWidthDef( State* inWidthDef )
{
	widthDef = inWidthDef;
}
void NxCursor::setForward( bool inForward )
{
	if(forward!=inForward) { // Forward mode changed
		forwardChanged = true ;
		if( type==2 ) { // LINE
			float len = parentTrajectory->getLength() ;
			float newInitO	= len - finalOffset ;
			float newFinalO	= len - initOffset ;
			setInitOffset( newInitO ) ;
			setFinalOffset( newFinalO ) ;
		}
		else { // CIRCLE
			float newInitA = finalAngle ;
			float newFinalA = initAngle ;
			setInitAngle( newInitA ) ;
			setFinalAngle( newFinalA ) ;
			angle = adjustAngleMode( angle ) ;
		}
	}
	forward = inForward ;
}
void NxCursor::setMode( int inMode )
{
	mode = inMode;
}
void NxCursor::setChangedDirection( bool chooser ) 
{
	forwardChanged = chooser ;
}
void NxCursor::setOffset( float inOffset )
{
	if(inOffset>=0) offset = inOffset;
	updateAnglePosition() ;
}
void NxCursor::setAngle( float inAngle, bool reajust )
{
	angle = inAngle;
	
	// ModeReajusting : set angle in intervall [0, 360[, then set angle depending on mode
	if(reajust) angle = adjustAngleMode( angle ) ;

	updateAnglePosition() ;
}
void NxCursor::advanceAngle( float inVal )
{
	float rad = parentTrajectory->getCircle().radius ;
	setAngle( angle + inVal*180/(PI*rad), false ) ;
}
void NxCursor::advanceOffset( float inVal )
{
	setOffset( offset + inVal)  ;
}
void NxCursor::setInitOffset( float inOffset )
{
	if(inOffset>=0) initOffset = inOffset;
}
void NxCursor::setFinalOffset( float inOffset )
{
	if(inOffset>=0) finalOffset = inOffset;
	if( parentTrajectory->getLength()==finalOffset ) customFinalOffset = false ;
	else customFinalOffset = true ;
}
void NxCursor::setInitAngle( float inAngle )
{
	initAngle = inAngle ;
	angle = adjustAngleMode( angle ) ;
}
void NxCursor::setFinalAngle( float inAngle )
{
	finalAngle = inAngle ;
	angle = adjustAngleMode( angle ) ;
}
void NxCursor::reEvalOffsets()
{
	float len = parentTrajectory->getLength() ;
	if (initOffset>=len)	initOffset = 0.0 ;
	if (initOffset<0)		initOffset = 0.0 ;
	if (finalOffset>=len)	customFinalOffset = false ;
	if (finalOffset<=0)		customFinalOffset = false ;
	if(!customFinalOffset)	finalOffset = len ;
	if (initOffset>=finalOffset) {
		float temp = initOffset ;
		initOffset = finalOffset ;
		finalOffset = temp ;
	}
	if(offset>finalOffset)	offset = finalOffset ;
	if(offset<initOffset)	offset = initOffset ;
}
void NxCursor::reEvalAngles()
{
	initAngle	= adjustAngle(initAngle) ;
	finalAngle	= adjustAngle(finalAngle) ;
	
	if(initAngle==finalAngle)
		customFinalAngle = false ;
	else
		customFinalAngle = true ;
	
	if( customFinalAngle && !isInBounds(angle) )
		angle = returnNearBoundAngle( adjustAngle(angle) ) ;
}
float NxCursor::adjustAngle( float inAngle )
{
	float newA = inAngle ;
	if(newA==360) newA = 0 ;
	if(newA>360)
		newA = fmod(newA, 360) ;
	if(newA<0)
		newA = 360 + fmod(newA, 360) ;

	return newA ;
}
float NxCursor::adjustAngleMode( float inAngle )
{
	float newA = adjustAngle( inAngle ) ;
		
	if(!customFinalAngle) { // ALL CIRCLE CURSOR
		if(forward) {
			if(newA<initAngle) newA += 360 ;
		}
		else {
			if(newA>initAngle) newA -= 360 ;
		}
	}
	else { // ONLY PART OF CIRCLE
		bool I_F = (finalAngle>initAngle) ;
		bool F_I = (finalAngle<initAngle) ;
		if(forward && F_I && newA<initAngle)	newA += 360 ;
		if(!forward && I_F && newA>initAngle)	newA -= 360 ;
	}
	
	return newA ;
}
bool NxCursor::isInBounds( float inAngle )
{
	bool res = true ;
	
	bool I_F = (finalAngle>initAngle) ;
	bool F_I = (finalAngle<initAngle) ;
	///////////// Must be in interval [ I , F ]
	float I = initAngle;
	float F = initAngle;
	if(I_F) {
		if(forward) { F=finalAngle; }
		else		{ I=finalAngle-360; }
	}
	if(F_I) {
		if(forward) { F=finalAngle+360; }
		else		{ I=finalAngle; }			
	}
		
	if( inAngle<I || inAngle>F ) res = false ; 
	
	return res ;
}
float NxCursor::returnNearBoundAngle( float inAngle )
{
	float finalToReturn = finalAngle ;
	
	bool I_F = (finalAngle>initAngle) ;
	bool F_I = (finalAngle<initAngle) ;
	if( forward && F_I )	finalToReturn +=360 ;
	if( !forward && I_F )	finalToReturn -=360 ;
	
	float dInit = fabs(inAngle-initAngle) ;
	float dFinal = fabs(inAngle-finalAngle) ;
	if(dInit>180)	dInit = 360 - dInit ;
	if(dFinal>180)	dFinal = 360 - dFinal ;
	if(dInit<=dFinal)	return initAngle ;
	else				return finalToReturn ;
}
void NxCursor::hardOffsetsReset()
{
	if(type==2) { // LINE
		initOffset = 0.0 ;
		finalOffset = ((NxTraject*)parentTrajectory)->getLength() ;
		customFinalOffset = false ;
	}
	else { // CIRCLE
		initAngle = 0.0 ;
		finalAngle = 0.0 ;
		customFinalAngle = false ;
	}
}
void NxCursor::setPlayFinished( bool chooser )
{
	playFinished = chooser ;
}
void NxCursor::setParentTrajectory( NxTraject* newParent )
{
	parentTrajectory = newParent;
}


///////////
// GET
///////////
int NxCursor::getParentID()
{
	return parentID;
}
State* NxCursor::getSpeedCurve() 
{	
	return speedDef;
}
State* NxCursor::getWidthCurve()
{
	return widthDef;
}
bool NxCursor::getForward()
{
	return forward ;
}
int NxCursor::getMode()
{
	return mode ;
}
int NxCursor::getType()
{
	return type ;
}
bool NxCursor::hasChangedDirection() 
{
	return forwardChanged ;
}
int NxCursor::getNbSpeed()
{
	return nbSpeed;
}
///////////////////////////////////////////////
// Speed / Width Evaluator
///////////////////////////////////////////////
float NxCursor::getSpeedWidth( float inTime, int chooser )
{
	// Local variables
	int nbStates;
	State* timeline = NULL;
	int i = 0;
	int millisTime = (int)( inTime*1000 );
	float result = 0.0;
	
	// Return 0.0 if no timeline def -> temp
	if ( (nbWidth == 0) && (chooser == 0) )
		return 0.0;
		
	if ( (nbSpeed == 0) && (chooser == 1) )
		return 0.0;
		
	// Choose between speed 1 / width 0
	if (chooser == 0)
	{
		timeline = widthDef;
		nbStates = nbWidth;
		result = width;
	}
	else
	{
		timeline = speedDef;
		nbStates = nbSpeed;
		result = speed;
	}
		float globalTime = timeline[0].duration;
		
	// Searching State
	while( i<nbStates && globalTime < millisTime )
	{
		i++;
		globalTime += timeline[i].duration;
	}
	
	///////////////////////
	// Evaluating 
	///////////////////////
	if (timeline[i].type == 1)
		result = (timeline[i].coeff * millisTime) / 1000.0;
		
	if (timeline[i].type == 2)
		result += cos(millisTime) * timeline[i].scale; 
	
	if (timeline[i].type == 3)
		result += sin(millisTime) * timeline[i].scale;
	
	if (timeline[i].type == 4)
		result += log( millisTime - (globalTime - timeline[i].duration) ) * timeline[i].scale;
	
	return result;
}

int NxCursor::getNbWidth()
{
	return nbWidth;
}
float NxCursor::getWidth()
{
	return width;
}
float NxCursor::getSpeed()
{
	return speed;
}
float NxCursor::getOffset()
{
	return offset;
}
float NxCursor::getAngle()
{
	return angle;
}
float NxCursor::getNormalizedAngle()
{
	if(type==2) // LINE
		return angle ;
	else // CIRCLE
		return adjustAngle(angle) ;
}
float NxCursor::getInitOffset()
{
	return initOffset;
}
float NxCursor::getFinalOffset()
{
	return finalOffset;
}
float NxCursor::getInitAngle()
{
	return initAngle;
}
float NxCursor::getFinalAngle()
{
	return finalAngle;
}
bool NxCursor::hasCustomFinalOffset()
{
	return customFinalOffset;
}
bool NxCursor::hasCustomFinalAngle()
{
	return customFinalAngle;
}
bool NxCursor::getPlayFinished() 
{
	return playFinished ;	
}
NxTraject* NxCursor::getParentTrajectory()
{
	return parentTrajectory;
}
NxPoint NxCursor::getInitOffsetPoint()
{
	if( type==2 ) { // LINE
		NxPoint tempP ;
		float trajAng = parentTrajectory->getAngle()*PI/180 ;
		if(forward) {
			tempP = parentTrajectory->getInitialPoint() ;
			tempP.x += initOffset*cos(trajAng) ;
			tempP.y += initOffset*sin(trajAng) ;
		}
		else {
			tempP = parentTrajectory->getFinalPoint() ;
			tempP.x -= initOffset*cos(trajAng) ;
			tempP.y -= initOffset*sin(trajAng) ;
		}
		return tempP ;
	}
	else { // CIRCLE
		NxPoint tempP	= parentTrajectory->getCircle().p ;
		float trajRad	= parentTrajectory->getCircle().radius ;
		tempP.x += trajRad*cos(initAngle*PI/180) ;
		tempP.y += trajRad*sin(initAngle*PI/180) ;
		return tempP ;
	}
}
NxPoint NxCursor::getFinalOffsetPoint()
{
	if( type==2 ) { // LINE
		NxPoint tempP ;
		float trajAng = parentTrajectory->getAngle()*PI/180 ;
		if(forward) {
			tempP = parentTrajectory->getInitialPoint() ;
			tempP.x += finalOffset*cos(trajAng) ;
			tempP.y += finalOffset*sin(trajAng) ;
		}
		else {
			tempP = parentTrajectory->getFinalPoint() ;
			tempP.x -= finalOffset*cos(trajAng) ;
			tempP.y -= finalOffset*sin(trajAng) ;		
		}
		return tempP ;
	}
	else { // CIRCLE
		NxPoint tempP	= parentTrajectory->getCircle().p ;
		float trajRad	= parentTrajectory->getCircle().radius ;
		tempP.x += trajRad*cos(finalAngle*PI/180) ;
		tempP.y += trajRad*sin(finalAngle*PI/180) ;
		return tempP ;
	}
}
