///////////////
// IanniX
///////////////

////////////////////////////
// NxObject.h
////////////////////////////

#ifndef NXOBJECT_H
#define NXOBJECT_H

#include <QtCore>
#include "NxTypes.h"
#include "../../graphics/math/NxSpline.h"

class NxObject
{
 public:
	// Constructors
  	NxObject();
  	NxObject( int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor );
	
  	// Access methods
  	void 	setCustomID( int inID );
  	void 	setGroupID( int inGroupID );
  	void 	setName( QString inName );
  	void 	setPosition( NxPoint inPos );
  	void 	setColor( Color inColor );
	
  	int 	getID();
  	int 	getCustomID();	
  	int 	getGroupID();
  	QString getName();
  	NxPoint getPosition();
  	Color 	getColor();
  	bool 	isSelected();	
	bool	isHighlighted() ;
	bool	isActive() ;
	int		getLayer() ;
  	int 	getObjectType() ;
	
  	// Bounding box handle
  	void 	setBoundingBox( NxPoint inPt1, NxPoint inPt2 );
  	void 	setBox( BBox inBox );
  	BBox 	getBoundingBox();
	
	virtual void	updateBB( float inSize );
	
  	void 	setSelected( bool flag );
	void	setHighlighted( bool flag ) ;
	void	setActive( bool flag ) ;
	void	setLayer( int inLay ) ;
	
  	// Destructor
  	virtual ~NxObject();
	
 protected:
	// Bounding box coordinates
  	BBox 	boundingBox;
  	NxPoint pos;
  	int 	objectType;
	Color	color ;
	int		customID ;
	
 private:
	int 	ID;
  	int 	groupID;
	int		layer ;
  	QString name;
	bool 	selected ;		// Default = false
	bool	highlighted ;	// Default = false
	bool	active ;		// Default = true
};

// **************************
// Inheriting class : NxCurve
// **************************
class NxCurve : public NxObject
{
 public:
	// Constructors
  	NxCurve();
  	NxCurve( int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor, QVector<NxPoint> inCurves, int inNbPoints, 
		Color inCircleColor, bool inTrans, QString inTexPath, QString inShape, float inRMin, float inRMax, float inScale, float inAngle, int inSpatialPeriod, QString inOscF, QString inHost, int inPort );  
	
  	// Destructor
  	~NxCurve();
	
  	// Access methods
  	void 	setCurve( QVector<NxPoint> inCurve );						
  	void 	setCenter( NxPoint inPoint );
  	void 	setRadius( float inF );
  	void 	setCircleColor( Color inColor );	
  	void 	setTransparency( bool flag );
  	void 	setTexturePath( QString inPath );
  	void 	setShape( QString inShape );
  	void 	setPoints( bool flag );
  	void 	updateBB( float inSize );
	void	splineGenerate() ;
	void 	setDrawType( int inType );
	float	reEvalRealMin( float inMin) ;
	float	reEvalRealMax( float inMax) ;
	void 	setRealValues( float inMin, float inMax );
	void	setRealMin( float inRMin ) ;
	void	setRealMax( float inRMax ) ;
	void	setSplineMinMax() ;
	void	setScale( float inScale ) ;
	void	setAngle( float inAngle ) ;
	void	setHost( QString inHost ) ;
	void	setOscFormat( QString inOscFormat ) ;
	void	setPort( int inPort ) ;
	void	setSpatialPeriod( int inP ) ;
	
  	QVector<NxPoint> getCurve();
  	NxSpline* getSpline();
  	Color 	getCircleColor();
  	QString getTexturePath();
  	QString getShape();
  	NxPoint getCenter();
	NxPoint getEnd() ;
  	float 	getRadius();
	float 	getRealMin();
	float 	getRealMax();
	float	getXRange() ;
	float	getYRange() ;
	float	getYRealRange() ;
	float 	getScale();
	float 	getAngle();
  	QString getHost();
	QString getOscFormat();
	int		getPort();
	int		getSpatialPeriod() ;
  	int 	size();
  	bool 	isTransparent();
  	bool 	displayPoints();
	int 	getDrawType();
	
	bool	isInside( float inX, float inY ) ;
	
	float	getLinearValue( float inTime ) ; //// get simple linear value (for scheduler)
	
 protected:
	
 private:
	int		spatialPeriod ; //// in ms
	
	QString	host, oscFormat;
	int		port;
  	int 	nbPoints;
  	int 	drawType;
	bool 	transparency;
  	bool 	showPoints;
  	QString shape; 
  	QString texturePath;
  	Color	circleColor;
  	QVector<NxPoint> curves;	
  	NxSpline* splineDef;
	bool	splineGenerated ; // true if splinedef already build
  	NxPoint center;
  	float 	radius, dX, dY, dZ;
	float 	realMin, realMax;
	float	xRange ;
	float	yRange ;
	float	scale ; // if scale = 5 : 1 s represents 5 units (curve values)
	float	angle ; // simple rotation around Z axis
};


// ****************************
// Inheriting class : NxTraject
// ****************************
class NxTraject : public NxObject
{
 public:
	// Constructors
  	NxTraject();	// Void constructor
  	NxTraject( int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor, int inType, int nbCursors, QList<int> inCursors);
	
  	// Destructor
  	~NxTraject();
		
  	// Access methods
  	int		getType();
  	QList<int> getCursorIDs();
  	int		getNbCursors();
  	int		getOrientation();
	Circle	getCircle();
	float	getRadius() ;
  	NxPoint getInitialPoint();
  	NxPoint getFinalPoint();	
	float	getLength() ;
	float	getAngle() ;
	
	void	setRadius( float inRadius) ;
  	void	setCircleDef( Circle inC ) ;
	void	setLength( float inLength ) ;
  	void	setLineDef( NxPoint inInit, NxPoint inEnd );
  	void	setInitialPoint( NxPoint inPt );
  	void	setFinalPoint( NxPoint inPt );
  	void	setType( int type );
  	void	setCursorIDs( QList<int> inIDs );
  	void	addChild( int ID );
  	void	removeChild( int ID );
	
	void	updateAngle() ;
	void	updateLength() ;
  	void	updateBB( float inSize );
	void	setOrientation( int inOrient );
	
	bool	isInside( float inX, float inY, float inBBsize ) ;
			
 protected:
	
 private:
	// NxTraject attributes
  	int nbCursors;
	
  	// IDs of the associated cursors
  	QList<int> cursors;
	
  	// Traject definition
  	int type, orientation;
	
  	QVector<NxPoint> curves ;
  	Circle circ;				// if Circle -> Type = 1	
  	NxPoint initPt, endPt;		// Straight Line -> Type = 2  
	float angle, length;
};


// *************************
// Inheriting class : NxTrigger
// *************************
class NxTrigger : public NxObject
{
 public:
	// ************
  	// Constructors
  	// ************
  	NxTrigger();
  	NxTrigger( int inID, int incID, int inGroupID, int inLay, QString inName, bool inActive, NxPoint inPos, Color inColor, QString inOscF, QString inHost, int inPort, int inIntValue);
	
  	// **********
  	// Destructor
  	// **********
  	~NxTrigger();
		
  	// ***************
  	// Access methods
  	// ***************	
	void	setHost( QString inHost ) ;
	void	setOscFormat( QString inOscFormat ) ;
	void	setPort( int inPort ) ;
	void	setValue( int inValue ) ;
  	QString getEventType();
  	QString getHost();
	QString getOscFormat();
	int		getPort();
	int		getValue()  ;
  	void	updateBB( float inSize );
	
	bool	isInside( float inX, float inY ) ;
	
 protected:
	
 private:
	// Trigger specific attributes
  	QString host, oscFormat;
	int port;
	float TriggerSize;
	int value ;
	
	// Circles for collision (fixed)
	QList<Circle> circles;
	int nbCircles;
	
	// Cursors
	QList<bool> colliders;
};


// ************************
// Inheriting class : NxCursor
// ************************
class NxCursor : public NxObject
{
 public:
	// Constructors
	NxCursor( int inID, int incID, int inGroupID, int inLay,  NxTraject* inParentTraj, QString inName, bool inActive, NxPoint inPos, Color inColor, bool inPlayF, bool inForward, int inMode,
		State* inSpeedDef, State* inWidthDef, int inNSpeed, int inNWidth, float inWidth, float inInitOffset, float inFinalOffset);
  	
	// Destructor
  	~NxCursor();
		
  	// Access methods
  	void changeParent( int newID );																
  	void setSpeedDef( State* inSpeedDef );
  	void setWidthDef( State* inWidthDef );
	void setMode( int inMode );
	void setForward( bool inForward );
	void setChangedDirection( bool chooser ) ;
	void setSpeed( float inSpeed );
	void setWidth( float inWidth );
	
  	void setOffset( float inOffset );
	void setAngle( float inAngle, bool reajust ) ;
	void advanceAngle( float inVal ) ;
	void advanceOffset( float inVal ) ;
	void setInitOffset( float inOffset );
	void setFinalOffset( float inOffset );
	void setInitAngle( float inAngle );
	void setFinalAngle( float inAngle );
	
	float	adjustAngle( float inAngle ) ;
	float	adjustAngleMode( float inAngle ) ;
	
	void	reEvalOffsets() ;
	void	reEvalAngles() ;
	bool	isInBounds( float inAngle ) ;
	float	returnNearBoundAngle( float inAngle) ;
	
	void hardOffsetsReset() ;
	void setPlayFinished( bool chooser ) ;
  	void setParentTrajectory( NxTraject* newParent );
	
  	int getParentID();
  	State* getSpeedCurve();
  	State* getWidthCurve();
  	int 	getMode() ; // 0/1/2 : single/loop/f_b
	bool 	getForward() ; // forward or backward
	bool	hasChangedDirection() ;
  	int 	getNbSpeed();
  	float 	getSpeedWidth( float inTime, int chooser );
	int 	getNbWidth();
	float 	getSpeed();
	float 	getWidth();
  	NxTraject* getParentTrajectory();
	int		getType() ;
	
  	float 	getOffset();
	float 	getAngle();
	float	getNormalizedAngle() ;
	
	float 	getInitOffset();
	float 	getFinalOffset();
	float 	getInitAngle();
	float 	getFinalAngle();
	
	bool 	hasCustomFinalOffset() ;
	bool	hasCustomFinalAngle() ;
	
	bool 	getPlayFinished() ;
	
	NxPoint	getInitOffsetPoint() ;
	NxPoint	getFinalOffsetPoint() ;
	
  	void 	updateBB( float inSize );
	void	updateAnglePosition() ;
	
	bool	isInside( float inX, float inY ) ;
 
 protected:
	
 private:
	// Cursor attributes
  	NxTraject* parentTrajectory;
  	int		parentID;
  	int		nbSpeed;
  	int		nbWidth;
	bool	forward ;	// forward or backward
	int		mode ;		// single / loop / f_b
	
	int		type ; // circle cursor / line cursor
	
	bool	forwardChanged ; // true if cursor changed forward/backward mode
	
	bool 	customFinalOffset, customFinalAngle ;
	bool 	playFinished ;
	
	// Timing values
	float speed, width;
	State* speedDef;
 	State* widthDef;
	
  	float offset;	// For lines
	float angle;	// For circles
	float initOffset, finalOffset ;		// For lines
	float initAngle, finalAngle ;		// For circles
};

#endif

