///////////////
// IanniX
///////////////

////////////////////////////
// NxScore.h
////////////////////////////

#ifndef NXSCORE_H
#define NXSCORE_H

#include <QtCore>

#include "NxTypes.h"
#include "../../xml/NxParser.h"

class NxScore : public QObject
{
 	Q_OBJECT
public:
	/////////////////////	
  	// Constructors
  	/////////////////////	
  	NxScore( QString inName, int inID, int inLength, int inHeight );
  	NxScore( QString inXMLpath );

  	/////////////////////
  	// Destructor
  	////////////////////
  	~NxScore();
	
  	//////////////////////////////
  	// Access methods
  	/////////////////////////////
  	void setEnvData( NxEnv inEnv );
  	void setCurves();			// parses and sets all new curves
  	void setCursors();			// same, with cursors
  	void setTrajects();			// same, with trajects
  	void setTriggers();			// same, with triggers
	void setTempo( float inTempo );
	void setDisplayStep( int inDisStep ) ;
	void setSchedulerStep( int inSchStep ) ;

  	NxCurve* 	getCurve( int inIndex );
  	NxTraject* 	getTraject( int inIndex );
  	NxCursor* 	getCursor( int inIndex );
  	NxTrigger* 	getTrigger( int inIndex );
  	
	QList<int> 	curvesIndexes();
	QList<int>	trajectsIndexes();
	QList<int> 	cursorsIndexes();
	QList<int> 	triggersIndexes();
	
	void maxOfIndexes() ;

	NxEnv getEnvironment();	
	float getTempo();
	
  	void addTrigger( NxPoint inPos, int ID, int incID, int groupID, int inLay, QString inName, Color inColor, QString inOscF, QString inHost, int inPort, int inIntValue);
  	void addCurve( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor1, Color inColor2, QVector<NxPoint> inCurveDef, int nbCtrlPoints, float inRMin, float inRMax, float inScale, float inAngle, int inSpatialPeriod, QString inOscF, QString inHost, int inPort );
  	void addTraject( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, int inType );
  	void addCursor( NxPoint inPos, int inID, int incID, NxTraject* inParent, int inGroupID, int inLay, QString inName, Color inColor, bool inForward, int inMode, float nWidth, float inInitOffset, float inFinalOffset );
  	void removeObject( int inID, int inType );
	
  	//////////////////////////////////
  	// Output XML writing
  	/////////////////////////////////
  	void saveToXML( QString inFilename );
  	void writeCursor( int ind );
  	void writeEnvironment();
  	void writeCurves();
  	void writeTrajects();
  	void writeTriggers();
		
  	/////////////////////////////////////
  	// Objects lists indexes
  	////////////////////////////////////
  	int NbCurves;
  	int NbCursors;
  	int NbTrajects;
  	int NbTriggers;
	/////////////////////////////////////
  	// Last indexes
  	////////////////////////////////////
	int cptCurve, cptTraject, cptCursor, cptTrigger ;	
	int getCptTrigger() ;
	int getCptTraject() ;
	int getCptCursor() ;
	int getCptCurve() ;
	
	bool isThereTrigger( int inIndex ) ;
	bool isThereTraject( int inIndex ) ;
	bool isThereCursor( int inIndex ) ;
	bool isThereCurve( int inIndex ) ;
			  
 protected:
		
	NxEnv envData;
		
 private:
  	
  	// NxScore initializes and holds every object's data in private structures
  	QMap<int, NxCurve*> 	curves;
  	QMap<int, NxCursor*> 	cursors;
	QMap<int, NxTraject*> 	trajects;
  	QMap<int, NxTrigger*>	triggers;			
	
  	// Output
  	QFile 		outFile;
	QTextStream out;
	
  	// Control variables
  	int lastRemoved;
	
  	// Playback variables
  	float globalTempo;		// Main parameter for playback -> Default value : 1.0
};

#endif

