///////////////
// IanniX
///////////////

////////////////////////////
// NxScheduler.cpp
////////////////////////////

#include "NxScheduler.h"

// Time lib
#include <unistd.h>
#include <math.h>

//////////////////////////////////
// THREAD CLASS
//////////////////////////////////

// Timing Scheduler for IanniX
/////////////////////////////////////////////////////

// THREADING PART
//////////////////////////////////////

///////////////////
// Constructor
///////////////////
NxScheduler::NxScheduler( NxGLScore *inGLBox )
{
	mainGLBox	= (NxGLScore*)inGLBox;
	score		= mainGLBox->getPartition() ;
	
	nbPackets 	= 0 ;
	
  	pCur 		= NULL;
	finished 	= true;
  	playing 	= false;

	mainTempo = (float)score->getEnvironment().globalTempo ;
	
	curvesOSCmF = new char[100] ;
	triggersOSCmF = new char[100] ;
	
	clock = QTime(0,0,0,0) ;
	useCirclesAlgo = false ;
	sendInitialFinalMessage = false ;
	advanceCursors = true ;
	
	addTrigger = addCurve = false ;
	readyToPlay = false ;

	updateObjectsIndexes() ;
	
	////////////////////////////////////////////// SLOTS to send osc messages to the IanniX Console
	qRegisterMetaType<curveOSC>();
	qRegisterMetaType<triggerOSC>();
	
	//sTimer = new QTimer( this );
}

//////////////////
// Destructor
//////////////////
NxScheduler::~NxScheduler()
{}
////////////////////////////
// Playback Parameters
////////////////////////////
void NxScheduler::useAutoSchedulerStep( bool chooser )
{
	autoSchedulerStep = chooser ;
}
bool NxScheduler::isAutoSchedulerStep() 
{
	return autoSchedulerStep ;
}
void NxScheduler::useCirclesAlgorithm( bool chooser, bool withReset )
{
	if ( chooser != useCirclesAlgo && withReset ) resetCurves() ;
	useCirclesAlgo = chooser ;
}
void NxScheduler::sendIFmessages( bool chooser )
{
	sendInitialFinalMessage = chooser ;
}
void NxScheduler::setOSCOutActive( bool chooser )	
{
	oscActive = chooser;
}
void NxScheduler::useBroadcastMode( bool inBroad )
{
	broadcastMode = inBroad ;
}
void NxScheduler::moveCursors( bool chooser )
{
	advanceCursors = chooser ;
}
////////////////////////////
// Time Settings
////////////////////////////
void NxScheduler::setPlaying( bool chooser )
{
	playing = chooser;
}
bool NxScheduler::isPlaying()
{
	return playing ;
}
void NxScheduler::changeTempo( float inTempo )
{
	mainTempo = inTempo;
}
void NxScheduler::setCurrentTime( float inTime )
{
	currentTime = inTime ;
}
void NxScheduler::setInitialTime( float inStartTime )	
{
	initialTime = inStartTime;
}
void NxScheduler::setFinalTime( float inStopTime )	
{
	finalTime = inStopTime;
}
void NxScheduler::resetTime()
{
	currentTime = initialTime ;
}
float NxScheduler::getCurrentTime()
{
	return currentTime ;
}
float NxScheduler::getInitialTime()
{
	return initialTime ;
}
float NxScheduler::getFinalTime()
{
	return finalTime ;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
///////////// UPDATE OBJECTS NUMBERS
///////////////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::updateObjectsIndexes()
{
	score = mainGLBox->getPartition();
	cursInd = score->cursorsIndexes() ;
	curvInd = score->curvesIndexes() ;
	trigInd = score->triggersIndexes() ;
	trajInd = score->trajectsIndexes() ;
	nbCursors	= cursInd.count() ;
	nbCurves	= curvInd.count() ;
	nbTriggers	= trigInd.count() ;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////// INITIALISATION
///////////////////////////////////////////////////////////////////////////////////////////////////
bool NxScheduler::isReadyToPlay() 
{
	return readyToPlay ;
}
void NxScheduler::setReadyToPlay( bool chooser ) 
{
	readyToPlay = chooser ;
}
void NxScheduler::initScheduler() // Called only at the begining and reset
{	
	readyToPlay = false ;
	currentTime = initialTime ;
	
	/////////////// INITIAL TIME SET - Here we shift all the cursors
	if(currentTime!=0.0 && mainTempo>0.0) {
		NxCursor* cur ;
		float len, initO, finalO, initA, finalA ;
		bool forw ;
		float tmpOffset, Offset, Angle ;
	
		for(int i=0; i<cursInd.count(); i++) {
			cur = score->getCursor( cursInd.at(i) ) ;
			forw = cur->getForward() ;
			if(cur->getType()==2) { // LINE
				len = cur->getParentTrajectory()->getLength() ;
				initO	= cur->getInitOffset() ;
				finalO	= cur->getFinalOffset() ;
				tmpOffset	= initO + initialTime*cur->getSpeed()*fabs(mainTempo) ;
				if( tmpOffset < finalO )	Offset = tmpOffset ;
				else						Offset = finalO ;
			
				if(forw) cur->setOffset( Offset ) ;
				else cur->setOffset( len - Offset ) ;
			}
			else {
				float rad = cur->getParentTrajectory()->getCircle().radius ;
				initA	= cur->getInitOffset() ;
				finalA	= cur->getFinalOffset() ;
				if(forw)	Angle	= initA + initialTime*cur->getSpeed()*fabs(mainTempo)*180/(PI*rad) ;
				else		Angle	= initA - initialTime*cur->getSpeed()*fabs(mainTempo)*180/(PI*rad) ;
				cur->setAngle( Angle, true ) ;
				cur->reEvalAngles() ;	
			}
		}
	}
	/////////////////////////////////////////////////////////////////
	
	// Initializing OSC buffer
 	b = &myBuf;
 	OSC_initBuffer(b, 1000, bytes);	
	
	resetSchedule() ;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////// LIVE OBJECT ADDING
///////////////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::addNewCursor()
{
	resetSchedule() ;
}
void NxScheduler::addNewCurve()
{
	if(!playing) resetSchedule() ;
	else addCurve = true ;
}
void NxScheduler::doAddNewCurve()
{
	mutex.lock() ;
	for(int i=0; i<cursInd.count(); i++) {	
		QList<float>	tmpValCTimers		= curvesTimers.at(i) ;	
		QList<float>	tmpValCDurations	= curvesDurations.at(i) ;
		QList<bool>		tmpValCCollisions	= curvesCollisions.at(i) ;	
		QList<float>	tmpValCLocalStep	= curvesLocalStep.at(i) ;			
		tmpValCTimers		<< 0.0 ;
		tmpValCDurations	<< 5.0 ;
		tmpValCCollisions	<< false ;
		tmpValCLocalStep	<< 0.0 ;
		curvesTimers.replace( i , tmpValCTimers ) ;
		curvesDurations.replace( i , tmpValCDurations ) ;
		curvesCollisions.replace( i , tmpValCCollisions ) ;
		curvesLocalStep.replace( i , tmpValCLocalStep ) ;
	}
	updateObjectsIndexes() ; // nbCurves actualized (+= 1)
	mutex.unlock() ;
}
void NxScheduler::addNewTrigger()
{
	if(!playing) resetSchedule() ;
	else addTrigger = true ;
}
void NxScheduler::doAddNewTrigger()
{
	mutex.lock() ;
  	for(int i=0; i<cursInd.count(); i++) {	
		QList<bool> tmpList1 = triggersSign1.at(i) ;
		QList<bool> tmpList2 = triggersSign2.at(i) ;
		tmpList1 << true ; // adding a value
		tmpList2 << true ; // adding a value
		triggersSign1.replace( i , tmpList1 ) ;
		triggersSign2.replace( i , tmpList2 ) ;
	}
	updateObjectsIndexes() ; // nbTriggers actualized (+= 1)
	firstShot = true ;
	mutex.unlock() ;
}
void NxScheduler::setFirstShot( bool chooser )
{
	firstShot = chooser ;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////// RESET ALL
///////////////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::resetSchedule() // Reinitialize arrays
{
	nbPackets = 0 ;
	
	mutex.lock() ;
	
	updateObjectsIndexes() ;
	
	resetCursors() ;
	resetCurves() ;
	resetTriggers() ;
	resetOSC() ;
	
	mutex.unlock() ;
	
	readyToPlay = true ;
}
////////////////////////////////////// Reset Cursors
void NxScheduler::resetCursors()
{	
	curLoopShot.clear() ;
	curDirections.clear() ;
	cursorsFinished.clear() ;
		
	for(int i=0; i<cursInd.count(); i++) {
		curDirections << ((NxCursor*)score->getCursor( cursInd.at(i) ))->getForward() ; // direction at time t
		cursorsFinished << false ;
		curLoopShot << false ;
	}
}
////////////////////////////////////// Reset Curves
void NxScheduler::resetCurves()
{	
	curvesTimers.clear() ;
	curvesDurations.clear() ;
	curvesCollisions.clear() ;
	curvesLocalStep.clear() ;
	
	resetSplines() ;
	
	for(int i=0; i<cursInd.count(); i++) {	
		QList<float>	valCTimers ;	
		QList<float>	valCDurations ;
		QList<bool>		valCCollisions ;	
		QList<float>	valCLocalStep ;			
		for (int j=0; j<curvInd.count(); j++) {
			valCTimers		<< 0.0 ;
			valCDurations	<< 5.0 ;
			valCCollisions	<< false ;
			valCLocalStep	<< 0.0 ;
		}
		curvesTimers		<< valCTimers ;
		curvesDurations		<< valCDurations ;
		curvesCollisions	<< valCCollisions ;
		curvesLocalStep		<< valCLocalStep ;
	}
}
////////////////////////////////////// Reset Splines
void NxScheduler::resetSplines()
{	
	for( int i=0; i<curvInd.count();i++ ) {		
		if( score->getCurve(curvInd.at(i))->getDrawType() == SplineStrip )
			score->getCurve(curvInd.at(i))->getSpline()->reset();
	}
}
////////////////////////////////////// Reset Triggers
void NxScheduler::resetTriggers()
{	
	triggersSign1.clear() ; /////////// trigger collisions / sign tests
	triggersSign2.clear() ; /////////// trigger collisions / sign tests
		
  	for(int i=0; i<cursInd.count(); i++) {	// for all cursors
		QList<bool> valTBool1 ;
		QList<bool> valTBool2 ;	
		for(int k=0; k<trigInd.count(); k++) { // for all triggers
			valTBool1 << true ;
			valTBool2 << true ;	
		}
		triggersSign1 << valTBool1  ;
		triggersSign2 << valTBool2  ;
	}
}
////////////////////////////////////// Reset OSC
void NxScheduler::resetOSC()
{	
	triggersMessages.clear() ;
	curvesMessages.clear() ;
}
///////////////////////////////////////////////////////////////////////////////////////////
///////// CONFIG / INITIAL / FINAL OSC MESSAGES
///////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::sendOSCConfigMessage()
{
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	
	printf("Sending OSC Configuration Message (%s:%d)\n", tmpEnv.OSCConfigMessageIP.toAscii().constData(), tmpEnv.UDPout) ;
	
	char* charOSC1 = new char[100];
	QString configOSC = tmpEnv.OSCConfigMessageIP  ;
	strncpy(charOSC1, configOSC.toAscii(), 99);
	envClientConfigUDP = new NxNetwork( charOSC1, tmpEnv.UDPout );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/dimensions",",ii");
	OSC_writeIntArg( b, tmpEnv.length ) ;
	OSC_writeIntArg( b, tmpEnv.height ) ;
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false ) ;
	OSC_resetBuffer( b ) ;
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/time",",f");
	OSC_writeFloatArg( b, currentTime ) ;		
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false ) ;
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/cursors",",i");
	OSC_writeIntArg( b, tmpEnv.nbCursors ) ;
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	NxCursor* curs ;
	for(int k=0 ; k<tmpEnv.nbCursors ; k++) { // CURSORS
		curs = score->getCursor( cursInd.at(k) ) ;
		int forwInt ;
		if(curs->getForward()) forwInt = 1 ;
		else forwInt = 0 ;
		OSC_writeAddressAndTypes(b, "/iannix/score/cursor",",iiiffii");
		OSC_writeIntArg( b, curs->getID() ) ;			// ID
		OSC_writeIntArg( b, curs->getCustomID() ) ;		// Custom ID
		OSC_writeIntArg( b, curs->getGroupID() ) ;		// Group ID
		OSC_writeFloatArg( b, curs->getSpeed() ) ;		// Speed
		OSC_writeFloatArg( b, curs->getWidth() ) ;		// Width
		OSC_writeIntArg( b, forwInt ) ;					// Forward or Backward
		OSC_writeIntArg( b, curs->getMode() ) ;			// Repeat Mode
		envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
		OSC_resetBuffer( b );
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/trajects",",i");	
	OSC_writeIntArg( b, tmpEnv.nbTrajects ) ;
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	NxTraject* traj ;
	for(int k=0 ; k<tmpEnv.nbTrajects ; k++) { // TRAJECTS
		traj = score->getTraject( trajInd.at(k) ) ;
		OSC_writeAddressAndTypes(b, "/iannix/score/traject",",iiif");	
		OSC_writeIntArg( b, traj->getID() ) ;			// ID
		OSC_writeIntArg( b, traj->getCustomID() ) ;		// Custom ID
		OSC_writeIntArg( b, traj->getGroupID() ) ;		// Group ID
		OSC_writeFloatArg( b, traj->getLength() ) ;		// Length
		envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
		OSC_resetBuffer( b );
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/curves",",i");
	OSC_writeIntArg( b, tmpEnv.nbCurves ) ;	
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	NxCurve *curv ;
	for(int k=0 ; k<tmpEnv.nbCurves ; k++) { // CURVES
		curv = score->getCurve( curvInd.at(k) ) ;
		OSC_writeAddressAndTypes(b, "/iannix/score/curve",",iiifff");
		OSC_writeIntArg( b, curv->getID() ) ;			// ID
		OSC_writeIntArg( b, curv->getCustomID() ) ;		// Custom ID
		OSC_writeIntArg( b, curv->getGroupID() ) ;		// Group ID
		OSC_writeFloatArg( b, curv->getRealMin() ) ;	// Real Min
		OSC_writeFloatArg( b, curv->getRealMax() ) ;	// Real Max
		OSC_writeFloatArg( b, curv->getXRange() ) ;		// Duration
		envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
		OSC_resetBuffer( b );
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/triggers",",i");
	OSC_writeIntArg( b, tmpEnv.nbTriggers ) ;
	envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );	
	NxTrigger *trig ;
	for(int k=0 ; k<tmpEnv.nbTriggers ; k++) { // TRIGGERS
		trig = score->getTrigger( trigInd.at(k) ) ;
		OSC_writeAddressAndTypes(b, "/iannix/score/trigger",",iiii");
		OSC_writeIntArg( b, trig->getID() ) ;			// ID
		OSC_writeIntArg( b, trig->getCustomID() ) ;		// Custom ID
		OSC_writeIntArg( b, trig->getGroupID() ) ;		// Group ID
		OSC_writeIntArg( b, trig->getValue() ) ;		// Int Value
		envClientConfigUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
		OSC_resetBuffer( b );	
	}
	///////////////////////////////////////////////////////////////////////////////////////////
	delete(envClientConfigUDP) ;
}
///////////////////////////////////////////////////////////////////////////////////////////
///////// INITIAL and FINAL OSC MESSAGES
///////////////////////////////////////////////////////////////////////////////////////////
void NxScheduler::sendOSCFirstMessage()
{
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	
	printf("Sending OSC Initial Message (255.255.255.255:%d) ...\n", tmpEnv.UDPout) ;
	
	char* charOSC1 = new char[100];
	QString firstOSC = QString("255.255.255.255")  ;
	strncpy(charOSC1, firstOSC.toAscii(), 99);
	envClientFirstUDP = new NxNetwork( charOSC1, tmpEnv.UDPout );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/play",",f");
	OSC_writeFloatArg( b, currentTime ) ;		
	envClientFirstUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	delete(envClientFirstUDP) ;
}
void NxScheduler::sendOSCLastMessage()
{
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	
	printf("Sending OSC Final Message (255.255.255.255:%d) ...\n", tmpEnv.UDPout) ;

	char* charOSC2 = new char[100];
	QString lastOSC = QString("255.255.255.255")  ;
	strncpy(charOSC2, lastOSC.toAscii(), 99);
	envClientLastUDP = new NxNetwork( charOSC2, tmpEnv.UDPout );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/stop",",f");
	OSC_writeFloatArg( b, currentTime ) ;		
	envClientLastUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	OSC_writeAddressAndTypes(b, "/iannix/score/packets",",i");
	OSC_writeIntArg( b, nbPackets ) ;		
	envClientLastUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), false );
	OSC_resetBuffer( b );
	///////////////////////////////////////////////////////////////////////////////////////////
	delete(envClientLastUDP) ;
}
/////////////////////
// MAIN TIMING METHOD
/////////////////////
void NxScheduler::run()
{	
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	NxCursor* curs ;
	
	printf("//////////////////////////////\n") ;
	if(useCirclesAlgo) printf("We use the Circles Algorithm\n") ;
	else printf("We use the Segment Algorithm\n") ;
	
	////////////////////// OSC ENVIRONMENT FIRST MESSAGE //////////////////////
	if (oscActive && sendInitialFinalMessage) sendOSCFirstMessage() ;

	//////////////////////////////////////////////////////////////////////
	finished = false ;
	next = true ;
	firstShot = true ;

	////////////////// TIME CONTROL //////////////////
	gPrecision = tmpEnv.schedulerStep ;		/// in ms
	dtime = gPrecision ;
	if(autoSchedulerStep) {
		gPrecision = 4 ;
		dtime = 4 ;
		printf("Auto Scheduler TimeStep Starting with: %d ms\n", gPrecision ) ;
	}
	else printf("Manual Scheduler TimeStep : %d ms\n", gPrecision ) ;

	float xStep ;			///// advance step of cursor	(in ms)
	int timeLeft = 0 ;		///// sleep time				(in ms)
	int loopCount = 50 ;

	clock.start() ;
	QTime tt0 = QTime(0,0,0,0) ;
	QTime tt1 = QTime(0,0,0,0) ;
	QTime tt2 = QTime(0,0,0,0) ;
	
	//connect( sTimer, SIGNAL(timeout()), this, SLOT(nextStep()) );
	//sTimer->start(10) ;
	//next = true ;
	///////////////////////////////////////////////////
	
	updateObjectsIndexes() ; // Update Objects indexes & NB (nbCursors / nbTriggers / ...)
	
	QList<float> curveTempStep ;
	
	while (finished == false)
    {
		tt0 = clock.currentTime() ; //// get init calcul Time
		//////////////////////////////////////////////////////////////////////////////////////////
		for (int i=0; i<nbCursors; i++) {
			curs = score->getCursor( cursInd.at(i) ) ;
			if(advanceCursors) updateCursorPos( i ) ;
			if (oscActive) {
				curveTempStep = curvesLocalStep.at(i) ;
				
				if( curs->getPlayFinished() || !cursorsFinished.at(i) ) triggersCollisionTest( i ) ;

				for (int k=0; k<nbCurves; k++) {
					float customPeriod = score->getCurve( curvInd.at(k) )->getSpatialPeriod() ;
					/////// get the spatial step advance (in ms)
					xStep = dtime * fabs(mainTempo) * curs->getSpeed() ;
					//if ( (xStep >= customPeriod) && (loopCount == 50) ) {
						//mainGLBox->setCPUWarning(true) ;
						//mainGLBox->setToolBarInfosTxt( QString("At Least one Cursor is Going too Fast\n( Cursor Step > Curve Spatial Period )") ) ;
						//}
					curveTempStep.replace(k, curveTempStep.at(k) + xStep ) ;
					if( (float)curveTempStep.at(k) >= customPeriod ) {
						//printf("////////// Test COLLISION\n") ;
						curveTempStep.replace(k, 0.0) ;
						
						if( curs->getPlayFinished() || !cursorsFinished.at(i) ) curvesCollisionTest( i, k ) ;
					} // end test collision
				} // end for (curves)
				
				curvesLocalStep.replace( i, curveTempStep ) ;
				cursorsFinished.replace( i, false ) ;
				curLoopShot.replace( i, false ) ;
				
				
			} // end if (oscActive)
		} // end for (cursors)

		if(firstShot) setFirstShot( false ) ; // put firstShot bool to false
		//////////////////////////////////////////////////////////////////////////////////////////
		
		//////////////////////////////////////////////////////////////////////////////////////////
		tt1 = clock.currentTime() ;
		//printf("timestep : %d\n", tt1.msecsTo(tt0)) ;
		timeLeft = gPrecision + tt1.msecsTo(tt0) ;
		if (timeLeft>0) usleep( timeLeft*1000 ) ;
		else {
			//mainGLBox->setCPUWarning(true) ;
			if (autoSchedulerStep) { ////// INCREASE SCHEDULER STEP //////
				if ( gPrecision == (int)floor(gPrecision*(1.4)) )
					gPrecision += 2 ;
				else
					gPrecision = (int)floor( gPrecision*(1.4) )  ;
				//mainGLBox->setCPUTimeStep( gPrecision ) ;
			}
		}
		
		//tt2 = clock.currentTime() ;
		//printf("time : %d\n", tt2.msecsTo(tt0)  ) ;
		//////////////////////////////////////////////////////////////////////////////////////////
		if(loopCount == 50) {
			loopCount = 0 ;
			if (timeLeft>2 && gPrecision>1 && autoSchedulerStep) { ////// REDUCE SCHEDULER STEP //////
				if ( gPrecision == (int)floor( gPrecision*(0.9)) )
					gPrecision -= 1 ;
				else
					gPrecision = (int)floor( gPrecision*(0.9) )  ;
			}
			//mainGLBox->setCPUWarning(false) ;
			//mainGLBox->setCPUTimeStep( gPrecision ) ; // in ms
		}//////////////////////////////////////////////////////////////////////////////////////////
		
		loopCount++ ;
		
		sendSavedOSC() ;
		
		/////////// LIVE ADDING OBJECT TEST ///////////
		if(addTrigger) {
			addTrigger = false ;
			doAddNewTrigger() ;
		}
		if(addCurve) {
			addCurve = false ;
			doAddNewCurve() ;
		}
		////////////// LIVE ADDING END ////////////////
		
		tt2 = clock.currentTime() ;		
		dtime = (float)fabs( tt2.msecsTo(tt0) ) ;
		currentTime += (float)dtime * 0.001 ;
		if (finalTime!=0 && currentTime >= finalTime) {
			currentTime = finalTime ;
			stop() ;
		}
		//////////////////////////////////////////////////////////////////////////////////////////
	}
	
	////////////////////// OSC ENVIRONMENT LAST MESSAGE //////////////////////
	if (oscActive && sendInitialFinalMessage) sendOSCLastMessage() ;

	////////////////////////////////////////////////////////////////////
	
	printf("//////////////////////////////\n") ;
	printf("Playing finished !\n");
	printf("UDP messages sent : %d \n", nbPackets );
	
	//mainGLBox->setToolBarInfosTxt( QString(" ") ) ;
	//mainGLBox->setCPUWarning(false) ;
	
	//sTimer->stop() ;
	finished = false ;
	playing = false ;

	nbPackets = 0 ;
}
void NxScheduler::updateCursorPos( int i )
{
	NxCursor* cur = score->getCursor( cursInd.at(i) ) ;
	
	bool forwCur	= cur->getForward() ;		// forward / backward
	int typeCur		= cur->getMode() ;			// 0 / 1 / 2 : single / loop / b_f
	bool dir		= curDirections.at(i) ;		// direction of the cursor i at time t
	float tempo		= mainTempo ;
	
	////// To allow direction inversion during playback
	if(cur->hasChangedDirection()) {
		curDirections.replace(i, !dir ) ;
		dir = !dir ;
		cur->setChangedDirection( false ) ;
	}
	
	if(cur->getSpeed()<0) tempo = - tempo ;
	
	//////////////////////////////////////////////////////////////////////////////////////////
	// CIRCLE CURSOR
	//////////////////////////////////////////////////////////////////////////////////////////
	if( ((NxTraject*)cur->getParentTrajectory())->getType()==1 ) {
		float initAngle		= cur->getInitAngle() ;
		float finalAngle	= cur->getFinalAngle() ;
		
		bool I_F = (finalAngle>initAngle) ;
		bool F_I = (finalAngle<initAngle) ;
		bool I_I = (finalAngle==initAngle) ;
		float currentAngle ;
		
		///////////// Interval [ I , F ]
		float I = initAngle;
		float F = initAngle;
		if(I_F) {
			if(forwCur) { F=finalAngle; }
			else		{ I=finalAngle-360; }
		}
		if(F_I) {
			if(forwCur) { F=finalAngle+360; }
			else		{ I=finalAngle; }			
		}
		if(I_I) {
			if(forwCur) { F=initAngle+360; }
			else		{ I=initAngle-360; }				
		}

		if(tempo<0.0)
			dir=!dir ;
			
		if(dir)		advanceAngle( cur ) ;
		else		regressAngle( cur ) ;	
		currentAngle = cur->getAngle() ;
		
		switch( typeCur ) {
			case 0 : // single
				if(dir && currentAngle>=F) { cur->setAngle( F, false ) ; cursorsFinished.replace( i, true ) ; }
				if(!dir && currentAngle<=I) { cur->setAngle( I, false ) ; cursorsFinished.replace( i, true ) ; }		
			break;
			case 1 : // loop
				if(dir && currentAngle>=F) { cur->setAngle( I+currentAngle-F, false ) ; if(!I_I) curLoopShot.replace(i, true) ; }
				if(!dir && currentAngle<=I) { cur->setAngle( F+currentAngle-I, false ) ; if(!I_I) curLoopShot.replace(i, true) ; }
			break;											
			case 2 : // for/bck
				if(tempo<0.0) dir=!dir ;
				if(currentAngle>F) { cur->setAngle( 2*F-currentAngle, false ) ; curDirections.replace(i, !dir ) ; }
				if(currentAngle<I) { cur->setAngle( 2*I-currentAngle, false ) ; curDirections.replace(i, !dir ) ; }
			break;
		} // end(switchType)
	} // end(circleCursor)
	else {
		//////////////////////////////////////////////////////////////////////////////////////////
		// LINE CURSOR
		//////////////////////////////////////////////////////////////////////////////////////////
		float initOffs	= cur->getInitOffset() ;		// initial offset
		float finalOffs	= cur->getFinalOffset() ;		// final offset
		
		if (tempo >= 0.0) {		
			switch( typeCur ) {
				case 0 : // single
					advanceOffset( cur ) ;
					if( cur->getOffset()>finalOffs ) { cur->setOffset( finalOffs ) ; cursorsFinished.replace( i, true ) ; }
				break;
				case 1 : // loop
					advanceOffset( cur ) ;
					if( cur->getOffset()>finalOffs ) { cur->setOffset( initOffs+cur->getOffset()-finalOffs ) ; curLoopShot.replace(i, true) ; }
				break;											
				case 2 : // for/bck
					if( dir )	advanceOffset( cur ) ;
					if( !dir )	regressOffset( cur ) ;		
					if( dir && cur->getOffset()>finalOffs )	{ cur->setOffset( 2*finalOffs-cur->getOffset() ) ; curDirections.replace(i, !dir ) ; }
					if( !dir && cur->getOffset()<initOffs )	{ cur->setOffset( 2*initOffs-cur->getOffset() ) ; curDirections.replace(i, !dir) ; }
				break;				
			}
		}
		else {
			switch( typeCur ) {
				case 0 : // single
					regressOffset( cur ) ;
					if( cur->getOffset()<initOffs ) { cur->setOffset( initOffs ) ; cursorsFinished.replace( i, true ) ; }
				break;
				case 1 : // loop
					regressOffset( cur ) ;
					if( cur->getOffset()<initOffs )	{ cur->setOffset( finalOffs+cur->getOffset()-initOffs ) ; curLoopShot.replace(i, true) ; }
				break;											
				case 2 : // for/bck
					dir = !dir ;
					if( dir )	advanceOffset( cur ) ;
					if( !dir )	regressOffset( cur ) ;
					if( cur->getOffset()<initOffs )		{ cur->setOffset( 2*initOffs-cur->getOffset() ) ; curDirections.replace(i, dir ) ; }
					if( cur->getOffset()>finalOffs )	{ cur->setOffset( 2*finalOffs-cur->getOffset() ) ; curDirections.replace(i, dir) ; }
				break;				
			}	
		} // end(elseTempo)
	} // end(lineCursor)
}
void NxScheduler::advanceAngle( NxCursor* cur ) 
{
	cur->advanceAngle( 0.001 * dtime * fabs(cur->getSpeed()) * fabs(mainTempo) ) ;
}
void NxScheduler::regressAngle( NxCursor* cur) 
{
	cur->advanceAngle( - 0.001 * dtime * fabs(cur->getSpeed()) * fabs(mainTempo) ) ;
}
void NxScheduler::advanceOffset( NxCursor* cur ) 
{
	cur->advanceOffset( 0.001 * dtime * fabs(cur->getSpeed()) * fabs(mainTempo)  ) ;
}
void NxScheduler::regressOffset( NxCursor* cur) 
{
	cur->advanceOffset( - 0.001 * dtime * fabs(cur->getSpeed()) * fabs(mainTempo)  ) ;
}
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void NxScheduler::nextStep() ////// UNUSED
{
	mutex.lock() ;
	//currentTime += dtime;
	next = true ;
	mutex.unlock() ;
	//printf("nextstep : time : %f\n", currentTime) ;
}

//////////////////////////// Stopping Method
void NxScheduler::stop()
{
	mutex.lock();
	finished = true ;
	mutex.unlock();
}
/////////////////////////////////////////////////////////////////////////////
/////////////////////////// TRIGGERS COLLISION TEST /////////////////////////
/////////////////////////////////////////////////////////////////////////////
void NxScheduler::triggersCollisionTest( int i ) // For one cursor
{	
	NxCursor 	*cursor ;
	NxTrigger 	*trigger ; 
	
	NxPoint trigP ;		// trigger position
	NxPoint cursP ;		// cursor position
	
	int layer ;			// Interactions only if objects in the same layer
	
	float wid, ang ;	// width and angle of the cursor
	float cosV, sinV ;	// projections of object center on the cursor
	float distT	;		// distance between cursor and trigger
		
	cursor = score->getCursor( cursInd.at(i) );
	layer = cursor->getLayer() ;
	if( cursor->isActive() ) { // only if cursor is active
		ang		= cursor->getAngle()*PI/180 ;		// get angle in radians
		wid		= cursor->getWidth() ;				// get width
		cursP	= cursor->getPosition() ;			// get position
			
		//////////////////////////////////////////////////////////
		//////////////////////// TRIGGERS ////////////////////////
		//////////----------------------------------------////////
		/////// arrays of triggers booleans
		QList<bool> tmpTBool1 = triggersSign1.at(i) ; // array of cursor i
		QList<bool> tmpTBool2 = triggersSign2.at(i) ; // array of cursor i
		/////// Trigger Loop
		for (int j=0; j<nbTriggers; j++) {				
			trigger = score->getTrigger( trigInd.at(j) ) ;	
			if( (trigger->isActive()) && (trigger->getLayer()==layer) ) { // only if in the same layer and active
				trigP = trigger->getPosition() ; // new collision test
				//printf("-++- TRIGGER ID %d\n", trigP.ID ) ;
				//////// COS projection /////////////////////////////////////////////////
				// vector C :	X    - sin(ang)		////////// (normalized)
				//				Y    cos(ang)		////////// (normalized)
				// vector T :	X    trigP.x - cursP.x
				//				Y    trigP.y - cursP.y
				//printf("/////////////////////////////////////////////////////////////\n") ;
				//printf("Positions: trigg %f %f curs: %f %f\n", trigP.x, trigP.y, cursP.x, cursP.y ) ; 
				//printf("Angle: %f\n", ang*180/PI ) ;
				float c1, c2, t1, t2 ;
				c1 = cos(ang) ;
				c2 = sin(ang) ;
				t1 = trigP.x - cursP.x ;
				t2 = trigP.y - cursP.y ;
				cosV = (c1*t1 + c2*t2 ) ; //---------// distance to the traject (perpendicular to cursor direction)
		
				distT = sqrt(t1*t1 + t2*t2)  ; //---------// distance between cursor center and trigger
				//printf("distance : %f\n",distT ) ; 
				//printf("cursor c1: %f c2: %f\n", c1, c2 ) ; 
				//printf("trigger t1: %f t2: %f\n", t1, t2 ) ; 
				//printf("Distance to the traject = %f \n", cosV) ;
				///////////////////////////////////////
				if( fabs(cosV) <=  (wid/2) ) { // only if trigger 'viewable'
					//////// SIN projection ///////////////////
					sinV = (c1*t2 - c2*t1) ; // (wid/2) ; //---------// distance to the cursor
					//////////////////////////////////////////
					//printf("Distance to the cursor = %f \n\n", sinV) ;
					if(sinV>0) tmpTBool1.replace( j, true ) ; // SIN projection
					else tmpTBool1.replace( j, false ) ;
					/////////// Collision Test - need to test :
					// sign change
					// avoid first shot banging (loops)
					// avoid circles problem
					if( !firstShot && !curLoopShot.at(i) && ( tmpTBool1.at(j) != tmpTBool2.at(j) ) && (distT<=wid/2) ) { 
						///////////////////////////////////////////////////////////////////////////// COLLISION !!!!
						//// SEND ////////
						//sendTriggerOSC( cursor, trigger ) ;
						triggerOSC mess ;
						mess.oscFormat	= trigger->getOscFormat() ;
						mess.host		= trigger->getHost() ;
						mess.port		= trigger->getPort() ;
						mess.cursRID	= cursor->getID() ;
						mess.cursGID	= cursor->getGroupID() ;
						mess.cursID		= cursor->getCustomID() ;
						mess.RID		= trigger->getID() ;
						mess.GID		= trigger->getGroupID() ;
						mess.ID			= trigger->getCustomID() ;
						mess.val		= trigger->getValue() ;
						mess.X			= trigger->getPosition().x ;
						mess.Y			= trigger->getPosition().y ;
						mess.Z			= trigger->getPosition().z ;
						triggersMessages.append( mess ) ;

					} // end if() -> collision
					tmpTBool2.replace( j, tmpTBool1.at(j) ) ;
					triggersSign1.replace( i , tmpTBool1 ) ;
					triggersSign2.replace( i , tmpTBool2 ) ;
				} // end if( trigger viewable )	
			} // end if( active and same layer )
		} // end for ( triggers )
	} // end if ( cursor active and same layer)
}
////////////////////////////////////////////////////////////////////////
////////////////     TRIGGERS COLLISION TEST END       /////////////////
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
////////////////     CURVES COLLISION TEST      ////////////////////////
////////////////////////////////////////////////////////////////////////
void NxScheduler::curvesCollisionTest( int i, int k ) // For one cursor
{	
	NxCursor	*cursor ;
	NxCurve		*curve ;
	NxSpline	*spline ;
	
	NxPoint curvP ;		// curve position
	NxPoint cursP ;		// cursor position
	
	int layer ; // Interactions only if objects in the same layer
	
	float wid, ang ;	// width and angle of the cursor
	float cosV, sinV ;	// projections of object center on the cursor
	float distC ;		// distance between cursor and curve
	
	float solx, soly, a1, a2, b1, b2 ; // local variables for algo 2
	
	cursor = score->getCursor( cursInd.at(i) );
	layer = cursor->getLayer() ;
	if( cursor->isActive() ) { // only if cursor is active
		ang		= cursor->getAngle()*PI/180.0 ;		// get angle in radians
		wid		= cursor->getWidth() ;				// get width
		cursP	= cursor->getPosition() ;			// get position

		if(useCirclesAlgo) {
			//////////////////////////////////////////////////////////	
			//////////////////////// CURVES ALGO 1 ///////////////////
			///////////-----------------------------------////////////	
			/////// arrays for curves collision test
			QList <float> tmpDurations	= curvesDurations.at(i) ;		/////// collision durations	
			QList <float> tmpTimers		= curvesTimers.at(i) ;			/////// local timers
			QList <bool> tmpCollisions	= curvesCollisions.at(i) ;		/////// in-collision booleans		
				
			float splineVal, splineAngle ;
			splineVal = 0 ; // initialisation
				
			curve = score->getCurve( curvInd.at(k) ) ;
			if( (curve->isActive()) && (curve->getLayer()==layer) ) { // only if same layer and active
				curvP = curve->getCenter() ;
				float cRadius = curve->getRadius() ;
		
				if( tmpCollisions.at(k) == false ) { //----------------------// test the collision 
					//printf("/////// Curve Radius : %f \n", cRadius) ;
					//////// COS projection /////////////////////////////////////////////////
					// vector C :	X    - sin(ang)		////////// (normalized)
					//				Y    cos(ang)		////////// (normalized)
					// vector T :	X    curvP.x - cursP.x
					//				Y    curvP.y - cursP.y
					//printf("/////////////////////////////////////////////////////////////\n") ;
					//printf("Positions: curv %f %f curs: %f %f\n", curvP.x, curvP.y, cursP.x, cursP.y ) ; 
					//printf("Angle: %f\n", ang ) ;
					float c1, c2, t1, t2 ;
					c1 = cos(ang) ;
					c2 = sin(ang) ;
					t1 = curvP.x - cursP.x ;
					t2 = curvP.y - cursP.y ;
					cosV = (c1*t1 + c2*t2 ) ; //---------// distance to the traject (perpendicular to cursor direction)
			
					distC = sqrt(t1*t1 + t2*t2)  ; // distance between cursor center and curve center
					//printf("distance : %f\n",distC ) ; 
					//printf("cursor c1: %f c2: %f\n", c1, c2 ) ; 
					//printf("curve t1: %f t2: %f\n", t1, t2 ) ; 
					//printf("Distance to the traject = %f \n", cosV) ;
					///////////////////////////////////////
					if( fabs(cosV) <=  ((wid/2) + cRadius) ) { // only if the curve circle is "viewable" ...
						sinV = (c1*t2 - c2*t1) ; //---------// distance to the cursor
						//printf("Distance to the cursor = %f \n", sinV) ;
						float d ;
						if( fabs(cosV) > (wid/2) ) { //// the circle is not read entirely, we have to calculate the real duration
							///////////////// Calculate duration...
							//		(cosV-(wid/2))^2 + (duration/2)^2 = cRadius^2
							//		d = duration/2
							d = sqrt( pow(cRadius, 2) - pow(fabs(cosV)-(wid/2), 2) ) ;
						}
						else d = cRadius ;
						//printf("Real duration of event : %f\n", 2*d) ;
				
						if( distC < ((wid/2)+cRadius) ) { // test only if not too far one from another
							//if( (sinV>0.0) && ((sinV-d)>0) && collisions.at(k) )	
							//if( (sinV>0.0) && ((sinV-d)>0) && !collisions.at(k) )	
							//if( (sinV>0.0) && ((sinV-d)<0) && collisions.at(k) )	
							//if( (sinV<0.0) && ((sinV+d)<0) && collisions.at(k) )		
							//if( (sinV<0.0) && ((sinV+d)<0) && !collisions.at(k) )								
							//if( (sinV<0.0) && ((sinV+d)>0) && collisions.at(k) )	
							if( firstShot && (fabs(sinV) < cRadius) ) { // cursor already int the curve circle when playing
								//printf("Already in the Curve Circle !!\n") ;
								tmpCollisions.replace( k , true ) ; 
								tmpTimers.replace( k , cRadius + sinV ) ; /// TO DO : depends on cursor position
								float realDur = cRadius + sqrt( pow( cRadius ,2) - pow( fabs(cosV) - wid/2 ,2) ) ;
								//printf("TimeIn of event (already in circle) = %f\n", cRadius + sinV ) ;
								//printf("TimeOut of event (already in circle) = %f\n", realDur ) ;
								tmpDurations.replace( k , realDur  ) ; 						
							}
							else { // To simplify...
								if( (sinV>0.0) && ((sinV-d)<0) && !tmpCollisions.at(k) ) { 
									tmpCollisions.replace( k , true ) ; 
									tmpTimers.replace( k , cRadius - d ) ; 
									tmpDurations.replace( k , cRadius + d ) ; 
								}
								if( (sinV<0.0) && ((sinV+d)>0) && !tmpCollisions.at(k) ) {	
									tmpCollisions.replace( k , true ) ; 
									tmpTimers.replace( k , cRadius - d ) ; 
									tmpDurations.replace( k , cRadius + d ) ; 
								}
							}
						} // end if( distC < ... )			
					} /// end if( 'viewable' )
				} /// end if( collision=false )
	
				if( tmpCollisions.at(k) == true ) { //----------------------// Already in collision
					///////////////////////////////////////////////////////////////////////////////////// COLLISION !!!!!		
					////// get curve value
					if(curve->getDrawType() == LineStrip) { /////----- LINES -----//////
						splineVal = curve->getLinearValue( tmpTimers.at(k) ) ;
					}
					if(curve->getDrawType() == SplineStrip) { /////----- SPLINES -----///////
						spline = curve->getSpline() ;
						splineVal = (float)spline->getValue( tmpTimers.at(k) );
					}
					splineAngle = curve->getAngle() - cursor->getNormalizedAngle() ; ////// get real angle in degrees

					curveOSC mess ;
					mess.oscFormat	= curve->getOscFormat() ;
					mess.host		= curve->getHost() ;
					mess.port		= curve->getPort() ;
					mess.cursRID	= cursor->getID() ;
					mess.cursGID	= cursor->getGroupID() ;
					mess.cursID		= cursor->getCustomID() ;
					mess.RID		= curve->getID() ;
					mess.GID		= curve->getGroupID() ;
					mess.ID			= curve->getCustomID() ;
					mess.angle		= splineAngle ;
					mess.val		= splineVal ;
					mess.X			= curve->getPosition().x ;
					mess.Y			= curve->getPosition().y ;
					mess.Z			= curve->getPosition().z ;
					curvesMessages.append( mess ) ;
					if(curve->getDrawType() == SplineStrip) spline->reset() ;
												
					//-------/// TO DO WARNING : it is not the same timeStep with circle trajects ....
					//float ddtimer = tmpTimers.at(k) + 0.001 * dtime * cursor->getSpeed() * fabs(mainTempo) ; 
					float ddtimer = tmpTimers.at(k) + curve->getSpatialPeriod() * 0.001 ; 
					tmpTimers.replace( k ,  ddtimer  ) ; ///// advance local time
					//printf( "Timer Value : %f -- Max duration : %f\n", tmpTimers.at(k) , tmpDurations.at(k));
					if( tmpTimers.at(k) > tmpDurations.at(k) ) { //// end of collision
						tmpCollisions.replace( k , false ) ;
						if(curve->getDrawType() == SplineStrip) spline->reset() ;
						tmpTimers.replace( k , 0.0 ) ;
						tmpDurations.replace( k , 999.0 ) ;
					}
				} /// end if( collision=true )
				
				curvesDurations.replace( i , tmpDurations ) ;
				curvesTimers.replace( i , tmpTimers ) ;
				curvesCollisions.replace( i , tmpCollisions ) ;
			} // end if (active and same layer )
		//////////////////////////////////// END ALGO 1
		} // end algo1
		else {
			//////////////////////////////////////////////////////////	
			//////////////////////// CURVES ALGO 2 ///////////////////
			///////////-----------------------------------////////////		
			float splineVal, splineAngle ;
			splineVal = 0 ; // initialisation
				
			NxSpline *spline ;
			curve = score->getCurve( curvInd.at(k) ) ;
			if( (curve->isActive()) && (curve->getLayer()==layer) ) {
				bool collisionIntersect = false ; 
				bool paralel = false ;
				bool curvVertical = false ;
				bool cursVertical = false ;
				float localTime ;
				curvP = curve->getCenter() ;
				float cRadius = curve->getXRange()/2 ;
				//printf("cRadius is : %f\n", cRadius) ;
				float cAngle = curve->getAngle()*PI/180.0 ; // in radians
				if( sqrt( pow(cursP.x-curvP.x,2) + pow(cursP.y-curvP.y,2) ) <= (cRadius+wid/2) ) { // test only if centers are not too far one from another ...
					//a1 = a2 = b1 = b2 = 0.0 ;
					a1 = 0 ; // Initialisation (avoid automatic paralell)
					a2 = 1 ; // Initialisation (avoid automatic paralell)
					if( fabs(cos(cAngle))>0.01 ) { // avoid Curve vertical direction
						a1 = sin(cAngle) / cos(cAngle) ;
						b1 = curvP.y - a1*curvP.x ;
						}
					else curvVertical = true ;
					if(fabs(cos(ang))>0.01 ) { // avoid Cursor vertical direction
						a2 = sin(ang) / cos(ang) ; 
						b2 = cursP.y - a2*cursP.x ;
						}
					else cursVertical = true ;
					if(curvVertical) {
						solx = curvP.x ;
						soly = a2*solx + b2 ;
						//printf("Curve is vertical !\n") ;
						}						
					if(cursVertical) {
						solx = cursP.x ;
						soly = a1*solx + b1 ;
						//printf("Cursor is vertical !\n") ;
						}
					if( ( a1==a2 && !(cursVertical || curvVertical) ) || (cursVertical && curvVertical) ) { 
						paralel = true ;
						//printf("Parallel !\n") ;
						}
					//printf("angles : curve %f cursor : %f \n", cAngle*180.0/PI,  ang*180.0/PI ) ;
					//printf("a1 a2 : %f %f\n", a1, a2) ;
					//printf("b1 b2 : %f %f\n", b1, b2) ;
					////////// now we solve the intersection :
					if( !paralel && !cursVertical && !curvVertical) { // avoid paralellism and vertical direction !
						solx = (b2-b1)/(a1-a2) ;
						soly = a1*solx + b1 ;
					}
					//printf("Intersection calculated : %f et %f\n", solx, soly) ;
					
					// Test if solution is on the segments (cursor and curve segment)
					if(	!paralel && ( sqrt( pow(solx-curvP.x,2) + pow(soly-curvP.y,2) )<=cRadius ) && ( sqrt( pow(solx-cursP.x,2) + pow(soly-cursP.y,2) )<=wid/2 )) {
						//printf("Collision!!\n") ;
						collisionIntersect = true ;
						NxPoint initPos = curve->getCenter() ;
						initPos.x -= cRadius*cos(cAngle) ;
						initPos.y -= cRadius*sin(cAngle) ;
						localTime = sqrt( pow(solx-initPos.x,2) + pow(soly-initPos.y,2) ) ;
					}
				
					if( !paralel && collisionIntersect ) { //----------------------// 
						///////////////////////////////////////////////////////////////////////////////////// COLLISION !!!!!		
						////// Get curve value
						if(curve->getDrawType() == LineStrip) { /////----- LINES -----//////
							splineVal = curve->getLinearValue( localTime ) ;
						}
						if(curve->getDrawType() == SplineStrip) { /////----- SPLINES -----/////
							spline = curve->getSpline() ;
							splineVal = spline->getValue( localTime );
						}
						splineAngle = curve->getAngle() - cursor->getNormalizedAngle() ; ////// get real angle in degrees
						
						curveOSC mess ;
						mess.oscFormat	= curve->getOscFormat() ;
						mess.host		= curve->getHost() ;
						mess.port		= curve->getPort() ;
						mess.cursRID	= cursor->getID() ;
						mess.cursGID	= cursor->getGroupID() ;
						mess.cursID		= cursor->getCustomID() ;
						mess.RID		= curve->getID() ;
						mess.GID		= curve->getGroupID() ;
						mess.ID			= curve->getCustomID() ;
						mess.angle		= splineAngle ;
						mess.val		= splineVal ;
						mess.X			= curve->getPosition().x ;
						mess.Y			= curve->getPosition().y ;
						mess.Z			= curve->getPosition().z ;
						curvesMessages.append( mess ) ;
						if( curve->getDrawType() == SplineStrip ) curve->getSpline()->reset() ;	
						
					} // end if( paralel && collision ) (collision) 
				} // end if (not too far from onother)
			} // end if (active and same layer)
		} // and else[ ] (algo 2)
	} // end if (cursor active)
}
////////////////////////////////////////////////////////////////////////
////////////////     CURVES COLLISION TEST END       ///////////////////
////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////
////////////////     OSC SEND FONCTIONS       ///////////////////////
/////////////////////////////////////////////////////////////////////
void NxScheduler::updateSendingOptions()
{
	NxEnv tmpEnv = ((NxScore*)mainGLBox->getPartition())->getEnvironment() ;
	scCID	= tmpEnv.scCID ;
	scCGID	= tmpEnv.scCGID ;
	scID	= tmpEnv.scID ;
	scGID	= tmpEnv.scGID ;
	scA		= tmpEnv.scA ;
	scV		= tmpEnv.scV ;
	stCID	= tmpEnv.stCID ;
	stCGID	= tmpEnv.stCGID ;
	stID	= tmpEnv.stID ;
	stGID	= tmpEnv.stGID ;
	stV		= tmpEnv.stV ;
	/////// Real IDs and Positions (after 0.640 version)
	scCRID	= tmpEnv.scCRID ;
	scRID	= tmpEnv.scRID ;
	scX		= tmpEnv.scX ;
	scY		= tmpEnv.scY ;
	scZ		= tmpEnv.scZ ;
	stCRID	= tmpEnv.stCRID ;
	stRID	= tmpEnv.stRID ;
	stX		= tmpEnv.stX ;
	stY		= tmpEnv.stY ;
	stZ		= tmpEnv.stZ ;
}
void NxScheduler::setOscConsoleActive( bool flag )
{
	oscConsoleActive = flag ;
}
void NxScheduler::sendSavedOSC()
{
	for (int j=0; j< triggersMessages.size(); j++) {
		sendTriggerOSC( triggersMessages.at(j) ) ;
		if(oscConsoleActive) emit sendTriggerMessage( triggersMessages.at(j) );
	}
	for (int j=0; j< curvesMessages.size(); j++) {
		sendCurveOSC( curvesMessages.at(j) ) ;
		if(oscConsoleActive) emit sendCurveMessage( curvesMessages.at(j) ); 
	}
		
	resetOSC() ;
}
void NxScheduler::sendCurveOSC( curveOSC mess ) 
{
	QString cC( "," ) ;
	if(scCRID)	cC.append("i") ;
	if(scCGID)	cC.append("i") ;
	if(scCID)	cC.append("i") ;
	if(scRID)	cC.append("i") ;
	if(scGID)	cC.append("i") ;
	if(scID)	cC.append("i") ;
	if(scA)		cC.append("f") ;
	if(scV)		cC.append("f") ;
	if(scX)		cC.append("f") ;
	if(scY)		cC.append("f") ;
	if(scZ)		cC.append("f") ;
	char curvChar[ cC.size()+1 ];
	strncpy(curvChar, cC.toAscii().data(), sizeof(curvChar)-1);
	curvChar[sizeof(curvChar)-1] = '\0' ;
	
	strncpy(curvesOSCmF, (mess.oscFormat).toAscii(), 99);
	OSC_writeAddressAndTypes(b, curvesOSCmF, curvChar) ;
	if(scCRID)	OSC_writeIntArg( b, mess.cursRID );		/// OSC Argument 1 - Cursor Real ID
	if(scCGID)	OSC_writeIntArg( b, mess.cursGID );		/// OSC Argument 2 - Cursor Group ID
	if(scCID)	OSC_writeIntArg( b, mess.cursID );		/// OSC Argument 3 - Cursor Custom ID
	if(scRID)	OSC_writeIntArg( b, mess.RID );			/// OSC Argument 4 - Curve Real ID
	if(scGID)	OSC_writeIntArg( b, mess.GID );			/// OSC Argument 5 - Curve Group ID
	if(scID)	OSC_writeIntArg( b, mess.ID );			/// OSC Argument 6 - Curve Custom ID
	if(scA)		OSC_writeFloatArg( b, mess.angle );		/// OSC Argument 7 - Curve Angle
	if(scV)		OSC_writeFloatArg( b, mess.val );		/// OSC Argument 8 - Curve Value
	if(scX)		OSC_writeFloatArg( b, mess.X );			/// OSC Argument 9 - Curve X
	if(scY)		OSC_writeFloatArg( b, mess.Y );			/// OSC Argument 10 - Curve Y
	if(scZ)		OSC_writeFloatArg( b, mess.Z );			/// OSC Argument 11 - Curve Z
	curveClientUDP = new NxNetwork( mess.host, mess.port );
	curveClientUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), broadcastMode);
	OSC_resetBuffer( b );
	delete(curveClientUDP) ;
	nbPackets++;
}
void NxScheduler::sendTriggerOSC( triggerOSC mess ) 
{
	QString tC( "," ) ;
	if(stCRID)	tC.append("i") ;
	if(stCGID)	tC.append("i") ;
	if(stCID)	tC.append("i") ;
	if(stRID)	tC.append("i") ;
	if(stGID)	tC.append("i") ;
	if(stID)	tC.append("i") ;
	if(stV)		tC.append("i") ;
	if(stX)		tC.append("f") ;
	if(stY)		tC.append("f") ;
	if(stZ)		tC.append("f") ;
	char trigChar[ tC.size()+1 ];
	strncpy(trigChar, tC.toAscii().data(), sizeof(trigChar)-1);
	trigChar[sizeof(trigChar)-1] = '\0' ;
	
	strncpy(triggersOSCmF, mess.oscFormat.toAscii(), 99);
	OSC_writeAddressAndTypes(b, triggersOSCmF , trigChar);
	if(stCRID)	OSC_writeIntArg( b, mess.cursRID );		/// OSC Argument 1 - Cursor Real ID
	if(stCGID)	OSC_writeIntArg( b, mess.cursGID );		/// OSC Argument 2 - Cursor Group ID
	if(stCID)	OSC_writeIntArg( b, mess.cursID );		/// OSC Argument 3 - Cursor Custom ID
	if(stRID)	OSC_writeIntArg( b, mess.RID );			/// OSC Argument 4 - Trigger Real ID
	if(stGID)	OSC_writeIntArg( b, mess.GID );			/// OSC Argument 5 - Trigger Group ID
	if(stID)	OSC_writeIntArg( b, mess.ID );			/// OSC Argument 6 - Trigger Custom ID
	if(stV)		OSC_writeIntArg( b, mess.val );			/// OSC Argument 7 - Trigger Int Value
	if(stX)		OSC_writeFloatArg( b, mess.X );			/// OSC Argument 8 - Trigger X
	if(stY)		OSC_writeFloatArg( b, mess.Y );			/// OSC Argument 9 - Trigger Y
	if(stZ)		OSC_writeFloatArg( b, mess.Z );			/// OSC Argument 10 - Trigger Z
	triggerClientUDP = new NxNetwork( mess.host, mess.port );
	triggerClientUDP->sendPacket(OSC_getPacket(b), OSC_packetSize(b), broadcastMode);
	OSC_resetBuffer( b );
	delete(triggerClientUDP) ;
	nbPackets++;
}
/////////////////////////////////////////////////////////////////////
// Function for displaying OSC packets
/////////////////////////////////////////////////////////////////////
void NxScheduler::PrintPacket(OSCbuf *b) 
{
	char *p = OSC_getPacket(b);
	int size = OSC_packetSize(b);
	unsigned int *intp;
	int i;

	printf("PrintPacket: packet at %p, size %d\n", p, size);
	if (p == 0 || size == 0) return;

	printf("Hex version:");
	for (i = 0, intp = (unsigned int *)p; i < size; i += 4, intp++) 
	{
		if (i % 40 == 0) printf("\n");
		printf("%x ", *intp);
	}

	printf("\n\nString version:");
	for (i = 0; i < size; i++) 
	{
		if (i % 40 == 0) printf("\n");
		if (isprint(p[i])) 
			printf("%c", p[i]); 
		else 
			printf("\\%x", p[i] & 0x000000ff);
	}
	printf("\n");
}
