///////////////
// IanniX
///////////////

////////////////////////////
// NxScheduler
////////////////////////////

#ifndef NXSCHEDULER_H
#define NXSCHEDULER_H

// IanniX
#include "../atoms/NxTypes.h"
#include "../../graphics/opengl/NxGLScore.h"
#include "../../widgets/NxOscConsole.h"
#include "../../network/NxNetwork.h"
#include "../../network/OSCout/OSC_Client.h"

// Qt Libs
#include <QtGui>

class NxGLScore ;
class NxOscConsole ;

class NxScheduler : public QThread
{
	Q_OBJECT
 public :
	////////////////////
  	// Constructor
  	////////////////////
  	NxScheduler( NxGLScore *inGLBox );
  
  	//////////////////
  	// Destructor	
  	//////////////////
  	~NxScheduler();

	bool	isPlaying() ;
	void	setPlaying( bool chooser ) ;
	bool	isReadyToPlay() ;
	void	setReadyToPlay( bool chooser ) ;
		
	/////////// PARAMETERS
	void	setOSCOutActive( bool chooser ) ;
	void	useAutoSchedulerStep( bool chooser ) ;
	bool	isAutoSchedulerStep() ;
	void	useCirclesAlgorithm( bool chooser, bool withReset ) ;
	void	sendIFmessages( bool chooser ) ;
	void	useBroadcastMode( bool inBroad ) ;
	void	moveCursors( bool chooser ) ;
	
	/////////// LIVE ADDING
	void	addNewCursor() ;
	void	addNewCurve() ;
	void	doAddNewCurve() ;
	void	addNewTrigger() ;
	void	doAddNewTrigger() ;
	bool	addTrigger, addCurve ;
	/////////// INITIALISATIONS
	void	initScheduler() ; // Timing calculation & prepare playback (only at init)
  	void	resetSchedule() ; // Reset All
	void	resetCursors() ;
  	void	resetCurves() ;
	void	resetSplines() ;
	void	resetTriggers() ;
	void	resetOSC() ;
	
	/////////// COLLISIONS
	void	updateCursorPos( int i ) ;
	
	void	advanceAngle( NxCursor* cur) ;
	void	regressAngle( NxCursor* cur) ;
	void	advanceOffset( NxCursor* cur) ;
	void	regressOffset( NxCursor* cur) ;
	
	void	triggersCollisionTest( int i ) ;
	void	curvesCollisionTest( int i, int k ) ;
	
	/////////// TIME
  	void	changeTempo( float inTempo );	// tempo switcher
	void	resetTime() ;
	void	setCurrentTime( float inTime ) ;
	float	getCurrentTime() ;
	void	setInitialTime( float inStartTime ) ;	// Plays a partition from time specified in parameter
	float	getInitialTime() ;
	void	setFinalTime( float inStopTime ) ;		// Stop playback at time specified in parameter
	float	getFinalTime() ;
	
	void	setOscConsoleActive( bool flag ) ;
	
	/////////// OSC
	void	updateSendingOptions() ;
	void	sendOSCConfigMessage() ;
	void	sendOSCFirstMessage() ;
	void	sendOSCLastMessage() ;
	void	sendSavedOSC() ;
	void	sendCurveOSC( curveOSC mess ) ;
	void	sendTriggerOSC( triggerOSC mess ) ;
	void	PrintPacket(OSCbuf *b) ;

 public slots :
	void run();	
	void stop();
	virtual void nextStep() ;

 signals:
 
	void sendTriggerMessage( triggerOSC mess );
	void sendCurveMessage( curveOSC mess );
		
 private :
 
	NxGLScore		*mainGLBox ;
	NxScore			*score;
  	NxEnv			scoreEnv ;
		
	bool	oscConsoleActive ;
	bool	playing ;
	bool	readyToPlay ; // to avoid crashes after reset
	
	void	setFirstShot( bool chooser ) ;
 
	// Objects Indexes
	void	updateObjectsIndexes() ;
	int		nbCursors ;
	int		nbTriggers ;
	int		nbCurves ;
	
	// Sending Options (what to send)
	bool	scCRID, scCID, scCGID, scRID, scID, scGID, scA, scV, scX, scY, scZ ;
	bool	stCRID, stCID, stCGID, stRID, stID, stGID, stV, stX, stY, stZ ;
	
	// Parameters
	bool	advanceCursors ;
	bool	autoSchedulerStep ;
	bool	useCirclesAlgo ;
	bool	sendInitialFinalMessage ;
	bool	broadcastMode ;
	bool	oscActive ;
	bool	firstShot ;		// avoid banging at the beginning (triggers val1 / val2 )
 	// Time Variables
  	float	currentTime, initialTime, finalTime ;
	float	mainTempo ;
	float	dtime ;
	bool	finished;
 	bool	next ;			// For the timer
	int		gPrecision ;	// Time Step
	
  	// Objects
  	NxCursor* pCur;
	QMutex mutex;
	
	QTime clock ;
	QTimer* sTimer ;
		
	/////// SAVED OSC Messages / by cursor
	QVector<triggerOSC>		triggersMessages ;
	QVector<curveOSC>		curvesMessages ;

	/////// CURSORS
	QList<bool>				cursorsFinished ; 	// cursor finished or not
	QList<bool>				curDirections ;		// cursor directions at time t
	QList<bool>				curLoopShot ;		// avoid cursors loop bang
	/////// TRIGGERS sign changement test
	QList< QList<bool> >	triggersSign1 ; 
	QList< QList<bool> >	triggersSign2 ;
	/////// CURVES sign changement test	
	QList< QList<float> >	curvesDurations ;
	QList< QList<float> >	curvesTimers ;
	QList< QList<bool> >	curvesCollisions ;
	QList< QList<float> >	curvesLocalStep ; 
	
	/////// OBJECTS Indexes
	QList<int> cursInd ;
	QList<int> curvInd ;
	QList<int> trigInd ;
	QList<int> trajInd ;

	// UDP handlers / OSC buffer
	char		*triggersOSCmF, *curvesOSCmF ;
	NxNetwork 	*triggerClientUDP, *curveClientUDP, *envClientConfigUDP, *envClientFirstUDP, *envClientLastUDP ;
	int 		nbPackets;

	char bytes[1000];
	OSCbuf myBuf;
  	OSCbuf *b;
};
	
#endif
