///////////////
// IanniX
///////////////

////////////////////////////
// NxSpline.cpp
////////////////////////////

#include "NxSpline.h"
#include <iostream>
#include <math.h>
#include <QtOpenGL>

/////////////////////////////////
// Class TrimCurve
/////////////////////////////////

//////////////////////////
// Constructors
//////////////////////////
TrimCurve::TrimCurve(float ax,float ay,float bx,float by,float cx,float cy,int ndiv) // UNUSED
{
	Ax = ax;
  	Ay = ay;
  	Bx = bx;
  	By = by;
  	Cx = cx;
  	Cy = cy;
  	Ndiv = ndiv;
}

TrimCurve::TrimCurve(float ax,float ay,float bx,float by,float cx,float cy) // UNUSED
{
	Ax = ax; 
  	Ay = ay;
  	Bx = bx; 
  	By = by;
  	Cx = cx; 
  	Cy = cy;
  	Ndiv = (int)( max( abs( (int)Ax ), abs( (int)Ay ) ) / DIV_FACTOR );
}
	
TrimCurve::TrimCurve() 
{
	// That's the one which is used
}

// Maximum of 2 floats
float TrimCurve::max( int f1, int f2 )
{
	if (f1 >= f2)
    	return f1;
	else 
    	return f2;
}

// Evaluating distance
float TrimCurve::evalDistance( float x1, float y1, float x2, float y2 )
{
	return ( sqrt( pow( x2 - x1, 2.0) + pow(y2 - y1, 2.0) ) );
}
	
void TrimCurve::PutCurve( float ax, float ay, float bx, float by, float cx, float cy ) 
{
	Ax = ax;
  	Ay = ay;
  	Bx = bx; 
  	By = by;
  	Cx = cx; 
  	Cy = cy;
  	Ndiv = (int) ( (max(fabs((int)Ax),fabs((int)Ay)) + 1)/DIV_FACTOR ) ;
	//printf("Ax vaut : %f, Ay vaut : %f\n", Ax, Ay) ;
	//printf("Et donc Ndiv vaut : %d\n", Ndiv) ;
}
	
void TrimCurve::draw(float x,float y,float z) 
{
	float OrigX, OrigY, NewX, NewY;
  	float  t, f, g, h;
  	if (Ndiv==0)
    	Ndiv=1;

  	OrigX = x; 
  	OrigY = y;
	
  	for(int i=1; i<=Ndiv ; i++) {
		t = 1.0f / (float)Ndiv * (float)i;
		f = t*t*(3.0f-2.0f*t);
		g = t*(t-1.0f)*(t-1.0f);
		h = t*t*(t-1.0f);
		NewX = x + Ax*f + Bx*g + Cx*h;
		NewY = y + Ay*f + By*g + Cy*h;
		
		// OpenGL rendering
		glVertex3f( OrigX, OrigY, z );
		glVertex3f( NewX, NewY, z );
		
		OrigX = NewX;  
		OrigY = NewY;
		}
}


// Counting...
int TrimCurve::GetCount()
{
	if (Ndiv==0) Ndiv=1;
  	int PointCount = 1;

  	for(int i=1; i<=Ndiv ; i++) {
		PointCount++;
		}
  	return PointCount;
}
	
void TrimCurve::GetCurve( float x,float y, NxPoint NxPoints[], int& PointCount)
{
	int X, Y ;
  	float  t, f, g, h;
  	if (Ndiv==0) Ndiv=1;

  	X = (int)x; 
  	Y= (int)y;
  	NxPoints[PointCount].x = X;
  	NxPoints[PointCount].y = Y;
  	PointCount++;

  	for(int i=1; i<=Ndiv ; i++) {
    	t = 1.0f / (float)Ndiv * (float)i;
      	f = t*t*(3.0f-2.0f*t);
      	g = t*(t-1.0f)*(t-1.0f);
      	h = t*t*(t-1.0f);
      	X = (int)(x + Ax*f + Bx*g + Cx*h);
      	Y = (int)(y + Ay*f + By*g + Cy*h);
      	NxPoints[PointCount].x = X;
      	NxPoints[PointCount].y = Y;
      	PointCount++;
		}
}

//////////////////////
// Class NxSpline
//////////////////////
	
///////////////////////
// Constructors
///////////////////////
NxSpline::NxSpline( NxPoint inPos, NxPoint pt[], int np, float inMin, float inMax, float inScale )
{
	scale = inScale ;
	pos = inPos ;

	NP = np;
  	Px = new float[NP]; // real X position of curve points in the score
  	Py = new float[NP]; // real Y position of curve points in the score
  	Ax = new float[NP];
  	Ay = new float[NP];
  	Bx = new float[NP];
  	By = new float[NP];
  	Cx = new float[NP];
  	Cy = new float[NP];
  	k = new float[NP];
  	Mat[0] = new float[NP];
  	Mat[1] = new float[NP];
  	Mat[2] = new float[NP];

  	realMin = inMin;
  	realMax = inMax;

  	for(int i=0;i<NP ;i++) {
 		Px[i] = (float) ( pt[i].x + inPos.x ) ;												// real X position of curve points in the score
      	Py[i] = (float) ( ( pt[i].y - inMin - fabs(inMax-inMin)/2 )*scale + inPos.y ) ;	// real Y position of curve points in the score
	}
  	// Calculating minimal values on spline
  	vMin = Py[0];
  	vMax = Py[0];
  	for(int i=1;i<NP ;i++) {
 		if (Py[i] > vMax) vMax = Py[i]; 
		if (Py[i] < vMin) vMin = Py[i];	
		}
	// Debug printf
	//printf( "NxSp  Range : Min = %f / Max = %f\n", realMin, realMax );
	//printf( "NxSp  Extrema Y : Min = %f / Max = %f\n", vMin, vMax );
}

NxSpline::NxSpline( NxPoint inPos, float px[] , float py[] , int np, float inScale ) 
{
	scale = inScale ;
	pos = inPos ;
	
	NP = np;
  	Px = new float[NP];
  	Py = new float[NP];
  	Ax = new float[NP];
  	Ay = new float[NP];
  	Bx = new float[NP];
  	By = new float[NP];
  	Cx = new float[NP];
  	Cy = new float[NP];
  	k = new float[NP];
  	Mat[0] = new float[NP];
  	Mat[1] = new float[NP];
  	Mat[2] = new float[NP];

  	for(int i=0;i<NP ;i++) {
    	Px[i] = px[i];  
      	Py[i] = py[i];
		}
}

NxSpline::~NxSpline()
{
	delete[] Px;
  	delete[] Py;
  	delete[] Ax;
  	delete[] Ay;
  	delete[] Bx;
  	delete[] By;
  	delete[] Cx;
  	delete[] Cy;
  	delete[] k;
  	delete[] Mat[0];
  	delete[] Mat[1];
  	delete[] Mat[2];
}


void NxSpline::Generate() 
{
	float AMag , AMagOld ;
  	int i;
  	// vector A
  	for(i= 0 ; i<=NP-2 ; i++ ) {
    	Ax[i] = Px[i+1] - Px[i];
      	Ay[i] = Py[i+1] - Py[i];
		}
  	
	// k
  	AMagOld = (float)sqrt(Ax[0]*Ax[0] + Ay[0]*Ay[0]);
  	for(i=0 ; i<=NP-3 ; i++) {
    	AMag = (float)sqrt(Ax[i+1]*Ax[i+1] + Ay[i+1]*Ay[i+1]);
      	k[i] = AMagOld / AMag;
      	AMagOld = AMag;
		}
  	k[NP-2] = 1.0f;

  	// Matrix
  	for(i=1; i<=NP-2;i++) {
    	Mat[0][i] = 1.0f;
      	Mat[1][i] = 2.0f*k[i-1]*(1.0f + k[i-1]);
      	Mat[2][i] = k[i-1]*k[i-1]*k[i];
		}
  	Mat[1][0] = 2.0f;
  	Mat[2][0] = k[0];
  	Mat[0][NP-1] = 1.0f;
  	Mat[1][NP-1] = 2.0f*k[NP-2];

  	// 
  	for(i=1; i<=NP-2;i++) {
    	Bx[i] = 3.0f*(Ax[i-1] + k[i-1]*k[i-1]*Ax[i]);
      	By[i] = 3.0f*(Ay[i-1] + k[i-1]*k[i-1]*Ay[i]);
		}
  	Bx[0] = 3.0f*Ax[0];
  	By[0] = 3.0f*Ay[0];
  	Bx[NP-1] = 3.0f*Ax[NP-2];
  	By[NP-1] = 3.0f*Ay[NP-2];

  	//
  	MatrixSolve(Bx);
  	MatrixSolve(By);

  	for(i=0 ; i<=NP-2 ; i++ ) {
    	Cx[i] = k[i]*Bx[i+1];
      	Cy[i] = k[i]*By[i+1];
    }
		
	// Saving TrimCurve
	
	// Calculating Steps
	varX = Px[0];
	varY = Py[0];
	actualTime = 0.0;
	curveDef = trimCurve();	
	seg = curveDef[0];
	currentX = Px[0];
	currentY = Py[0];
	indexRT = 0;
	divRT = 0;
}

void NxSpline::MatrixSolve(float B[])
{
	float* Work = new float[NP] ;
  	float* WorkB = new float[NP] ;
  	int i, j;
  	for(i=0;i<=NP-1;i++) {
    	Work[i] = B[i] / Mat[1][i];
      	WorkB[i] = Work[i];
		}

  	for(j=0 ; j<10 ; j++) { ///  need convergence judge
    	Work[0] = (B[0] - Mat[2][0]*WorkB[1])/Mat[1][0];
      	for(i=1; i<NP-1 ; i++ ) {
	  		Work[i] = (B[i]-Mat[0][i]*WorkB[i-1]-Mat[2][i]*WorkB[i+1])/Mat[1][i];
			}
      	Work[NP-1] = (B[NP-1] - Mat[0][NP-1]*WorkB[NP-2])/Mat[1][NP-1];

      	for(i=0 ; i<=NP-1 ; i++ ) {
	  		WorkB[i] = Work[i];
			}
		}
		
  	for(i=0 ; i<=NP-1 ; i++ ) {
    	B[i] = Work[i];
		}
		
  	delete[] Work;
  	delete[] WorkB;
}

void NxSpline::draw() 
{
  	TrimCurve c;
  	for(int i=0; i<NP-1 ; i++) {
    	c.PutCurve(Ax[i],Ay[i],Bx[i],By[i],Cx[i],Cy[i]);
    	c.draw( Px[i], Py[i], pos.z );
	}	
}

int NxSpline::GetCurveCount()
{
	TrimCurve c;
  	int count = 0;
  	for(int i=0; i<NP-1 ; i++){
  		c.PutCurve(Ax[i],Ay[i],Bx[i],By[i],Cx[i],Cy[i]);
		count += c.GetCount();
		}
  	return count;
}

void NxSpline::GetCurve(NxPoint NxPoints[], int& PointCount)
{
	TrimCurve c;
  	for(int i=0; i<NP-1 ; i++) {
    	c.PutCurve(Ax[i],Ay[i],Bx[i],By[i],Cx[i],Cy[i]);
      	c.GetCurve(Px[i],Py[i], NxPoints, PointCount);
		}
}

//////////// closed cubic NxSpline ////////////////////
void NxSpline::GenClosed() 
{
	float AMag , AMagOld , AMag0;
  	int i;
	
  	// vector A
  	for( i= 0 ; i<=NP-2 ; i++ ) {
    	Ax[i] = Px[i+1] - Px[i];
      	Ay[i] = Py[i+1] - Py[i];
		}
		
  	Ax[NP-1] = Px[0] - Px[NP-1];
  	Ay[NP-1] = Py[0] - Py[NP-1];

  	// k
  	AMag0 = AMagOld = (float)sqrt(Ax[0]*Ax[0] + Ay[0]*Ay[0]);
  	for( i=0 ; i<=NP-2 ; i++) {
    	AMag = (float)sqrt(Ax[i+1]*Ax[i+1] + Ay[i+1]*Ay[i+1]);
      	k[i] = AMagOld / AMag;
      	AMagOld = AMag;
		}
		
  	k[NP-1]=AMagOld/AMag0; 

  	// Matrix
  	for(i=1; i<=NP-1;i++) {
    	Mat[0][i] = 1.0f;
      	Mat[1][1] = 2.0f*k[i-1]*(1.0f + k[i-1]);
      	Mat[2][i] = k[i-1]*k[i-1]*k[i];
		}
  	Mat[0][0] = 1.0f;
  	Mat[1][0] = 2.0f*k[NP-1]*(1.0f + k[NP-1]);
  	Mat[2][0] = k[NP-1]*k[NP-1]*k[0];

  	// 
  	for(i=1; i<=NP-1;i++) {
    	Bx[i] = 3.0f*(Ax[i-1] + k[i-1]*k[i-1]*Ax[i]);
      	By[i] = 3.0f*(Ay[i-1] + k[i-1]*k[i-1]*Ay[i]);
		}
  	Bx[0] = 3.0f*(Ax[NP-1] + k[NP-1]*k[NP-1]*Ax[0]);
  	By[0] = 3.0f*(Ay[NP-1] + k[NP-1]*k[NP-1]*Ay[0]);

  	//
  	MatrixSolveEX(Bx);
  	MatrixSolveEX(By);

  	for(i=0 ; i<=NP-2 ; i++ ) {
    	Cx[i] = k[i]*Bx[i+1];
      	Cy[i] = k[i]*By[i+1];
		}
  	Cx[NP-1] = k[NP-1]*Bx[0];
  	Cy[NP-1] = k[NP-1]*By[0];
}

///// tridiagonal matrix + elements of [0][0], [N-1][N-1] //// 
void NxSpline::MatrixSolveEX(float B[]) 
{
	float* Work = new float[NP];
  	float* WorkB = new float[NP];
  	int i, j;
  	for( i=0;i<=NP-1;i++ )  {
    	Work[i] = B[i] / Mat[1][i];
      	WorkB[i] = Work[i];
		}
  	for( j=0 ; j<10 ; j++) {  	// need judge of convergence
    	Work[0] = (B[0]-Mat[0][0]*WorkB[NP-1]-Mat[2][0]*WorkB[1])/Mat[1][0];
      	for( i=1; i<NP-1 ; i++ ) {
	  		Work[i] = (B[i]-Mat[0][i]*WorkB[i-1]-Mat[2][i]*WorkB[i+1])/Mat[1][i];
			}
		Work[NP-1] = (B[NP-1]-Mat[0][NP-1]*WorkB[NP-2]-Mat[2][NP-1]*WorkB[0])/Mat[1][NP-1];
      	for(i=0 ; i<=NP-1 ; i++ ) {
	  		WorkB[i] = Work[i];
			}
    }
  	for(i=0 ; i<=NP-1 ; i++ ) {
    	B[i] = Work[i];
		}
}

void NxSpline::drawClosed() 
{
	TrimCurve c;
  	for(int i=0; i<NP ; i++) {
    	c.PutCurve(Ax[i],Ay[i],Bx[i],By[i],Cx[i],Cy[i]);
      	c.draw(Px[i],Py[i], pos.z);
		}
}


// NxSpline value for t
TrimCurve* NxSpline::trimCurve()
{
	TrimCurve* c = new TrimCurve[NP];
  	for(int i=0; i<NP-1 ; i++) {
 		c[i].PutCurve(Ax[i],Ay[i],Bx[i],By[i],Cx[i],Cy[i]);
		}	
	return c;	
}


// Updating Real Values
void NxSpline::setRealValues( float inRMin, float inRMax )
{
	realMin = inRMin;
	realMax = inRMax;
}
void NxSpline::setRealMin( float inRMin )
{
	realMin = inRMin;
}
void NxSpline::setRealMax( float inRMax )
{
	realMax = inRMax;
}
void NxSpline::setScale( float inScale )
{
	scale = inScale;
}

// Returns a Y-axis value given a time
float NxSpline::getValue( float inTime )
{
  	float  t,f,g,h;
	float Z, Ybis; 
	actualTime = inTime;
	
	//printf("varX : %f\n",varX) ;
	while ( varX < (Px[0]+actualTime) )
	{
		divRT++;
		if ( divRT > 1000 ) {
			// Moving to next segment
			indexRT++;
			seg = curveDef[indexRT];
			currentX = Px[indexRT];
			currentY = Py[indexRT];
			divRT = 0;
		}
			
		// Evaluate new coordinates
		t = 1.0f / 1000.0 * (float)divRT;
		f = t*t*(3.0f-2.0f*t);
		g = t*(t-1.0f)*(t-1.0f);
		h = t*t*(t-1.0f);
		varX = currentX + seg.Ax*f + seg.Bx*g + seg.Cx*h;
		varY = currentY + seg.Ay*f + seg.By*g + seg.Cy*h;
	}
	
	// Calculating Real Value
	// Z = (varY - vMin) / ( vMax - vMin ); // OLD ONE
	//printf("vMin vMax : %f %f\n", vMin, vMax) ;
	//printf("varY : %f \n", varY) ;
	//printf("posY : %f \n", pos.y) ;
	
	Z = pos.y - varY ; 
	//Ybis = Z/scale + (realMax-realMin)/2 + realMin ;
	Ybis = (-Z/scale + (realMax-realMin)/2 + realMin) ;

	//printf("real max : %f real min : %f \n", realMax, realMin ) ;
	//printf("Z : %f \n", Z ) ;
	//printf("Ybis : %f \n", Ybis ) ;
	//Ybis = Z * realMax + (1 - Z) * realMin ; // OLD ONE in the score direction
	//Ybis = realMax - Ybis ; // NEW ONE in intuitive direction
	//varY = ( (realMax - realMin)/(vMax - vMin) ) * (varY - vMin) + realMin ;
	//printf("GENERATE : %f\n", Ybis) ;
	return ( Ybis );
}

// Inverted Function
float NxSpline::getInverseValue( float inTime ) // NOT USED at this time
{
	float t, f, g, h;
	actualTime = inTime;
	float Z, Ybis;	

	while ( varX > (Px[0] + actualTime) )
	{
		divRT--;
		if (divRT < 0) {
			// Moving to previous segment
			indexRT--;
			seg = curveDef[indexRT];
			currentX = Px[indexRT];
			currentY = Py[indexRT];
			divRT = 1000;
			}

		// Evaluate new coordinates
		t = 1.0f / 1000.0 * (float)divRT;
		f = t*t*(3.0f-2.0f*t);
		g = t*(t-1.0f)*(t-1.0f);
		h = t*t*(t-1.0f);
		varX = currentX + seg.Ax*f + seg.Bx*g + seg.Cx*h;
		varY = currentY + seg.Ay*f + seg.By*g + seg.Cy*h;
	}

	// Calculating Real Value
	Z = (varY - vMin) / ( vMax - vMin ) ;
	Ybis = Z * realMax + (1-Z) * realMin ;
	//varY = ( (realMax - realMin)/(vMax - vMin) ) * (varY - vMin) + realMin ;
	//printf("GENERATE inversevalue : %f\n", Ybis) ;
	return ( Ybis );
}
 

// Resetting Spline
void NxSpline::reset()
{
	//printf("spline reset\n") ;
	varX = Px[0];
	varY = Py[0];
	actualTime = 0.0;
	curveDef = trimCurve();	
	seg = curveDef[0];
	currentX = Px[0];
	currentY = Py[0];
	indexRT = 0;
	divRT = 0;
}

