///////////////
// IanniX
///////////////

////////////////////////////
// NxGLScore.h
////////////////////////////

#ifndef NXGLSCORE_H
#define NXGLSCORE_H

// Qt
#include <QtGui>
#include <QtOpenGL>

// IanniX
#include "../math/NxSpline.h"
#include "../../widgets/NxObjectCreator.h"
#include "../../widgets/NxObjectEditor.h"
#include "../../widgets/NxControlCenter.h"
#include "../../widgets/NxOscConsole.h"
#include "../../widgets/NxPreferences.h"
#include "../../widgets/NxTime.h"
#include "../../base/atoms/NxTypes.h"
#include "../../base/atoms/NxScore.h"
#include "../../base/scheduler/NxScheduler.h"
#include "../../network/NxNetwork.h"
#include "../../network/NxListener.h"
#include "../../widgets/NxView.h" 

class NxView;
class NxControlCenter;
class NxObjectCreator;
class NxObjectEditor;
class NxScheduler;
class NxTime;
class NxListener ;
class NxOscConsole ;
class NxPreferences ;

///////////////////////////////////////
// Main OpenGL class
///////////////////////////////////////		
class NxGLScore : public QGLWidget
{
  	Q_OBJECT
	
 public:
  	NxGLScore( QWidget* parent, NxView* daddy, QString inFile, int inL, int inH, bool isNew );
  	~NxGLScore();
	
	/////////////////////////////////
	// UTILITIES
	/////////////////////////////////
	void		setCPUWarning( bool act ) ;				/// Red light if CPU pb
	void		setToolBarInfosTxt( QString inTxt ) ;	/// Set ToolBar Info txt
	void		setCPUTimeStep( int val ) ;				/// CPU TimeStep
	GLfloat*	colorConv( Color inColor ) ;			/// (Color -> GLfloat*) converter
	void		updateScoreColors() ;					/// Score and Bckg colors
	float		getZoomScale() ;
	float		getBBsize() ;
	
	void		selectAll() ;
	void		deselectAll() ;		// deselect and dehighlight selected objects
	void		addObjectToSelection( NxObject* inObj ) ;
  	
	QString inXMLfile;
	
	/////////// UDP Listener
	QUdpSocket		*udpReceiver ; 
	NxListener		*listener ;
	bool	UDPListenerCreated ;
	
	float	BBsize ;				// Bounding Boxes fixed size (triggers and straight lines) - depends on zoom
	bool	deactivateGraphics ;	// Only OSC Mode
	int		gridStepInt ;			// Public because we need it to Duplicate (in NxView)
	
	
	void updateScoreEnvironmentVariables() ;
	void updateCheckedMenus() ;
	void updateLocalDisplayParameters() ;
	////// MODES
	void switchVisualMode() ;
	void switchOSCoutMode() ;
	void switchOSCinMode() ;
	void showHideTrajects() ;
	void showHideCursors() ;
	void showHideCurves() ;
	void showHideTriggers() ;
	QList<int> getGroupFilterList() ;
	QList<int> getLayerFilterList() ;
	void showHideGroups( int gp, bool filter ) ;
	void showHideLayers( int lay, bool filter ) ;
	////// DISPLAY
	void change2D3DView() ;
	void showHideScore() ;
	void showHideCrosses() ;
	void showHideCurvesCircles() ;
	void showHideCurvesRect() ;
	void showHideScreenInfos() ;
	void showHideObjectsNames() ;
	void showHideGrid() ;
	void switchSnapToGrid() ;
	void showHideCursorsArea() ;
	////// TIME
	void switchBroadcastMode() ;
	void switchCirclesAlgo() ;
	void switchSendIFmessages() ;
	void switchMoveCursors() ;

	/////////////////////////////////
	// TIMING FUNCTIONS
	/////////////////////////////////
	void	sendConfigOSC() ;
	void	playPause() ;
	void	pause() ;
	void	stopAndReset() ;
	bool	isPlaying() ; // says if the score is playing or not
	void	stopGLTimer() ;
	void	startGLTimer();
	void	setTempo( float inTempo );
	void	setTimeline( State* inStates, int chooser );
	void	hardOffsetsReset() ;

	/////////////////////////////////
	// Object handlers
	/////////////////////////////////
	NxView*				getMainWindow() ;
	NxControlCenter*	getControlCenter() ;
	NxScore*			getPartition();
	NxScheduler*		getScheduler() ;
	NxPoint				getCursor() ;
	QList<NxObject*>	getSelected() ;
	NxObject*			getHighlighted() ;
	int					getNumberSelected() ;
		
  	/////////////////////////////////
  	// Object Management
  	/////////////////////////////////
  	void addNewTraject( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, int inChx, int inAxis, float inRadius );
  	void addNewTrigger( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, QString inOscF ,QString inHost, int inPort, int inIntValue );
	void addNewCursor( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, bool inForward, int inMode, float inFactor, float inWidth, float inInitOffset, float inFinalOffset );
	void addRandomCurve( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor1, Color inColor2, int nbCtrlPoints, float rangeMin, float rangeMax, int newDuration, float rMin, float rMax, float inScale, int inSpatialPeriod, QString inOscF, QString inHost, int inPort );
	void addEditCurve( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor1, Color inColor2, float rMin, float rMax, float inScale, int inSpatialPeriod, QString inOscF, QString inHost, int inPort );
	QVector<NxPoint> randomCurve( int nbpoints, float inRangeMin, float inRangeMax, int duration );

  	//////////////////////////////////////////////////////////////////////////////////////////
  	////////////////////////////// OSC Server Control ////////////////////////////////////////
  	//////////////////////////////////////////////////////////////////////////////////////////
  	void updateUDPListener( bool chooser ) ; // to connect or disconnect the listener
	
	void oscNewTrigger( float inX, float inY, float inZ, int inCID, int inGID, int inVal, int colR, int colG, int colB ) ;
	void oscTriggerValue( int inID, int inValue ) ;
	
	void oscCurveCreationBegin( float inPx, float inPy, float inPz, int inCID, int inGID, int c1R, int c1G, int c1B ) ;
	void oscCurveCreationAddPt( float inPx, float inPy ) ;
	void oscCurveCreationEnd() ;	
	void oscCurveModifPt( int inID, int nb, float inX, float inY ) ;	
	void oscCurveAddPt( int inID, float inX, float inY ) ;
	void oscCurveRmPt( int inID, int inN ) ;
	void oscCurveAngle( int inID, float inAngle ) ;
	void oscCurveScale( int inID, float inScale ) ;
	void oscCurveStyle( int inID, int inStyle ) ;
	void oscCurveMin( int inID, float inMin ) ;
	void oscCurveMax( int inID, float inMax ) ;
	
	void oscNewCircle( float inX, float inY, float inZ, float inRadius, int inCID, int inGID, int inR, int inG, int inB ) ;
	void oscNewLine( float inX1, float inY1, float inZ1, float inX2, float inY2, float inZ2, int inCID, int inGID, int inR, int inG, int inB ) ;
	void oscTrajectRadius( int inID, float inRadius ) ;
	void oscTrajectLength( int inID, float inLength ) ;
	void oscTrajectPoint( int inID, int firstPt, float inX, float inY, float inZ ) ;
	
	void oscNewCursor( int inID, int inCID, int inGID, float inSpeed, float inWidth, int inForward, int inMode, int inR, int inG, int inB ) ;
	void oscCursorOffset( int inID, float inOffsetPC ) ;
	void oscCursorInitOffset( int inID, float inOffsetPC ) ;
	void oscCursorFinalOffset( int inID, float inOffsetPC ) ;
	void oscCursorForward( int inID, int playb ) ;
	void oscCursorMode( int inID, int mode ) ;
	void oscCursorWidth( int inID, float wid ) ;
	void oscCursorSpeed( int inID, float speed ) ;
	
	void oscDeleteObject( int type, int inID) ;
	void oscObjectLayer( int type, int inID, int inLay ) ;
	void oscObjectName( int type, int inID, const char *inName ) ;
	void oscObjectCID( int type, int inID, int inCID ) ;
	void oscObjectGID( int type, int inID, int inGID ) ;
	void oscObjectActivity( int type, int inID, int chooser) ;
	void oscObjectPosition( int type, int inID, float inx, float iny, float inz ) ;
	void oscObjectColor( int type, int inID, int inR, int inG, int inB ) ;
	
	void oscObjectHost( int type, int inID, const char *inHost ) ;
	void oscObjectPort( int type, int inID, int inPort ) ;
	
	void oscShowHideObject( int type, int chooser) ;
	void osc3DRotateX( float ang ) ;
	void osc3DRotateY( float ang ) ;
	
	void oscSendInfos() ;
	void oscSetTempo( float inTempo ) ;
	void oscMoveCursors( int chooser ) ;
	void oscBroadcastMode( int chooser )  ;
	void oscOSCMode( int chooser )  ;
	////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////
	
  	////////// UTILITIES
	float	pixToRealX(float _x) ;
	float	pixToRealY(float _y) ;
	float	snapXY( float _a ) ;
	int		realToPixX(float _x) ;
	int		realToPixY(float _y) ;
	float	getLineAngle( NxPoint inP1, NxPoint inP2 ) ;
	float	shiftAngleRound( float inAngle ) ;
	bool	isShiftXLocked( NxPoint inP1, NxPoint inP2 ) ;
	bool	isInside( NxPoint inPoint, NxPoint inP1, NxPoint inP2 );
	
	void	updateCursorsAnglePosition( NxTraject *inTraject ) ;
	void	updateGroupFilterList() ;
	void	updateLayerFilterList() ;
	
	/////////////////////////////////
	// RENDERING
	/////////////////////////////////
	void	repaint() ;
	void	updateAntialiasing() ; // set antialiasing as the checked menu of NxView
  	void	setAntialiasing( bool state );
  	void	frustumView();
	
	void	rotationSetX( float angX );
	void	rotationSetY( float angY );
	void	rotationAddX( float offset );
  	void	rotationAddY( float offset );

	void	updateTrigg_LinesBB() ;		// to actualize BBs sizes / zoom
	
	void	setZoom( float factor );
	void	zoomBy( float inZoom ) ; // setZoom() but conserve center
	void	fitToPageView() ;
	void	hardResetView() ; // (after score dimensions change...)
	void	checkSelection( NxPoint inP1, NxPoint inP2 ) ;					// Selection rectangle
	bool	isLineInBox( NxPoint A, NxPoint B, NxPoint p1, NxPoint p2 ) ;	// Lines tool
  	void	zoomRect( NxPoint inP1, NxPoint inP2 ) ;		// Zoom rectangle
  	void	checkHighlighted( int curX, int cuY ) ;								// Object Selection
  	void	selectCurvePoints( NxCurve* inCurve, int inX, int inY ) ;			// Curve point selection
	void	selectCurveAnglePoint( int inX, int inY ) ;		// Curve angle edition point
	void	selectTrajectPoints( NxTraject* inTraject, int inX, int inY ) ;		// Traject edition points selection
	void	selectCursorPoints( int inX, int inY ) ;		// Cursor width points selection
  	void	destroyObject( NxObject* inSelected );
	void	displayToSelectObjects( QMenu* inMenu ) ;		// Select Object from ALL
  	void	displayToPasteObjects( QMenu* inMenu );			// Paste an Object from ALL
	void	displayToViewUnderObjects( QMenu* inMenu ) ;	// Select Object under cursor
  	void	updateIndexes();
  	void	assignCursors();
	void	setDrawMode( int inMode );
  	void	flagsOff();
	void	resetSchedule();
	
	/////////// CURVE EDITION ///////////
	void	removeCurvePoint() ;	// to delete a point on a curve
	void	addCurvePoint() ;		// add a point to a curve
	void	flattenCurve( NxCurve *toFlattenCurve ) ; // to flatten a curve
	void	changeCurveBounds( NxCurve* inCurve, float newMin, float newMax, bool resc, bool chgescale, bool transObj ) ;
	
	void	setCursorsActive( NxTraject *inTraject, bool act ) ;

 public slots :
 
	virtual void processUDPMessage() ; // UDP Listener
		
	virtual void refreshView();
  	virtual bool save();
  	virtual bool saveAs();
  	virtual void removeSelected();
	virtual void editSpeed();
	virtual void editWidth();
	virtual void openControl();
	virtual void openOscConsole();
	virtual void closeOscConsole();
  	virtual void open3DCenter();
	virtual void openPreferences() ;
	virtual void openCreator();
	/////// Contextuals / settings ///// (To be Accessed from NxView Menus)
	virtual void editGroup() ;
	virtual void setGridOpacity( float inOp ) ;
	virtual void setCursorZonesOpacity( float inOp ) ;
	virtual void setNamesOpacity( float inOp ) ;
	virtual void changeGridStep() ;
	virtual void setDynamicGridStep( int val ) ;
	virtual void setGridStep() ;
	virtual void switchAutoGridStep() ;
	virtual void changePlaybackTime() ;
	virtual void setPlaybackTime() ;
	
 protected:
 	void initializeGL();
  	void paintGL();
	void resizeGL( int w, int h );
	
	void checkEditionModes() ;
	void exitEditionModes() ;
	
	void setCursorOpenHand() ;
	void setCursorClosedHand() ;
	void setCursorNormal() ;
	
	////////////////////////////////// RENDERING TOOLS
	void	renderTriggers();
  	void	renderTrajects();
  	void	renderCurves();
  	void	renderCursors( NxTraject *inTraject );
	
  	void	drawBoundingBox( BBox inBox ) ;
	void	drawCursorBoundingBox( NxCursor* inCurs ) ;
	void	drawPointBox( NxPoint inP, QColor inColor ) ;			// simple point white rectangle draw
	void	drawRect( NxPoint p1, NxPoint p2) ;		// simple rectangle draw
	void	drawCross( NxPoint p1, float inSize ) ;	// simple cross draw
  	void	drawLine( NxPoint inP1, NxPoint inP2, QColor inColor ) ;
	void	drawDashedLine( NxPoint inP1, NxPoint inP2, QColor inColor ) ;
	void	drawCursorZone( NxCursor *inCursor ) ;
  	void	drawCircle( GLfloat R, GLfloat cX, GLfloat cY, GLfloat cZ, int mode, QColor inColor );
	void	drawDashedArc( Circle inC, float inA1, float inA2, QColor inColor ) ;
  	void	drawCursor( NxCursor *inCursor, Color inColor ) ;
  	void	drawTrigger( NxPoint inP, Color inColor );
	void	drawObjectName( NxObject *inObj );
	void	drawPointText( QString inText, NxPoint inP, int decx, int decy );
	
  	// Event handlers
  	virtual void keyPressEvent( QKeyEvent* e );
  	virtual void keyReleaseEvent( QKeyEvent* e );
  	virtual void mousePressEvent( QMouseEvent* );
  	virtual void mouseMoveEvent( QMouseEvent * e );
  	virtual void mouseDoubleClickEvent( QMouseEvent* );
  	virtual void mouseReleaseEvent( QMouseEvent* );
  	virtual void contextMenuEvent( QContextMenuEvent* );
		
 protected slots :
 
	//////////////// CONTEXTUAL MENU SLOTS ////////////////
	virtual void selectTraject( QAction *action ) ;
	virtual void selectCursor( QAction *action ) ;
	virtual void selectCurve( QAction *action ) ;
	virtual void selectTrigger( QAction *action ) ;
	virtual void pasteTraject( QAction * action ) ;
	virtual void pasteTrigger( QAction * action ) ;
	virtual void pasteCurve( QAction * action ) ;
	///// OBJECTS
	virtual void switchActivity() ;
	///// CURSORS
	virtual void changeCurrentOffset() ;
	virtual void setChangedCurrentOffset() ;
	///// GROUPS
	virtual void editObject() ;
	virtual void editSelection() ;
	virtual void deleteGroup() ;
	virtual void setDeletedGroup() ;
	
 private:
 
	NxScheduler		*timeScheduler ;
	NxView			*mParent ;
	
 	GLfloat scaleX, scaleY, scaleZ ;
	GLfloat RScaleX, RScaleY ;
  	GLfloat transX, transY;
  	GLfloat selX, selY;
	
  	int		NbSLICES;
	float	curveXMinStep ; // minimal x-distance between curves points (x step)	
  	
	float	oldAngleX, oldAngleY, angleX, angleY ; // Rotation angles for the 3D View
	bool	mouseRotation ;
		
	bool	doubleClickOk ; // to avoid launching editors when moving and clicking fast
	bool	mousePressed ;
	
	////////////////// ENVIRONMENT VARIABLES //////////////////
	/////// Local
	bool	neverSaved ;
	float	gridDecX, gridDecY;
	NxPoint lineRulePt1 ;			// for manual Rule Line and for angle measuring
	bool	lineRuleMode ;
	bool	lineRuleSetFirstPt ;
	bool	gridMove ;
	bool	lockedSelection ;		// selection by contextual menu
	/////// MODES
	bool	visualMode ;
	bool	oscInMode, oscOutMode ;
	bool 	showTrajects, showCursors, showCurves, showTriggers ; // Objects Rendering
	bool	showAllL, showAllG ;
	int		showOnlyL, showOnlyG ;
	/////// DISPLAY
	bool	showGrid ;
	bool	crossesDraw ;
	bool	scoreDraw ;
	bool	displayOn ;
	bool	showNames ;
	bool	circlesDraw;	
	bool	rectDraw ;	
	bool	snapGrid ;	
	bool	autoGridStep ;
	float	gridStep ;
	bool	showCursorsArea ;
	bool	antialiasing ;
	/////// TIME / OSC
	bool	useCirclesAlgo ;
	bool	sendIFmessages ;
	bool	moveCursors ;
	bool	broadcastMode ;
	
	////////////////////////////////////////////////////////////////////////////////////////
	////////////////////// CONTEXTUAL MENU FIELDS //////////////////////////////////////////
	QLineEdit	*timiF, *timfF ;						// DISPLAY // TIME
	QLineEdit	*gridStepF, *radF, *lenF, *layerF ;
	QSlider		*gStepSlider ;
	QLineEdit	*initoF, *finaloF, *curOffF ;			// CURSORS
	QLineEdit	*gpDelF ;								// GROUP Delete
	QCheckBox	*trajdC, *cursdC, *curvdC, *triggdC ;	// Filter objects to delete
	
	NxCursor	*toChgOffsetsCursor ;
	
  	NxObjectCreator*	objectCreator;
  	NxControlCenter*	controlCenter;
	NxObjectEditor*		objectEditor;
	NxOscConsole*		oscConsole;
	NxPreferences*		preferencesDialog ;
	
  	NxTime* timeWidthEditor;
	NxTime* timeSpeedEditor;
  	QMenu	*contextMenu ;		// Right-clic contextual Menu
	QMenu	*subUnderMenu ;		// Under cursor selected objects
  	QFileDialog* fileChooser;
  	QDialog		*cGridDialog, *trajDialog ;
	QDialog		*transDialog, *gpLayerDialog, *gpDeleteDialog, *gpActDialog ; // GROUPS

	QDialog		*curOffsDialog ;
	QDialog		*cPlaybackTimeDialog ; // TIME // DISPLAY
	////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////
	
	Color	selectedColor, BBColor, bColor, sColor, gColor, inactiveColor, hiddenColor ;
	float	gridOpacity, cursZonesOpacity, namesOpacity ;
	
  	/////////////// Position Variables
	int mouseXPixPos, mouseYPixPos ;
  	float posX, posY ;
  	int mouseX, mouseY ;
  	float diffX, diffY ;
	NxPoint timePos ;
	NxPoint timePosPix ;
	
  	//////////////////////////////////// Interactions flags
	bool dPressed, shiftPressed ;
  	bool firstClick, frustum, onClick, onDrag, zoomMode, scoreDragMode, selectionMode ;
	bool dragging ;
	bool cursorMode, cursorMoving, firstPoint, mouseZoom ;
	
	////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////// TRAJECTS and CURVES EDITION
	////////////////////////////////////////////////////////////////////////////////////////////////
	NxPoint tempP ; ///// position of moved point (for both curve and traject edition)
	///////////// LINE RULE MODE
	float		lineRuleAngle, lineRuleLength ;
	///////////// TRAJECT EDITION
	bool		trajectCreation, lineEdition, circleEdition ;
	NxTraject	*createdTraject, *editedLine, *editedCircle ;
	NxPoint		lineEdPt1, lineEdPt2, circleEdPt ;
	float		editedTrajectAngle ;
	///////////// CURSOR EDITION
	bool		cursorEdition ;
	NxCursor	*editedCursor ;
	NxPoint		cursorEdPt1, cursorEdPt2, initOffPt, finalOffPt ;
	///////////// CURVE ANGLE EDITION
	bool		curveAngleEdition ;
	NxCurve		*editedAngleCurve ;
	NxPoint		curveAngleEdPt ;
	float		curveAngleEd ;
  	///////////// CURVE POINTS EDITION
	NxCurve		*editedPointsCurve ;
	///////////// CURVE EDITION
	bool	curveCreation, curveEdition ;
  	QVector<NxPoint> editedCurve ;
  	NxPoint 	savPos;
  	QString 	savName;
  	int savID, savGroupID, currentPoints;
	NxPoint	 firstCurvPos ; // first point while curve creation
	float 		curvEdInfoX, curvEdInfoY ; // on screen coordinates display info
	float		curvEdScale, curvEdMax, curvEdMin, curvEdYmin, curvEdYmax, curvEdXmin, curvEdXmax ;
	float		curvEdPosX, curvEdPosY ;
	float		curvEdYRange ;
	bool		curvEdPtChoosed ;
	float		curvEd1x, curvEd1y, curvEd2x, curvEd2y ;
	bool		curvEdlast, curvEdfirst ;
	bool		curvEdOnlyClic ;
	////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////// OSC CURVE CREATION
	////////////////////////////////////////////////////////////////////////////////////////////////
	bool				oscCurveCreation, oscCurveListening ;
	QVector<NxPoint>	oscEditedCurve ;
	NxPoint				oscCurvePos ;
	int					oscCurve1R, oscCurve1G, oscCurve1B ; // Color 1
	int					oscCurveCID, oscCurveGID ;
	int					oscNbPoints ;
	float				oscCurveRealMin, oscCurveRealMax ;
	////////////////////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////////////////////
	
  	// Strings & Fonts
  	QString str;
	
	///////// SELECTION LIST (superposition)
	QVector<NxTraject*>		selectedTrajectList ;
	QVector<NxCursor*>		selectedCursorList ;
	QVector<NxCurve*>		selectedCurveList ;
	QVector<NxTrigger*>		selectedTriggerList ;
	
  	///////// SELECTED OBJECTS
	bool		objectDragging, scoreDragging ;
	int			numberSelected ;
	int			trajID;
	int			selTrajectPoint, selCursorPoint ;
	QList<NxPoint>		selectedPositions ;
  	QList<NxObject*>	selected ;
	NxObject*			highlighted ;	
	NxObject*	edited;

  	//////////// Partition & objets indexes
  	NxScore		*score;
	//////////// OBJECTS ( rendered / checkingSelected / assignCursors() / editWidthSpeed() )
	NxTraject	*renderedTraj, *traj, *assParent ;
	NxCurve		*renderedCurv, *curve ;
	NxCursor	*renderedCurs, *curs, *assCurs, *editCursor ;
	NxTrigger	*renderedTrig, *trig ;
	//////////// Mouse released event
	NxCursor	*tmpCurs ;
	NxTraject	*tmpSelTraj ;
	
  	Circle circ, newC ;
  	BBox inBox, bb;
	
  	QVector<NxPoint> tmpCurves;
	QVector<NxPoint> selCurve;
  	NxSpline* coolSpline;
  	NxPoint p, tmpPos, objectPos, clickPos, clickPixPos, newPos ;
  	float bX, bY, newrad, radius ;
  	float realMin, realMax;
	float nHeight ;
	int nbCurves, nbTrajects, nbCursors, nbTriggers, length, lengthC, height, trajShape;
  	int nbCercles, randomizer;
	int drawMode;
	
  	///// Display Refresh Timer
	QTimer *glTimer;
	
  	// Generic decls
  	int selfW, selfH, initSelfH, initSelfW ;
	int nbCtrlPoints, newDuration, pointID, IDindex;
  	float rangeMin, rangeMax;
};

#endif
