///////////////
// IanniX
///////////////

////////////////////////////
// NxControlCenter.cpp
////////////////////////////

#include "NxControlCenter.h"
#include "NxControlCenterDelegate.h"
#include <cstdlib>

///////////////////////////////////
// Main Constructor
///////////////////////////////////
NxControlCenter::NxControlCenter( NxGLScore* parent )
  : QDialog( parent )
{
	QFont f( "Lucida Grande", 12 );
	setFont( f );
	
	setWindowTitle( tr( "IanniX Control Center" ) );
	backgColor = QColor(0,0,0,3) ;
  	
	score			= parent->getPartition();
  	parentGLBox 	= parent;
	
	nbCurves		= score->curvesIndexes().count();
	nbTrajects		= score->trajectsIndexes().count();
	nbTriggers		= score->triggersIndexes().count();
	nbCursors		= score->cursorsIndexes().count();
		
	// Widgets
  	controlBox 		= new QTabWidget;
	curvePage 		= new QWidget(controlBox) ;
	trajectPage 	= new QWidget(controlBox) ;
  	triggerPage 	= new QWidget(controlBox) ;
  	cursorPage 		= new QWidget(controlBox) ;
	
	buildCurvesItems() ;
	buildTrajectsItems() ;
	buildCursorsItems() ;
	buildTriggersItems() ;

	// Settings
  	updateIndexes();
  	controlBox->setCurrentIndex( 0 );
	
  	// Adding Sections
  	controlBox->addTab( curvePage, "Curves" );
  	controlBox->addTab( trajectPage, "Trajectories" );
  	controlBox->addTab( cursorPage, "Cursors" );
	controlBox->addTab( triggerPage, "Triggers" );
	this->setTabOrder( curvePage, trajectPage ) ;
	this->setTabOrder( trajectPage, triggerPage ) ;
	this->setTabOrder( triggerPage, cursorPage ) ;
	this->setTabOrder( cursorPage, curvePage ) ;
	
	// First time filling tables
	addItems() ;
	updateIndexes() ;
	
	// Resizing
	printf("- Control Center : Resizing Tables\n") ;
	resizeTables() ;
	
	// Initializing signals (cells modifying)
	connect( curveList,  	SIGNAL( itemChanged(QStandardItem*) ), this, SLOT( updateCurve( QStandardItem* ) ) );
	connect( trajectList,  	SIGNAL( itemChanged(QStandardItem*) ), this, SLOT( updateTraject( QStandardItem* ) ) );
	connect( cursorList,  	SIGNAL( itemChanged(QStandardItem*) ), this, SLOT( updateCursor( QStandardItem* ) ) );
	connect( triggerList,  	SIGNAL( itemChanged(QStandardItem*) ), this, SLOT( updateTrigger( QStandardItem* ) ) );
	
  	// Layout	
  	mainLayout = new QVBoxLayout( this );
  	mainLayout->addWidget( controlBox );
	
//	QAction *closeAction = new QAction( this ) ;
//	closeAction->setShortcut( Qt::CTRL + Qt::Key_W );
//	connect(closeAction, SIGNAL( changed() ), this, SLOT( close() ) ) ;
	
  	// Global settings
  	//resize( QSize( 950, 660 ).expandedTo(minimumSizeHint()) );
  	//setMaximumSize( 950, 660 );
	////setWindowModality( Qt::ApplicationModal ) ;
}

//////////////////////
// Destructor
//////////////////////
NxControlCenter::~NxControlCenter()
{
	// Done by Qt...
}
////////////////////////////////////////
// Resizing tables
////////////////////////////////////////
void NxControlCenter::resizeTables()
{
	curveTable->resizeRowsToContents() ;
	curveTable->resizeColumnsToContents() ;
	trajectTable->resizeRowsToContents() ;
	trajectTable->resizeColumnsToContents() ;
	cursorTable->resizeRowsToContents() ;
	cursorTable->resizeColumnsToContents() ;
	triggerTable->resizeRowsToContents() ;
	triggerTable->resizeColumnsToContents() ;
}
////////////////////////////////////////
// First time building
////////////////////////////////////////
void NxControlCenter::buildCurvesItems() 
{
	QGroupBox *curveProxy		= new QGroupBox(curvePage) ;
	curveTable					= new QTableView(curvePage) ;
	curveList					= new QStandardItemModel(0, 12, curvePage);
	curveProxyModel				= new QSortFilterProxyModel;
	curveProxyModel->setDynamicSortFilter(true);

	curveProxyModel->setSourceModel(curveList);
	curveTable->setModel(curveProxyModel) ;
	curveTable->setSortingEnabled(true);
	curveTable->setEditTriggers( QAbstractItemView::CurrentChanged | QAbstractItemView::DoubleClicked | QAbstractItemView::SelectedClicked ) ;
    curveTable->setAlternatingRowColors(true);
	
	NxControlCenterDelegate *delegate = new NxControlCenterDelegate( this ) ;
	delegate->setObjectType(0) ;
	curveTable->setItemDelegate( delegate );

	curveSortCaseSensitivityCheckBox	= new QCheckBox(tr("Case sensitive sorting"));
    curveFilterCaseSensitivityCheckBox	= new QCheckBox(tr("Case sensitive filter"));
	
	curveFilterPatternLineEdit = new QLineEdit(curveProxy) ;
	    
	curveFilterSyntaxComboBox = new QComboBox(curveProxy) ;
    curveFilterSyntaxComboBox->addItem(tr("Regular expression"), QRegExp::RegExp);
    curveFilterSyntaxComboBox->addItem(tr("Wildcard"), QRegExp::Wildcard);
    curveFilterSyntaxComboBox->addItem(tr("Fixed string"), QRegExp::FixedString);
    QLabel *filterSyntaxLabel = new QLabel(tr("Filter by"));
	curveFilterSyntaxComboBox->setCurrentIndex(2);

    curveFilterColumnComboBox = new QComboBox(curveProxy) ;
    curveFilterColumnComboBox->addItem(tr("ID"));
    curveFilterColumnComboBox->addItem(tr("Name"));
    curveFilterColumnComboBox->addItem(tr("Custom ID"));
	curveFilterColumnComboBox->addItem(tr("Group ID"));
	curveFilterColumnComboBox->addItem(tr("Layer"));
	curveFilterColumnComboBox->addItem(tr("Duration"));
	curveFilterColumnComboBox->addItem(tr("Style"));
	curveFilterColumnComboBox->addItem(tr("Real Min"));
	curveFilterColumnComboBox->addItem(tr("Real Max"));
	curveFilterColumnComboBox->addItem(tr("Host IP"));
	curveFilterColumnComboBox->addItem(tr("UDP Port"));
	curveFilterColumnComboBox->addItem(tr("OSC Format"));
    QLabel *filterColumnLabel = new QLabel(tr("in"));
	curveFilterColumnComboBox->setCurrentIndex(1);

	QHBoxLayout *proxyLayout = new QHBoxLayout(curveProxy) ;
    proxyLayout->addWidget(filterSyntaxLabel);
    proxyLayout->addWidget(curveFilterSyntaxComboBox);
	proxyLayout->addWidget(curveFilterPatternLineEdit);
    proxyLayout->addWidget(filterColumnLabel);
    proxyLayout->addWidget(curveFilterColumnComboBox);
    proxyLayout->addWidget(curveFilterCaseSensitivityCheckBox);
    proxyLayout->addWidget(curveSortCaseSensitivityCheckBox);
    curveProxy->setLayout(proxyLayout);
	
	QVBoxLayout *curvesLay = new QVBoxLayout(curvePage) ;
	curvesLay->addWidget(curveProxy);
    curvesLay->addWidget(curveTable);
    curvePage->setLayout(curvesLay);
	
	curvesFilterColumnChanged() ;
	
	connect(curveFilterPatternLineEdit,			SIGNAL(textChanged(const QString &)), this,	SLOT(curvesFilterRegExpChanged()));
    connect(curveFilterSyntaxComboBox,			SIGNAL(currentIndexChanged(int)), this,		SLOT(curvesFilterRegExpChanged()));
    connect(curveFilterColumnComboBox,			SIGNAL(currentIndexChanged(int)), this,		SLOT(curvesFilterColumnChanged()));
    connect(curveFilterCaseSensitivityCheckBox,	SIGNAL(toggled(bool)), this,				SLOT(curvesFilterRegExpChanged()));
    connect(curveSortCaseSensitivityCheckBox,	SIGNAL(toggled(bool)), this,				SLOT(curvesSortChanged()));

	curveList->setHeaderData(0, Qt::Horizontal, QObject::tr("ID"));
	curveList->setHeaderData(1, Qt::Horizontal, QObject::tr("Name"));
	curveList->setHeaderData(2, Qt::Horizontal, QObject::tr("Custom ID"));
	curveList->setHeaderData(3, Qt::Horizontal, QObject::tr("Group ID"));
	curveList->setHeaderData(4, Qt::Horizontal, QObject::tr("Layer"));
	curveList->setHeaderData(5, Qt::Horizontal, QObject::tr("Duration (s)"));
	curveList->setHeaderData(6, Qt::Horizontal, QObject::tr("Style"));
	curveList->setHeaderData(7, Qt::Horizontal, QObject::tr("Real Min"));
	curveList->setHeaderData(8, Qt::Horizontal, QObject::tr("Real Max"));
	curveList->setHeaderData(9, Qt::Horizontal, QObject::tr("Host IP"));
	curveList->setHeaderData(10, Qt::Horizontal, QObject::tr("UDP Port"));
	curveList->setHeaderData(11, Qt::Horizontal, QObject::tr("OSC Format"));
}
void NxControlCenter::buildTrajectsItems()
{
	QGroupBox *trajectProxy		= new QGroupBox(trajectPage) ;
	trajectTable				= new QTableView(trajectPage) ;
	trajectList					= new QStandardItemModel(0, 7, trajectPage);
	trajectProxyModel			= new QSortFilterProxyModel;
	trajectProxyModel->setDynamicSortFilter(true);
	trajectProxyModel->setSourceModel(trajectList);
	trajectTable->setModel(trajectProxyModel) ;
	trajectTable->setSortingEnabled(true);
	trajectTable->setEditTriggers( QAbstractItemView::CurrentChanged | QAbstractItemView::DoubleClicked | QAbstractItemView::SelectedClicked ) ;
    trajectTable->setAlternatingRowColors(true);

	NxControlCenterDelegate *delegate = new NxControlCenterDelegate( this ) ;
	delegate->setObjectType(1) ;
	trajectTable->setItemDelegate( delegate );

	trajectSortCaseSensitivityCheckBox		= new QCheckBox(tr("Case sensitive sorting"));
    trajectFilterCaseSensitivityCheckBox	= new QCheckBox(tr("Case sensitive filter"));

	trajectFilterPatternLineEdit = new QLineEdit(trajectProxy) ;
	trajectFilterSyntaxComboBox = new QComboBox(trajectProxy) ;
    trajectFilterSyntaxComboBox->addItem(tr("Regular expression"), QRegExp::RegExp);
    trajectFilterSyntaxComboBox->addItem(tr("Wildcard"), QRegExp::Wildcard);
    trajectFilterSyntaxComboBox->addItem(tr("Fixed string"), QRegExp::FixedString);
    QLabel *filterSyntaxLabel = new QLabel(tr("Filter by"));
	trajectFilterSyntaxComboBox->setCurrentIndex(2);

	trajectFilterColumnComboBox = new QComboBox(trajectProxy) ;
    trajectFilterColumnComboBox->addItem(tr("ID"));
    trajectFilterColumnComboBox->addItem(tr("Name"));
    trajectFilterColumnComboBox->addItem(tr("Custom ID"));
	trajectFilterColumnComboBox->addItem(tr("Group ID"));
	trajectFilterColumnComboBox->addItem(tr("Layer"));
	trajectFilterColumnComboBox->addItem(tr("Type"));
	trajectFilterColumnComboBox->addItem(tr("Length"));
    QLabel *filterColumnLabel = new QLabel(tr("in"));
	trajectFilterColumnComboBox->setCurrentIndex(1);

	QHBoxLayout *proxyLayout = new QHBoxLayout(trajectProxy) ;
    proxyLayout->addWidget(filterSyntaxLabel);
    proxyLayout->addWidget(trajectFilterSyntaxComboBox);
	proxyLayout->addWidget(trajectFilterPatternLineEdit);
    proxyLayout->addWidget(filterColumnLabel);
    proxyLayout->addWidget(trajectFilterColumnComboBox);
    proxyLayout->addWidget(trajectFilterCaseSensitivityCheckBox);
    proxyLayout->addWidget(trajectSortCaseSensitivityCheckBox);
    trajectProxy->setLayout(proxyLayout);

	QVBoxLayout *trajectLay = new QVBoxLayout(trajectPage) ;
	trajectLay->addWidget(trajectProxy);
    trajectLay->addWidget(trajectTable);
	trajectPage->setLayout(trajectLay);

	trajectsFilterColumnChanged() ;
	
	connect(trajectFilterPatternLineEdit,			SIGNAL(textChanged(const QString &)), this,	SLOT(trajectsFilterRegExpChanged()));
    connect(trajectFilterSyntaxComboBox,			SIGNAL(currentIndexChanged(int)), this,		SLOT(trajectsFilterRegExpChanged()));
    connect(trajectFilterColumnComboBox,			SIGNAL(currentIndexChanged(int)), this,		SLOT(trajectsFilterColumnChanged()));
    connect(trajectFilterCaseSensitivityCheckBox,	SIGNAL(toggled(bool)), this,				SLOT(trajectsFilterRegExpChanged()));
    connect(trajectSortCaseSensitivityCheckBox,		SIGNAL(toggled(bool)), this,				SLOT(trajectsSortChanged()));

	trajectList->setHeaderData(0, Qt::Horizontal, QObject::tr("ID"));
	trajectList->setHeaderData(1, Qt::Horizontal, QObject::tr("Name"));
	trajectList->setHeaderData(2, Qt::Horizontal, QObject::tr("Custom ID"));
	trajectList->setHeaderData(3, Qt::Horizontal, QObject::tr("Group ID"));
	trajectList->setHeaderData(4, Qt::Horizontal, QObject::tr("Layer"));
	trajectList->setHeaderData(5, Qt::Horizontal, QObject::tr("Type"));
	trajectList->setHeaderData(6, Qt::Horizontal, QObject::tr("Length"));
}
void NxControlCenter::buildCursorsItems()
{
	QGroupBox *cursorProxy		= new QGroupBox(cursorPage) ;
	cursorTable					= new QTableView(cursorPage) ;
	cursorList					= new QStandardItemModel(0, 10, cursorPage);
	cursorProxyModel			= new QSortFilterProxyModel;
	cursorProxyModel->setDynamicSortFilter(true);
	
	cursorProxyModel->setSourceModel(cursorList);
	cursorTable->setModel(cursorProxyModel) ;
	cursorTable->setSortingEnabled(true);
	cursorTable->setEditTriggers( QAbstractItemView::CurrentChanged | QAbstractItemView::DoubleClicked | QAbstractItemView::SelectedClicked ) ;
    cursorTable->setAlternatingRowColors(true);
	
	NxControlCenterDelegate *delegate = new NxControlCenterDelegate( this ) ;
	delegate->setObjectType(2) ;
	cursorTable->setItemDelegate( delegate );

	cursorSortCaseSensitivityCheckBox	= new QCheckBox(tr("Case sensitive sorting"));
    cursorFilterCaseSensitivityCheckBox	= new QCheckBox(tr("Case sensitive filter"));
	
	cursorFilterPatternLineEdit = new QLineEdit(cursorProxy) ;
	    
	cursorFilterSyntaxComboBox = new QComboBox(cursorProxy) ;
    cursorFilterSyntaxComboBox->addItem(tr("Regular expression"), QRegExp::RegExp);
    cursorFilterSyntaxComboBox->addItem(tr("Wildcard"), QRegExp::Wildcard);
    cursorFilterSyntaxComboBox->addItem(tr("Fixed string"), QRegExp::FixedString);
    QLabel *filterSyntaxLabel = new QLabel(tr("Filter by"));
	cursorFilterSyntaxComboBox->setCurrentIndex(2);

	cursorFilterColumnComboBox = new QComboBox(cursorProxy) ;
    cursorFilterColumnComboBox->addItem(tr("ID"));
    cursorFilterColumnComboBox->addItem(tr("Name"));
    cursorFilterColumnComboBox->addItem(tr("Custom ID"));
	cursorFilterColumnComboBox->addItem(tr("Group ID"));
	cursorFilterColumnComboBox->addItem(tr("Layer"));
	cursorFilterColumnComboBox->addItem(tr("Parent ID"));
	cursorFilterColumnComboBox->addItem(tr("Speed"));
	cursorFilterColumnComboBox->addItem(tr("Width"));
	cursorFilterColumnComboBox->addItem(tr("Playback"));
	cursorFilterColumnComboBox->addItem(tr("Mode"));
    QLabel *filterColumnLabel = new QLabel(tr("in"));
	cursorFilterColumnComboBox->setCurrentIndex(1);
	
	QHBoxLayout *proxyLayout = new QHBoxLayout(cursorProxy) ;
    proxyLayout->addWidget(filterSyntaxLabel);
    proxyLayout->addWidget(cursorFilterSyntaxComboBox);
	proxyLayout->addWidget(cursorFilterPatternLineEdit);
    proxyLayout->addWidget(filterColumnLabel);
    proxyLayout->addWidget(cursorFilterColumnComboBox);
    proxyLayout->addWidget(cursorFilterCaseSensitivityCheckBox);
    proxyLayout->addWidget(cursorSortCaseSensitivityCheckBox);
    cursorProxy->setLayout(proxyLayout);
	
	QVBoxLayout *cursorLay = new QVBoxLayout(cursorPage) ;
	cursorLay->addWidget(cursorProxy);
    cursorLay->addWidget(cursorTable);
    cursorPage->setLayout(cursorLay);
	
	cursorsFilterColumnChanged() ;
	
	connect(cursorFilterPatternLineEdit,			SIGNAL(textChanged(const QString &)), this,	SLOT(cursorsFilterRegExpChanged()));
    connect(cursorFilterSyntaxComboBox,				SIGNAL(currentIndexChanged(int)), this,		SLOT(cursorsFilterRegExpChanged()));
    connect(cursorFilterColumnComboBox,				SIGNAL(currentIndexChanged(int)), this,		SLOT(cursorsFilterColumnChanged()));
    connect(cursorFilterCaseSensitivityCheckBox,	SIGNAL(toggled(bool)), this,				SLOT(cursorsFilterRegExpChanged()));
    connect(cursorSortCaseSensitivityCheckBox,		SIGNAL(toggled(bool)), this,				SLOT(cursorsSortChanged()));

	cursorList->setHeaderData(0, Qt::Horizontal, QObject::tr("ID"));
	cursorList->setHeaderData(1, Qt::Horizontal, QObject::tr("Name"));
	cursorList->setHeaderData(2, Qt::Horizontal, QObject::tr("Custom ID"));
	cursorList->setHeaderData(3, Qt::Horizontal, QObject::tr("Group ID"));
	cursorList->setHeaderData(4, Qt::Horizontal, QObject::tr("Layer"));
	cursorList->setHeaderData(5, Qt::Horizontal, QObject::tr("Parent ID"));
	cursorList->setHeaderData(6, Qt::Horizontal, QObject::tr("Speed"));
	cursorList->setHeaderData(7, Qt::Horizontal, QObject::tr("Width"));
	cursorList->setHeaderData(8, Qt::Horizontal, QObject::tr("Playback"));
	cursorList->setHeaderData(9, Qt::Horizontal, QObject::tr("Mode"));
}
void NxControlCenter::buildTriggersItems()
{
	QGroupBox *triggerProxy			= new QGroupBox(triggerPage) ;
	triggerTable					= new QTableView(triggerPage) ;
	triggerList						= new QStandardItemModel(0, 9, triggerPage);
	triggerProxyModel				= new QSortFilterProxyModel;
	triggerProxyModel->setDynamicSortFilter(true);
	
	triggerProxyModel->setSourceModel(triggerList);
	triggerTable->setModel(triggerProxyModel) ;
	triggerTable->setSortingEnabled(true);
	triggerTable->setEditTriggers( QAbstractItemView::CurrentChanged | QAbstractItemView::DoubleClicked | QAbstractItemView::SelectedClicked ) ;
    triggerTable->setAlternatingRowColors(true);
	
	NxControlCenterDelegate *delegate = new NxControlCenterDelegate( this ) ;
	delegate->setObjectType(3) ;
	triggerTable->setItemDelegate( delegate );

	triggerSortCaseSensitivityCheckBox		= new QCheckBox(tr("Case sensitive sorting"));
    triggerFilterCaseSensitivityCheckBox	= new QCheckBox(tr("Case sensitive filter"));
	
	triggerFilterPatternLineEdit = new QLineEdit(triggerProxy) ;
	    
	triggerFilterSyntaxComboBox = new QComboBox(triggerProxy) ;
    triggerFilterSyntaxComboBox->addItem(tr("Regular expression"), QRegExp::RegExp);
    triggerFilterSyntaxComboBox->addItem(tr("Wildcard"), QRegExp::Wildcard);
    triggerFilterSyntaxComboBox->addItem(tr("Fixed string"), QRegExp::FixedString);
    QLabel *filterSyntaxLabel = new QLabel(tr("Filter by"));
	triggerFilterSyntaxComboBox->setCurrentIndex(2);

	triggerFilterColumnComboBox = new QComboBox(triggerProxy) ;
    triggerFilterColumnComboBox->addItem(tr("ID"));
    triggerFilterColumnComboBox->addItem(tr("Name"));
    triggerFilterColumnComboBox->addItem(tr("Custom ID"));
	triggerFilterColumnComboBox->addItem(tr("Group ID"));
	triggerFilterColumnComboBox->addItem(tr("Layer"));
	triggerFilterColumnComboBox->addItem(tr("Host IP"));
	triggerFilterColumnComboBox->addItem(tr("UDP Port"));
	triggerFilterColumnComboBox->addItem(tr("OSC Format"));
	triggerFilterColumnComboBox->addItem(tr("Value"));
    QLabel *filterColumnLabel = new QLabel(tr("in"));
	triggerFilterColumnComboBox->setCurrentIndex(1);
	
	QHBoxLayout *proxyLayout = new QHBoxLayout(triggerProxy) ;
    proxyLayout->addWidget(filterSyntaxLabel);
    proxyLayout->addWidget(triggerFilterSyntaxComboBox);
	proxyLayout->addWidget(triggerFilterPatternLineEdit);
    proxyLayout->addWidget(filterColumnLabel);
    proxyLayout->addWidget(triggerFilterColumnComboBox);
    proxyLayout->addWidget(triggerFilterCaseSensitivityCheckBox);
    proxyLayout->addWidget(triggerSortCaseSensitivityCheckBox);
    triggerProxy->setLayout(proxyLayout);
	
	QVBoxLayout *triggerLay = new QVBoxLayout(triggerPage) ;
	triggerLay->addWidget(triggerProxy);
    triggerLay->addWidget(triggerTable);
    triggerPage->setLayout(triggerLay);
	
	triggersFilterColumnChanged() ;
	
	connect(triggerFilterPatternLineEdit,			SIGNAL(textChanged(const QString &)), this,	SLOT(triggersFilterRegExpChanged()));
    connect(triggerFilterSyntaxComboBox,			SIGNAL(currentIndexChanged(int)), this,		SLOT(triggersFilterRegExpChanged()));
    connect(triggerFilterColumnComboBox,			SIGNAL(currentIndexChanged(int)), this,		SLOT(triggersFilterColumnChanged()));
    connect(triggerFilterCaseSensitivityCheckBox,	SIGNAL(toggled(bool)), this,				SLOT(triggersFilterRegExpChanged()));
    connect(triggerSortCaseSensitivityCheckBox,		SIGNAL(toggled(bool)), this,				SLOT(triggersSortChanged()));

	triggerList->setHeaderData(0, Qt::Horizontal, QObject::tr("ID"));
	triggerList->setHeaderData(1, Qt::Horizontal, QObject::tr("Name"));
	triggerList->setHeaderData(2, Qt::Horizontal, QObject::tr("Custom ID"));
	triggerList->setHeaderData(3, Qt::Horizontal, QObject::tr("Group ID"));
	triggerList->setHeaderData(4, Qt::Horizontal, QObject::tr("Layer"));
	triggerList->setHeaderData(5, Qt::Horizontal, QObject::tr("Host IP"));
	triggerList->setHeaderData(6, Qt::Horizontal, QObject::tr("UDP Port"));
	triggerList->setHeaderData(7, Qt::Horizontal, QObject::tr("OSC Format"));
	triggerList->setHeaderData(8, Qt::Horizontal, QObject::tr("Value"));
}

////////////////////////////////////////////
// Return the row number of an ID
////////////////////////////////////////////
int NxControlCenter::curvesRowNumber( int inID )
{
	int rNb = -1 ;
	QList<QStandardItem*> liste ;
	liste = curveList->findItems( QString().setNum( inID ), Qt::MatchExactly, 0 ) ;
	rNb = liste.at(0)->row() ;
	return rNb ;
}
int NxControlCenter::trajectsRowNumber( int inID )
{
	int rNb = -1 ;
	QList<QStandardItem*> liste ;
	liste = trajectList->findItems( QString().setNum( inID ), Qt::MatchExactly, 0 ) ;
	rNb = liste.at(0)->row() ;
	return rNb ;
}
int NxControlCenter::cursorsRowNumber( int inID )
{
	int rNb = -1 ;
	QList<QStandardItem*> liste ;
	liste = cursorList->findItems( QString().setNum( inID ), Qt::MatchExactly, 0 ) ;
	rNb = liste.at(0)->row() ;
	return rNb ;
}
int NxControlCenter::triggersRowNumber( int inID )
{
	int rNb = -1 ;
	QList<QStandardItem*> liste ;
	liste = triggerList->findItems( QString().setNum( inID ), Qt::MatchExactly, 0 ) ;
	rNb = liste.at(0)->row() ;
	return rNb ;
}

////////////////////////////////////////////////////////////////////////
// ADDING LINES (rows)
////////////////////////////////////////////////////////////////////////
void NxControlCenter::addCurveLine(QStandardItemModel *model, NxCurve *curve, bool resize)
 {
	int rowCount = model->rowCount() ;
	model->insertRow(rowCount);
	model->setData(model->index(rowCount, 0), curve->getID() );
	model->item(rowCount,0)->setEditable( false ) ;
	model->setData(model->index(rowCount, 1), curve->getName() );
	model->setData(model->index(rowCount, 2), curve->getCustomID() );
	model->setData(model->index(rowCount, 3), curve->getGroupID() );
	model->setData(model->index(rowCount, 4), curve->getLayer() );
	model->setData(model->index(rowCount, 5), (float)curve->getXRange() );
	model->item(rowCount,5)->setEditable( false ) ;
	QString str ;
	if ( curve->getDrawType() == LineStrip )
		str = QString("Lines") ;
	else
		str = QString("Splines") ;
	model->setData(model->index(rowCount, 6), str );
	model->setData(model->index(rowCount, 7), (float)curve->getRealMin() );
	model->setData(model->index(rowCount, 8), (float)curve->getRealMax() );
	model->setData(model->index(rowCount, 9), curve->getHost() );
	model->setData(model->index(rowCount, 10), curve->getPort() );
	model->setData(model->index(rowCount, 11), curve->getOscFormat() );
	
	if(resize) {
		curveTable->resizeRowsToContents() ;
		curveTable->resizeColumnsToContents() ;
	}
}
void NxControlCenter::addTrajectLine(QStandardItemModel *model, NxTraject *traject, bool resize)
{
	int rowCount = model->rowCount() ;
	model->insertRow(rowCount);
	model->setData(model->index(rowCount, 0), traject->getID() );
	model->item(rowCount,0)->setEditable( false ) ;
	model->setData(model->index(rowCount, 1), traject->getName() );
	model->setData(model->index(rowCount, 2), traject->getCustomID() );
	model->setData(model->index(rowCount, 3), traject->getGroupID() );
	model->setData(model->index(rowCount, 4), traject->getLayer() );
	QString str;
	if ( traject->getType() == 1 )
		str = QString("Circle") ;
	else
		str = QString("Line") ;
	model->setData(model->index(rowCount, 5), str );
	model->item(rowCount,5)->setEditable( false ) ;
	model->setData(model->index(rowCount, 6), traject->getLength() );
	model->item(rowCount,6)->setEditable( false ) ;
	
	if(resize) {
		trajectTable->resizeRowsToContents() ;
		trajectTable->resizeColumnsToContents() ;
	}
}
void NxControlCenter::addCursorLine(QStandardItemModel *model, NxCursor *cursor, bool resize)
{	
	int rowCount = model->rowCount() ;
	model->insertRow(rowCount);
	model->setData(model->index(rowCount, 0), cursor->getID() );
	model->item(rowCount,0)->setEditable( false ) ;
	model->setData(model->index(rowCount, 1), cursor->getName() );
	model->setData(model->index(rowCount, 2), cursor->getCustomID() );
	model->setData(model->index(rowCount, 3), cursor->getGroupID() );
	model->setData(model->index(rowCount, 4), cursor->getLayer() );
	model->setData(model->index(rowCount, 5), cursor->getParentID() );
	model->item(rowCount,5)->setEditable( false ) ;
	model->setData(model->index(rowCount, 6), cursor->getSpeed() );
	model->setData(model->index(rowCount, 7), cursor->getWidth() );
	QString strPlayb, strMode ;
	if( cursor->getForward() ) strPlayb = QString("Forward") ;
	else strPlayb = QString("Backward") ;
	if( cursor->getMode() == 0 ) strMode = QString("Single") ;
	else if( cursor->getMode() == 1 ) strMode = QString("Repeat") ;
	else strMode = QString("Fwd / Bckwd") ;
	model->setData(model->index(rowCount, 8), strPlayb );
	model->setData(model->index(rowCount, 9), strMode );
	
	if(resize) {
		cursorTable->resizeRowsToContents() ;
		cursorTable->resizeColumnsToContents() ;
	}
}
void NxControlCenter::addTriggerLine(QStandardItemModel *model, NxTrigger *trigger, bool resize)
{
	int rowCount = model->rowCount() ;
	model->insertRow(rowCount);
	model->setData(model->index(rowCount, 0), trigger->getID() );
	model->item(rowCount,0)->setEditable( false ) ;
	model->setData(model->index(rowCount, 1), trigger->getName() );
	model->setData(model->index(rowCount, 2), trigger->getCustomID() );
	model->setData(model->index(rowCount, 3), trigger->getGroupID() );
	model->setData(model->index(rowCount, 4), trigger->getLayer() );
	model->setData(model->index(rowCount, 5), trigger->getHost() );
	model->setData(model->index(rowCount, 6), trigger->getPort() );
	model->setData(model->index(rowCount, 7), trigger->getOscFormat() );
	model->setData(model->index(rowCount, 8), trigger->getValue() );
	
	if(resize) {
		triggerTable->resizeRowsToContents() ;
		triggerTable->resizeColumnsToContents() ;
	}
}

/////////////////////////////////////////////////////////////
// First Time filling method (OPEN EXISTING SCORE)
/////////////////////////////////////////////////////////////
void NxControlCenter::addItems()
{		
	//////////////////////////////// CURVES
	printf("- Control Center : Adding Curves\n") ;
  	QList<int> indexesCurv = score->curvesIndexes();
	for(int i=0;i<indexesCurv.count(); i++)
	{
		addCurveLine(curveList, score->getCurve( indexesCurv.at(i) ), false) ;
	}
	//////////////////////////////// TRAJECTS
	printf("- Control Center : Adding Trajects\n") ;
  	QList<int> indexesTra = score->trajectsIndexes();
	for(int i=0;i<indexesTra.count(); i++)
	{
		addTrajectLine(trajectList, score->getTraject( indexesTra.at(i) ), false) ;
	}
	//////////////////////////////// CURSORS
	printf("- Control Center : Adding Cursors\n") ;
  	QList<int> indexesCurs = score->cursorsIndexes();
	for(int i=0;i<indexesCurs.count(); i++)
	{
		addCursorLine(cursorList, score->getCursor( indexesCurs.at(i) ), false) ;
	}
	//////////////////////////////// TRIGGERS
	printf("- Control Center : Adding Triggers\n") ;
  	QList<int> indexesTri = score->triggersIndexes();
	for(int i=0;i<indexesTri.count(); i++)
	{
		addTriggerLine(triggerList, score->getTrigger( indexesTri.at(i) ), false) ;
	}
}

/////////////////////////////////////////////////////
// Dynamic addition/delete to tables
/////////////////////////////////////////////////////
void NxControlCenter::updateItems( int choice, bool add, int inID, bool resize ) // add or remove row
{
	int rowNb ; 
	score = parentGLBox->getPartition() ; // local score update
	updateIndexes() ; // update local indexes (nbTriggers, etc...)
	
	switch (choice) 
	{	
		////////////////////////////////////// CURVES
    	case Curve :
      		if (add) // adding a row
			{
				NxCurve *tCurve = score->getCurve(inID) ;
				addCurveLine( curveList, tCurve, resize ) ;
				if(resize) curveTable->resizeRowsToContents() ;
			}
      		else // removing a row
			{
				rowNb = curvesRowNumber( inID ) ;
				curveList->removeRow( rowNb ) ;
			}
			if(resize) for(int u=0;u<curveList->columnCount();u++) curveTable->resizeColumnToContents(u) ;
		break;
		////////////////////////////////////// TRAJECTS
    	case Trajectory :
      		if (add) // adding a row
			{
				NxTraject *tTraject = score->getTraject(inID) ;
				addTrajectLine( trajectList, tTraject, resize ) ;
				if(resize) trajectTable->resizeRowsToContents() ;
			}
      		else // removing a row
			{
				rowNb = trajectsRowNumber( inID ) ;
				trajectList->removeRow( rowNb ) ;
			}
			if(resize) for(int u=0;u<trajectList->columnCount();u++) trajectTable->resizeColumnToContents(u) ;
		break;
		////////////////////////////////////// CURSORS
    	case 2 :
      		if (add) // adding a row
			{
				NxCursor *tCursor = score->getCursor(inID) ;
				addCursorLine( cursorList, tCursor, resize ) ;
				if(resize) cursorTable->resizeRowsToContents() ;
			}
      		else // removing a row
			{
				rowNb = cursorsRowNumber( inID ) ;
				cursorList->removeRow( rowNb ) ;
			}
			if(resize) for(int u=0;u<cursorList->columnCount();u++) cursorTable->resizeColumnToContents(u) ;
		break;
		////////////////////////////////////// TRIGGERS
    	case Trigger :
      		if (add) // adding a row
			{
				NxTrigger *tTrigger = score->getTrigger(inID) ;
				addTriggerLine( triggerList, tTrigger, resize ) ;
				if(resize) triggerTable->resizeRowsToContents() ;
			}
      		else // removing a row
			{
				rowNb = triggersRowNumber( inID ) ;
				triggerList->removeRow( rowNb ) ;
			}
			if(resize) for(int u=0;u<triggerList->columnCount();u++) triggerTable->resizeColumnToContents(u) ;
		break;
	}
}

/////////////////////////////////////////////////////////////////////////////////
// Parameters updates (if changed from another place)
/////////////////////////////////////////////////////////////////////////////////
void NxControlCenter::updateParameters( int choice, int inID, bool resize )
{	
	QList<int> curvInd	= score->curvesIndexes();
	QList<int> trajInd	= score->trajectsIndexes();
	QList<int> cursInd	= score->cursorsIndexes();
	QList<int> trigInd	= score->triggersIndexes();
	NxCurve		*curv ;
	NxTraject	*traj ;
	NxCursor	*curs ;
	NxTrigger	*trig ;
	int idx = 0	; // Initialisation
	
	switch (choice) {
		case Curve : { ///////////////////////// CURVE
			idx = curvesRowNumber(inID) ;
			curv = score->getCurve(inID) ;
			curveList->item(idx,1)->setText( curv->getName() ) ;
			curveList->item(idx,2)->setText( QString().setNum( curv->getCustomID() ) ) ;
			curveList->item(idx,3)->setText( QString().setNum( curv->getGroupID() ) ) ;
			curveList->item(idx,4)->setText( QString().setNum( curv->getLayer() ) ) ;
 			curveList->item(idx,5)->setText( QString().setNum( (float)curv->getXRange() ) ) ;
			QString strDrawT ;
			if( curv->getDrawType()==8 )	strDrawT = QString("Lines") ;
			else							strDrawT = QString("Splines") ;
			curveList->item(idx,6)->setText( strDrawT ) ;
			curveList->item(idx,7)->setText( QString().setNum( (float)curv->getRealMin() ) ) ;
			curveList->item(idx,8)->setText( QString().setNum( (float)curv->getRealMax() ) ) ;
			curveList->item(idx,9)->setText( curv->getHost() ) ;
			curveList->item(idx,10)->setText( QString().setNum( curv->getPort() ) ) ;
			curveList->item(idx,11)->setText( curv->getOscFormat() ) ;
			
			if(resize) for(int u=0;u<curveList->columnCount();u++) curveTable->resizeColumnToContents(u) ; }
		break;
		case Trajectory : { ///////////////////////// TRAJECT
			idx = trajectsRowNumber(inID) ;
			traj = score->getTraject(inID) ;
			trajectList->item(idx,1)->setText( traj->getName() ) ;
			trajectList->item(idx,2)->setText( QString().setNum( traj->getCustomID() ) ) ;
			trajectList->item(idx,3)->setText( QString().setNum( traj->getGroupID() ) ) ;
			trajectList->item(idx,4)->setText( QString().setNum( traj->getLayer() ) ) ;
			trajectList->item(idx,6)->setText( QString().setNum( traj->getLength() ) ) ;
			
			if(resize) for(int u=0;u<trajectList->columnCount();u++) trajectTable->resizeColumnToContents(u) ; }
		break;
		case 2 : { ///////////////////////// CURSOR
			idx = cursorsRowNumber(inID) ;
			curs = score->getCursor(inID) ;
			cursorList->item(idx,1)->setText( curs->getName() ) ;
			cursorList->item(idx,2)->setText( QString().setNum( curs->getCustomID() ) ) ;
			cursorList->item(idx,3)->setText( QString().setNum( curs->getGroupID() ) ) ;
			cursorList->item(idx,4)->setText( QString().setNum( curs->getLayer() ) ) ;
			cursorList->item(idx,6)->setText( QString().setNum( curs->getSpeed() ) ) ;
			cursorList->item(idx,7)->setText( QString().setNum( curs->getWidth() ) ) ;
			QString strPlayb, strMode ;
			if( curs->getForward() )	strPlayb = QString("Forward") ;
			else						strPlayb = QString("Backward") ;
			if( curs->getMode() == 0 )		strMode = QString("Single") ;
			else if( curs->getMode() == 1 )	strMode = QString("Repeat") ;
			else							strMode = QString("Fwd / Bckwd") ;
			cursorList->item(idx,8)->setText( strPlayb ) ;
			cursorList->item(idx,9)->setText( strMode ) ;
			
			if(resize) for(int u=0;u<cursorList->columnCount();u++) cursorTable->resizeColumnToContents(u) ; }
		break;
		case Trigger : { ///////////////////////// TRIGGER
			idx = triggersRowNumber(inID) ;
			trig = score->getTrigger(inID) ;
			triggerList->item(idx,1)->setText( trig->getName() ) ;
			triggerList->item(idx,2)->setText( QString().setNum( trig->getCustomID() ) ) ;
			triggerList->item(idx,3)->setText( QString().setNum( trig->getGroupID() ) ) ;
			triggerList->item(idx,4)->setText( QString().setNum( trig->getLayer() ) ) ;
			triggerList->item(idx,5)->setText( trig->getHost() ) ;
			triggerList->item(idx,6)->setText( QString().setNum( trig->getPort() ) ) ;
			triggerList->item(idx,7)->setText( trig->getOscFormat() ) ;
			triggerList->item(idx,8)->setText( QString().setNum( trig->getValue() ) ) ;
			
			if(resize) for(int u=0;u<triggerList->columnCount();u++) triggerTable->resizeColumnToContents(u) ; }
		break;
	}
}

///////////////////////////////////////////////////////
// Dynamic Table Updates (cell modified locally)
///////////////////////////////////////////////////////
void NxControlCenter::updateCurve( QStandardItem* item )
{
	int row			= item->row() ;
	int valueType	= item->column() ;
	int id			= (curveList->item(row, 0)->text()).toInt() ;
	NxCurve *curve 	= score->getCurve( id ) ;
	QString str = item->text() ;
	
	switch (valueType)
	{
		case 1 : 
			curve->setName( str );
			curveTable->resizeColumnToContents(1) ;
		break;			
		case 2 : // Custom ID
			curve->setCustomID( str.toInt() );
			curveTable->resizeColumnToContents(2) ;
		break;
		case 3 : // GroupID
			int oldGID = curve->getGroupID() ;
			curve->setGroupID( str.toInt() );
			curveTable->resizeColumnToContents(3) ;
			if(oldGID!=str.toInt()) parentGLBox->updateGroupFilterList() ;
		break;
		case 4 : // Layer
			int oldLay = curve->getLayer() ;
			curve->setLayer( str.toInt() );
			curveTable->resizeColumnToContents(4) ;
			if(oldLay!=str.toInt()) parentGLBox->updateLayerFilterList() ;
		break;
		case 5:
			curveTable->resizeColumnToContents(5) ;
		break;
		case 7 : // Real min
			curve->setRealMin( str.toFloat() );
			curveList->item(row, valueType)->setText( QString("%1").arg(curve->getRealMin()) ) ;
			if(curve->getDrawType() == SplineStrip) curve->splineGenerate() ;
			curveTable->resizeColumnToContents(7) ;
		break; 
		case 8 : // Real max
			curve->setRealMax( str.toFloat() );
			curveList->item(row, valueType)->setText( QString("%1").arg(curve->getRealMax()) ) ;
			if(curve->getDrawType() == SplineStrip) curve->splineGenerate() ;
			curveTable->resizeColumnToContents(8) ;
		break;
		case 9 : // Host
			curve->setHost( str );
			curveTable->resizeColumnToContents(9) ;
		break;
		case 10 : // Port
			curve->setPort( str.toInt() );
			curveTable->resizeColumnToContents(10) ;
		break;
		case 11 : // OSC Format
			curve->setOscFormat( str );
			curveTable->resizeColumnToContents(11) ;
		break;	
	}
}
void NxControlCenter::updateTraject( QStandardItem* item )
{
	int row			= item->row() ;
	int valueType	= item->column() ;
	int id			= (trajectList->item(row, 0)->text()).toInt() ;
	NxTraject *traject 	= score->getTraject( id ) ;
	QString str = item->text() ;
	
	switch (valueType)
	{
		case 1 : // Name
			traject->setName( str );
			trajectTable->resizeColumnToContents(1) ;
		break;			
		case 2 : // Custom ID
			traject->setCustomID( str.toInt() );
			trajectTable->resizeColumnToContents(2) ;
		break;
		case 3 : // GroupID
			int oldGID = traject->getGroupID() ;
			traject->setGroupID( str.toInt() );
			trajectTable->resizeColumnToContents(3) ;
			if(oldGID!=str.toInt()) parentGLBox->updateGroupFilterList() ;
		break;
		case 4 : // Layer
			int oldLay = traject->getLayer() ;
			traject->setLayer( str.toInt() );
			trajectTable->resizeColumnToContents(4) ;
			if(oldLay!=str.toInt()) parentGLBox->updateLayerFilterList() ;
		break;
	}
}
void NxControlCenter::updateCursor( QStandardItem* item )
{
	int row			= item->row() ;
	int valueType	= item->column() ;
	int id			= (cursorList->item(row, 0)->text()).toInt() ;
	NxCursor *cursor 	= score->getCursor( id ) ;
	QString str = item->text() ;
	
	switch (valueType)
	{
		case 1 : // Name
			cursor->setName( str );
			cursorTable->resizeColumnToContents(1) ;
		break;			
		case 2 : // Custom ID
			cursor->setCustomID( str.toInt() );
			cursorTable->resizeColumnToContents(2) ;
		break;
		case 3 : // GroupID
			int oldGID = cursor->getGroupID() ;
			cursor->setGroupID( str.toInt() );
			cursorTable->resizeColumnToContents(3) ;
			if(oldGID!=str.toInt()) parentGLBox->updateGroupFilterList() ;
		break;
		case 4 : // Layer
			int oldLay = cursor->getLayer() ;
			cursor->setLayer( str.toInt() );
			cursorTable->resizeColumnToContents(4) ;
			if(oldLay!=str.toInt()) parentGLBox->updateLayerFilterList() ;
		break;
		case 5: // Parent ID
			cursorTable->resizeColumnToContents(5) ;
		break;
		case 6 : // Speed
			cursor->setSpeed( str.toFloat() );
			cursorTable->resizeColumnToContents(6) ;
		break; 
		case 7 : // Width
			cursor->setWidth( str.toFloat() );
			cursorTable->resizeColumnToContents(7) ;
		break;
	}
}
void NxControlCenter::updateTrigger( QStandardItem* item )
{
	int row			= item->row() ;
	int valueType	= item->column() ;
	int id			= (triggerList->item(row, 0)->text()).toInt() ;
	NxTrigger *trigger 	= score->getTrigger( id ) ;
	QString str = item->text() ;
	
	switch (valueType)
	{
		case 1 : // Name
			trigger->setName( str );
			triggerTable->resizeColumnToContents(1) ;
		break;			
		case 2 : // Custom ID
			trigger->setCustomID( str.toInt() );
			triggerTable->resizeColumnToContents(2) ;
		break;
		case 3 : // GroupID
			int oldGID = trigger->getGroupID() ;
			trigger->setGroupID( str.toInt() );
			triggerTable->resizeColumnToContents(3) ;
			if(oldGID!=str.toInt()) parentGLBox->updateGroupFilterList() ;
		break;
		case 4 : // Layer
			int oldLay = trigger->getLayer() ;
			trigger->setLayer( str.toInt() );
			triggerTable->resizeColumnToContents(4) ;
			if(oldLay!=str.toInt()) parentGLBox->updateLayerFilterList() ;
		break;
		case 5 : // Host
			trigger->setHost( str );
			triggerTable->resizeColumnToContents(5) ;
		break;
		case 6 : // Port
			trigger->setPort( str.toInt() );
			triggerTable->resizeColumnToContents(6) ;
		break;
		case 7 : // OSC Format
			trigger->setOscFormat( str );
			triggerTable->resizeColumnToContents(7) ;
		break;
		case 8 : // Value
			trigger->setValue( str.toInt() );
			triggerTable->resizeColumnToContents(8) ;
		break;
	}
}
//////////////////////////////////////////////////////////////////////////
// ComboBoxe updates
//////////////////////////////////////////////////////////////////////////
void NxControlCenter::updateCurveDrawStyle( int inRow, int flag )
{
	int id	= (curveList->item(inRow,0)->text()).toInt() ;
	NxCurve *curve = score->getCurve( id ) ;
	curve->setDrawType( 8 - flag ) ;
}
void NxControlCenter::updateCursorModes( int inCol, int inRow, int flag )
{
	int id	= (cursorList->item(inRow,0)->text()).toInt() ;
	NxCursor *cursor = score->getCursor( id ) ;
	if(inCol==8)	cursor->setForward( (flag==0) ) ;
	else			cursor->setMode( flag ) ;
}
//////////////////////////////////////////////////////////////////////////
// Sorting / Filtering Tools
//////////////////////////////////////////////////////////////////////////
/////////////// CURVES
void NxControlCenter::curvesFilterRegExpChanged()
{
    QRegExp::PatternSyntax syntax =
            QRegExp::PatternSyntax(curveFilterSyntaxComboBox->itemData(
                    curveFilterSyntaxComboBox->currentIndex()).toInt());
    Qt::CaseSensitivity caseSensitivity =
            curveFilterCaseSensitivityCheckBox->isChecked() ? Qt::CaseSensitive
                                                       : Qt::CaseInsensitive;

    QRegExp regExp(curveFilterPatternLineEdit->text(), caseSensitivity, syntax);
    curveProxyModel->setFilterRegExp(regExp);
	
	curveTable->resizeRowsToContents() ;
	curveTable->resizeColumnsToContents() ;
}
void NxControlCenter::curvesFilterColumnChanged()
{
    curveProxyModel->setFilterKeyColumn(curveFilterColumnComboBox->currentIndex());
	
	curveTable->resizeRowsToContents() ;
	curveTable->resizeColumnsToContents() ;
}
void NxControlCenter::curvesSortChanged()
{
    curveProxyModel->setSortCaseSensitivity(
            curveSortCaseSensitivityCheckBox->isChecked() ? Qt::CaseSensitive
                                                     : Qt::CaseInsensitive);
													 
	curveTable->resizeRowsToContents() ;
	curveTable->resizeColumnsToContents() ;
}
/////////////// TRAJECTS
void NxControlCenter::trajectsFilterRegExpChanged()
{
    QRegExp::PatternSyntax syntax =
            QRegExp::PatternSyntax(trajectFilterSyntaxComboBox->itemData(
                    trajectFilterSyntaxComboBox->currentIndex()).toInt());
    Qt::CaseSensitivity caseSensitivity =
            trajectFilterCaseSensitivityCheckBox->isChecked() ? Qt::CaseSensitive
                                                       : Qt::CaseInsensitive;

    QRegExp regExp(trajectFilterPatternLineEdit->text(), caseSensitivity, syntax);
    trajectProxyModel->setFilterRegExp(regExp);
	
	trajectTable->resizeRowsToContents() ;
	trajectTable->resizeColumnsToContents() ;
}
void NxControlCenter::trajectsFilterColumnChanged()
{
    trajectProxyModel->setFilterKeyColumn(trajectFilterColumnComboBox->currentIndex());
	
	trajectTable->resizeRowsToContents() ;
	trajectTable->resizeColumnsToContents() ;
}
void NxControlCenter::trajectsSortChanged()
{
    trajectProxyModel->setSortCaseSensitivity(
            trajectSortCaseSensitivityCheckBox->isChecked() ? Qt::CaseSensitive
                                                     : Qt::CaseInsensitive);
													 
	trajectTable->resizeRowsToContents() ;
	trajectTable->resizeColumnsToContents() ;
}
/////////////// CURSORS
void NxControlCenter::cursorsFilterRegExpChanged()
{
    QRegExp::PatternSyntax syntax =
            QRegExp::PatternSyntax(cursorFilterSyntaxComboBox->itemData(
                    cursorFilterSyntaxComboBox->currentIndex()).toInt());
    Qt::CaseSensitivity caseSensitivity =
            cursorFilterCaseSensitivityCheckBox->isChecked() ? Qt::CaseSensitive
                                                       : Qt::CaseInsensitive;

    QRegExp regExp(cursorFilterPatternLineEdit->text(), caseSensitivity, syntax);
    cursorProxyModel->setFilterRegExp(regExp);
	
	cursorTable->resizeRowsToContents() ;
	cursorTable->resizeColumnsToContents() ;
}
void NxControlCenter::cursorsFilterColumnChanged()
{
    cursorProxyModel->setFilterKeyColumn(cursorFilterColumnComboBox->currentIndex());
	
	cursorTable->resizeRowsToContents() ;
	cursorTable->resizeColumnsToContents() ;
}
void NxControlCenter::cursorsSortChanged()
{
    cursorProxyModel->setSortCaseSensitivity(
            cursorSortCaseSensitivityCheckBox->isChecked() ? Qt::CaseSensitive
                                                     : Qt::CaseInsensitive);
													 
	cursorTable->resizeRowsToContents() ;
	cursorTable->resizeColumnsToContents() ;
}
/////////////// TRIGGERS
void NxControlCenter::triggersFilterRegExpChanged()
{
    QRegExp::PatternSyntax syntax =
            QRegExp::PatternSyntax(triggerFilterSyntaxComboBox->itemData(
                    triggerFilterSyntaxComboBox->currentIndex()).toInt());
    Qt::CaseSensitivity caseSensitivity =
            triggerFilterCaseSensitivityCheckBox->isChecked() ? Qt::CaseSensitive
                                                       : Qt::CaseInsensitive;

    QRegExp regExp(triggerFilterPatternLineEdit->text(), caseSensitivity, syntax);
    triggerProxyModel->setFilterRegExp(regExp);
	
	triggerTable->resizeRowsToContents() ;
	triggerTable->resizeColumnsToContents() ;
}
void NxControlCenter::triggersFilterColumnChanged()
{
    triggerProxyModel->setFilterKeyColumn(triggerFilterColumnComboBox->currentIndex());
	
	triggerTable->resizeRowsToContents() ;
	triggerTable->resizeColumnsToContents() ;
}
void NxControlCenter::triggersSortChanged()
{
    triggerProxyModel->setSortCaseSensitivity(
            triggerSortCaseSensitivityCheckBox->isChecked() ? Qt::CaseSensitive
                                                     : Qt::CaseInsensitive);
													 
	triggerTable->resizeRowsToContents() ;
	triggerTable->resizeColumnsToContents() ;
}


////////////////////////////////////
// Updating Indexes
////////////////////////////////////
void NxControlCenter::updateIndexes()
{
	nbCurves 	= score->curvesIndexes().count();
  	nbCursors 	= score->cursorsIndexes().count();
  	nbTrajects 	= score->trajectsIndexes().count();
  	nbTriggers 	= score->triggersIndexes().count();
	
	controlBox->setTabText ( 0, QString("%1 Curves").arg(nbCurves) ) ;
	controlBox->setTabText ( 1, QString("%1 Trajects").arg(nbTrajects) ) ;
	controlBox->setTabText ( 2, QString("%1 Cursors").arg(nbCursors) ) ;
	controlBox->setTabText ( 3, QString("%1 Triggers").arg(nbTriggers) ) ;
}

////////////////////////////
// Closing Event
////////////////////////////
void NxControlCenter::closeEvent( QCloseEvent* )
{
	releaseKeyboard() ;
}

///////////////////////////////////////
// Key Events (deletting rows)
///////////////////////////////////////
void NxControlCenter::keyReleaseEvent( QKeyEvent* e )
{
	NxObject* obj ;
	int rID ;
	switch ( e->key() )
	{
		case Qt::Key_Backspace :
			for (int i = nbCurves-1 ; i >=0; i-- )
				if ( curveList->item(i,0)->checkState() == Qt::Checked )
				{
					rID = curveList->item(i,0)->text().toInt() ;
					obj = parentGLBox->getPartition()->getCurve( rID ) ;
					curveList->removeRow( i );
					parentGLBox->destroyObject( obj );
				}
			parentGLBox->updateIndexes();
			updateIndexes();
		break;
	}
}

