///////////////
// IanniX
///////////////

////////////////////////////
// NxControlCenter
////////////////////////////

#ifndef NxControlCenter_H
#define NxControlCenter_H

#include <QtGui>

// IanniX classes
#include "../graphics/opengl/NxGLScore.h"
#include "../base/atoms/NxObject.h"
#include <math.h>

class NxGLScore ;
class NxScore ;

class NxControlCenter : public QDialog
{
  Q_OBJECT

 public:
	NxControlCenter( NxGLScore* parent = 0 );
  	~NxControlCenter();

	void addCurveLine(QStandardItemModel *model, NxCurve *curve, bool resize) ;
	void addTrajectLine(QStandardItemModel *model, NxTraject *traject, bool resize);
	void addCursorLine(QStandardItemModel *model, NxCursor *cursor, bool resize);
	void addTriggerLine(QStandardItemModel *model, NxTrigger *trigger, bool resize);
	
	void resizeTables() ;
	
	void buildCurvesItems() ;
	void buildTrajectsItems() ;
	void buildCursorsItems() ;
	void buildTriggersItems() ;
	
	void addItems();
  	void updateIndexes();
  	void updateItems( int choice, bool add, int inID, bool resize ) ;	// add or remove row
	void updateParameters( int choice, int inID, bool resize ) ;		// update a row
	
	void updateCurveDrawStyle( int inRow, int flag ) ;
	void updateCursorModes( int inCol, int inRow, int flag ) ;
	
	QColor backgColor ; // Backg color for the non modifiable values
		
 protected:

 protected slots:
	virtual void updateCurve( QStandardItem* item );
	virtual void updateTraject( QStandardItem* item );
	virtual void updateCursor( QStandardItem* item );
	virtual void updateTrigger( QStandardItem* item );
	
  	virtual void closeEvent( QCloseEvent* );
	virtual void keyReleaseEvent( QKeyEvent* e );
	
	void curvesFilterRegExpChanged();
	void curvesFilterColumnChanged();
	void curvesSortChanged();
	void trajectsFilterRegExpChanged();
	void trajectsFilterColumnChanged();
	void trajectsSortChanged();
	void cursorsFilterRegExpChanged();
	void cursorsFilterColumnChanged();
	void cursorsSortChanged();
	void triggersFilterRegExpChanged();
	void triggersFilterColumnChanged();
	void triggersSortChanged();
	
 private:
	int curvesRowNumber( int inID ) ;
	int trajectsRowNumber( int inID ) ;
	int cursorsRowNumber( int inID ) ;
	int triggersRowNumber( int inID ) ;
	
  	QTabWidget* controlBox ;
  	QWidget					*curvePage, *trajectPage, *triggerPage, *cursorPage;
  	QStandardItemModel		*curveList, *trajectList, *triggerList, *cursorList;
	QTableView				*curveTable, *trajectTable, *cursorTable, *triggerTable ;
	QSortFilterProxyModel	*curveProxyModel, *trajectProxyModel, *cursorProxyModel, *triggerProxyModel ;

	// Filtering and Sorting GUI
	QCheckBox	*curveSortCaseSensitivityCheckBox, *trajectSortCaseSensitivityCheckBox, *cursorSortCaseSensitivityCheckBox, *triggerSortCaseSensitivityCheckBox ;
	QCheckBox	*curveFilterCaseSensitivityCheckBox, *trajectFilterCaseSensitivityCheckBox, *cursorFilterCaseSensitivityCheckBox, *triggerFilterCaseSensitivityCheckBox ;
	QComboBox	*curveFilterColumnComboBox, *trajectFilterColumnComboBox, *cursorFilterColumnComboBox, *triggerFilterColumnComboBox ;
	QComboBox	*curveFilterSyntaxComboBox, *trajectFilterSyntaxComboBox, *cursorFilterSyntaxComboBox, *triggerFilterSyntaxComboBox ;
	QLineEdit	*curveFilterPatternLineEdit, *trajectFilterPatternLineEdit, *cursorFilterPatternLineEdit, *triggerFilterPatternLineEdit ;
	
  	// Main layout
  	QVBoxLayout* mainLayout;
  	QVBoxLayout* boxLayout;
  
  	// Partition pointer
  	NxScore* score;
  	NxGLScore* parentGLBox;
	
  	// Local indexes
  	int nbCurves, nbCursors, nbTrajects, nbTriggers;
};

#endif

