///////////////
// IanniX
///////////////

////////////////////////////
// NxObjectCreator.cpp
////////////////////////////

#include "NxObjectCreator.h"

NxObjectCreator::NxObjectCreator( NxGLScore* parent )
  : QDialog( parent )
{
	QFont f( "Lucida Grande", 12 );
	setFont( f );
	
	parentGLBox = parent;
	scoreEnv = parentGLBox->getPartition()->getEnvironment();
	
	setWindowFlags(Qt::Dialog);
	setWindowTitle( tr( "IanniX Object Creator" ) );
	
	getScoreMaxCpt() ; // get the last indexes
	
  	// Variables
  	infX = infY = 0.0;
	trajectSelected = false ;
	isCircle = false ;
  	
	pickedColor1.ID = 0 ;
	pickedColor2.ID = 0 ;
	color1Picked = false ;
	color2Picked = false ;
	
	// Tabbing Widgets
	tabwidget = new QTabWidget(this);

	// Setting Child Widgets
	mainBox 	= new QGroupBox(tabwidget);
	mainBox->setFlat(true) ;
	curveBox 	= new QGroupBox ;
	curveBox->setFlat(true) ;
	trajectBox 	= new QGroupBox ;
	trajectBox->setFlat(true) ;
	triggerBox 	= new QGroupBox ;
	triggerBox->setFlat(true) ;
	cursorBox 	= new QGroupBox ;
	cursorBox->setFlat(true) ;
	QGroupBox* buttBox = new QGroupBox(tabwidget);
	
	// Initializing Child Widgets
	initMainBox();
	initCurveBox();
	initTrajectBox();
	initTriggerBox();	
	initCursorBox();
	
	tabwidget->addTab( trajectBox, "Trajectory" );
	tabwidget->addTab( cursorBox, "Cursor" );
	tabwidget->addTab( curveBox, "Curve" );
	tabwidget->addTab( triggerBox, "Trigger" );
	
	mainEditorLayout = new QHBoxLayout;
	QVBoxLayout *mainLayout = new QVBoxLayout;
	QHBoxLayout *hl			= new QHBoxLayout;
		
	// Dialog Buttons 
	cancel	= new QPushButton(this) ;
	apply 	= new QPushButton(this) ;
	cancel->setText("Cancel");
	apply->setText("->");
	apply->setShortcut(Qt::Key_Return) ;
	
	hl->addWidget(cancel) ;
	hl->addWidget(apply) ;
	buttBox->setLayout(hl) ;

	mainEditorLayout->addWidget(tabwidget) ;
	mainEditorLayout->addWidget(mainBox) ;

	mainLayout->addLayout(mainEditorLayout) ;
	mainLayout->addWidget(buttBox) ;
	setLayout(mainLayout) ;
	
	mainBox->setEnabled( false );
	
	// Sizing
	//resize( QSize( 350, 400 ).expandedTo(minimumSizeHint()) );
	setMinimumSize( 670, 580 ) ;
	setMaximumSize( 670, 580 ) ;
	move( 250, 80 ) ;
	setWindowModality( Qt::ApplicationModal ) ;
			
  	// Signals connections
  	connect( chooseColor1, 	SIGNAL( clicked() ), this, SLOT( pickColor1() ) );
	connect( chooseColor2, 	SIGNAL( clicked() ), this, SLOT( pickColor2() ) );
	connect( apply,			SIGNAL( clicked() ), this, SLOT( setSecondInformations() ) );
	connect( cancel,		SIGNAL( clicked() ), this, SLOT( close() ) );
}
void NxObjectCreator::setCircleCursorLabels()
{
	offsetBox1->setTitle("Initial Angle") ;
	offsetBox2->setTitle("Final Angle") ;
	off1L->setText("Angle (degrees)");
	off1pL->setText("Angle (%)");
	off2L->setText("Angle (degrees)");
	off2pL->setText("Angle (%)");
}
void NxObjectCreator::setLineCursorLabels()
{
	offsetBox1->setTitle("Initial Offset") ;
	offsetBox2->setTitle("Initial Offset") ;
	off1L->setText("Offset (s)");
	off1pL->setText("Offset (%)");
	off2L->setText("Offset (s)");
	off2pL->setText("Offset (%)");
}
void NxObjectCreator::raiseEditor( float infX, float infY, NxObject* inSelected )
{	
	scoreEnv = parentGLBox->getPartition()->getEnvironment();
	getScoreMaxCpt() ;
	
	////// Put the left part active
	disconnect( apply,	SIGNAL( clicked() ), this, SLOT( setObject() ) );
	connect( apply,		SIGNAL( clicked() ), this, SLOT( setSecondInformations() ) );
	apply->setText("->");
	mainBox->setEnabled(false) ;
	tabwidget->setEnabled(true) ;
	formX->setEnabled(true) ;
	formY->setEnabled(true) ;
	formZ->setEnabled(true) ;
	
	/////// Put the default values (Mapping / trigger value / host / port ...)
	QString str;
	realMinF->setText( str.setNum( scoreEnv.mappingDefaultMin ) ) ; 
	realMaxF->setText( str.setNum( scoreEnv.mappingDefaultMax ) ) ;
	triggersOSCMF->setText( scoreEnv.OSCTriggersDefaultMFormat ) ; 
	IPField->setText( scoreEnv.UDPhost ) ;
	curvesOSCMF->setText( scoreEnv.OSCCurvesDefaultMFormat ) ;
	curvesIPF->setText( scoreEnv.UDPhost ) ;			
  	curvesportF->setText(str.setNum( scoreEnv.UDPout ) ) ;
	portField->setText(str.setNum( scoreEnv.UDPout ) ) ;
	
	valField->setText(str.setNum( scoreEnv.OSCTriggersDefaultIntValue ) ) ;
	/////// Put the mouse coordinates
	formX->setText( str.setNum( infX, 'f', 4) );
  	formY->setText( str.setNum( infY, 'f', 4) );
	
	nameF->setText("");
	IDF->setText("");

	cursorBox->setEnabled(false) ;
	tabwidget->setTabEnabled(1,false) ;
	
	// Setting Caption
	if (inSelected != NULL)
	if (inSelected->getObjectType() == Trajectory)
	{
		QString sType;
		switch ( inSelected->getObjectType() )
		{
			case Trajectory :
				sType = "Trajectory";
				trajectSelected = true ;
				tabwidget->setTabEnabled(1,true) ;
				cursorBox->setEnabled(true) ;
				tabwidget->setCurrentIndex(1) ;
				if(((NxTraject*)inSelected)->getType()==2) {  // LINE
					setLineCursorLabels() ;
					isCircle = false ;
					parentLength = ((NxTraject*)inSelected)->getLength() ;
					off1F->setText( "0" ) ;
					off2F->setText( QString("%1").arg( parentLength ) ) ;
				}
				else { // CIRCLE
					setCircleCursorLabels() ;
					isCircle = true ;
					off1F->setText( "0" ) ;
					off2F->setText( QString("360") ) ;
				}
			break;	
		}
		str = QString("%1 / ID = %2 / Name = %3").arg(sType).arg(inSelected->getCustomID()).arg(inSelected->getName());
		setWindowTitle( str );
		
		IDF->setText( QString("%1").arg( inSelected->getCustomID() ) );
		
	}
	else
		setWindowTitle( "IanniX" );
		
  	show() ;
	setFocus() ;
}

void NxObjectCreator::raiseNetBox(bool b) 
{
	netBox->setEnabled( b );
}

void NxObjectCreator::setSecondInformations()
{
	connect( apply,		SIGNAL( clicked() ), this, SLOT( setObject() ) );
	disconnect( apply,	SIGNAL( clicked() ), this, SLOT( setSecondInformations() ) );
	apply->setText("Apply");
	tabwidget->setEnabled(false) ;
	mainBox->setEnabled(true) ;
	
	chooseColor2->setEnabled( false );

	if ( tabwidget->tabText( tabwidget->currentIndex() ) ==  "Curve" )
	{
		IDF->setText( QString("%1").arg(cptCurve) ) ;
		nameF->setText( QString("Curv%1").arg(cptCurve+1) ) ;
		chooseColor2->setEnabled( true );
	}
	if ( tabwidget->tabText( tabwidget->currentIndex() ) == "Trajectory" )
	{	
		IDF->setText( QString("%1").arg(cptTraject) ) ;
		nameF->setText( QString("Traj%1").arg(cptTraject+1) ) ;		
	}
	if ( tabwidget->tabText( tabwidget->currentIndex() ) == "Trigger" )
	{	
		IDF->setText( QString("%1").arg(cptTrigger) ) ;
		nameF->setText( QString("Trigg%1").arg(cptTrigger+1) ) ;		
	}
	if ( tabwidget->tabText( tabwidget->currentIndex() ) == "Cursor" )
	{
		IDF->setText( QString("%1").arg(cptCursor) ) ;
		nameF->setText( QString("Curs%1").arg(cptCursor+1) ) ;	
		formX->setEnabled(false) ;
		formY->setEnabled(false) ;
		formZ->setEnabled(false) ;
		if( !trajectSelected ) {
			parentGLBox->flagsOff() ; // unuseful in theory
			accept() ;
			QMessageBox::about(this,"Warning", "You have to select a Trajectory \nto make a Cursor !") ;
		}
		trajectSelected = false ;
	}
}

//////////////////////////////////////////
// Graphical Object Setting
/////////////////////////////////////////
void NxObjectCreator::setObject()
{
	NxPoint p;
  	// Global parameters
  	bool ok;
  	p.ID = 0; // default value
  	p.x = QString( formX->text() ).toFloat();
  	p.y = QString( formY->text() ).toFloat();
  	p.z = QString( formZ->text() ).toFloat();
	int ID ;   	
	int groupID = groupF->text().toInt( &ok, 10 ) ;
	int cusID = IDF->text().toInt( &ok, 10 ) ;
	int layer = layerF->text().toInt( &ok, 10 ) ;

	newName = nameF->text();
 	
 	///////////////////////////////////////////////////
  	// Calling NxGLScore Methods
	///////////////////////////////////////////////////
	
	////////////////////////
	// New Curve
	////////////////////////
	if ( tabwidget->tabText( tabwidget->currentIndex() ) ==  "Curve" )
	{
		if(!color1Picked) pickedColor1.c = QColor(255,0,0,255) ;
		if(!color2Picked) pickedColor2.c = QColor(0,180,50,255) ;

		// Retrieving Curves specific parameters
		ID = cptCurve;
		int duration, nbControl;
		float vMin, vMax, realMin, realMax, scale;
		
		QString curvHost = curvesIPF->text() ;
		QString curvOscM = curvesOSCMF->text() ;
		int curvPort = curvesportF->text().toInt() ;
		int inSpatialPeriod = scoreEnv.defaultSpatialPeriod ; /////////////// DEFAULT SPATIAL PERIOD VALUE... (20 ms - cf nxscore.cpp)
		scale = scaleF->text().toFloat() ;
		realMin = realMinF->text().toFloat();
		realMax = realMaxF->text().toFloat();
		
		if(newName == "") newName = QString("Curv%1").arg(ID + 1) ;
		if(IDF->text() == "") cusID = ID + 1 ;
		
		//printf("min, max : %f %f\n",realMin, realMax) ;
		if ( curvesB->isChecked() ) parentGLBox->setDrawMode( SplineStrip );
		else parentGLBox->setDrawMode( LineStrip );
			
		if ( randButton->isChecked() )
		{
			nbControl = pointsF->text().toInt( &ok, 10 );
			duration = durationF->text().toInt( &ok, 10 );
			vMin = rangeMinF->text().toFloat() ;
			vMax = rangeMaxF->text().toFloat() ;
			parentGLBox->addRandomCurve( p, ID, cusID, groupID, layer, newName, pickedColor1, pickedColor2, nbControl, vMin, vMax, duration, realMin, realMax, scale, inSpatialPeriod, curvOscM, curvHost, curvPort );
		}
		if (editButton->isChecked() )
		{
			parentGLBox->addEditCurve(p , ID, cusID, groupID, layer, newName, pickedColor1, pickedColor2, realMin, realMax, scale, inSpatialPeriod, curvOscM, curvHost, curvPort );
		}
	}
		
	/////////////////////////
	// New Traject
	/////////////////////////
	if ( tabwidget->tabText( tabwidget->currentIndex() ) == "Trajectory" )
	{
		if(!color1Picked) pickedColor1.c = QColor(200,200,0,255) ;	
		
		ID = cptTraject;
		int chx = 0;
		float radius = 0.0;
		int axis = Both;	
			
		if(newName == "") newName = QString("Traj%1").arg(ID + 1) ;	
		if(IDF->text() == "") cusID = ID + 1 ;
			
		if ( circButton->isChecked() )
		{	
			chx = 1;
			if ( radC->isChecked() )	radius = radiusF->text().toFloat() ;
			else						radius = circumF->text().toFloat() / (2*PI) ;
		}
			
		if ( lineButton->isChecked() ) chx = 2;
			
		if ( horizontalB->isChecked() )		axis = Horizontal;
		else if ( verticalB->isChecked() )	axis = Vertical;
		else if ( bothB->isChecked() )		axis = Both;
			
		parentGLBox->addNewTraject( p, ID, cusID, groupID, layer, newName, pickedColor1, chx, axis, radius );
	}
	
	/////////////////////////
  	// New Trigger
	/////////////////////////
  	if ( tabwidget->tabText( tabwidget->currentIndex() ) == "Trigger" )
	{
		if(!color1Picked) pickedColor1.c = QColor(0,0,200,255) ;
	
		ID = cptTrigger;
		QString oscM = triggersOSCMF->text();
		QString host = IPField->text();
		int port = portField->text().toInt( &ok, 10 ) ;
		int inIntValue = valField->text().toInt( &ok, 10 ) ;
		
		if(newName == "") newName = QString("Trigg%1").arg(ID + 1) ;
		if(IDF->text() == "") cusID = ID + 1 ;
		
		parentGLBox->addNewTrigger( p, ID, cusID, groupID, layer, newName, pickedColor1, oscM, host, port, inIntValue );
	}
	
	/////////////////////////
  	// New Cursor
	/////////////////////////
  	if ( tabwidget->tabText( tabwidget->currentIndex() ) == "Cursor" )
	{
		if(!color1Picked) pickedColor1.c = QColor(200,0,0,255) ;
				
		ID = cptCursor;
		float cFactor = factorF->text().toFloat();
		float cWidth = cursorWidthF->text().toFloat();
		/////// Offsets ////////////////////////////////////////////////////////
		float inInitOffset ;
		float inFinalOffset ;
		//////////////// INITIAL OFFSET
		if(off1Btn->isChecked() )  /// normal offset
			inInitOffset = off1F->text().toFloat();
		else {
			if(isCircle)
				inInitOffset = (off1pF->text().toFloat())*360/100 ;
			else
				inInitOffset = (off1pF->text().toFloat())*parentLength/100 ;
		}
		/////////////// FINAL OFFSET
		if(off2Btn->isChecked() ) /// normal offset
			inFinalOffset = off2F->text().toFloat();
		else {
			if(isCircle)
				inFinalOffset = (off2pF->text().toFloat())*360/100 ;
			else
				inFinalOffset = (off2pF->text().toFloat())*parentLength/100 ;
		}
		//////////////////////////////////////////////////////////////////////
		/////// get Mode Values
		bool inForward ;
		int inMode = 0 ; // initialisation
		if( fwdBtn->isChecked() ) inForward = true ;
			else inForward = false ;
		if( singleBtn->isChecked() )	inMode = 0 ;
		if( repeatBtn->isChecked() )	inMode = 1 ;
		if( fBtn->isChecked() )			inMode = 2 ;
		
		newName = QString("Curs%1").arg( ID + 1 ) ;
		if(IDF->text() == "") cusID = ID + 1 ;
		
		parentGLBox->addNewCursor( p, ID, cusID, groupID, layer, newName, pickedColor1, inForward, inMode, cFactor, cWidth, inInitOffset, inFinalOffset );
	}
	
	parentGLBox->updateGroupFilterList() ;
	parentGLBox->updateLayerFilterList() ;
	
  	// Incrementing ID counter
	parentGLBox->getPartition()->maxOfIndexes() ;
	
	// Releasing Keyboard
	parentGLBox->flagsOff();
	//this->releaseKeyboard();
  	//parentGLBox->grabKeyboard();

	color1Picked = false ;
	color2Picked = false ;
	accept();
}

void NxObjectCreator::pickColor1() 
{
	pickedColor1.c = colorPicker->getColor();
	if( !(pickedColor1.c.red()==0 && pickedColor1.c.green()==0 && pickedColor1.c.blue()==0) ) 
		color1Picked = true ;
	else
		printf("Default Color Choosed\n") ;
}

void NxObjectCreator::pickColor2() 
{
	pickedColor2.c = colorPicker->getColor();	
	if( !(pickedColor2.c.red()==0 && pickedColor2.c.green()==0 && pickedColor2.c.blue()==0) ) 
		color2Picked = true ;
	else
		printf("Default Color Choosed\n") ;
}

/////////////////////
// Destructor
////////////////////
NxObjectCreator::~NxObjectCreator()
{
  // no need to delete child widgets, Qt does it all for us
}

////////////////////////////
// Events Handler
///////////////////////////
void NxObjectCreator::closeEvent( QCloseEvent* )
{
	parentGLBox->flagsOff();
  	//this->hide();
	//this->releaseKeyboard();
	//parentGLBox->grabKeyboard();
}

void NxObjectCreator::giveKeyboard()
{
	//this->releaseKeyboard();
	//parentGLBox->grabKeyboard();
}

/////////////////////////////////////
// Initializing Widgets
/////////////////////////////////////

void NxObjectCreator::initMainBox()
{
	// Groupboxes & layouts
	QGroupBox* IDBox 	= new QGroupBox("Position / Identification", mainBox );
	QGroupBox* colBox 	= new QGroupBox("Color", mainBox );
	QGroupBox* paramBox = new QGroupBox("Special", mainBox );

	IDBox->setAlignment( Qt::AlignHCenter );
	colBox->setAlignment( Qt::AlignHCenter );
	paramBox->setAlignment( Qt::AlignHCenter );
	paramBox->setEnabled( false ) ;
	
	formX		= new QLineEdit("0.0", IDBox );
  	formY		= new QLineEdit("0.0", IDBox );
  	formZ		= new QLineEdit("0.0", IDBox );
  	posXLabel 	= new QLabel("<b>X</b>", IDBox );
  	posYLabel 	= new QLabel("<b>Y</b>", IDBox );
  	posZLabel 	= new QLabel("<b>Z</b>", IDBox );	
	
	nameF 	= new QLineEdit( IDBox );
  	IDF 	= new QLineEdit( IDBox );
  	groupF 	= new QLineEdit("0", IDBox ); 
	layerF	= new QLineEdit("0", IDBox ); 
  	nameL 	= new QLabel("<b>Name</b>", IDBox );
  	IDL 	= new QLabel("<b>ID</b>", IDBox );
	groupL 	= new QLabel("<b>Group ID</b>", IDBox );
	layerL 	= new QLabel("<b>Layer</b>", IDBox );

	// Special Sizes
	nameF->setMaximumSize( 120, 20 );
	IDF->setMaximumSize( 50, 20 );
	groupF->setMaximumSize( 50, 20 );
	layerF->setMaximumSize( 50, 20 );
	
  	// Special parameters
  	fileButton 		= new QRadioButton("Select from file...", paramBox );
  	fileDialog 		= new QFileDialog( paramBox );
	fileDialog->setEnabled( FALSE );
	
  	personalButton 	= new QRadioButton("Custom" , paramBox );
  	texture 		= new QCheckBox("Texture", paramBox );
  	transparency 	= new QCheckBox("Transparency", paramBox );
  	remanence 		= new QCheckBox("Remanence", paramBox );
	
  	chooseColor1 	= new QPushButton("Choose Color 1", colBox );
	chooseColor2 	= new QPushButton("Choose Color 2", colBox );
	
	/*chooseColor1->setBackgroundColor( pickedColor1.c ) ;
	chooseColor2->setBackgroundColor( pickedColor2.c ) ; */

	//Sizes
	formX->setMaximumSize( 100, 20 );
	formY->setMaximumSize( 100, 20 );
	formZ->setMaximumSize( 100, 20 );
	
	// Layouts
	QGridLayout* idLay 	= new QGridLayout;
	QGridLayout* posLay = new QGridLayout;
	QGridLayout* gridL 	= new QGridLayout;
	QVBoxLayout* vBox 	= new QVBoxLayout;
	QHBoxLayout* idpos 	= new QHBoxLayout;
	QVBoxLayout* colLay = new QVBoxLayout;
	
	idLay->addWidget( nameL		,0 ,0 );
	idLay->addWidget( nameF		,0 ,1 );
	idLay->addWidget( IDL		,1 ,0 );
	idLay->addWidget( IDF		,1 ,1 );
	idLay->addWidget( groupL	,2 ,0 );
	idLay->addWidget( groupF	,2 ,1 );
	idLay->addWidget( layerL	,3 ,0 );
	idLay->addWidget( layerF	,3 ,1 );
	
	posLay->addWidget( posXLabel	,0 ,0 );
	posLay->addWidget( formX		,0 ,1 );
	posLay->addWidget( posYLabel	,1 ,0 );
	posLay->addWidget( formY		,1 ,1 );
	posLay->addWidget( posZLabel	,2 ,0 );
	posLay->addWidget( formZ		,2 ,1 );
	
	gridL->addWidget( fileButton, 		1, 0 );
  	gridL->addWidget( fileDialog, 		1, 1 );
  	gridL->addWidget( personalButton, 	2, 0 );
  	gridL->addWidget( texture, 			2, 1 );
  	gridL->addWidget( transparency, 	3, 0 );
  	gridL->addWidget( remanence, 		3, 1 );
	
	vBox->addWidget( IDBox );
 	vBox->addWidget( colBox );
 	vBox->addWidget( paramBox );
	
	idpos->addLayout(posLay) ;
	idpos->addLayout(idLay) ;	
	
	colLay->addWidget(chooseColor1) ;
	colLay->addWidget(chooseColor2) ;
	
	IDBox->setLayout(idpos);
	colBox->setLayout(colLay);
	paramBox->setLayout(gridL);
	
	mainBox->setLayout(vBox);
}

void NxObjectCreator::initCurveBox()
{
	QVBoxLayout* vBoxL = new QVBoxLayout;
	
  	// Groupboxes
	QGroupBox *typeGroup 	= new QGroupBox("Drawing Style", curveBox );
	QGroupBox *styleGroup 	= new QGroupBox("Edition Mode", curveBox );
	QButtonGroup* typeBox 	= new QButtonGroup( typeGroup ); 
  	QButtonGroup* styleBox 	= new QButtonGroup( styleGroup );
	QGroupBox *mappingBox	= new QGroupBox("Mapping", curveBox ); 
  	QGroupBox *paramsBox	= new QGroupBox("Parameters", curveBox );
	QGroupBox *oscBox		= new QGroupBox("OSC Messages", curveBox );
	
	// Layouts
	QHBoxLayout* hLay1 		= new QHBoxLayout;
  	QHBoxLayout* hLay2 		= new QHBoxLayout;
	QGridLayout* boxLay2 	= new QGridLayout;
	QGridLayout* mappingLay = new QGridLayout;
	QGridLayout* oscLay		= new QGridLayout;
		
  	// Type
 	editButton 		= new QRadioButton("Graphical Edition", typeGroup ) ;
  	randButton 		= new QRadioButton("Random", typeGroup ) ;
	styleBox->addButton(editButton);
	styleBox->addButton(randButton);
	// Style
	linesB 	= new QRadioButton("Lines", styleGroup ) ;
	curvesB = new QRadioButton("Splines", styleGroup ) ;
	typeBox->addButton(linesB);
	typeBox->addButton(curvesB);
  	// Mapping
	QLabel* scaleL		= new QLabel( "Graphic Scale (in s)", mappingBox );
	QLabel* realMinL 	= new QLabel( "Affected Minimum", mappingBox );
	QLabel* realMaxL 	= new QLabel( "Affected Maximum", mappingBox );
	scaleF		= new QLineEdit( "1", mappingBox );	
	realMinF 	= new QLineEdit( mappingBox );		
	realMaxF 	= new QLineEdit( mappingBox );
	// Parameters
	pointsF 	= new QLineEdit("8", paramsBox );							
  	durationF 	= new QLineEdit("5000", paramsBox );		
  	rangeMinF 	= new QLineEdit( paramsBox );		
  	rangeMaxF 	= new QLineEdit( paramsBox );
  	QLabel* pointsL 	= new QLabel("Number of control points", paramsBox );				
  	QLabel* durationL 	= new QLabel("Duration (in ms)", paramsBox );
  	QLabel* rangeMinL 	= new QLabel("Value Min",  paramsBox );
  	QLabel* rangeMaxL 	= new QLabel("Value Max",  paramsBox );
	// OSC
	curvesOSCMF		= new QLineEdit( oscBox );	
	curvesIPF		= new QLineEdit( oscBox );							
  	curvesportF 	= new QLineEdit( oscBox );
	QLabel* curvesOSCML = new QLabel("Format", oscBox );		
  	QLabel* curvesIPL 	= new QLabel("Host IP", oscBox );				
  	QLabel* curvesportL = new QLabel("UDP Port", oscBox );

  	// Adding widgets
  	hLay1->addWidget( editButton );
  	hLay1->addWidget( randButton );
	hLay2->addWidget( linesB );
	hLay2->addWidget( curvesB );
		
  	boxLay2->addWidget( pointsL, 1, 0 );
  	boxLay2->addWidget( pointsF, 1, 1 );
  	boxLay2->addWidget( durationL, 2, 0 );
  	boxLay2->addWidget( durationF, 2, 1 );
  	boxLay2->addWidget( rangeMinL, 3, 0 );
  	boxLay2->addWidget( rangeMinF, 3, 1 );
  	boxLay2->addWidget( rangeMaxL, 4, 0 );
  	boxLay2->addWidget( rangeMaxF, 4, 1 );

	mappingLay->addWidget( scaleL,		0, 0 );
	mappingLay->addWidget( scaleF,		0, 1 );
	mappingLay->addWidget( realMinL,	1, 0 );
	mappingLay->addWidget( realMinF,	1, 1 );
	mappingLay->addWidget( realMaxL,	2, 0 );
	mappingLay->addWidget( realMaxF,	2, 1 );
	
	oscLay->addWidget( curvesOSCML,		0, 0 );
	oscLay->addWidget( curvesOSCMF,		0, 1 );
	oscLay->addWidget( curvesIPL,		1, 0 );
	oscLay->addWidget( curvesIPF,		1, 1 );
	oscLay->addWidget( curvesportL,		2, 0 );
	oscLay->addWidget( curvesportF,		2, 1 );
	
	vBoxL->addWidget( styleGroup );
	vBoxL->addWidget( typeGroup );
	vBoxL->addWidget( mappingBox );	
	vBoxL->addWidget( paramsBox );    
	vBoxL->addWidget( oscBox );             	
	
	// Layouts
	styleGroup->setLayout( hLay1 );
	typeGroup->setLayout( hLay2 );
	paramsBox->setLayout( boxLay2 );
	mappingBox->setLayout( mappingLay ) ;
	oscBox->setLayout( oscLay ) ;
	curveBox->setLayout(vBoxL);	
	
  	// Values for Fields & Sizes
	rangeMinF->setText( QString("%1").arg(	int(scoreEnv.mappingDefaultMin) )  ) ;
  	rangeMaxF->setText( QString("%1").arg(	int(scoreEnv.mappingDefaultMax) )  ) ;
	realMinF->setText( QString("%1").arg(	int(scoreEnv.mappingDefaultMin) ) ) ;	
	realMaxF->setText( QString("%1").arg(	int(scoreEnv.mappingDefaultMax) ) ) ;	
	//// OSC
	curvesOSCMF->setText( scoreEnv.OSCCurvesDefaultMFormat ) ;
	curvesIPF->setText( scoreEnv.UDPhost ) ;
  	curvesportF->setText( QString("%1").arg( scoreEnv.UDPout ) ) ;
	
	typeGroup->setAlignment( Qt::AlignHCenter );
  	styleGroup->setAlignment( Qt::AlignHCenter );	
  	mappingBox->setAlignment( Qt::AlignHCenter );
  	paramsBox->setAlignment( Qt::AlignHCenter );
	oscBox->setAlignment( Qt::AlignHCenter );
	editButton->setChecked( true );

	// Default settings
  	pointsF->setEnabled( false );
  	durationF->setEnabled( false );
  	rangeMinF->setEnabled( false );
  	rangeMaxF->setEnabled( false );
	linesB->setChecked( true );
	
	realMinF->setFocus() ;
	
  	// Connecting Buttons
  	connect( randButton, SIGNAL( toggled(bool) ), pointsF, SLOT( setEnabled(bool) ) );
  	connect( randButton, SIGNAL( toggled(bool) ), rangeMinF, SLOT( setEnabled(bool) ) );
  	connect( randButton, SIGNAL( toggled(bool) ), rangeMaxF, SLOT( setEnabled(bool) ) );
  	connect( randButton, SIGNAL( toggled(bool) ), durationF, SLOT( setEnabled(bool) ) );
}

void NxObjectCreator::initTrajectBox()
{	
	QVBoxLayout* setL = new QVBoxLayout;
	
  	// Groupboxes with layouts
  	QGroupBox *typeGroup = new QGroupBox("Shape", trajectBox);
	QGroupBox *lineGroup = new QGroupBox("Line", trajectBox);
	QGroupBox *circleBox = new QGroupBox("Circle", trajectBox );
	
	QButtonGroup* typeBox 	= new QButtonGroup( trajectBox );
	QButtonGroup* lineBox 	= new QButtonGroup( trajectBox );
	
	QHBoxLayout* lay1 = new QHBoxLayout;
  	QGridLayout* lay2 = new QGridLayout;
	QHBoxLayout* lay4 = new QHBoxLayout;
	
  	// Widgets
	lineButton 		= new QRadioButton("Line", trajectBox) ;
  	circButton 		= new QRadioButton("Circle", trajectBox) ;
	typeBox->addButton(lineButton);
	typeBox->addButton(circButton);
	
	verticalB 	= new QRadioButton("Vertical", lineGroup );
	horizontalB = new QRadioButton("Horizontal", lineGroup) ;
	bothB 		= new QRadioButton("Free", lineGroup ) ;
	horizontalB->setChecked( true );
	lineBox->addButton(verticalB);
	lineBox->addButton(horizontalB);
	lineBox->addButton(bothB);
	
  	// Parameters Circle
	radC	= new QRadioButton ;
	circumC	= new QRadioButton ;
	radiusF = new QLineEdit("1", circleBox );							
 	radiusL = new QLabel("Radius", circleBox );
	circumF = new QLineEdit("5", circleBox );
	circumL = new QLabel("Length", circleBox );		
	
  	// Adding widgets
	lay1->addWidget( lineButton );
  	lay1->addWidget( circButton );
		
	lay2->addWidget( radC,		0, 0 );
  	lay2->addWidget( radiusL,	0, 1 );
  	lay2->addWidget( radiusF,	0, 2 );
	lay2->addWidget( circumC,	1, 0 );
	lay2->addWidget( circumL,	1, 1 );
	lay2->addWidget( circumF,	1, 2 );
	
	lay4->addWidget( verticalB );
	lay4->addWidget( horizontalB );
	lay4->addWidget( bothB );
	
	typeGroup->setLayout(lay1);
	lineGroup->setLayout(lay4);
	circleBox->setLayout(lay2);
	
  	setL->addWidget( typeGroup );
	setL->addWidget( lineGroup );
	setL->addWidget( circleBox );
	setL->addStretch(2) ;
	
	trajectBox->setLayout(setL);
	
  	// Values for Fields & Sizes
  	lineButton ->setChecked( true );
  	typeGroup->setAlignment( Qt::AlignHCenter);
  	circleBox->setAlignment( Qt::AlignHCenter );
	lineGroup->setAlignment( Qt::AlignHCenter );
		
	// Signals & Slots
	circleBox->setEnabled( false );
	radC->setChecked(true) ;
	circumF->setEnabled(false) ;
	connect( radC,			SIGNAL( toggled( bool ) ), radiusF, 	SLOT( setEnabled( bool ) ) );
	connect( circumC,		SIGNAL( toggled( bool ) ), circumF, 	SLOT( setEnabled( bool ) ) );
	connect( circButton, 	SIGNAL( toggled( bool ) ), circleBox, 	SLOT( setEnabled( bool ) ) );
	connect( lineButton, 	SIGNAL( toggled( bool ) ), lineGroup, 	SLOT( setEnabled( bool ) ) );
}

void NxObjectCreator::initCursorBox()
{	
	// Containers
	QGroupBox *playGroup 	= new QGroupBox("Playback", cursorBox );
	QGroupBox *modeGroup 	= new QGroupBox("Mode", cursorBox );
	
	QButtonGroup* playBox 	= new QButtonGroup( playGroup );
	QButtonGroup* modeBox 	= new QButtonGroup( modeGroup );
	QGroupBox* factorBox 	= new QGroupBox("Speed / Width", cursorBox );
	offsetBox1	= new QGroupBox("Initial Offset", cursorBox );
	offsetBox2	= new QGroupBox("Final Offset", cursorBox );
	
	QVBoxLayout* vBox 		= new QVBoxLayout;
	QHBoxLayout* playL 		= new QHBoxLayout;
	QHBoxLayout* modeL 		= new QHBoxLayout;
	QGridLayout* factorLay	= new QGridLayout;
	QGridLayout* offsLay1	= new QGridLayout;
	QGridLayout* offsLay2	= new QGridLayout;
	
	// Widgets
	fwdBtn		= new QRadioButton("Forward", modeGroup ) ;
	bwdBtn		= new QRadioButton("Backward", modeGroup ) ;
	singleBtn	= new QRadioButton("Single", playGroup ) ;
	repeatBtn	= new QRadioButton("Repeat", playGroup ) ;
	fBtn 		= new QRadioButton("Fwd/Bckwd", playGroup ) ;
	// Offsets radio
	off1Btn		= new QRadioButton;
	off1pBtn	= new QRadioButton;
	off2Btn		= new QRadioButton;
	off2pBtn	= new QRadioButton;
	
	// Adding Buttons
	playBox->addButton(fwdBtn);
	playBox->addButton(bwdBtn);
	modeBox->addButton(singleBtn);
	modeBox->addButton(repeatBtn);
	modeBox->addButton(fBtn);
	
	factorL 		= new QLabel("Speed Factor", factorBox );
	factorF 		= new QLineEdit("1.0", factorBox );
	cursorWidthL 	= new QLabel("Width (in s)", factorBox );
	cursorWidthF 	= new QLineEdit("2", factorBox );
	// Offsets
	off1F 		= new QLineEdit("0",			offsetBox1 );
	off1L		= new QLabel("Offset (in s)",	offsetBox1 );
	off1pF 		= new QLineEdit("0",			offsetBox1 );
	off1pL		= new QLabel("Offset (in %)",	offsetBox1 );
	off2F 		= new QLineEdit("0",			offsetBox2 );
	off2L		= new QLabel("Offset (in s)",	offsetBox2 );
	off2pF 		= new QLineEdit("100",			offsetBox2 );
	off2pL		= new QLabel("Offset (in %)",	offsetBox2 );
	
	// Layout
	playL->addWidget( fwdBtn );
	playL->addWidget( bwdBtn );
	modeL->addWidget( singleBtn );
	modeL->addWidget( repeatBtn );
	modeL->addWidget( fBtn );
	factorLay->addWidget( factorL		, 0, 0 );
	factorLay->addWidget( factorF		, 0, 1 );
	factorLay->addWidget( cursorWidthL	, 1, 0 );
	factorLay->addWidget( cursorWidthF	, 1 ,1 );
	offsLay1->addWidget(off1Btn	, 0, 0 ) ;
	offsLay1->addWidget(off1L	, 0, 1 ) ;
	offsLay1->addWidget(off1F	, 0, 2 ) ;
	offsLay1->addWidget(off1pBtn, 1, 0 ) ;
	offsLay1->addWidget(off1pL	, 1, 1 ) ;
	offsLay1->addWidget(off1pF	, 1, 2 ) ;
	offsLay2->addWidget(off2Btn	, 0, 0 ) ;
	offsLay2->addWidget(off2L	, 0, 1 ) ;
	offsLay2->addWidget(off2F	, 0, 2 ) ;
	offsLay2->addWidget(off2pBtn, 1, 0 ) ;
	offsLay2->addWidget(off2pL	, 1, 1 ) ;
	offsLay2->addWidget(off2pF	, 1, 2 ) ;
	playGroup->setLayout(playL);
	modeGroup->setLayout(modeL);
	factorBox->setLayout(factorLay);
	offsetBox1->setLayout(offsLay1) ;
	offsetBox2->setLayout(offsLay2) ;
	
	vBox->addWidget( playGroup );
	vBox->addWidget( modeGroup );
	vBox->addWidget( factorBox );
	vBox->addWidget( offsetBox1 );	
	vBox->addWidget( offsetBox2 );
	vBox->addStretch(2) ;
	
	cursorBox->setLayout(vBox);
	
	// Texts
	playGroup->setAlignment( Qt::AlignHCenter);
	modeGroup->setAlignment( Qt::AlignHCenter);
	factorBox->setAlignment( Qt::AlignHCenter);
	offsetBox1->setAlignment( Qt::AlignHCenter);
	offsetBox2->setAlignment( Qt::AlignHCenter);
	
	// Default Settings
	fwdBtn->setChecked( true );
  	singleBtn->setChecked( true );
	off1Btn->setChecked(true) ;
	off1pF->setEnabled(false) ;
	off2Btn->setChecked(true) ;
	off2pF->setEnabled(false) ;
	
	// Signals
	connect( off1Btn, 	SIGNAL( toggled( bool ) ), off1F, 	SLOT( setEnabled( bool ) ) );
	connect( off1pBtn, 	SIGNAL( toggled( bool ) ), off1pF, 	SLOT( setEnabled( bool ) ) );
	connect( off2Btn, 	SIGNAL( toggled( bool ) ), off2F, 	SLOT( setEnabled( bool ) ) );
	connect( off2pBtn, 	SIGNAL( toggled( bool ) ), off2pF, 	SLOT( setEnabled( bool ) ) );
}

void NxObjectCreator::initTriggerBox()
{	
	QGroupBox* netBox 	= new QGroupBox("OSC Messages", triggerBox );
	netBox->setAlignment( Qt::AlignHCenter );
 
	QGridLayout* lay1 	= new QGridLayout;
	QVBoxLayout* vBox 	= new QVBoxLayout;
	
	// Widgets
	QLabel *triggersOSCML 	= new QLabel("Format", netBox );
	QLabel *IPLabel			= new QLabel("Host IP", netBox );
	QLabel *portLabel		= new QLabel("UDP Port", netBox );
	QLabel *valLabel		= new QLabel("Int Value", netBox );
	triggersOSCMF	= new QLineEdit( netBox );
	IPField			= new QLineEdit( netBox );
	portField		= new QLineEdit( netBox );
	valField		= new QLineEdit( netBox );
	
	// Adding Widgets
	lay1->addWidget( triggersOSCML	, 0, 0 );
	lay1->addWidget( triggersOSCMF	, 0, 1 );
	lay1->addWidget( IPLabel		, 1, 0 );
	lay1->addWidget( IPField		, 1, 1 );
	lay1->addWidget( portLabel		, 2, 0 );
	lay1->addWidget( portField		, 2, 1 );
	lay1->addWidget( valLabel		, 3, 0 );
	lay1->addWidget( valField		, 3, 1 );

	vBox->addWidget( netBox );
	vBox->addStretch(2) ;
	
	// Layouts
	netBox->setLayout(lay1);
	triggerBox->setLayout(vBox);
	
	netBox->setMaximumSize(800, 200) ;
	 
	// Texts
	triggersOSCMF->setText( scoreEnv.OSCTriggersDefaultMFormat );
	IPField->setText( scoreEnv.UDPhost );
	portField->setText( QString("%1").arg(scoreEnv.UDPout) );
	valField->setText( QString("%1").arg(scoreEnv.OSCTriggersDefaultIntValue) );
}

void NxObjectCreator::getScoreMaxCpt()
{
	cptTrigger = parentGLBox->getPartition()->getCptTrigger() ;
	cptCurve = parentGLBox->getPartition()->getCptCurve() ;
	cptCursor = parentGLBox->getPartition()->getCptCursor() ;
	cptTraject = parentGLBox->getPartition()->getCptTraject() ;
}

