///////////////
// IanniX
///////////////

////////////////////////////
// NxObjectCreator.h
////////////////////////////

#ifndef NXOBJECTCREATOR_H
#define NXOBJECTCREATOR_H

#include <QtGui>
#include <QtOpenGL>
#include "../graphics/opengl/NxGLScore.h"
#include "../base/atoms/NxTypes.h"
#include "../base/atoms/NxObject.h"

class NxGLScore ;

class NxObjectCreator : public QDialog
{
	Q_OBJECT

 public:
	NxObjectCreator( NxGLScore* parent );
  	~NxObjectCreator();
  
  	QLabel* mainTitle;
  	QFrame* titleLine;
  
	// Child Widgets
	QGroupBox *mainBox, *curveBox, *trajectBox, *cursorBox, *triggerBox;
	
	//
	QPushButton *apply, *cancel ;
	QHBoxLayout *mainEditorLayout ;
	// Layouts
	QGridLayout* mainLay;
	QGridLayout* curveLay;
	QGridLayout* trajectLay;
	QGridLayout* cursorLay;
	QGridLayout* triggerLay;
	
  	// Identification
  	QLineEdit *nameF, *IDF, *groupF, *layerF ;
  	QLabel *nameL, *IDL, *groupL, *layerL ;
	
  	// Object params
  	QRadioButton *fileButton, *personalButton;
  	QFileDialog *fileDialog;
  	QCheckBox *texture, *transparency, *remanence;
  	QPushButton *chooseColor1, *chooseColor2;
	
  	// Position Handler
  	QLineEdit *formX, *formY, *formZ;
  	QLabel *posXLabel, *posYLabel, *posZLabel;
  	QPushButton* placeObject, *quitEd;
	
  	// Color Picker
  	QColorDialog* colorPicker ;
  	Color pickedColor1, pickedColor2;
	bool color1Picked, color2Picked ;

  	// Network conf
  	QGroupBox* netBox;
  	QLineEdit *triggersOSCMF, *IPField, *portField, *valField;
	QComboBox* typeChooser;
	
  	// Layouts
 	QBoxLayout* vLayout;
  	QHBoxLayout* idL;
  	QHBoxLayout* typeL;
  	QGridLayout* gridL;
  	QHBoxLayout* posL;
  	QHBoxLayout* netL;
	
	// Widgets
	QLineEdit	*pointsF, *durationF, *rangeMinF, *rangeMaxF, *realMinF, *realMaxF, *scaleF;
	QLineEdit	*curvesOSCMF, *curvesIPF, *curvesportF ; // Curves OSC
	QRadioButton *radC, *circumC ;
	QLabel *circumL, *radiusL, *factorL;
	QLineEdit *circumF, *radiusF, *factorF;
	QLabel *cursorWidthL;
	QLineEdit *cursorWidthF;
	QRadioButton *fwdBtn, *bwdBtn, *singleBtn, *repeatBtn, *fBtn ;
	QLabel *off1L, *off1pL, *off2L, *off2pL ;
	QLineEdit *off1F, *off1pF, *off2F, *off2pF ;
	QRadioButton *off1Btn, *off1pBtn, *off2Btn, *off2pBtn ;
	
	QRadioButton *editButton, *randButton, *fromFileButton;
	QRadioButton *circButton, *lineButton, *BSplineButton; 
	QRadioButton *linesB, *curvesB;
	QRadioButton *horizontalB, *verticalB, *bothB;
	QTabWidget *tabwidget;
	
  	// Public methods
  	void raiseEditor( float infX, float infY, NxObject* inSelected );
  	void initMainBox();
	void initCurveBox();
	void initTrajectBox();
	void initCursorBox();
	void initTriggerBox();
		
	void getScoreMaxCpt() ;	
		
 public slots:

	virtual void giveKeyboard();
	
 protected:
	
	void setCircleCursorLabels() ;
	void setLineCursorLabels() ;
	
	float parentLength ; // Length of parent trajectory when cursor creation
	bool trajectSelected ;
	bool isCircle ;
	
	QGroupBox	*offsetBox1, *offsetBox2 ;
	
  	NxGLScore* parentGLBox;
  	NxEnv	scoreEnv;
  	float valX, valY, valZ; 
  	float infX, infY;
	float vMin, vMax;
  	int objectsCounter;
	int cptCurve, cptTrigger, cptTraject, cptCursor;
  	QString objectsString;
  	QString path, newName;
	
	Color			objectPickedColor ;
	QColorDialog	*colDialogPicker ;
	bool			dialogColorPicked ;
	
 protected slots:
	virtual void raiseNetBox( bool b );
	virtual void setSecondInformations() ;
  	virtual void setObject();
  	virtual void pickColor1();
  	virtual void pickColor2();	
  	virtual void closeEvent( QCloseEvent* e );
};

#endif

