///////////////
// IanniX
///////////////

////////////////////////////
// NxObjectEditor.cpp
////////////////////////////

#include "NxObjectEditor.h"

NxObjectEditor::NxObjectEditor( NxGLScore* parent )
  : QDialog( parent )
{
	QFont f( "Lucida Grande", 11 );
	setFont( f );
	
	parentGLBox	= parent;
	objectPickedColor.ID = 0 ;
	
	setWindowFlags(Qt::Dialog);
	setWindowTitle( tr( "IanniX Object Editor" ) );
	setFixedSize( 800, 590 ) ;
	
	firstBuildItems() ;
	
	setWindowModality( Qt::ApplicationModal ) ;
	
}
/////////////////////
// Destructor
////////////////////
NxObjectEditor::~NxObjectEditor()
{}

////////////////////////////////////////////////////////////////////////
// BUILD OBJECTS
////////////////////////////////////////////////////////////////////////
void NxObjectEditor::firstBuildItems()
{	
	Gsettings					= new QGroupBox(this) ;
	Gchoice						= new QGroupBox("Apply on", this) ;
		Gtype = new QGroupBox(Gchoice) ;
	Gparam						= new QGroupBox("Properties", this) ;
		Gmove = new QGroupBox("Position", Gparam) ;
	Gcurves						= new QGroupBox("Curves", this) ;
		GcurvesBounds = new QGroupBox("Curves Bounds", this) ;
			Gbounds	= new QGroupBox(GcurvesBounds) ;
			Gcheck	= new QGroupBox(GcurvesBounds) ;
	Gbounds->setFlat(true) ;
	Gcheck->setFlat(true) ;
	Gtrajects					= new QGroupBox("Trajectories", this) ;
	Gtriggers					= new QGroupBox("Triggers", this) ;
	Gcursors					= new QGroupBox("Cursors", this) ;
	Gosc						= new QGroupBox("OSC Settings", this) ;
	QGroupBox* Gbutt			= new QGroupBox(this) ;
	
	//////////////////////////////////////////// Background colors
	Gchoice->setAutoFillBackground(true) ;
	Gparam->setAutoFillBackground(true) ;
	Gosc->setAutoFillBackground(true) ;
	Gtrajects->setAutoFillBackground(true) ;
	Gtriggers->setAutoFillBackground(true) ;
	Gcursors->setAutoFillBackground(true) ;
	Gcurves->setAutoFillBackground(true) ;
	GcurvesBounds->setAutoFillBackground(true) ;

	cGray.setColor(Gchoice->backgroundRole(), QColor(212,208,200) );
	cRed.setColor(Gchoice->backgroundRole(), QColor(200,160,160) );
	cLight.setColor(Gchoice->backgroundRole(), QColor(200,200,200) );
	cDark.setColor(Gchoice->backgroundRole(), QColor(170,170,170) );
	
	setDefaultColors() ;
	
	QHBoxLayout* Lsettings	= new QHBoxLayout(Gsettings) ;
	QVBoxLayout* Lleft		= new QVBoxLayout() ;
	QVBoxLayout* Lcenter	= new QVBoxLayout() ;
	QVBoxLayout* Lright		= new QVBoxLayout() ;
	
	QGridLayout* Lchoice	= new QGridLayout(Gchoice) ;
	QGridLayout* Ltype		= new QGridLayout(Gtype) ;
	QGridLayout* Lparam		= new QGridLayout(Gparam) ;
	QHBoxLayout* Lmove		= new QHBoxLayout(Gmove) ;
	QGridLayout* Lcurves	= new QGridLayout(Gcurves) ;
	QGridLayout* LcurvesBounds	= new QGridLayout(GcurvesBounds) ;
		QGridLayout* Lbounds	= new QGridLayout(Gbounds) ;
		QVBoxLayout* Lcheck		= new QVBoxLayout(Gcheck) ;
	QGridLayout* Ltrajects	= new QGridLayout(Gtrajects) ;
	QGridLayout* Ltriggers	= new QGridLayout(Gtriggers) ;
	QGridLayout* Lcursors	= new QGridLayout(Gcursors) ;
	QGridLayout* Losc		= new QGridLayout(Gosc) ;
	QHBoxLayout* Lbutt		= new QHBoxLayout(Gbutt) ;
	
	QVBoxLayout* chpLay		= new QVBoxLayout() ;
	QGridLayout* posLay		= new QGridLayout() ;
	
	QVBoxLayout* Lgen		= new QVBoxLayout(this) ;

	////////////////////////////////////////////////////////////////////// APPLY ON
	applyOnGroupIDC		= new QComboBox(Gchoice) ;
	applyOnGroupIDC->addItem( "Only Group ID" );
	applyOnGroupIDC->addItem( "All Group IDs" );
	applyOnLayerC		= new QComboBox(Gchoice) ;
	applyOnLayerC->addItem( "Only Layer" );
	applyOnLayerC->addItem( "All Layers" );
	fGroupF				= new QLineEdit(Gchoice) ;
	fLayerF				= new QLineEdit(Gchoice) ;
	trajC	= new QCheckBox("Trajectories"	, Gtype) ;
	cursC	= new QCheckBox("Cursors"		, Gtype) ;
	curvC	= new QCheckBox("Curves"		, Gtype) ; 
	trigC	= new QCheckBox("Triggers"		, Gtype) ;
	Ltype->addWidget( trajC ,  0, 0 ) ;
	Ltype->addWidget( cursC ,  1, 0 ) ;
	Ltype->addWidget( curvC ,  0, 1 ) ;
	Ltype->addWidget( trigC ,  1, 1 ) ;
	Gtype->setLayout( Ltype ) ;
	Lchoice->addWidget( Gtype,				0, 0, 1, 0 ) ;
	Lchoice->addWidget( applyOnGroupIDC ,	1, 0 ) ;
	Lchoice->addWidget( fGroupF ,			1, 1 ) ;
	Lchoice->addWidget( applyOnLayerC ,		2, 0 ) ;
	Lchoice->addWidget( fLayerF ,			2, 1 ) ;
	Gchoice->setLayout(Lchoice) ;
	////////////////////////////////////////////////////////////////////// MOVE
	xtC		= new QCheckBox(Gmove) ; 
	ytC		= new QCheckBox(Gmove) ; 
	ztC		= new QCheckBox(Gmove) ; 
	shiftXC	= new QComboBox(Gmove) ;
	shiftXC->addItem( "Set X" );
	shiftXC->addItem( "Shift X by" );
	shiftYC	= new QComboBox(Gmove) ;
	shiftYC->addItem( "Set Y" );
	shiftYC->addItem( "Shift Y by" );
	shiftZC	= new QComboBox(Gmove) ;
	shiftZC->addItem( "Set Z" );
	shiftZC->addItem( "Shift Z by" );
	xtF		= new QLineEdit("0", Gmove) ;
	ytF		= new QLineEdit("0", Gmove) ;
	ztF		= new QLineEdit("0", Gmove) ;
	xtF->setMaximumSize( 80, 21 );
	ytF->setMaximumSize( 80, 21 );
	ztF->setMaximumSize( 80, 21 );
	posLay->addWidget(xtC ,  0, 0 ) ;
	posLay->addWidget(ytC ,  1, 0 ) ;
	posLay->addWidget(ztC ,  2, 0 ) ;
	posLay->addWidget(shiftXC ,  0, 1 ) ;
	posLay->addWidget(shiftYC ,  1, 1 ) ;
	posLay->addWidget(shiftZC ,  2, 1 ) ;
	posLay->addWidget(xtF ,  0, 2 ) ;
	posLay->addWidget(ytF ,  1, 2 ) ;
	posLay->addWidget(ztF ,  2, 2 ) ;
	Lmove->addLayout( chpLay ) ;
	Lmove->addLayout( posLay ) ;
	Gmove->setLayout(Lmove) ;
	////////////////////////////////////////////////////////////////////// PARAMETERS
	setActC			= new QCheckBox(Gparam) ; 
	actC			= new QComboBox(Gparam) ;
	actC->addItem( "Active" );
	actC->addItem( "Inactive" );
	newNameC		= new QCheckBox("Name", Gparam) ; 
	newNameL		= new QLineEdit(Gparam) ;
	newLayerC		= new QCheckBox("Layer", Gparam) ; 
	newLayerL		= new QLineEdit(Gparam) ;
	newGroupIDC		= new QCheckBox("Group ID", Gparam) ; 
	newGroupIDL		= new QLineEdit(Gparam) ;
	newCustomIDC	= new QCheckBox("Custom ID", Gparam) ; 
	newCustomIDL	= new QLineEdit(Gparam) ;
	colorC			= new QCheckBox("Color", Gparam) ; 
	colorB			= new QPushButton("Color", Gparam) ;
	Lparam->addWidget(newNameC,		0, 0 ) ;
	Lparam->addWidget(newNameL,		0, 1 ) ;
	Lparam->addWidget(newCustomIDC,	1, 0 ) ;
	Lparam->addWidget(newCustomIDL,	1, 1 ) ;
	Lparam->addWidget(newGroupIDC,	2, 0 ) ;
	Lparam->addWidget(newGroupIDL,	2, 1 ) ;
	Lparam->addWidget(newLayerC,	3, 0 ) ;
	Lparam->addWidget(newLayerL,	3, 1 ) ;
	Lparam->addWidget(colorC,		4, 0 ) ;
	Lparam->addWidget(colorB,		4, 1 ) ;
	Lparam->addWidget(setActC,		5, 0 ) ;
	Lparam->addWidget(actC,			5, 1 ) ;
	Lparam->addWidget(Gmove,		6, 0, 2, 0) ;
	Gparam->setLayout(Lparam) ;
	////////////////////////////////////////////////////////////////////// CURVES BOUNDS
	setBoundsC		= new QCheckBox("Change Bounds", GcurvesBounds) ;
	rMinF			= new QLineEdit(GcurvesBounds) ;
	rMaxF			= new QLineEdit(GcurvesBounds) ;
	QLabel* minL	= new QLabel("Min", GcurvesBounds) ;
	QLabel* maxL	= new QLabel("Max", GcurvesBounds) ;
	cRescale		= new QCheckBox("Rescale Points to fit Bounds", GcurvesBounds) ;
	cRescaleG		= new QCheckBox("Keep Graphical Aspect", GcurvesBounds) ;
	cRescaleTrans	= new QCheckBox("Keep Position", GcurvesBounds) ;
	Lbounds->addWidget(minL ,		0, 0 ) ;
	Lbounds->addWidget(rMinF ,		0, 1 ) ;
	Lbounds->addWidget(maxL ,		1, 0 ) ;
	Lbounds->addWidget(rMaxF ,		1, 1 ) ;
	Gbounds->setLayout(Lbounds) ;
	Lcheck->addWidget( cRescale ) ;
	Lcheck->addWidget( cRescaleG ) ;
	Lcheck->addWidget( cRescaleTrans ) ;
	Gcheck->setLayout(Lcheck) ;
	LcurvesBounds->addWidget(setBoundsC) ;
	LcurvesBounds->addWidget(Gbounds) ;
	LcurvesBounds->addWidget(Gcheck) ;
	GcurvesBounds->setLayout(LcurvesBounds) ;
	////////////////////////////////////////////////////////////////////// CURVES
	setStyleC		= new QCheckBox("Style", Gcurves) ;
	setStyleQ		= new QComboBox(Gcurves) ;
	setStyleQ->addItem( "Lines" );
	setStyleQ->addItem( "Splines" );
	setStyleC->setChecked(false) ;
	setStyleQ->setEnabled(false) ;
	setAngleC		= new QCheckBox("Angle", Gcurves) ;
	shiftAngleC		= new QComboBox(Gcurves) ;
	shiftAngleC->addItem( "Set" );
	shiftAngleC->addItem( "Shift by" );
	newAngleF		= new QLineEdit("0", Gcurves) ;
	setScaleC	= new QCheckBox("Graphical Scale", Gcurves) ;
	shiftScaleC	= new QComboBox(Gcurves) ;
	shiftScaleC->addItem( "Set" );
	shiftScaleC->addItem( "Shift by" );
	newScaleF		= new QLineEdit("0", Gcurves) ;
	scanPeriodC		= new QCheckBox("Scan Period", Gcurves) ;
	shiftPeriodC	= new QComboBox(Gcurves) ;
	shiftPeriodC->addItem( "Set" );
	shiftPeriodC->addItem( "Shift by" );
	scanPeriodL		= new QSpinBox( Gcurves ) ;
	scanPeriodL->setMinimum( 20 ) ;
	scanPeriodL->setMaximum( 9999 ) ;
	flattenC		= new QCheckBox("Flatten Curve", Gcurves) ;
	Lcurves->addWidget(flattenC,		0, 0 ) ;
	Lcurves->addWidget(setStyleC,		1, 0 ) ;
	Lcurves->addWidget(setStyleQ,		1, 1 ) ;
	Lcurves->addWidget(setAngleC,		2, 0 ) ;
	Lcurves->addWidget(shiftAngleC,		2, 1 ) ;
	Lcurves->addWidget(newAngleF,		2, 2 ) ;
	Lcurves->addWidget(setScaleC,		3, 0 ) ;
	Lcurves->addWidget(shiftScaleC,		3, 1 ) ;
	Lcurves->addWidget(newScaleF,		3, 2 ) ;
	Lcurves->addWidget(scanPeriodC,		4, 0 ) ;
	Lcurves->addWidget(shiftPeriodC,	4, 1 ) ;
	Lcurves->addWidget(scanPeriodL,		4, 2 ) ;
	Gcurves->setLayout(Lcurves) ;
	////////////////////////////////////////////////////////////////////// TRIGGERS
	intValueC		= new QCheckBox("Int Value", Gtriggers) ; 
	intValueL		= new QLineEdit(Gtriggers) ;
	Ltriggers->addWidget(intValueC,	0, 0 ) ;
	Ltriggers->addWidget(intValueL,	0, 1 ) ;
	Gtriggers->setLayout(Ltriggers) ;
	////////////////////////////////////////////////////////////////////// TRAJECTS
	lengthC		= new QCheckBox("Length", Gtrajects) ; 
	lengthL		= new QLineEdit(Gtrajects) ;
	radiusC		= new QCheckBox("Radius", Gtrajects) ; 
	radiusL		= new QLineEdit(Gtrajects) ;
	Ltrajects->addWidget(lengthC,	0, 0 ) ;
	Ltrajects->addWidget(lengthL,	0, 1 ) ;
	Ltrajects->addWidget(radiusC,	1, 0 ) ;
	Ltrajects->addWidget(radiusL,	1, 1 ) ;
	Gtrajects->setLayout(Ltrajects) ;
	////////////////////////////////////////////////////////////////////// CURSORS
	initAngleC			= new QCheckBox("Initial Angle", Gcursors) ; 
	initAngleL			= new QLineEdit(Gcursors) ;
	finalAngleC			= new QCheckBox("Final Angle", Gcursors) ; 
	finalAngleL			= new QLineEdit(Gcursors) ;
	initOffsetC			= new QCheckBox("Initial Offset", Gcursors) ; 
	initOffsetL			= new QLineEdit(Gcursors) ;
	finalOffsetC		= new QCheckBox("Final Offset", Gcursors) ; 
	finalOffsetL		= new QLineEdit(Gcursors) ;
	speedC			= new QCheckBox("Speed", Gcursors) ; 
	speedL			= new QLineEdit(Gcursors) ;
	widthC			= new QCheckBox("Width", Gcursors) ; 
	widthL			= new QLineEdit(Gcursors) ;
	whenFinishedC	= new QCheckBox("When Finished", Gcursors) ; 
	playMuteC		= new QComboBox(Gcursors) ;
	playMuteC->addItem( "Play" );
	playMuteC->addItem( "Mute" );
	forwardC		= new QCheckBox("Playback", Gcursors) ; 
	forwardQ		= new QComboBox(Gcursors) ;
	forwardQ->addItem( "Forward" );
	forwardQ->addItem( "Backward" );
	modeC			= new QCheckBox("Mode", Gcursors) ; 
	modeQ			= new QComboBox(Gcursors) ;
	modeQ->addItem( "Single" );
	modeQ->addItem( "Repeat" );
	modeQ->addItem( "Fwd / Bckwd" );
	Lcursors->addWidget(initAngleC,		0, 0 ) ;
	Lcursors->addWidget(initAngleL,		0, 1 ) ;
	Lcursors->addWidget(finalAngleC,	1, 0 ) ;
	Lcursors->addWidget(finalAngleL,	1, 1 ) ;
	Lcursors->addWidget(initOffsetC,	2, 0 ) ;
	Lcursors->addWidget(initOffsetL,	2, 1 ) ;
	Lcursors->addWidget(finalOffsetC,	3, 0 ) ;
	Lcursors->addWidget(finalOffsetL,	3, 1 ) ;
	Lcursors->addWidget(speedC,			4, 0 ) ;
	Lcursors->addWidget(speedL,			4, 1 ) ;
	Lcursors->addWidget(widthC,			5, 0 ) ;
	Lcursors->addWidget(widthL,			5, 1 ) ;
	Lcursors->addWidget(whenFinishedC,	6, 0 ) ;
	Lcursors->addWidget(playMuteC,		6, 1 ) ;
	Lcursors->addWidget(forwardC,		7, 0 ) ;
	Lcursors->addWidget(forwardQ,		7, 1 ) ;
	Lcursors->addWidget(modeC,			8, 0 ) ;
	Lcursors->addWidget(modeQ,			8, 1 ) ;
	Gcursors->setLayout(Lcursors) ;
	////////////////////////////////////////////////////////////////////// OSC
	setOscFormatC	= new QCheckBox("Format", Gosc) ;
	setIPC			= new QCheckBox("Host IP", Gosc) ;
	setPortC		= new QCheckBox("UDP Port", Gosc) ;
	newOscFormatL	= new QLineEdit( Gosc ) ;
	newIPL			= new QLineEdit( Gosc ) ;
	newPortL		= new QLineEdit( Gosc ) ;
	Losc->addWidget( setOscFormatC,	0, 0 ) ;
	Losc->addWidget( newOscFormatL,	0, 1 ) ;
	Losc->addWidget( setIPC,		1, 0 ) ;
	Losc->addWidget( newIPL,		1, 1 ) ;
	Losc->addWidget( setPortC,		2, 0 ) ;
	Losc->addWidget( newPortL,		2, 1 ) ;
	Gosc->setLayout(Losc) ;
	////////////////////////////////////////////////////////////////////// BUTTONS
	QPushButton* apply	= new QPushButton("Apply", Gbutt) ;
	QPushButton* cancel	= new QPushButton("Cancel", Gbutt) ;
	Lbutt->addWidget(cancel) ;
	Lbutt->addWidget(apply) ;
	Gbutt->setLayout(Lbutt) ;

	//////////////////////////////////////////// LAYOUT
	Lleft->addWidget(Gchoice);
	Lleft->addWidget(Gparam);
	Lcenter->addWidget(GcurvesBounds);
	Lcenter->addWidget(Gtrajects);
	Lcenter->addWidget(Gtriggers);
	Lcenter->addWidget(Gosc);
	Lright->addWidget(Gcurves);
	Lright->addWidget(Gcursors);
	Lsettings->addLayout(Lleft) ;
	Lsettings->addLayout(Lcenter) ;
	Lsettings->addLayout(Lright) ;
	Gsettings->setLayout(Lsettings) ;
	Lgen->addWidget(Gsettings) ;
	Lgen->addWidget(Gbutt) ;
	this->setLayout(Lgen) ;
	
	apply->setShortcut(Qt::Key_Return) ;
	
	connect(applyOnGroupIDC, SIGNAL(currentIndexChanged(int)), this,	SLOT(setEditByGroupEnabled(int)));
	connect(applyOnLayerC, SIGNAL(currentIndexChanged(int)), this,		SLOT(setEditByLayerEnabled(int)));
	
	connect( curvC, SIGNAL( toggled(bool) ),	this,		SLOT( setEditGroupsEnabled(bool) ) );
	connect( trigC, SIGNAL( toggled(bool) ),	this,		SLOT( setEditGroupsEnabled(bool) ) );
	connect( trajC, SIGNAL( toggled(bool) ),	this,		SLOT( setEditGroupsEnabled(bool) ) );
	connect( cursC, SIGNAL( toggled(bool) ),	this,		SLOT( setEditGroupsEnabled(bool) ) );
	
	connect( newNameC, SIGNAL( toggled(bool) ),		newNameL,			SLOT( setEnabled(bool) ) );
	connect( newGroupIDC, SIGNAL( toggled(bool) ),	newGroupIDL,		SLOT( setEnabled(bool) ) );
	connect( newLayerC, SIGNAL( toggled(bool) ),	newLayerL,			SLOT( setEnabled(bool) ) );
	connect( newCustomIDC, SIGNAL( toggled(bool) ),	newCustomIDL,		SLOT( setEnabled(bool) ) );
	connect( setActC, SIGNAL( toggled(bool) ),		actC,				SLOT( setEnabled(bool) ) );
	
	connect( xtC, SIGNAL( toggled(bool) ),		xtF,			SLOT( setEnabled(bool) ) );
	connect( ytC, SIGNAL( toggled(bool) ),		ytF,			SLOT( setEnabled(bool) ) );
	connect( ztC, SIGNAL( toggled(bool) ),		ztF,			SLOT( setEnabled(bool) ) );
	connect( xtC, SIGNAL( toggled(bool) ),		shiftXC,		SLOT( setEnabled(bool) ) );
	connect( ytC, SIGNAL( toggled(bool) ),		shiftYC,		SLOT( setEnabled(bool) ) );
	connect( ztC, SIGNAL( toggled(bool) ),		shiftZC,		SLOT( setEnabled(bool) ) );
	
	connect( setStyleC, SIGNAL( toggled(bool) ),	setStyleQ,			SLOT( setEnabled(bool) ) );
	connect( setAngleC, SIGNAL( toggled(bool) ),	shiftAngleC,	SLOT( setEnabled(bool) ) );
	connect( setAngleC, SIGNAL( toggled(bool) ),	newAngleF,		SLOT( setEnabled(bool) ) );
	connect( scanPeriodC, SIGNAL( toggled(bool) ),	shiftPeriodC,	SLOT( setEnabled(bool) ) );
	connect( scanPeriodC, SIGNAL( toggled(bool) ),	scanPeriodL,	SLOT( setEnabled(bool) ) );
	connect( setScaleC, SIGNAL( toggled(bool) ),	shiftScaleC,	SLOT( setEnabled(bool) ) );
	connect( setScaleC, SIGNAL( toggled(bool) ),	newScaleF,		SLOT( setEnabled(bool) ) );
	
	connect( flattenC,	SIGNAL( toggled(bool) ),		this,			SLOT( setEditBoundsEnabled(bool) ) );
	connect( setBoundsC,	SIGNAL( toggled(bool) ),	this,			SLOT( setEditFlattenEnabled(bool) ) );
	connect( setBoundsC,	SIGNAL( toggled(bool) ),	Gcheck,		SLOT( setEnabled(bool) ) );
	connect( setBoundsC,	SIGNAL( toggled(bool) ),	Gbounds,	SLOT( setEnabled(bool) ) );
	connect( cRescale,	SIGNAL( toggled(bool) ), cRescaleG,			SLOT( setEnabled(bool) ) );
	connect( cRescale,	SIGNAL( toggled(bool) ), cRescaleG,			SLOT( setChecked(bool) ) );
	connect( cRescaleG,	SIGNAL( toggled(bool) ), cRescaleTrans,		SLOT( setEnabled(bool) ) );

	connect( intValueC, SIGNAL( toggled(bool) ),	intValueL,		SLOT( setEnabled(bool) ) );
	connect( lengthC, SIGNAL( toggled(bool) ),		lengthL,		SLOT( setEnabled(bool) ) );
	connect( radiusC, SIGNAL( toggled(bool) ),		radiusL,		SLOT( setEnabled(bool) ) );
	
	connect( lengthC, SIGNAL( toggled(bool) ),		this,		SLOT( setEditLengthEnabled(bool) ) );
	connect( radiusC, SIGNAL( toggled(bool) ),		this,		SLOT( setEditRadiusEnabled(bool) ) );
	
	connect( initAngleC, SIGNAL( toggled(bool) ),	initAngleL,				SLOT( setEnabled(bool) ) );
	connect( finalAngleC, SIGNAL( toggled(bool) ),	finalAngleL,			SLOT( setEnabled(bool) ) );
	connect( initOffsetC, SIGNAL( toggled(bool) ),	initOffsetL,			SLOT( setEnabled(bool) ) );
	connect( finalOffsetC, SIGNAL( toggled(bool) ),	finalOffsetL,			SLOT( setEnabled(bool) ) );
	connect( speedC, SIGNAL( toggled(bool) ),		speedL,				SLOT( setEnabled(bool) ) );
	connect( widthC, SIGNAL( toggled(bool) ),		widthL,				SLOT( setEnabled(bool) ) );
	connect( whenFinishedC, SIGNAL( toggled(bool) ),	playMuteC,	SLOT( setEnabled(bool) ) );
	connect( forwardC, SIGNAL( toggled(bool) ),	forwardQ,			SLOT( setEnabled(bool) ) );
	connect( modeC, SIGNAL( toggled(bool) ),	modeQ,				SLOT( setEnabled(bool) ) );
	
	connect( setOscFormatC, SIGNAL( toggled(bool) ),	newOscFormatL,	SLOT( setEnabled(bool) ) ) ;
	connect( setIPC, SIGNAL( toggled(bool) ),	newIPL,				SLOT( setEnabled(bool) ) ) ;
	connect( setPortC, SIGNAL( toggled(bool) ),	newPortL,			SLOT( setEnabled(bool) ) ) ;
	
	connect( colorC, SIGNAL( toggled(bool) ),	colorB,		SLOT( setEnabled(bool) ) );
	connect( colorB, SIGNAL( clicked() ),		this,		SLOT( pickObjectColor() ) ) ;
	
	connect( cancel, SIGNAL( clicked() ),		this,	SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ),		this,	SLOT( applyChanges() ) );
}
void NxObjectEditor::pickObjectColor() 
{
	objectPickedColor.c = colDialogPicker->getColor();	
	if( !(objectPickedColor.c.red()==0 && objectPickedColor.c.green()==0 && objectPickedColor.c.blue()==0) ) 
		dialogColorPicked = true ;
	else {
		dialogColorPicked = false ;
		printf("Color wasn't Choosed\n") ;
	}
}
void NxObjectEditor::setEditLengthEnabled( bool flag )
{
	if(circleOnlySelected && flag) radiusC->setChecked( false ) ;
}
void NxObjectEditor::setEditRadiusEnabled( bool flag )
{
	if(circleOnlySelected && flag) lengthC->setChecked( false ) ;
}
void NxObjectEditor::setEditBoundsEnabled( bool flag )
{
	if(flag) {
		setBoundsC->setChecked(false) ;
		setScaleC->setChecked(false) ;
		GcurvesBounds->setEnabled(false) ;
		setScaleC->setEnabled(false) ;
	}
	else {
		GcurvesBounds->setEnabled(true) ;
		setScaleC->setEnabled(true) ;	
	}
}
void NxObjectEditor::setEditFlattenEnabled( bool flag )
{
	if(flag) flattenC->setChecked(false) ;
}
void NxObjectEditor::setEditByGroupEnabled( int flag ) 
{
	bool enable = (flag==0) ;
	fGroupF->setVisible( enable ) ;
}
void NxObjectEditor::setEditByLayerEnabled( int flag ) 
{
	bool enable = (flag==0) ;
	fLayerF->setVisible( enable ) ;
}

/////////////////////////////////////////////////////////////////
// SET FIELDS VALUES
/////////////////////////////////////////////////////////////////
void NxObjectEditor::setDefaultValues() // default values
{
	NxEnv env = parentGLBox->getPartition()->getEnvironment() ;
	
	applyOnGroupIDC->setCurrentIndex(1);	// 1 = All GID
	applyOnLayerC->setCurrentIndex(1);		// 1 = All layers
	fGroupF->setVisible( false ) ;
	fLayerF->setVisible( false ) ;
	fGroupF->setText("0") ;
	fLayerF->setText("0") ;
	//////////////////////////// General Parameters
	newNameL->setText("NewName") ;
	newLayerL->setText("0") ;
	newGroupIDL->setText("0") ;	
	newCustomIDL->setText("0") ;
	shiftXC->setCurrentIndex(0);
	shiftYC->setCurrentIndex(0);
	shiftZC->setCurrentIndex(0);
	xtF->setText("0") ;
	ytF->setText("0") ;
	ztF->setText("0") ;
	//////////////////////////// Curves
	setStyleQ->setCurrentIndex(0); // 0 = Lines
	shiftAngleC->setCurrentIndex(0); // 0 = set angle
	shiftScaleC->setCurrentIndex(0); // 0 = set scale
	shiftPeriodC->setCurrentIndex(0); // 0 = set scan period
	scanPeriodL->setValue(env.defaultSpatialPeriod) ;
	newScaleF->setText("1") ;
	newAngleF->setText("0") ;
	rMinF->setText(QString("%1").arg( env.mappingDefaultMin )) ;
	rMaxF->setText(QString("%1").arg( env.mappingDefaultMax )) ;
	//////////////////////////// Triggers + trajects
	intValueL->setText(QString("%1").arg(env.OSCTriggersDefaultIntValue)) ;
	lengthL->setText("") ;
	radiusL->setText("") ;
	//////////////////////////// Cursors
	initAngleL->setText("") ;
	finalAngleL->setText("") ;
	initOffsetL->setText("") ;
	finalOffsetL->setText("") ;
	speedL->setText("") ;
	widthL->setText("") ;
	playMuteC->setCurrentIndex(0); // 0 = mute
	forwardQ->setCurrentIndex(0); // 0 = forward
	modeQ->setCurrentIndex(0); // 0 = single
	//////////////////////////// OSC
	newOscFormatL->setText( "/iannix/" ) ;
	newIPL->setText( env.UDPhost ) ;
	newPortL->setText( QString("%1").arg( env.UDPout) ) ;
}
void NxObjectEditor::setObjectValues( NxObject* inObject ) // default values
{
	//////////////////////////// General Parameters
	newNameL->setText( inObject->getName() ) ;
	newLayerL->setText( QString().setNum(inObject->getLayer()) ) ;
	newGroupIDL->setText( QString().setNum(inObject->getGroupID()) ) ;	
	newCustomIDL->setText( QString().setNum(inObject->getCustomID()) ) ;
	xtF->setText( QString().setNum(inObject->getPosition().x) ) ;
	ytF->setText( QString().setNum(inObject->getPosition().y) ) ;
	ztF->setText( QString().setNum(inObject->getPosition().z) ) ;
	switch( inObject->getObjectType() )
	{
		case Curve: {
			NxCurve* curve = (NxCurve*)inObject ;
			setStyleQ->setCurrentIndex( 8 - curve->getDrawType() ); // 0 = Lines
			shiftAngleC->setCurrentIndex(0); // 0 = set angle
			shiftPeriodC->setCurrentIndex(0); // 0 = set scan period
			scanPeriodL->setValue( curve->getSpatialPeriod() ) ;
			newScaleF->setText( QString().setNum(curve->getScale()) ) ;
			newAngleF->setText( QString().setNum(curve->getAngle()) ) ;
			rMinF->setText( QString().setNum(curve->getRealMin()) ) ;
			rMaxF->setText( QString().setNum(curve->getRealMax()) ) ;
			newOscFormatL->setText( curve->getOscFormat() ) ;
			newIPL->setText( curve->getHost() ) ;
			newPortL->setText( QString().setNum(curve->getPort())  ) ;
			}
		break;
		case Trigger: {
			NxTrigger* trigger = (NxTrigger*)inObject ;
			intValueL->setText( QString().setNum(trigger->getValue()) ) ;
			newOscFormatL->setText( trigger->getOscFormat() ) ;
			newIPL->setText( trigger->getHost() ) ;
			newPortL->setText( QString().setNum(trigger->getPort()) ) ;
			}
		break;
		case 2:
			NxCursor* cursor = (NxCursor*)inObject ;
			if(cursor->getType()==1) { // CIRCLE
				initAngleL->setText( QString().setNum(cursor->getInitAngle()) ) ;
				finalAngleL->setText( QString().setNum(cursor->getFinalAngle()) ) ;
				initOffsetL->setText("") ;
				finalOffsetL->setText("") ;
			}
			else { // LINE
				initOffsetL->setText( QString().setNum(cursor->getInitOffset()) ) ;
				finalOffsetL->setText( QString().setNum(cursor->getFinalOffset()) ) ;
				initAngleL->setText("") ;
				finalAngleL->setText("") ;
			}
			speedL->setText( QString().setNum(cursor->getSpeed()) ) ;
			widthL->setText( QString().setNum(cursor->getWidth()) ) ;
			if(cursor->getPlayFinished())
				playMuteC->setCurrentIndex(1) ;
			else
				playMuteC->setCurrentIndex(0) ;
			if(cursor->getForward())
				forwardQ->setCurrentIndex(0) ;
			else
				forwardQ->setCurrentIndex(1) ;				
			modeQ->setCurrentIndex( cursor->getMode() ); // 0 = single
		break;
		case Trajectory:
			NxTraject* traject = (NxTraject*)inObject ;
			lengthL->setText( QString().setNum(traject->getLength()) ) ;
			if(traject->getType()==1) // CIRCLE
				radiusL->setText( QString().setNum(traject->getRadius()) ) ;
		break;
	}
}
void NxObjectEditor::setChecked() // de-check all
{
	circleOnlySelected = false ;
	
	radiusC->setEnabled( true ) ;
	initAngleC->setEnabled( true ) ;
	finalAngleC->setEnabled( true ) ;
	initOffsetC->setEnabled( true ) ;
	finalOffsetC->setEnabled( true ) ;
	
	trajC->setChecked( true ) ;
	cursC->setChecked( true ) ;
	curvC->setChecked( true ) ;
	trigC->setChecked( true ) ;
	//////////////////////////// GroupBoxes
	Gtype->setEnabled(true) ;
	Gosc->setEnabled(true);
	Gcursors->setEnabled(true);
	Gtriggers->setEnabled(true);
	Gcurves->setEnabled(true);
	Gchoice->setEnabled(true);
	Gmove->setEnabled(true) ;
	//////////////////////////// General Parameters
	actC->setCurrentIndex(0);
	actC->setEnabled(false) ;
	setActC->setChecked( false ) ;
	newNameC->setChecked( false ) ;
	newGroupIDC->setChecked( false ) ;
	newLayerC->setChecked( false ) ;
	newCustomIDC->setChecked( false ) ;
	newNameL->setEnabled( false ) ;
	newGroupIDL->setEnabled( false ) ;
	newLayerL->setEnabled( false ) ;
	newCustomIDL->setEnabled( false ) ;
	xtC->setChecked(false) ;
	ytC->setChecked(false) ;
	ztC->setChecked(false) ;
	shiftXC->setEnabled(false) ;
	shiftYC->setEnabled(false) ;
	shiftZC->setEnabled(false) ;
	xtF->setEnabled(false) ;
	ytF->setEnabled(false) ;
	ztF->setEnabled(false) ;
	colorC->setChecked( false ) ;
	colorB->setEnabled( false ) ;
	//////////////////////////// Curves
	setStyleC->setChecked(false) ;
	setStyleQ->setEnabled(false) ;
	setAngleC->setChecked(false) ;
	shiftAngleC->setEnabled(false) ;
	newAngleF->setEnabled(false) ;
	setScaleC->setChecked(false) ;
	shiftScaleC->setEnabled(false) ;
	newScaleF->setEnabled(false) ;
	scanPeriodC->setChecked(false) ;
	shiftPeriodC->setEnabled(false) ;
	scanPeriodL->setEnabled(false) ;
	flattenC->setChecked( false ) ;
	setBoundsC->setChecked(false) ;
	cRescaleG->setChecked(false) ;
	cRescaleTrans->setChecked(false) ;
	Gbounds->setEnabled(false) ;
	Gcheck->setEnabled(false) ;
	//////////////////////////// Triggers + Trajects
	intValueC->setChecked( false ) ;
	intValueL->setEnabled (false ) ;
	lengthC->setChecked( false ) ;
	lengthL->setEnabled (false ) ;
	radiusC->setChecked( false ) ;
	radiusL->setEnabled (false ) ;
	//////////////////////////// Cursors
	initAngleC->setChecked( false ) ;
	finalAngleC->setChecked( false ) ;
	initOffsetC->setChecked( false ) ;
	finalOffsetC->setChecked( false ) ;
	speedC->setChecked( false ) ;
	widthC->setChecked( false ) ;
	whenFinishedC->setChecked( false ) ;
	forwardC->setChecked( false ) ;
	modeC->setChecked( false ) ;
	initOffsetL->setEnabled (false ) ;
	finalOffsetL->setEnabled (false ) ;
	speedL->setEnabled (false ) ;
	widthL->setEnabled (false ) ;
	playMuteC->setEnabled (false ) ;
	forwardQ->setEnabled (false ) ;
	modeQ->setEnabled (false ) ;
	//////////////////////////// OSC
	setOscFormatC->setChecked( false ) ;
	setIPC->setChecked( false ) ;
	setPortC->setChecked( false ) ;
	newOscFormatL->setEnabled (false ) ;
	newIPL->setEnabled (false ) ;
	newPortL->setEnabled (false ) ;
}
/////////////////////////////////////////////////////////////////
// RAISE EDITORS
/////////////////////////////////////////////////////////////////
void NxObjectEditor::closeEvent( QCloseEvent* )
{
	releaseKeyboard();
}
void NxObjectEditor::showEditGroup()
{
	setWindowTitle( tr( "IanniX Group Editor" ) );
	setDefaultValues() ;
	setChecked() ;
	setDefaultColors() ;
	dialogColorPicked = false ;
	//////////////////////////////////////////// SETS OBJECTS TO MODIFY
	applyAll = false ; // group/layer filtering
	objectList.clear() ;
	NxScore *score = parentGLBox->getPartition() ;
	QList<int> curvInd = score->curvesIndexes();
	QList<int> cursInd = score->cursorsIndexes();
	QList<int> trigInd = score->triggersIndexes();
	QList<int> trajInd = score->trajectsIndexes();
	for (int i=0; i<curvInd.count(); i++)
		objectList.append( score->getCurve( curvInd.at(i) ) ) ;
	for (int i=0; i<cursInd.count(); i++)
		objectList.append( score->getCursor( cursInd.at(i) ) ) ;
	for (int i=0; i<trigInd.count(); i++)
		objectList.append( score->getTrigger( trigInd.at(i) ) ) ;
	for (int i=0; i<trajInd.count(); i++)
		objectList.append( score->getTraject( trajInd.at(i) ) ) ;
	
	show() ;
	setFocus();
	activateWindow() ;
	showNormal() ;
}
void NxObjectEditor::showEditSelection( QList<NxObject*> inSelected )
{
	setWindowTitle( QString("IanniX Selection Editor - %1 Objects").arg(inSelected.size()) );
	setDefaultValues() ;
	setChecked() ;
	setDefaultColors() ;
	dialogColorPicked = false ;
	//////////////////////////////////////////// SETS OBJECTS TO MODIFY
	applyAll = true ; // No group/layer filtering
	objectList.clear() ;
	objectList = inSelected ;
	
	show() ;
	setFocus();
	activateWindow() ;
	showNormal() ;
}
void NxObjectEditor::showEditObject( NxObject* inObject )
{
	setWindowTitle( tr( "IanniX Object Editor" ) );
	setDefaultValues() ;
	setObjectValues( inObject ) ;
	setChecked() ;
	dialogColorPicked = false ;
	//////////////////////////////////////////// DISABLE Groupboxes
	Gosc->setEnabled(false);
	Gcursors->setEnabled(false);
	Gtriggers->setEnabled(false);
	Gcurves->setEnabled(false);
	Gtrajects->setEnabled(false);
	Gchoice->setEnabled(false);
	Gchoice->setPalette(cGray);
	trajC->setChecked(false) ;
	cursC->setChecked(false) ;
	curvC->setChecked(false) ;
	trigC->setChecked(false) ;
	////////////////////////////////// Cursor Init/Final
	initAngleC->setEnabled( false ) ;
	finalAngleC->setEnabled( false ) ;
	initOffsetC->setEnabled( false ) ;
	finalOffsetC->setEnabled( false ) ;
	switch( inObject->getObjectType() )
	{
		case Curve:
			curvC->setChecked(true) ;
			Gcurves->setEnabled(true);
			Gosc->setEnabled(true);
		break;
		case 2:
			cursC->setChecked(true) ;
			Gcursors->setEnabled(true);
			if( ((NxCursor*)inObject)->getType()==1 ) { // CIRCLE
				initAngleC->setEnabled(true) ;
				finalAngleC->setEnabled(true) ;
			}
			else { // LINE
				initOffsetC->setEnabled(true) ;
				finalOffsetC->setEnabled(true) ;
			}
		break;
		case Trigger:
			trigC->setChecked(true) ;
			Gtriggers->setEnabled(true);
			Gosc->setEnabled(true);
		break;
		case Trajectory:
			trajC->setChecked(true) ;
			Gtrajects->setEnabled(true);
			if(((NxTraject*)inObject)->getType()==2) // LINE
				radiusC->setEnabled(false) ;
			else
				circleOnlySelected = true ;
		break;
	}
	setEditGroupsEnabled(true) ;
	/////////////////////////////////////////////////// SETS OBJECTS TO MODIFY
	applyAll = true ; // No group/layer filtering
	objectList.clear() ;
	objectList.append( inObject ) ;
	
	show() ;
	setFocus();
	activateWindow() ;
	showNormal() ;
}

/////////////////////////////////////////////////////////////////
// APPLY CHANGES
/////////////////////////////////////////////////////////////////
void NxObjectEditor::applyChanges()
{
	setEdited( objectList ) ;
	accept() ;
}
void NxObjectEditor::setEdited( QList<NxObject*> inList ) // APPLY CHANGES
{
	////////////////////////////////////////////////////////////////// APPLY ON
	bool allGroups = (applyOnGroupIDC->currentIndex()==1 || applyAll) ;
	bool allLayers = (applyOnLayerC->currentIndex()==1 || applyAll) ;
	int gpeID			= fGroupF->text().toInt() ;		// apply on group ID
	int layer			= fLayerF->text().toInt() ;		// apply on Layer
	////////////////////////////////////////////////////////////////// PARAMETERS
	bool setAct			= setActC->isChecked() ;
	bool setNewName		= newNameC->isChecked() ;
	bool setNewLayer	= newLayerC->isChecked() ;
	bool setNewGroupID	= newGroupIDC->isChecked() ;
	bool setNewCustomID	= newCustomIDC->isChecked() ;
	bool newAct			= (actC->currentIndex()==0)  ;
	QString newName		= newNameL->text() ;
	int newLayer		= newLayerL->text().toInt() ;
	int newGroupID		= newGroupIDL->text().toInt() ;
	int newCustomID		= newCustomIDL->text().toInt() ;
	////////////////////////////////////////////////////////////////// MOVE
	bool shiftX		= (shiftXC->currentIndex()==1)  ;
	bool shiftY		= (shiftYC->currentIndex()==1)  ;
	bool shiftZ		= (shiftZC->currentIndex()==1)  ;
	bool setX		= xtC->isChecked() ;
	bool setY		= ytC->isChecked() ;
	bool setZ		= ztC->isChecked() ;
	float tx	= xtF->text().toFloat() ;
	float ty	= ytF->text().toFloat() ;
	float tz	= ztF->text().toFloat() ;
	////////////////////////////////////////////////////////////////// CURVES	
	bool flatC				= flattenC->isChecked() ;
	bool setStyle			= setStyleC->isChecked() ;
	bool setAngle			= setAngleC->isChecked() ;
	bool setScanPeriod		= scanPeriodC->isChecked() ;
	bool setScale			= setScaleC->isChecked() ;
	bool shiftAngle			= (shiftAngleC->currentIndex()==1) ;	// set/shift angle ?
	bool shiftPeriod		= (shiftPeriodC->currentIndex()==1) ;	// set/shift scan period ?
	bool shiftScale			= (shiftScaleC->currentIndex()==1) ;	// set/shift scan period ?
	int newStyle			= 8 - setStyleQ->currentIndex() ;
	float newAngle			= newAngleF->text().toFloat() ;
	float newScale			= newScaleF->text().toFloat() ;
	int newScanPeriod		= scanPeriodL->value() ;
	////////////////////////////////////////////////////////////////// CURVES BOUNDS
	bool changeBounds		= setBoundsC->isChecked() ;
	bool rescaleAll			= cRescale->isChecked() ;
	bool rescaleGAll		= cRescaleG->isChecked() ;
	bool rescaleTransAll	= cRescaleTrans->isChecked() ;
	float newRMin			= rMinF->text().toFloat() ;
	float newRMax			= rMaxF->text().toFloat() ;
	////////////////////////////////////////////////////////////////// TRIGGERS + TRAJECTS
	bool setIntValue	= intValueC->isChecked() ;
	int newIntValue		= intValueL->text().toInt() ;
	bool setLength		= lengthC->isChecked() ;
	float newLength		= lengthL->text().toFloat() ;
	bool setRadius		= radiusC->isChecked() ;
	float newRadius		= radiusL->text().toFloat() ;
	////////////////////////////////////////////////////////////////// CURSORS
	bool setInitAngle		= initAngleC->isChecked() ;
	bool setFinalAngle		= finalAngleC->isChecked() ;	
	bool setInitOffset		= initOffsetC->isChecked() ;
	bool setFinalOffset		= finalOffsetC->isChecked() ;
	bool setSpeed		= speedC->isChecked() ;
	bool setWidth		= widthC->isChecked() ;
	bool setFinished	= whenFinishedC->isChecked() ;
	bool setForward		= forwardC->isChecked() ;
	bool setMode		= modeC->isChecked() ;
	float newInitAngle		= initAngleL->text().toFloat() ;
	float newFinalAngle		= finalAngleL->text().toFloat() ;
	float newInitOffset		= initOffsetL->text().toFloat() ;
	float newFinalOffset	= finalOffsetL->text().toFloat() ;
	float newSpeed		= speedL->text().toFloat() ;
	float newWidth		= widthL->text().toFloat() ;
	bool playFinished	= (playMuteC->currentIndex()==0) ;
	bool newForward		= (forwardQ->currentIndex()==0) ;
	int newMode			= modeQ->currentIndex() ;
	////////////////////////////////////////////////////////////////// OSC
	bool setNewOscFormat	= setOscFormatC->isChecked() ;
	bool setNewIP			= setIPC->isChecked() ;
	bool setNewPort			= setPortC->isChecked() ;
	QString newOscFormat	= newOscFormatL->text() ;
	QString newIP			= newIPL->text() ;
	int newPort				= newPortL->text().toInt() ;
	////////////////////////////////////////////////////////////////// COLOR
	bool setColor		= colorC->isChecked() ;
	
	NxObject	*obj ;
	NxTraject	*traj ;
	NxCursor	*curs ;
	NxCurve		*curv ;
	NxTrigger	*trigg ;
	
	NxPoint oldP ;
	
	for(int o=0;o<inList.size();o++)
	{
		obj = inList.at(o) ;
		switch( ((NxObject*)obj)->getObjectType() )
		{
			case Curve: { ///////////////////////////////////// CURVES
				if(curvC->isChecked()) {
					curv = (NxCurve*)obj ;
					if( (allGroups || curv->getGroupID() == gpeID) && (allLayers || curv->getLayer() == layer) ) {
						oldP = curv->getPosition() ;
						if(setX) if(shiftX) oldP.x += tx ; else oldP.x = tx ;
						if(setY) if(shiftY) oldP.y += ty ; else oldP.y = ty ;
						if(setZ) if(shiftZ) oldP.z += tz ; else oldP.z = tz ;
						curv->setPosition( oldP ) ;
						/////// ANGLES
						if(setAngle) {
							if(shiftAngle)
								curv->setAngle( curv->getAngle() + newAngle ) ;
							else
								curv->setAngle( newAngle ) ;
							curv->updateBB( parentGLBox->getBBsize() ) ;
						}
						/////// GRAPHICAL SCALE
						if(setScale) {
							if(shiftScale)
								curv->setScale( curv->getScale() + newScale ) ;
							else 
								curv->setScale( newScale ) ;
						}
						/////// SCAN PERIOD
						if(setScanPeriod) {
							if(shiftPeriod) {
								float newS = curv->getSpatialPeriod() + newScanPeriod ;
								if(newS>=20)
									curv->setSpatialPeriod( newS ) ;
								else
									curv->setSpatialPeriod( 20 ) ;
							}
							else 
								curv->setSpatialPeriod( newScanPeriod ) ;
						}
						if(setAct)							curv->setActive(newAct) ;				
						if(setNewName)						curv->setName(newName) ;
						if(setNewLayer)						curv->setLayer(newLayer) ;
						if(setNewGroupID)					curv->setGroupID(newGroupID) ;
						if(setNewCustomID)					curv->setCustomID(newCustomID) ;
						if(setColor && dialogColorPicked)	curv->setColor( objectPickedColor ) ;
						if(setNewOscFormat)					curv->setOscFormat( newOscFormat ) ;
						if(setNewIP)						curv->setHost( newIP ) ;
						if(setNewPort)						curv->setPort( newPort ) ;
						if(setStyle)						curv->setDrawType( newStyle ) ;
						if(changeBounds)			
							parentGLBox->changeCurveBounds( curv, newRMin, newRMax, rescaleAll, rescaleGAll, rescaleTransAll ) ;
						if(flatC)
							parentGLBox->flattenCurve( curv ) ;
						curv->updateBB( parentGLBox->getBBsize() ) ;
						parentGLBox->getControlCenter()->updateParameters( Curve, curv->getID(), false ) ;
					}
				}
			}
			break ;
			case Trajectory: { ///////////////////////////////////// TRAJECTS
				if(trajC->isChecked()) {
					traj = (NxTraject*)obj ;
					if( (allGroups || traj->getGroupID() == gpeID) && (allLayers || traj->getLayer() == layer) ) {
						oldP = traj->getPosition() ;
						if(setX) if(shiftX) oldP.x += tx ; else oldP.x = tx ;
						if(setY) if(shiftY) oldP.y += ty ; else oldP.y = ty ;
						if(setZ) if(shiftZ) oldP.z += tz ; else oldP.z = tz ;
						traj->setPosition( oldP ) ;
						if(setLength)						traj->setLength( newLength ) ;
						if(setRadius)						traj->setRadius( newRadius ) ;
						traj->updateBB( parentGLBox->getBBsize() ) ;
						if(setAct)							{ traj->setActive(newAct) ; parentGLBox->setCursorsActive(traj,newAct) ; }
						if(setNewName)						traj->setName(newName) ;
						if(setNewLayer)						traj->setLayer(newLayer) ;
						if(setNewGroupID)					traj->setGroupID(newGroupID) ;
						if(setNewCustomID)					traj->setCustomID(newCustomID) ;
						if(setColor && dialogColorPicked)	traj->setColor( objectPickedColor ) ;
						parentGLBox->getControlCenter()->updateParameters( Trajectory, traj->getID(), false ) ;
						parentGLBox->updateCursorsAnglePosition(traj) ;
					}
				}
			}
			break ;
			case Trigger: { ///////////////////////////////////// TRIGGERS
				if(trigC->isChecked()) {
					trigg = (NxTrigger*)obj ;
					if( (allGroups || trigg->getGroupID() == gpeID) && (allLayers || trigg->getLayer() == layer) ) {
						oldP = trigg->getPosition() ;
						if(setX) if(shiftX) oldP.x += tx ; else oldP.x = tx ;
						if(setY) if(shiftY) oldP.y += ty ; else oldP.y = ty ;
						if(setZ) if(shiftZ) oldP.z += tz ; else oldP.z = tz ;
						trigg->setPosition( oldP ) ;
						trigg->updateBB( parentGLBox->getBBsize() ) ;
						if(setAct)							trigg->setActive(newAct) ;
						if(setNewName)						trigg->setName(newName) ; 
						if(setNewLayer)						trigg->setLayer(newLayer) ;
						if(setNewGroupID)					trigg->setGroupID(newGroupID) ;
						if(setNewCustomID)					trigg->setCustomID(newCustomID) ;
						if(setColor && dialogColorPicked)	trigg->setColor( objectPickedColor ) ;
						if(setNewOscFormat)					trigg->setOscFormat( newOscFormat ) ;
						if(setNewIP)						trigg->setHost( newIP ) ;
						if(setNewPort)						trigg->setPort( newPort ) ;
						if(setIntValue)						trigg->setValue( newIntValue ) ;
						parentGLBox->getControlCenter()->updateParameters( Trigger, trigg->getID(), false ) ;
					}
				}
			}
			break ;
			case 2: { ///////////////////////////////////// CURSORS
				if(cursC->isChecked()) {
					curs = (NxCursor*)obj ;
					if( (allGroups || curs->getGroupID() == gpeID) && (allLayers || curs->getLayer() == layer) ) {
						if(setAct)							curs->setActive(newAct) ;
						if(setNewName)						curs->setName(newName) ;
						if(setNewLayer)						curs->setLayer(newLayer) ;
						if(setNewGroupID)					curs->setGroupID(newGroupID) ;
						if(setNewCustomID)					curs->setCustomID(newCustomID) ;
						if(setColor && dialogColorPicked)	curs->setColor(objectPickedColor) ;
						if(setSpeed)						curs->setSpeed(newSpeed) ;
						if(setWidth)						curs->setWidth(newWidth) ;
						if(setFinished)						curs->setPlayFinished(playFinished) ;
						if(setForward)						curs->setForward(newForward) ;
						if(setMode)							curs->setMode(newMode) ;
						if(curs->getType()==2) { ////////// LINE
							if(setInitOffset)				curs->setInitOffset(newInitOffset) ;
							if(setFinalOffset)				curs->setFinalOffset(newFinalOffset) ;
							curs->reEvalOffsets() ;
						}
						else { ////////// CIRCLE
							if(setInitAngle)				curs->setInitAngle(newInitAngle) ;
							if(setFinalAngle)				curs->setFinalAngle(newFinalAngle) ;
							curs->reEvalAngles() ;
						}
						
						parentGLBox->getControlCenter()->updateParameters( 2, curs->getID(), false ) ;
					}
				}
			}
			break ;
		}
	}
	if(setNewLayer)		parentGLBox->updateLayerFilterList() ;
	if(setNewGroupID)	parentGLBox->updateGroupFilterList() ;
	parentGLBox->getControlCenter()->resizeTables() ;
	parentGLBox->assignCursors() ;
}
void NxObjectEditor::setEditGroupsEnabled( bool ) 
{
	Gosc->setEnabled( curvC->isChecked() || trigC->isChecked() ) ;
	Gcurves->setEnabled( curvC->isChecked() ) ;
	GcurvesBounds->setEnabled( curvC->isChecked() ) ;
	Gtriggers->setEnabled( trigC->isChecked() ) ;
	Gtrajects->setEnabled( trajC->isChecked() ) ;
	Gcursors->setEnabled( cursC->isChecked() ) ;
	Gmove->setEnabled( curvC->isChecked() || trajC->isChecked() || trigC->isChecked() ) ;
		
	if( Gosc->isEnabled() )	Gosc->setPalette(cDark);
	else					Gosc->setPalette(cGray);
	
	if( curvC->isChecked()) {Gcurves->setPalette(cLight) ; GcurvesBounds->setPalette(cLight) ;}
	else					{Gcurves->setPalette(cGray) ; GcurvesBounds->setPalette(cGray) ;}

	if( trigC->isChecked())	Gtriggers->setPalette(cLight) ;
	else					Gtriggers->setPalette(cGray) ;
							
	if( trajC->isChecked())	Gtrajects->setPalette(cLight) ;
	else					Gtrajects->setPalette(cGray) ;
						
	if( cursC->isChecked())	Gcursors->setPalette(cLight) ;
	else					Gcursors->setPalette(cGray) ;
	
	Gparam->setEnabled(	curvC->isChecked() || trajC->isChecked() || trigC->isChecked() || cursC->isChecked() ) ;
	if(Gparam->isEnabled()) Gparam->setPalette(cDark) ;
	else					Gparam->setPalette(cGray) ;
}
void NxObjectEditor::setDefaultColors()
{
	Gchoice->setPalette(cRed);
	Gparam->setPalette(cDark);
	Gosc->setPalette(cDark);
	Gtriggers->setPalette( cLight ) ;
	Gtrajects->setPalette( cLight ) ;
	Gcursors->setPalette( cLight ) ;
	Gcurves->setPalette( cLight ) ;
	GcurvesBounds->setPalette( cLight ) ;
}