///////////////
// IanniX
///////////////

////////////////////////////
// NxOscConsole.cpp
////////////////////////////

#include "NxOscConsole.h"
#include <cstdlib>

///////////////////////////////////
// Main Constructor
///////////////////////////////////
NxOscConsole::NxOscConsole( NxGLScore* parent, NxScheduler* inScheduler )
{	
	parentGLBox		= (NxGLScore*)parent ;
	timeScheduler	= (NxScheduler*)inScheduler ;
	
	console = new QDialog ;
	
	//QFont f( "Lucida Grande", 10);
	//console->setFont( f );
	console->setWindowTitle( tr( "IanniX OSC Console" ) );
	//console->setBaseSize(660,360) ;
	
	space = QString("    ") ;
	outListening = inListening = false ;
	
	// Maximum number messages
	maxCurv = maxTrig = 20 ;
	maxIn = 30 ;
	
	QTabBar *oscTabs		= new QTabBar(console) ;
	QStackedWidget *pages	= new QStackedWidget(console) ;
	
	//oscTabs->setMaximumSize(100, 30) ;
	
	QVBoxLayout *genLay = new QVBoxLayout(console) ;
	
	QWidget *inBox			= new QWidget ;
	QWidget *outBox			= new QWidget ;
	QWidget *curvBox		= new QWidget(oscTabs) ;
	QWidget *trigBox		= new QWidget(oscTabs) ;
	
	QVBoxLayout *outLay	= new QVBoxLayout(outBox) ;
	QVBoxLayout *inLay	= new QVBoxLayout(inBox) ;
	
	//////////////////////////////////////////////////////////////// BUTTONS
	QGroupBox	*buttons = new QGroupBox(console) ;
	buttons->setFlat(true) ;
	QHBoxLayout *buttLay = new QHBoxLayout(buttons) ;
	
	QPushButton *clearB = new QPushButton("Clear", buttons) ;
	clearB->setMaximumSize(60,20) ;
	startB = new QPushButton("Start", buttons) ;
	startB->setMaximumSize(60,20) ;
	stopB = new QPushButton("Stop", buttons) ;
	stopB->setMaximumSize(60,20) ;
	stopB->setEnabled(false) ;
	roundC = new QCheckBox("Round Float Values", buttons) ;
	roundC->setChecked(false) ;
	roundV = false ;
	
	buttLay->addWidget(clearB) ;
	buttLay->addWidget(stopB) ;
	buttLay->addWidget(startB) ;
	buttLay->addWidget(roundC) ;
	buttLay->addStretch(1) ;
	buttons->setLayout(buttLay) ;
	
	//////////////////////////////////////////////////////////////// LISTS
	outCurveView		= new QListWidget(curvBox) ;
	outTriggerView		= new QListWidget(trigBox) ;
	inView				= new QListWidget(inBox) ;
	
	outCurveView->setAlternatingRowColors(true) ;
	outTriggerView->setAlternatingRowColors(true) ;
	inView->setAlternatingRowColors(true) ;
	
	outLay->addWidget(outCurveView) ;
	outLay->addWidget(outTriggerView) ;
	outBox->setLayout(outLay) ;
	
	inLay->addWidget(inView) ;
	inBox->setLayout(inLay) ;
	
	pages->addWidget(inBox) ;
	pages->addWidget(outBox) ;
	oscTabs->addTab("OSC in") ;
	oscTabs->addTab("OSC out") ;
	pages->setCurrentIndex(1) ;
	oscTabs->setCurrentIndex(1) ;
	
	genLay->addWidget(buttons) ;
	genLay->addWidget(oscTabs) ;
	genLay->addWidget(pages) ;
	console->setLayout(genLay) ;

	connect(oscTabs,SIGNAL(currentChanged(int)), pages, SLOT(setCurrentIndex(int)) ) ;
	
	connect(roundC,	SIGNAL(toggled(bool)), this, SLOT(roundValues(bool)) ) ;
	
	connect(clearB, SIGNAL(clicked()), this, SLOT(clearMessages()) ) ;
	connect(startB, SIGNAL(clicked()), this, SLOT(startOutListening()) ) ;
	connect(stopB,	SIGNAL(clicked()), this, SLOT(stopOutListening()) ) ;
	
	connect(roundC,	SIGNAL(toggled(bool)), this, SLOT(roundValues(bool)) ) ;
	
	connect(console, SIGNAL(rejected()), this, SLOT(quit()) ) ;
	connect(console, SIGNAL(rejected()), parentGLBox, SLOT(closeOscConsole()) ) ;
}
//////////////////////////////////////////////
// Destructor
//////////////////////////////////////////////
NxOscConsole::~NxOscConsole()
{}
void NxOscConsole::openWindow()
{
	console->show() ;
	console->setFocus();
	console->activateWindow() ;
}
void NxOscConsole::run()
{	
	exec() ;
}
void NxOscConsole::roundValues( bool flag )
{
	roundV = flag ;
}
void NxOscConsole::startOutListening()
{
	if(!outListening) {
		printf("IanniX OSC Console Start\n") ;
		outListening = true ;
		startB->setEnabled(false) ;
		stopB->setEnabled(true) ;
		inListening = true ;
		timeScheduler->setOscConsoleActive(true) ;
		connect( timeScheduler, SIGNAL(sendTriggerMessage(triggerOSC)),	this, SLOT(addOscOutTriggerMessage(triggerOSC)) ) ;
		connect( timeScheduler, SIGNAL(sendCurveMessage(curveOSC)),		this, SLOT(addOscOutCurveMessage(curveOSC)) ) ;
	}
}
void NxOscConsole::stopOutListening()
{
	if(outListening) {
		printf("IanniX OSC Console Stop\n") ;
		outListening = false ;
		startB->setEnabled(true) ;
		stopB->setEnabled(false) ;
		inListening = false ;
		timeScheduler->setOscConsoleActive(false) ;
		disconnect( timeScheduler, SIGNAL(sendTriggerMessage(triggerOSC)),	this, SLOT(addOscOutTriggerMessage(triggerOSC)) ) ;
		disconnect( timeScheduler, SIGNAL(sendCurveMessage(curveOSC)),		this, SLOT(addOscOutCurveMessage(curveOSC)) ) ;
	}
}
//////////////////////////////////////////////////////////////////////
// Erase all messages
//////////////////////////////////////////////////////////////////////
void NxOscConsole::clearMessages()
{
	for(int a=outCurveView->count()-1;a>=0;a--)
		outCurveView->takeItem(a) ;
	for(int a=outTriggerView->count()-1;a>=0;a--)
		outTriggerView->takeItem(a) ;
	for(int a=inView->count()-1;a>=0;a--)
		inView->takeItem(a) ;
}
////////////////////////////////////////////////////////////////////////////
// Add an OSC IN message
////////////////////////////////////////////////////////////////////////////
void NxOscConsole::addOscInMessage( QString inStr, QList<QVariant> argList, bool isOK )
{
	if(inListening) {
		QString str ;
		QString str2 ;
	
		if(isOK)	str.append("OK ") ;
		else		str.append("??? ") ;
	
		str.append( space ) ;
		str.append( inStr) ;
		str.append( space ) ;

		for(int m=0;m<argList.count();m++)
		{
			if(	roundV && (argList.at(m).type() == QVariant::Double) )
				str2 = QString().setNum( rounder((float)argList.at(m).toDouble()) ) ;
			else
				str2 = argList.at(m).toString() ;
				
			str.append( QString("%1 ").arg( str2 ) );
		}
	
		inView->addItem(str) ;
		
		if( inView->count()>=(maxIn+1) )
			inView->takeItem(0) ;
			
		inView->scrollToBottom() ;
	}
}
float NxOscConsole::rounder( float inVal )
{
	return floor(inVal*10.0)/10.0 ;
}
////////////////////////////////////////////////////////////////////////////
// Add an OSC OUT message
////////////////////////////////////////////////////////////////////////////
void NxOscConsole::addOscOutCurveMessage( curveOSC inMess )
{
	NxEnv localEnv = parentGLBox->getPartition()->getEnvironment() ;
	QString str ;
	
	int ccRID	= inMess.cursRID ;		/// OSC Argument 1 - Cursor Real ID
	int ccGID	= inMess.cursGID ;		/// OSC Argument 2 - Cursor Group ID
	int ccCID	= inMess.cursID ;		/// OSC Argument 3 - Cursor Custom ID
	int cRID	= inMess.RID ;			/// OSC Argument 4 - Curve Real ID
	int cGID	= inMess.GID ;			/// OSC Argument 5 - Curve Group ID
	int cCID	= inMess.ID ;			/// OSC Argument 6 - Curve Custom ID
	float cAngle	= inMess.angle ;	/// OSC Argument 7 - Curve Angle
	float cValue	= inMess.val ;		/// OSC Argument 8 - Curve Value
	float cX		= inMess.X ;		/// OSC Argument 9 - Curve X
	float cY		= inMess.Y ;		/// OSC Argument 10 - Curve Y
	float cZ		= inMess.Z ;		/// OSC Argument 11 - Curve Z
	
	if(roundV) {
		cAngle = rounder(cAngle) ;
		cValue = rounder(cValue) ;
		cX = rounder(cX) ;
		cY = rounder(cY) ;
		cZ = rounder(cZ) ;
	}
	
	str.append( QString("Curve") ) ;
	str.append( space ) ;
	str.append( QString("%1:%2").arg(inMess.host).arg(inMess.port) ) ;
	str.append( space ) ;
	str.append( QString("%1 ").arg(inMess.oscFormat) ) ;
	str.append( space ) ;
	if(localEnv.scCRID) str.append( QString("%1 ").arg(ccRID) ) ;
	if(localEnv.scCGID) str.append( QString("%1 ").arg(ccGID) ) ;
	if(localEnv.scCID) str.append( QString("%1 ").arg(ccCID) ) ;
	str.append( space ) ;
	if(localEnv.scRID) str.append( QString("%1 ").arg(cRID) ) ;
	if(localEnv.scGID) str.append( QString("%1 ").arg(cGID) ) ;
	if(localEnv.scID) str.append( QString("%1 ").arg(cCID) ) ;
	str.append( space ) ;
	if(localEnv.scA) str.append( QString("%1 ").arg(cAngle) ) ;
	if(localEnv.scV) str.append( QString("%1 ").arg(cValue) ) ;
	str.append( space ) ;
	if(localEnv.scX) str.append( QString("%1 ").arg(cX) ) ;
	if(localEnv.scY) str.append( QString("%1 ").arg(cY) ) ;
	if(localEnv.scZ) str.append( QString("%1 ").arg(cZ) ) ;
	
	outCurveView->addItem(str) ;
	
	if( outCurveView->count()>=(maxCurv+1) )
		outCurveView->takeItem(0) ;

	outCurveView->scrollToBottom() ;
}
void NxOscConsole::addOscOutTriggerMessage( triggerOSC inMess )
{
	NxEnv localEnv = parentGLBox->getPartition()->getEnvironment() ;
	QString str ;
	
	int tcRID	= inMess.cursRID ;		/// OSC Argument 1 - Cursor Real ID
	int tcGID	= inMess.cursGID ;		/// OSC Argument 2 - Cursor Group ID
	int tcCID	= inMess.cursID ;		/// OSC Argument 3 - Cursor Custom ID
	int tRID	= inMess.RID ;			/// OSC Argument 4 - Trigger Real ID
	int tGID	= inMess.GID ;			/// OSC Argument 5 - Trigger Group ID
	int tCID	= inMess.ID ;			/// OSC Argument 6 - Trigger Custom ID
	int tValue	= inMess.val ;			/// OSC Argument 7 - Trigger Int Value
	float tX	= inMess.X ;			/// OSC Argument 8 - Trigger X
	float tY	= inMess.Y ;			/// OSC Argument 9 - Trigger Y
	float tZ	= inMess.Z ;			/// OSC Argument 10 - Trigger Z
	
	if(roundV) {
		tX = rounder(tX) ;
		tY = rounder(tY) ;
		tZ = rounder(tZ) ;
	}
	
	str.append( QString("Trigger") ) ;
	str.append( space ) ;
	str.append( QString("%1:%2").arg(inMess.host).arg(inMess.port) ) ;
	str.append( space ) ;
	str.append( QString("%1 ").arg(inMess.oscFormat) ) ;
	str.append( space ) ;
	if(localEnv.stCRID) str.append( QString("%1 ").arg(tcRID) ) ;
	if(localEnv.stCGID) str.append( QString("%1 ").arg(tcGID) ) ;
	if(localEnv.stCID) str.append( QString("%1 ").arg(tcCID) ) ;
	str.append( space ) ;
	if(localEnv.stRID) str.append( QString("%1 ").arg(tRID) ) ;
	if(localEnv.stGID) str.append( QString("%1 ").arg(tGID) ) ;
	if(localEnv.stID) str.append( QString("%1 ").arg(tCID) ) ;
	str.append( space ) ;
	if(localEnv.stV) str.append( QString("%1 ").arg(tValue) ) ;
	str.append( space ) ;
	if(localEnv.stX) str.append( QString("%1 ").arg(tX) ) ;
	if(localEnv.stY) str.append( QString("%1 ").arg(tY) ) ;
	if(localEnv.stZ) str.append( QString("%1 ").arg(tZ) ) ;
	
	outTriggerView->addItem(str) ;
	
	if( outTriggerView->count()>=(maxTrig+1) )
		outTriggerView->takeItem(0) ;

	outTriggerView->scrollToBottom() ;
}


