///////////////
// IanniX
///////////////

////////////////////////////
// NxPreferences.cpp
////////////////////////////

#include "NxPreferences.h"

NxPreferences::NxPreferences( NxGLScore* parent )
  : QDialog( parent )
{
	QFont f( "Lucida Grande", 12 );
	setFont( f );
		
	parentGLBox	= parent;

	setWindowTitle( "Score Settings" );
	setFixedSize( 460, 500 );
	setWindowModality( Qt::ApplicationModal ) ;
	
	firstBuildItems() ;
}
/////////////////////
// Destructor
////////////////////
NxPreferences::~NxPreferences()
{}
////////////////////////////////////////////////////////////////////////
// OPEN DIALOG
////////////////////////////////////////////////////////////////////////
void NxPreferences::openPreferences()
{
	parentGLBox->releaseKeyboard();
	localEnv = parentGLBox->getPartition()->getEnvironment() ;
	setValues() ;
	show() ;
}
////////////////////////////////////////////////////////////////////////
// BUILD OBJECTS
////////////////////////////////////////////////////////////////////////
void NxPreferences::firstBuildItems()
{
		QTabWidget* prefTabs 	= new QTabWidget(this);
		
		backColorPicked = false ;
		scoreColorPicked = false ;
		gridColorPicked = false ;
		pickedBackColor.ID = 0 ;
		pickedScoreColor.ID = 0 ;
		pickedGridColor.ID = 0 ;
		
		QGroupBox* mainSettings		= new QGroupBox(prefTabs) ; 
		QGroupBox* networkSettings	= new QGroupBox(prefTabs) ;
		QGroupBox* speedTable	= new QGroupBox(prefTabs) ;
			QGroupBox* speedSettings	= new QGroupBox(speedTable) ;
		QGroupBox* defaultTable = new QGroupBox(prefTabs) ;
			QGroupBox* defaultSettings	= new QGroupBox(defaultTable) ;
		QGroupBox* buttons			= new QGroupBox(this) ;
		
		QVBoxLayout* vLay			= new QVBoxLayout(this); // Vertical Layout
		vLay->setAlignment( Qt::AlignHCenter );
		
		QHBoxLayout* hbutts			= new QHBoxLayout(buttons);
		
		QVBoxLayout* mainLay = new QVBoxLayout(mainSettings);			// Main + Colors + Grid Opacity
			QGridLayout* gridOpLay	= new QGridLayout;					// opacity sliders
			QGridLayout* displayLay	= new QGridLayout;					// display options
			QHBoxLayout* colorLay	= new QHBoxLayout;					// colors
			QGridLayout* mainLay1	= new QGridLayout;					// main
		QVBoxLayout* oscLay		= new QVBoxLayout(networkSettings);		// OSC
		QVBoxLayout* speedTabLay = new QVBoxLayout(speedTable) ;		// Speed
			QGridLayout* speedLay	= new QGridLayout(speedSettings);	
		QVBoxLayout* defaultTabLay = new QVBoxLayout(defaultTable) ;
			QGridLayout* defaultLay = new QGridLayout(defaultSettings);	// Default Settings
			
			
		QGroupBox*	oscGp	= new QGroupBox(networkSettings) ;
		oscGp->setFlat( true ) ;
		QGroupBox*	oscC	= new QGroupBox("OSC Curves Collisions Values", networkSettings) ;
		QGroupBox*	oscT	= new QGroupBox("OSC Triggers Collisions Values", networkSettings) ;
		QGridLayout* oscGpLay	= new QGridLayout(oscGp) ;
		QGridLayout* oscScLay	= new QGridLayout(oscC) ;
		QGridLayout* oscStLay	= new QGridLayout(oscT) ;
		
		//////////////////// Buttons
		QPushButton *apply 	= new QPushButton("Apply") ;
		QPushButton *cancel = new QPushButton("Cancel") ;
		hbutts->addWidget(cancel);
		hbutts->addWidget(apply);
		buttons->setLayout(hbutts) ;
		buttons->setFlat( true ) ;
		//////////////////// Main Settings
		////////// Grid Opacity Slider ////////////////////
		QLabel *gridOpL			= new QLabel("Grid Opacity", mainSettings );
		gridOpSlider	= new QSlider ;
		gridOpSlider->setOrientation( Qt::Horizontal );
		gridOpSlider->setRange(2, 40);
		gridOpSlider->setTickPosition(QSlider::NoTicks);
		////////// Cursor ZONES Opacity Slider ////////////////////
		QLabel *zonesOpL		= new QLabel("Cursors Zones Opacity", mainSettings );
		zonesOpSlider	= new QSlider ;
		zonesOpSlider->setOrientation( Qt::Horizontal );
		zonesOpSlider->setRange(2, 35);
		zonesOpSlider->setTickPosition(QSlider::NoTicks);
		////////// Objects Names Display Opacity Slider ////////////////////
		QLabel *namesOpL		= new QLabel("Object Names Opacity", mainSettings );
		namesOpSlider	= new QSlider ;
		namesOpSlider->setOrientation( Qt::Horizontal );
		namesOpSlider->setRange(2, 100);
		namesOpSlider->setTickPosition(QSlider::NoTicks);
		//////// Display options
		sWorkspace		= new QCheckBox("Show Workspace", mainSettings );
		sWorkspace->setChecked( localEnv.scoreDraw ) ;
		sScreenInfos	= new QCheckBox("Show On Screen Infos", mainSettings );
		sScreenInfos->setChecked( localEnv.displayOn ) ;
		sNames			= new QCheckBox("Show Objects Names", mainSettings );
		sNames->setChecked( localEnv.namesDisplay) ;
		sGrid			= new QCheckBox("Show Grid", mainSettings );
		sGrid->setChecked( localEnv.showGrid ) ;
		sCrosses		= new QCheckBox("Show Center Crosses", mainSettings );
		sCrosses->setChecked( localEnv.crossesDraw ) ;
		sCircles		= new QCheckBox("Show Curves Circles", mainSettings );
		sCircles->setChecked( localEnv.circlesDraw ) ;
		sRectangles		= new QCheckBox("Show Curves Rectangles", mainSettings );
		sRectangles->setChecked( localEnv.rectDraw ) ;
		sCursZones		= new QCheckBox("Show Cursors Impact Zones", mainSettings );
		sCursZones->setChecked( localEnv.showCursorsArea ) ;
		//////// Colors
		chooseBackColor		= new QPushButton("Background Color", mainSettings );
		chooseScoreColor	= new QPushButton("Score Color", mainSettings ) ;
		chooseGridColor		= new QPushButton("Grid Color", mainSettings ) ;
		//////// Main settings
		nameL 	= new QLabel("Score Name", mainSettings );
		nameF 	= new QLineEdit( mainSettings );
		IDL 	= new QLabel("ID", mainSettings );
		IDF 	= new QLineEdit( mainSettings );
		lengthL = new QLabel("Current Length (in s)", mainSettings );
		lengthF = new QLineEdit( mainSettings );
		heightL = new QLabel("Current Height (in s)", mainSettings );
		heightF = new QLineEdit( mainSettings );
		colorLay->addWidget(chooseBackColor) ;
		colorLay->addWidget(chooseScoreColor) ;
		colorLay->addWidget(chooseGridColor) ;
		mainLay1->addWidget( nameL,		0, 0 );
		mainLay1->addWidget( IDL, 		1, 0 );
		mainLay1->addWidget( lengthL, 	2, 0 );
		mainLay1->addWidget( heightL, 	3, 0 );
		mainLay1->addWidget( nameF,		0, 1 );
		mainLay1->addWidget( IDF, 		1, 1 );
		mainLay1->addWidget( lengthF, 	2, 1 );
		mainLay1->addWidget( heightF, 	3, 1 );
		gridOpLay->addWidget(gridOpL,		0, 0 );
		gridOpLay->addWidget(gridOpSlider,	0, 1 );
		gridOpLay->addWidget(zonesOpL,		1, 0 );
		gridOpLay->addWidget(zonesOpSlider, 1, 1 );
		gridOpLay->addWidget(namesOpL,		2, 0 );
		gridOpLay->addWidget(namesOpSlider, 2, 1 );
		displayLay->addWidget( sWorkspace,		0, 0 ) ;
		displayLay->addWidget( sScreenInfos,	1, 0 ) ;
		displayLay->addWidget( sNames,			2, 0 ) ;
		displayLay->addWidget( sGrid,			3, 0 ) ;
		displayLay->addWidget( sCrosses,		0, 1 ) ;
		displayLay->addWidget( sCircles,		1, 1 ) ;
		displayLay->addWidget( sRectangles,		2, 1 ) ;
		displayLay->addWidget( sCursZones,		3, 1 ) ;
		mainLay->addLayout(mainLay1) ;
		mainLay->addLayout(displayLay) ;
		mainLay->addLayout(gridOpLay) ;
		mainLay->addLayout(colorLay) ;
		mainSettings->setLayout(mainLay) ;
				
		//////////////////// OSC Settings	
		QLabel *configIPL	= new QLabel("OSC OUT Config Message IP", networkSettings );	
		QLabel *inportL		= new QLabel("OSC IN Port", networkSettings );
		configIPF		= new QLineEdit( networkSettings );
		inPortF			= new QLineEdit( networkSettings );
		/// Curves / Triggers options
		oscScCRID	= new QCheckBox("Cursor ID", networkSettings ) ;
		oscScCID	= new QCheckBox("Cursor Custom ID", networkSettings ) ;
		oscScCGID	= new QCheckBox("Cursor Group ID", networkSettings ) ;
		oscScRID	= new QCheckBox("ID", networkSettings ) ;
		oscScID		= new QCheckBox("Custom ID", networkSettings ) ;
		oscScGID	= new QCheckBox("Group ID", networkSettings ) ;
		oscScA		= new QCheckBox("Angle", networkSettings ) ;
		oscScV		= new QCheckBox("Value", networkSettings ) ;
		oscScX		= new QCheckBox("X", networkSettings ) ;
		oscScY		= new QCheckBox("Y", networkSettings ) ;
		oscScZ		= new QCheckBox("Z", networkSettings ) ;
		oscStCRID	= new QCheckBox("Cursor ID", networkSettings ) ;
		oscStCID	= new QCheckBox("Cursor Custom ID", networkSettings ) ;
		oscStCGID	= new QCheckBox("Cursor Group ID", networkSettings ) ;
		oscStRID	= new QCheckBox("ID", networkSettings ) ;
		oscStID		= new QCheckBox("Custom ID", networkSettings ) ;
		oscStGID	= new QCheckBox("Group ID", networkSettings ) ;
		oscStV		= new QCheckBox("Int Value", networkSettings ) ;
		oscStX		= new QCheckBox("X", networkSettings ) ;
		oscStY		= new QCheckBox("Y", networkSettings ) ;
		oscStZ		= new QCheckBox("Z", networkSettings ) ;
		/// layout
		oscScLay->addWidget( oscScCRID, 0, 0 ) ;
		oscScLay->addWidget( oscScCGID, 1, 0 ) ;
		oscScLay->addWidget( oscScCID,	2, 0 ) ;
		oscScLay->addWidget( oscScRID,	0, 1 ) ;
		oscScLay->addWidget( oscScGID,	1, 1 ) ;
		oscScLay->addWidget( oscScID,	2, 1 ) ;
		oscScLay->addWidget( oscScA,	0, 2 ) ;
		oscScLay->addWidget( oscScV,	1, 2 ) ;
		oscScLay->addWidget( oscScX,	0, 3 ) ;
		oscScLay->addWidget( oscScY,	1, 3 ) ;
		oscScLay->addWidget( oscScZ,	2, 3 ) ;
		oscStLay->addWidget( oscStCRID, 0, 0 ) ;
		oscStLay->addWidget( oscStCGID, 1, 0 ) ;
		oscStLay->addWidget( oscStCID,	2, 0 ) ;
		oscStLay->addWidget( oscStRID,	0, 1 ) ;
		oscStLay->addWidget( oscStGID,	1, 1 ) ;
		oscStLay->addWidget( oscStID,	2, 1 ) ;
		oscStLay->addWidget( oscStV,	0, 2 ) ;
		oscStLay->addWidget( oscStX,	0, 3 ) ;
		oscStLay->addWidget( oscStY,	1, 3 ) ;
		oscStLay->addWidget( oscStZ,	2, 3 ) ;
		oscC->setLayout( oscScLay ) ;
		oscT->setLayout( oscStLay ) ;
		/// OSC Message Preview
		oscTriggerMessageL	= new QLabel(networkSettings) ;
		oscCurveMessageL	= new QLabel(networkSettings) ;
		oscTriggerMessageL->setAutoFillBackground(true) ;
		oscCurveMessageL->setAutoFillBackground(true) ;
		oscTriggerMessageL->setForegroundRole(QPalette::WindowText);
		oscTriggerMessageL->setPalette(QPalette(QColor(237,151,116))) ;
		oscCurveMessageL->setForegroundRole(QPalette::WindowText) ;
		oscCurveMessageL->setPalette(QPalette(QColor(237,151,116)));
		///
		oscGpLay->addWidget( configIPL,		0, 0 );
		oscGpLay->addWidget( inportL,		1, 0 );
		oscGpLay->addWidget( configIPF,		0, 1 );
		oscGpLay->addWidget( inPortF,		1, 1 );
		oscGp->setLayout( oscGpLay ) ;
		oscLay->addWidget(oscGp);
		oscLay->addWidget(oscC);
		oscLay->addWidget(oscCurveMessageL) ;
		oscLay->addWidget(oscT);
		oscLay->addWidget(oscTriggerMessageL) ;
		networkSettings->setLayout(oscLay) ;
		
		//////////////////// TIME
		QLabel *tempoL 	= new QLabel("Speed", speedSettings );
		tempoF 	= new QLineEdit( speedSettings );
		QLabel *dStepL 	= new QLabel("Display FPS", speedSettings );
		dStepF 	= new QSpinBox( speedSettings );
		manualSchStep = new QCheckBox("Custom Scheduler Step (in ms)", speedSettings ) ;
		sStepF 	= new QSpinBox( speedSettings );
		////// Scheduler step deactivated
		manualSchStep->setEnabled(false) ;
		sStepF->setEnabled(false) ;
		dStepF->setMinimum(1) ;
		dStepF->setMaximum(60) ;
		sStepF->setMinimum(2) ;
		sStepF->setMaximum(100) ;
		speedLay->addWidget( tempoL, 0, 0 );
		speedLay->addWidget( tempoF, 0, 1 );
		speedLay->addWidget( dStepL, 1, 0 );
		speedLay->addWidget( dStepF, 1, 1 );
		speedLay->addWidget( manualSchStep, 2, 0 );
		speedLay->addWidget( sStepF, 2, 1 );
		speedSettings->setLayout(speedLay) ;
		
		//////////////////// DEFAULT VALUES
		hostL			= new QLabel("Host IP", defaultSettings );
		hostF			= new QLineEdit( defaultSettings );
		outportL		= new QLabel("UDP OSC Out Port", defaultSettings );
		outPortF		= new QLineEdit( defaultSettings );
		QLabel *cOSCFormatL	= new QLabel("OSC Curves Format", defaultSettings );
		QLabel *tOSCFormatL	= new QLabel("OSC Triggers Format", defaultSettings );
		mformat1F		= new QLineEdit( defaultSettings );
		mformat2F		= new QLineEdit( defaultSettings );
		ivalueL			= new QLabel("OSC Triggers Int Value", defaultSettings );
		ivalueF			= new QLineEdit( defaultSettings );
		QLabel* mapp1L	= new QLabel("Curves Mapping Minimum", defaultSettings );
		QLabel* mapp2L	= new QLabel("Curves Mapping Maximum", defaultSettings );
		mapp1F			= new QLineEdit( defaultSettings );
		mapp2F			= new QLineEdit( defaultSettings );
		defaultSpatialPeriod	= new QSpinBox( defaultSettings ) ;
		QLabel *defaultSP		= new QLabel( "Curves Scan Period (in ms)" ) ;
		defaultSpatialPeriod->setMinimum( 20 ) ;
		defaultSpatialPeriod->setMaximum( 9999 ) ;
		defaultLay->addWidget( hostL,	0, 0 );
		defaultLay->addWidget( hostF,	0, 1 );
		defaultLay->addWidget( outportL,1, 0 );
		defaultLay->addWidget( outPortF,1, 1 );
		defaultLay->addWidget( cOSCFormatL,2, 0 );
		defaultLay->addWidget( mformat1F,2, 1 );
		defaultLay->addWidget( tOSCFormatL,3, 0 );
		defaultLay->addWidget( mformat2F,3, 1 );
		defaultLay->addWidget( ivalueL,	4, 0 );
		defaultLay->addWidget( ivalueF,	4, 1 );
		defaultLay->addWidget( mapp1L,	5, 0 );
		defaultLay->addWidget( mapp1F,	5, 1 );
		defaultLay->addWidget( mapp2L,	6, 0 );
		defaultLay->addWidget( mapp2F,	6, 1 );
		defaultLay->addWidget( defaultSP,			7, 0 );
		defaultLay->addWidget( defaultSpatialPeriod,7, 1 );
		defaultSettings->setLayout(defaultLay) ;
				
		/////////////////////////////////////////////
		speedTabLay->addWidget(speedSettings) ;
		speedTabLay->addStretch(2) ;
		speedTable->setLayout(speedTabLay) ;
		defaultTabLay->addWidget(defaultSettings) ;
		defaultTabLay->addStretch(2) ;
		defaultTable->setLayout(defaultTabLay) ;
		/////////////////////////////////////////////
		mainSettings->setFlat(true) ;
		networkSettings->setFlat(true) ;
		defaultTable->setFlat(true) ;
		speedTable->setFlat(true) ;
		prefTabs->addTab( mainSettings, "Score" ) ;
		prefTabs->addTab( networkSettings, "OSC" ) ;
		prefTabs->addTab( speedTable, "Time" ) ;
		prefTabs->addTab( defaultTable, "Default Values" ) ;
		
		vLay->addWidget(prefTabs) ;
		vLay->addWidget(buttons) ;
		this->setLayout( vLay ) ;
		apply->setShortcut(Qt::Key_Return) ;

		connect( mformat1F, SIGNAL( textChanged(QString) ), this, SLOT( updateOscMessages(QString) ) ) ;
		connect( mformat2F, SIGNAL( textChanged(QString) ), this, SLOT( updateOscMessages(QString) ) ) ;
		
		connect( oscScCRID, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScCID, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScCGID, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScRID, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScID, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScGID, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScA, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScV, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScX, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScY, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		connect( oscScZ, SIGNAL( toggled(bool) ), this, SLOT( updateOscCurveMessage(bool) ) ) ;
		
		connect( oscStCRID, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStCID, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStCGID, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStRID, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStID, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStGID, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStV, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStX, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStY, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		connect( oscStZ, SIGNAL( toggled(bool) ), this, SLOT( updateOscTriggerMessage(bool) ) ) ;
		
		connect( gridOpSlider, SIGNAL( sliderMoved(int) ), this, SLOT( setGridOpacity(int) ) ) ;
		connect( zonesOpSlider, SIGNAL( sliderMoved(int) ), this, SLOT( setCursorZonesOpacity(int) ) ) ;
		connect( namesOpSlider, SIGNAL( sliderMoved(int) ), this, SLOT( setNamesOpacity(int) ) ) ;
		
		connect( manualSchStep, SIGNAL( toggled(bool) ), sStepF, SLOT( setEnabled(bool) ) ) ;
		connect( chooseScoreColor, 	SIGNAL( clicked() ), this, SLOT( pickScoreColor() ) ) ;
		connect( chooseBackColor, 	SIGNAL( clicked() ), this, SLOT( pickBackColor() ) ) ;
		connect( chooseGridColor, 	SIGNAL( clicked() ), this, SLOT( pickGridColor() ) ) ;
		connect( cancel, SIGNAL( clicked() ), this, SLOT( close() ) ) ;
		connect( apply,	SIGNAL( clicked() ), this, SLOT( setPreferences() ) ) ;
}
void NxPreferences::setValues()
{
	QString str;
	/////////////////////// MAIN
	gridOpSlider->setValue( localEnv.gridOp*100 );
	zonesOpSlider->setValue( localEnv.zonesOp*100 );
	namesOpSlider->setValue( localEnv.namesOp*100 );
	nameF->setText( localEnv.name );
	heightF->setText( str.setNum( localEnv.height ) );
	lengthF->setText( str.setNum( localEnv.length ) );
	IDF->setText( str.setNum( localEnv.ID ) );
	//////// DISPLAY OPTIONS Checkboxes
	sWorkspace->setChecked(localEnv.scoreDraw) ;
	sScreenInfos->setChecked(localEnv.displayOn) ;
	sNames->setChecked(localEnv.namesDisplay) ;
	sGrid->setChecked(localEnv.showGrid) ;
	sCrosses->setChecked(localEnv.crossesDraw) ;
	sCircles->setChecked(localEnv.circlesDraw) ;
	sRectangles->setChecked(localEnv.rectDraw) ;
	sCursZones->setChecked(localEnv.showCursorsArea) ;
	/////////////////////// SPEED
	tempoF->setText( str.setNum( localEnv.globalTempo ) ) ;
	dStepF->setValue( (int)floor(1000/localEnv.displayStep) ) ;
	sStepF->setValue( localEnv.schedulerStep ) ;
	/////////////////////// OSC NETWORK
	configIPF->setText( localEnv.OSCConfigMessageIP ) ;
	hostF->setText( localEnv.UDPhost );
	inPortF->setText( str.setNum( localEnv.UDPin ) );
	oscScCRID->setChecked( localEnv.scCRID ) ;
	oscScCID->setChecked( localEnv.scCID ) ;
	oscScCGID->setChecked( localEnv.scCGID ) ;
	oscScRID->setChecked( localEnv.scRID ) ;
	oscScID->setChecked( localEnv.scID ) ;
	oscScGID->setChecked( localEnv.scGID ) ;
	oscScA->setChecked( localEnv.scA ) ;
	oscScV->setChecked( localEnv.scV ) ;
	oscScX->setChecked( localEnv.scX ) ;
	oscScY->setChecked( localEnv.scY ) ;
	oscScZ->setChecked( localEnv.scZ ) ;
	oscStCRID->setChecked( localEnv.stCRID ) ;
	oscStCID->setChecked( localEnv.stCID ) ;
	oscStCGID->setChecked( localEnv.stCGID ) ;
	oscStRID->setChecked( localEnv.stRID ) ;
	oscStID->setChecked( localEnv.stID ) ;
	oscStGID->setChecked( localEnv.stGID ) ;
	oscStV->setChecked( localEnv.stV ) ;
	oscStX->setChecked( localEnv.stX ) ;
	oscStY->setChecked( localEnv.stY ) ;
	oscStZ->setChecked( localEnv.stZ ) ;
	/////////////////////// DEFAULT VALUES
	mformat1F->setText( localEnv.OSCCurvesDefaultMFormat );
	mformat2F->setText( localEnv.OSCTriggersDefaultMFormat );
	mapp1F->setText( str.setNum( localEnv.mappingDefaultMin ) ) ;
	mapp2F->setText( str.setNum( localEnv.mappingDefaultMax ) ) ;
	defaultSpatialPeriod->setValue( localEnv.defaultSpatialPeriod ) ;
	ivalueF->setText( str.setNum( localEnv.OSCTriggersDefaultIntValue ) );
	outPortF->setText( str.setNum( localEnv.UDPout ) );
	
	updateOscTriggerMessage(true) ;
	updateOscCurveMessage(true) ;
	
	manualSchStep->setChecked( !parentGLBox->getScheduler()->isAutoSchedulerStep() ) ;
	sStepF->setEnabled( !parentGLBox->getScheduler()->isAutoSchedulerStep() ) ;
	
	/// Deactive manual scheduler step
	//manualSchStep->setEnabled( false ) ;
	//sStepF->setEnabled( false ) ;
}
void NxPreferences::updateOscMessages(QString)
{
	updateOscTriggerMessage(true) ;
	updateOscCurveMessage(true) ;
}
void NxPreferences::updateOscTriggerMessage(bool)
{
	QString str = QString(mformat2F->text()).append(" ") ;
	if(oscStCRID->isChecked()) str.append(" cID") ;
	if(oscStCID->isChecked()) str.append(" cCID") ;
	if(oscStCGID->isChecked()) str.append(" cGID") ;
	if(oscStRID->isChecked()) str.append(" ID") ;
	if(oscStID->isChecked()) str.append(" CID") ;
	if(oscStGID->isChecked()) str.append(" GID") ;
	if(oscStV->isChecked()) str.append(" V") ;
	if(oscStX->isChecked()) str.append(" X") ;
	if(oscStY->isChecked()) str.append(" Y") ;
	if(oscStZ->isChecked()) str.append(" Z") ;
	oscTriggerMessageL->setText(str) ;
}
void NxPreferences::updateOscCurveMessage(bool)
{
	QString str = QString(mformat1F->text()).append(" ") ;
	if(oscScCRID->isChecked()) str.append(" cID") ;
	if(oscScCID->isChecked()) str.append(" cCID") ;
	if(oscScCGID->isChecked()) str.append(" cGID") ;
	if(oscScRID->isChecked()) str.append(" ID") ;
	if(oscScID->isChecked()) str.append(" CID") ;
	if(oscScGID->isChecked()) str.append(" GID") ;
	if(oscScA->isChecked()) str.append(" A") ;
	if(oscScV->isChecked()) str.append(" V") ;
	if(oscScX->isChecked()) str.append(" X") ;
	if(oscScY->isChecked()) str.append(" Y") ;
	if(oscScZ->isChecked()) str.append(" Z") ;
	oscCurveMessageL->setText(str) ;
}
void NxPreferences::pickScoreColor() 
{
	pickedScoreColor.c = colorPicker->getColor();	
	scoreColorPicked = true ;
}
void NxPreferences::pickBackColor() 
{	
	pickedBackColor.c = colorPicker->getColor();	
	backColorPicked = true ;
}
void NxPreferences::pickGridColor() 
{	
	pickedGridColor.c = colorPicker->getColor();	
	gridColorPicked = true ;
}
void NxPreferences::setGridOpacity( int inOp ) 
{	
	parentGLBox->setGridOpacity( inOp/100.0 ) ;
}
void NxPreferences::setCursorZonesOpacity( int inOp ) 
{
	parentGLBox->setCursorZonesOpacity( inOp/100.0 ) ;
}
void NxPreferences::setNamesOpacity( int inOp ) 
{
	parentGLBox->setNamesOpacity( inOp/100.0 ) ;
}
void NxPreferences::setPreferences()
{
	bool resize = false ;
	NxEnv tmpEnv = parentGLBox->getPartition()->getEnvironment();
	/////////////////////// MAIN
	if(scoreColorPicked)	tmpEnv.scoreColor = pickedScoreColor ;
	if(backColorPicked)		tmpEnv.backColor = pickedBackColor ;
	if(gridColorPicked)		tmpEnv.gridColor = pickedGridColor ;
	tmpEnv.name = nameF->text();
	tmpEnv.ID = IDF->text().toInt();
	int nL = lengthF->text().toInt() ;
	int nH = heightF->text().toInt() ;
	if( tmpEnv.length != nL || tmpEnv.height != nH ) resize = true ;
	tmpEnv.length = nL ;
	tmpEnv.height = nH ;
	//////// DISPLAY OPTIONS Checkboxes
	tmpEnv.scoreDraw		= sWorkspace->isChecked() ;
	tmpEnv.displayOn		= sScreenInfos->isChecked() ;
	tmpEnv.namesDisplay		= sNames->isChecked() ;
	tmpEnv.showGrid			= sGrid->isChecked() ;
	tmpEnv.crossesDraw		= sCrosses->isChecked() ;
	tmpEnv.circlesDraw		= sCircles->isChecked() ;
	tmpEnv.rectDraw			= sRectangles->isChecked() ;
	tmpEnv.showCursorsArea	= sCursZones->isChecked() ;
	/////////////////////// SPEED
	tmpEnv.globalTempo = tempoF->text().toFloat() ; 
	tmpEnv.displayStep = (int)floor( 1000.0/(dStepF->value()) ) ;
	if( manualSchStep->isChecked() ) {
		tmpEnv.schedulerStep = sStepF->value() ;
		parentGLBox->getScheduler()->useAutoSchedulerStep( false ) ;
	}
	else parentGLBox->getScheduler()->useAutoSchedulerStep( true ) ;
	tmpEnv.defaultSpatialPeriod = defaultSpatialPeriod->value() ;
	/////////////////////// OSC NETWORK
	tmpEnv.UDPhost = hostF->text() ;
	tmpEnv.UDPin = inPortF->text().toInt() ;
	tmpEnv.UDPout = outPortF->text().toInt() ;
	tmpEnv.OSCCurvesDefaultMFormat = mformat1F->text() ;
	tmpEnv.OSCTriggersDefaultMFormat = mformat2F->text() ;
	tmpEnv.OSCTriggersDefaultIntValue = ivalueF->text().toInt() ;
	tmpEnv.OSCConfigMessageIP = configIPF->text() ;
	// osc sending options (what to send)
	tmpEnv.scCRID	= oscScCRID->isChecked() ;
	tmpEnv.scCID	= oscScCID->isChecked() ;
	tmpEnv.scCGID	= oscScCGID->isChecked() ;
	tmpEnv.scRID	= oscScRID->isChecked() ;
	tmpEnv.scID		= oscScID->isChecked() ;
	tmpEnv.scGID	= oscScGID->isChecked() ;
	tmpEnv.scA		= oscScA->isChecked() ;
	tmpEnv.scV		= oscScV->isChecked() ;
	tmpEnv.scX		= oscScX->isChecked() ;
	tmpEnv.scY		= oscScY->isChecked() ;
	tmpEnv.scZ		= oscScZ->isChecked() ;
	tmpEnv.stCRID	= oscStCRID->isChecked() ;
	tmpEnv.stCID	= oscStCID->isChecked() ;
	tmpEnv.stCGID	= oscStCGID->isChecked() ;
	tmpEnv.stRID	= oscStRID->isChecked() ;
	tmpEnv.stID		= oscStID->isChecked() ;
	tmpEnv.stGID	= oscStGID->isChecked() ;
	tmpEnv.stV		= oscStV->isChecked() ;
	tmpEnv.stX		= oscStX->isChecked() ;
	tmpEnv.stY		= oscStY->isChecked() ;
	tmpEnv.stZ		= oscStZ->isChecked() ;
	/////////////////////// MAPPING
	tmpEnv.mappingDefaultMin = mapp1F->text().toFloat() ;
	tmpEnv.mappingDefaultMax = mapp2F->text().toFloat() ;
	
	parentGLBox->getPartition()->setEnvData( tmpEnv );
	parentGLBox->stopGLTimer() ;
	parentGLBox->startGLTimer() ; ////// refresh the display step
	parentGLBox->updateIndexes();
	
	QString nStr = QString("[%1x%2] ").arg(tmpEnv.length).arg(tmpEnv.height) ;
	nStr.append( tmpEnv.name ) ;
	parentGLBox->getMainWindow()->setCurrentTabText(nStr) ;
	parentGLBox->getMainWindow()->changeTempo( int( tmpEnv.globalTempo*100 ) ) ;
	
	parentGLBox->updateScoreColors() ;
	parentGLBox->updateLocalDisplayParameters() ;
	parentGLBox->updateCheckedMenus() ;
	////////////// OSC UDP Listener
	parentGLBox->updateUDPListener( false ) ;
	if(tmpEnv.oscInMode) parentGLBox->updateUDPListener( true ) ;
	////////////// OSC What to Send
	parentGLBox->getScheduler()->updateSendingOptions() ;
	
	if(resize) parentGLBox->hardResetView() ;
	
	accept() ;
}

