///////////////
// IanniX
///////////////

////////////////////////////
// NxView.cpp
////////////////////////////

// Class Header
#include "NxView.h"

///////////////////////////////
// Subclass NxTabs
//////////////////////////////
NxTabs::NxTabs( QWidget* parent, QTabBar* tb )
  : QTabWidget( parent )
{
	setTabBar( tb );
}

NxTabs::~NxTabs()
{}

//////////////////////////////////
// Subclass NxTabBar
/////////////////////////////////
NxTabBar::NxTabBar( QWidget* parent )
  : QTabBar( 0 )
{
	mainWindow = parent;
}

NxTabBar::~NxTabBar()
{}


////////////////////////////
// NxTabBar slots
////////////////////////////
void NxTabBar::contextMenuEvent( QContextMenuEvent* )
{
	QMenu* tabContext = new QMenu;
    
  	// Adding Items
  	tabContext->addAction( "&New Tab", 						this,		SLOT( newScore()) );
  	tabContext->addAction( "&Close Current Tab", 			this, 		SLOT( closeTab()) );
  	tabContext->addAction( "&Close All Tabs but Active", 	this, 		SLOT( closeButActive()) );
	tabContext->addAction( "&Close All Tabs",				this, 		SLOT( closeAllTabs()) );
	
  	tabContext->exec( QCursor::pos() );
  	delete tabContext;
}

void NxTabBar::setViewer(NxView *inViewer)
{
	viewer = inViewer;
}
void NxTabBar::newScore()
{
	viewer->newScore() ;
}
void NxTabBar::closeTab()
{
	viewer->closeScore();
}
void NxTabBar::closeAllTabs()
{
	viewer->closeAll();
}
void NxTabBar::closeButActive()
{
	viewer->closeAllButActive();
}

/////////////////////////////
// Class NxView
/////////////////////////////
NxView::NxView( QWidget* )
  : QMainWindow()
{
  	// Main Widget configuration
  	languageChange();
  	resize( QSize(800, 600) );
	move( 140, 30 ) ;
	setMinimumSize(800, 480) ;
	setWindowTitle( "IanniX Viewer" );
	
	//QFont f("Lucida Grande", 8);
	//setFont(f) ;
	
	//////////////// INITIALISATIONS
	firstLaunch		= true ;
	noScoreOpened	= true ;
	allClosedSaved	= true ;
	
	antialiasing	= true ;
	fullscreen		= false ;
	
	totalNbScore	= 0 ;
  	index 			= 0 ;
  	settingsOk 		= true ;
	external 		= NULL ;
	decObjectX		= 0 ;	
	incObjectID		= 1 ;

  	initMenuBar() ;
  	initToolBar() ;
  	initStatusBar() ;	
	initCloseDialog() ;
  	
	//////////////////////////////////// Main Layout
	myTabBar = new NxTabBar;
	myTabs = new NxTabs( this, myTabBar );
	myTabBar->setViewer(this);
	setCentralWidget(myTabs);
	
	showNoScoreText() ;
	enableMenuToolsSlider(false) ;
  	//////////////////////////////////// FileChooser
  	fileChooser = new QFileDialog;
  	fileChooser->setFilter( "XML files (*.xml *.html *.htm)" );
	
  	connect( myTabs, SIGNAL( currentChanged( QWidget* ) ),	this, SLOT( giveKeyboard(QWidget*) ) );
	connect( tempoSlider, SIGNAL( sliderReleased() ),		this, SLOT( reGiveHand() ) );
	connect( tempoSlider, SIGNAL( sliderMoved( int ) ),		this, SLOT( changeTempo( int ) ) );
	connect( tempoSlider, SIGNAL( actionTriggered( int ) ), this, SLOT( changeTempoTrigg( int ) ) );
}
/////////////////////
// Destructor
////////////////////
NxView::~NxView()
{
	GLwidgets.clear();
  	delete myTabs;
  	delete fileChooser;
}
void NxView::showNoScoreText()
{
	noScoreText = new QGroupBox(myTabs) ;
	QBoxLayout *noScoreLay = new QBoxLayout(QBoxLayout::TopToBottom, noScoreText) ;
	QLabel *noScoreT = new QLabel(noScoreText) ;
	noScoreT->setAlignment( Qt::AlignCenter ) ;
	noScoreT->setText("Choose <b>New Score</b> or <b>Open Score</b> in the File Menu.<br><br>You can also choose a <b>Tutorial</b> in the Help Menu.") ;
	noScoreLay->addWidget(noScoreT) ;
	noScoreText->setLayout(noScoreLay) ;
	
	myTabs->addTab(noScoreText,"") ;
}
void NxView::enableMenuToolsSlider( bool flag )
{
	if(!flag) {
		tbTempoF->setText("1") ;
		tbTime->setText(" Time : 000:00:000 ") ;
	}
	///////////////////////// MenuBar
	editA->setEnabled(flag) ;
	modulesA->setEnabled(flag) ;
	displayA->setEnabled(flag) ;
	timeA->setEnabled(flag) ;
	oscA->setEnabled(flag) ;
	///////////////////////// ToolBar
	tBar->setEnabled(flag) ;
	///////////////////////// TempoBar
	tempoSlider->setEnabled(flag) ;
}

///////////////////////////// PLAYBACK BUTTONS
void NxView::playScore() 
{		
	if(!noScoreOpened) GLwidgets.at(focusTab)->playPause() ;
}
void NxView::pauseScore() 
{		
	if(!noScoreOpened) GLwidgets.at(focusTab)->pause() ;
}
void NxView::resetScore() 
{
	if(!noScoreOpened) GLwidgets.at(focusTab)->stopAndReset() ;
}
void NxView::sendConfigOSC()
{
	if(!noScoreOpened) GLwidgets.at(focusTab)->sendConfigOSC() ;
}
//////////////////////////////////////
// ContextMenu Handler
/////////////////////////////////////
void NxView::contextMenuEvent( QContextMenuEvent*)
{
	/*
  	contextMenu = new QMenu;
  	// Items
  	contextMenu->addAction( "&New document", 	this, SLOT( newScore() ));
  	contextMenu->exec( QCursor::pos() );
	*/
}

////////////////////////////
// MENU BAR INIT
///////////////////////////
void NxView::initMenuBar()
{
  	// Popup Menus initialization
  	QMenu *fileM		= new QMenu("File");
  	QMenu *editM		= new QMenu("Edit");
  	QMenu *helpM		= new QMenu("Help");
  	QMenu *modulesM		= new QMenu("Modules");
	QMenu *displayM		= new QMenu("Display");
	QMenu *timeM		= new QMenu("Time");
	QMenu *oscM			= new QMenu("OSC");
	
	QMenu *tutosM		= new QMenu("Tutorials");
	
  	//////// File
	fileM->addAction( "&Preferences", 	this, SLOT( showPreferences() )		, Qt::CTRL + Qt::Key_P);
	fileM->addSeparator();
	fileM->addAction( "&New Score",				this, SLOT( newScore() ) , Qt::CTRL + Qt::Key_N);
  	fileM->addAction( "&Open Score",			this, SLOT( openFromFile() ) , Qt::CTRL + Qt::Key_O);
	fileM->addSeparator();
	fileM->addAction( "&Close Current Score",	this, SLOT( closeScore() ) , Qt::CTRL + Qt::Key_W);
  	fileM->addAction( "&Close All Scores",		this, SLOT( closeAll() ) , Qt::CTRL + Qt::SHIFT + Qt::Key_W );
	fileM->addAction( "&Close All But Active", 	this, SLOT( closeAllButActive() ) );
  	fileM->addSeparator();
  	fileM->addAction( "&Save", 				this, SLOT( save() )  , Qt::CTRL + Qt::Key_S );
  	fileM->addAction( "&Save As",			this, SLOT( saveAs() ) , Qt::CTRL + Qt::SHIFT + Qt::Key_S );
  	fileM->addAction( "&Save All Scores",	this, SLOT( saveAll() ) );
	fileM->addSeparator();
	fileM->addAction( "&Quit",			this, SLOT( quitDialog() )			, Qt::CTRL + Qt::Key_Q);
  
  	//////// Edit
	editM->addAction( "&Select All", 		this, SLOT( selectAll() )	, Qt::CTRL + Qt::Key_A );
  	editM->addAction( "&Deselect All", 		this, SLOT( deselectAll() )  , Qt::CTRL + Qt::SHIFT + Qt::Key_A );
	editM->addSeparator();
  	editM->addAction( "&Copy", 			this, SLOT( copy() )  , Qt::CTRL + Qt::Key_C );
  	editM->addAction( "&Cut", 			this, SLOT( cut() )  , Qt::CTRL + Qt::Key_X );
  	editM->addAction( "&Paste", 		this, SLOT( basicPaste() )  , Qt::CTRL + Qt::Key_V );
	editM->addAction( "&Duplicate", 	this, SLOT( duplicate() )  , Qt::CTRL + Qt::Key_D );
  	editM->addSeparator();
  	editM->addAction( "&Change Font", 	this, SLOT( font() ) );
	
  	//////// Modules
  	modulesM->addAction( "&Object Creator", 		this, SLOT( showObjectCreator() ) );
	modulesM->addAction( "&Object Editor", 			this, SLOT( showGroupEditor() )  ,		Qt::CTRL + Qt::Key_E);
  	modulesM->addAction( "&Control Center", 		this, SLOT( showControl() )  ,			Qt::CTRL + Qt::Key_Z);
	modulesM->addSeparator();
	modulesM->addAction( "&OSC Console", 			this, SLOT( showOscConsole() ) );
  	//modulesM->addAction( "&Advanced Render Module", 	this, SLOT( showAdvanced() ) );

	/*// Externals
	externals->addAction( "&Select Default software...", this, SLOT( tmp() ) );
	externals->addSeparator();
	externals->addAction( "&Launch PureData (with OSC library)", 	this, SLOT( launchPd() ) );
	externals->addAction( "&Launch Max/MSP", 						this, SLOT( launchMaxMSP() ) );*/
	
	////////// OSC Settings
	visualModeChoose = new QAction(tr("&Process Graphics"), this);
	connect(visualModeChoose, SIGNAL(triggered()), this, SLOT(switchVisualMode()) ) ;
	visualModeChoose->setCheckable(true) ;
	oscOutModeChoose = new QAction(tr("&Process OSC Out"), this);
	connect(oscOutModeChoose, SIGNAL(triggered()), this, SLOT(switchOSCoutMode()) ) ;
	oscOutModeChoose->setCheckable(true) ;
	oscInModeChoose = new QAction(tr("&Process OSC In"), this);
	connect(oscInModeChoose, SIGNAL(triggered()), this, SLOT(switchOSCinMode()) ) ;
	oscInModeChoose->setCheckable(true) ;
	broadcastChoose = new QAction(tr("&Broadcast mode"), this);
	connect(broadcastChoose, SIGNAL(triggered()), this, SLOT(switchBroadcastMode()) ) ;
	broadcastChoose->setCheckable(true) ;
	oscSendIFChoose = new QAction(tr("&Send Initial and Final OSC Messages"), this);
	connect(oscSendIFChoose, SIGNAL(triggered()), this, SLOT(switchSendIFmessages()) ) ;
	oscSendIFChoose->setCheckable(true) ;
	////////// TIME Settings
	algoChoose = new QAction(tr("&Use Circles Algorithm"), this);
	connect(algoChoose, SIGNAL(triggered()), this, SLOT(switchCirclesAlgo()) ) ;
	algoChoose->setCheckable(true) ;
	moveCursorsChoose = new QAction(tr("&Move Cursors"), this);
	connect(moveCursorsChoose, SIGNAL(triggered()), this, SLOT(switchMoveCursors()) ) ;
	moveCursorsChoose->setCheckable(true) ;
	algoChoose->setChecked(true) ;
	////////// DISPLAY Settings
	gridShow = new QAction(tr("&Show Grid"), this);
	connect(gridShow, SIGNAL(triggered()), this, SLOT(showHideGrid()) ) ;
	gridShow->setCheckable(true) ;
	crossesShow = new QAction(tr("&Show Center Crosses"), this);
	connect(crossesShow, SIGNAL(triggered()), this, SLOT(showHideCrosses()) ) ;
	crossesShow->setCheckable(true) ;	
	workspaceShow = new QAction(tr("&Show Workspace"), this);
	connect(workspaceShow, SIGNAL(triggered()), this, SLOT(showHideScore()) ) ;
	workspaceShow->setCheckable(true) ;	
	infosShow = new QAction(tr("&Show On Screen Infos"), this);
	connect(infosShow, SIGNAL(triggered()), this, SLOT(showHideScreenInfos()) ) ;
	infosShow->setCheckable(true) ;	
	namesShow = new QAction(tr("&Show Objects Names"), this);
	connect(namesShow, SIGNAL(triggered()), this, SLOT(showHideObjectsNames()) ) ;
	namesShow->setCheckable(true) ;	
	circlesShow = new QAction(tr("&Show Curves Circles"), this);
	connect(circlesShow, SIGNAL(triggered()), this, SLOT(showHideCurvesCircles()) ) ;
	circlesShow->setCheckable(true) ;	
	curvesRectShow = new QAction(tr("&Show Curves Rectangles"), this);
	connect(curvesRectShow, SIGNAL(triggered()), this, SLOT(showHideCurvesRect()) ) ;
	curvesRectShow->setCheckable(true) ;	
	cursArea = new QAction(tr("&Show Cursors Impact Zones"), this);
	connect(cursArea, SIGNAL(triggered()), this, SLOT(showHideCursorsArea()) ) ;
	cursArea->setCheckable(true) ;	
	autoManualStep = new QAction(tr("&Auto Grid Step"), this);
	connect(autoManualStep, SIGNAL(triggered()), this, SLOT(switchAutoGridStep()) ) ;
	autoManualStep->setCheckable(true) ;	
	snapGrid = new QAction(tr("&Snap To Grid"), this);
	connect(snapGrid, SIGNAL(triggered()), this, SLOT(switchSnapToGrid()) ) ;
	snapGrid->setCheckable(true) ;	
	view3d = new QAction(tr("&3D View"), this);
	connect(view3d, SIGNAL(triggered()), this, SLOT(change2D3DView()) ) ;
	view3d->setCheckable(true) ;
	antialias = new QAction(tr("&Antialiasing"), this);
	connect(antialias, SIGNAL(triggered()), this, SLOT(switchAntialiasing()) ) ;
	antialias->setCheckable(true) ;
	antialias->setChecked(true) ;
	fullscreenMode = new QAction(tr("&Fullscreen..."), this);
	connect(fullscreenMode, SIGNAL(triggered()), this, SLOT(switchFullscreen()) ) ;
	fullscreenMode->setShortcut( Qt::CTRL + Qt::Key_F ) ;
	antialias->setCheckable(true) ;
	
	/////// OSC ///////////////////////////////////////////////////////////////////////////////////
	oscM->addAction( broadcastChoose ) ;
	oscM->addSeparator();
	oscM->addAction( visualModeChoose ) ;
	oscM->addAction( oscOutModeChoose ) ;
	oscM->addAction( oscInModeChoose ) ;
	oscM->addSeparator();
	oscM->addAction( oscSendIFChoose ) ;
	/////// TIME ///////////////////////////////////////////////////////////////////////////////////
	timeM->addAction( algoChoose ) ;
	timeM->addSeparator();
	timeM->addAction( moveCursorsChoose ) ;
	timeM->addSeparator();
	timeM->addAction( "&Set Playback ...",	this, SLOT( setPlaybackTime() ) );
	timeM->addAction( "&Hard Offsets Reset",	this, SLOT( hardOffsetsReset() ) );
	timeM->addSeparator();
	timeM->addAction( "&Reset Speed Factor",		this, SLOT( resetTempo() )  , Qt::CTRL + Qt::Key_T);
	/////// DISPLAY ///////////////////////////////////////////////////////////////////////////////////
	displayM->addAction( fullscreenMode ) ;
	displayM->addSeparator() ;
	displayM->addAction( "&Reset View",	this, SLOT( resetView() )  , Qt::CTRL + Qt::Key_R);
	displayM->addSeparator() ;
	displayM->addAction( workspaceShow ) ;
	displayM->addAction( infosShow ) ;
	displayM->addAction( namesShow ) ;
	displayM->addAction( crossesShow ) ;
	displayM->addAction( circlesShow ) ;
	displayM->addAction( curvesRectShow ) ;
	displayM->addAction( cursArea ) ;
	displayM->addSeparator();
	displayM->addAction( gridShow );
	displayM->addAction( autoManualStep );
	displayM->addAction( "&Set Grid Step ...",	this, SLOT( setGridStep() ) );
	displayM->addAction( snapGrid );
	displayM->addSeparator();
	displayM->addAction( view3d );
	displayM->addSeparator();
	displayM->addAction( antialias );
	
	/////// HELP ///////////////////////////////////////////////////////////////////////////////////
	/////// Tutorials
	QAction *tuto1 = new QAction(tr("&Tutorial 1"), this);
	QAction *tuto2 = new QAction(tr("&Tutorial 2"), this);
	QAction *tuto3 = new QAction(tr("&Tutorial 3"), this);
	QAction *tuto4 = new QAction(tr("&Tutorial 4"), this);
	QAction *tuto5 = new QAction(tr("&Tutorial 5"), this);
	tutosM->addAction(tuto1) ;
	tutosM->addAction(tuto2) ;
	tutosM->addAction(tuto3) ;
	//tutosM->addAction(tuto4) ;
	//tutosM->addAction(tuto5) ;
	
	connect(tutosM, SIGNAL(triggered(QAction*)), this, SLOT(launchTutorial(QAction*)) ) ;
	
	helpM->addAction( "&About",					this, SLOT( about() ) );
	helpM->addAction( "&Keyboard Shortcuts", 	this, SLOT( showShortcuts() ) );
	helpM->addSeparator();
	helpM->addMenu( tutosM ) ;
	
	////////////////////////////////////////////////////////////////////////////////////////////////
	// Adding Menus
	////////////////////////////////////////////////////////////////////////////////////////////////
	menuBar()->addMenu(fileM);
	editA		= menuBar()->addMenu(editM);
	modulesA	= menuBar()->addMenu(modulesM);
	oscA		= menuBar()->addMenu(oscM);
	timeA		= menuBar()->addMenu(timeM);
	displayA	= menuBar()->addMenu(displayM);
	menuBar()->addSeparator();
	menuBar()->addMenu(helpM);
}

///////////////////////////////////////////////////////////////////////////////
// ToolBar
///////////////////////////////////////////////////////////////////////////////
void NxView::initToolBar()
{	
	////////// ICONS
	//cpuOnIcon	= QIcon(":/pixmaps/cpuOn.png") ;
	//cpuOffIcon	= QIcon(":/pixmaps/cpuOff.png") ;
	playIcon	= QIcon(":/pixmaps/play.png") ;
	pauseIcon	= QIcon(":/pixmaps/pause.png") ;
	stopIcon	= QIcon(":/pixmaps/stop.png") ;
	resetIcon	= QIcon(":/pixmaps/reset.png") ;
	configIcon	= QIcon(":/pixmaps/config.png") ;

	tBar = new QToolBar(this) ;
	tBar->setMovable(false) ;

	//QGroupBox *cpuGroup = new QGroupBox ;
	//cpuGroup->setFixedSize(100,55) ; 
	QGroupBox *playbuttsGroup = new QGroupBox(tBar) ;
	playbuttsGroup->setFixedSize(130,55) ; 
	QGroupBox *timeGroup = new QGroupBox(tBar) ;
	timeGroup->setFixedSize(145,55) ;
	QGroupBox *tempoGroup = new QGroupBox(tBar) ;
	tempoGroup->setFixedSize(135,55) ; 
	QGroupBox *renderGroup = new QGroupBox(tBar) ;
	renderGroup->setFixedSize(280,55) ; 
	QGroupBox *configGroup = new QGroupBox(tBar) ;
	configGroup->setFixedSize(55,55) ; 

	//QHBoxLayout *lay			= new QHBoxLayout(tBar) ;
	//QHBoxLayout *cpuLay			= new QHBoxLayout(cpuGroup) ;
	QHBoxLayout *playbuttsLay	= new QHBoxLayout(playbuttsGroup) ;
	QHBoxLayout *timeLay		= new QHBoxLayout(timeGroup) ;
	QHBoxLayout *tempoLay		= new QHBoxLayout(tempoGroup) ;
	QGridLayout *renderLay		= new QGridLayout(renderGroup) ;
	QHBoxLayout *configLay		= new QHBoxLayout(configGroup) ;
	
	//tbCpuValue	= new QLabel("0 ms") ;
	//tbCpu		= new QToolButton ;
	playPause	= new QToolButton ;
	QToolButton *stop	= new QToolButton;
	QToolButton *reset	= new QToolButton;
	QToolButton *config = new QToolButton;
	tbTime		= new QLabel ;
	tbTempoF	= new QLineEdit(tempoGroup) ;
	tbTempoF->setMinimumHeight(20) ;
	tbTempo		= new QLabel("Speed :", tempoGroup ) ;
	tbInfoTxt	= new QLabel ;
	
	QLabel *rTrajectsL	= new QLabel( "Trajects", renderGroup ) ;
	QLabel *rCursorsL	= new QLabel( "Cursors", renderGroup ) ;
	QLabel *rCurvesL	= new QLabel( "Curves", renderGroup ) ;
	QLabel *rTriggersL	= new QLabel( "Triggers", renderGroup ) ;
	rTrajectsL->setScaledContents( true ) ;
	rCursorsL->setScaledContents( true ) ;
	rCurvesL->setScaledContents( true ) ;
	rTriggersL->setScaledContents( true ) ;
	rTrajectsC	= new QCheckBox( renderGroup ) ;
	rCursorsC	= new QCheckBox( renderGroup ) ;
	rCurvesC	= new QCheckBox( renderGroup ) ;
	rTriggersC	= new QCheckBox( renderGroup ) ;
	rTrajectsC->setChecked(true) ;
	rCursorsC->setChecked(true) ;
	rCurvesC->setChecked(true) ;
	rTriggersC->setChecked(true) ;
	layerFilterC = new QComboBox(renderGroup) ;
	groupFilterC = new QComboBox(renderGroup) ;
	layerFilterC->addItem("All Layers") ;
	groupFilterC->addItem("All Groups") ;
	
	cpuWarning = false ;
	//tbCpu->setIcon( cpuOffIcon ) ;
    playPause->setIcon( playIcon ) ;
	stop->setIcon( stopIcon ) ;
	reset->setIcon( resetIcon ) ;
	config->setIcon( configIcon ) ;
	
	//cpuLay->addWidget( tbCpu ) ;
	//cpuLay->addWidget( tbCpuValue ) ;
	//cpuGroup->setLayout( cpuLay ) ;
	playbuttsLay->addWidget( reset ) ;
	playbuttsLay->addWidget( stop ) ;
	playbuttsLay->addWidget( playPause ) ;
	playbuttsGroup->setLayout( playbuttsLay ) ;
	timeLay->addWidget( tbTime ) ;
	timeGroup->setLayout( timeLay ) ;
	tempoLay->addWidget( tbTempo ) ;
	tempoLay->addWidget( tbTempoF ) ;
	tempoGroup->setLayout( tempoLay ) ;
	renderLay->addWidget(rTrajectsC,	0,1) ;
	renderLay->addWidget(rTrajectsL,	0,0) ;
	renderLay->addWidget(rCursorsC,		1,1) ;
	renderLay->addWidget(rCursorsL,		1,0) ;
	renderLay->addWidget(rCurvesC,		0,3) ;
	renderLay->addWidget(rCurvesL,		0,2) ;
	renderLay->addWidget(rTriggersC,	1,3) ;
	renderLay->addWidget(rTriggersL,	1,2) ;
	renderLay->addWidget(groupFilterC,	0,4) ;
	renderLay->addWidget(layerFilterC,	1,4) ;
	renderGroup->setLayout( renderLay ) ;
	configLay->addWidget( config ) ;
	configGroup->setLayout( configLay ) ;
	//tBar->addWidget( cpuGroup ) ;
	tBar->addWidget( playbuttsGroup ) ;	
	tBar->addWidget( timeGroup ) ;
	tBar->addWidget( tempoGroup ) ;
	tBar->addWidget( renderGroup ) ;
	tBar->addWidget( configGroup ) ;
	//tBar->addStretch(50) ;
	//tBar->addWidget( tbInfoTxt ) ;
	//tBar->addSpacing(5) ;
	
	//delete(tBar->layout()) ;
	//tBar->setLayout(lay) ;
	
	connect(rTrajectsC, SIGNAL( released() ),	this, SLOT( showHideTrajects() ) ) ;
	connect(rCursorsC,	SIGNAL( released() ),	this, SLOT( showHideCursors() ) ) ;
	connect(rCurvesC,	SIGNAL( released() ),	this, SLOT( showHideCurves() ) ) ;
	connect(rTriggersC, SIGNAL( released() ),	this, SLOT( showHideTriggers() ) ) ;
	
	//connect(tbCpu,		SIGNAL( clicked() ),			this, SLOT( showCPUinfo() ) );
	connect(tbTempoF,	SIGNAL( editingFinished() ),	this, SLOT( setTbTempo() ) );
	connect(reset,		SIGNAL( clicked()  ),			this, SLOT( resetScore() ) );
	connect(stop,		SIGNAL( clicked()  ),			this, SLOT( pauseScore() ) );
	connect(playPause,	SIGNAL( clicked()  ),			this, SLOT( playScore() ) );
	
	connect(config,		SIGNAL( clicked()  ),			this, SLOT( sendConfigOSC() ) );
	
	addToolBar(tBar) ;
}

//////////////////////
// Status Bar
//////////////////////
void NxView::initStatusBar()
{
  	// Creating slider
  	tempoSlider = new QSlider;
  	tempoSlider->setOrientation( Qt::Horizontal );
	
	// Default Tempo : 1.0
	tempoSlider->setRange(-1000, 1000);
	tempoSlider->setValue( 100 );
  	//tempoSlider->setTracking( true );
  	tempoSlider->setTickInterval( 10 );
	tempoSlider->setTickPosition(QSlider::NoTicks);

	//tempoSlider->setTickPosition(QSlider::TicksBelow);
	tempoSlider->setPalette( QPalette( QColor(155, 55, 0) ) ) ;
  	// Adding it to status bar
  	statusBar()->addWidget( tempoSlider, 1 );
}

void NxView::initCloseDialog()
{
	closeDialog = new QDialog(this) ;
	closeDialog->setFixedSize(430,120) ;
	
	QGroupBox* buttG	= new QGroupBox(closeDialog);
	QVBoxLayout* vL 	= new QVBoxLayout ;
	QHBoxLayout* buttL 	= new QHBoxLayout ;
	buttG->setFlat( true ) ;
	
	txtL			= new QLabel("Do you want to save changes before closing ?", closeDialog ) ;
	quitB			= new QPushButton("Don't Save",			buttG );
	saveAndQuitB	= new QPushButton("Save All and Quit",	buttG );
	cancelB			= new QPushButton("Cancel",				buttG );
	
	buttL->addWidget(cancelB) ;
	buttL->addSpacing(65) ;
	buttL->addWidget(quitB) ;
	buttL->addWidget(saveAndQuitB) ;
	buttG->setLayout(buttL) ;
	txtL->setAlignment( Qt::AlignHCenter );
	vL->addWidget(txtL) ;
	vL->addWidget(buttG) ;
	closeDialog->setLayout(vL) ;
	
	//cancelB->setShortcut(Qt::Key_Escape) ;
	//quitB->setShortcut(Qt::Key_Space) ;
	//saveAndQuitB->setShortcut(Qt::Key_Return) ;
	
	closeDialog->setWindowModality( Qt::ApplicationModal ) ;
	
	connect( cancelB,		SIGNAL( clicked() ), closeDialog,	SLOT( close() ) ) ;
	connect( quitB,			SIGNAL( clicked() ), qApp,			SLOT( quit() ) ) ;
	connect( saveAndQuitB,	SIGNAL( clicked() ), this,			SLOT( saveAndQuit() ) ) ;
	quitDialogMode = true ;
}
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
// OPEN / CLOSE / SAVE SCORES - SLOTS
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
void NxView::openFinderScore( QString inScore )
{
	printf("- Dragged score to open : %s\n", inScore.toAscii().constData() ) ;
	launchGLBox( inScore, false );
}
void NxView::newScore()
{
	QString str;
	openSettings();
	
  	if ( settingsOk ) {
  		// Opening Pop Up Menu for Settings
     	if ( partName.length() > 1 ) 
			str = partName;
		else {
			str = QString("NewScore%1").arg(totalNbScore+1);
			totalNbScore +=1 ;
		}
		
		launchGLBox( str, true );				
      	
		// Finally, updating score parameters
      	NxEnv tmpEnv = GLwidgets.at(index-1)->getPartition()->getEnvironment();
      	tmpEnv.name = str;
      	tmpEnv.ID = partID;
      	tmpEnv.length = partLength;
      	tmpEnv.height = partHeight;
		GLwidgets[index-1]->getPartition()->setEnvData( tmpEnv );
		GLwidgets[index-1]->updateIndexes();
    }
}
void NxView::openFromFile()
{
	QList<QString> XMLfilesToOpen ;
	
	XMLfilesToOpen = fileChooser->getOpenFileNames(
					0,
					"Select one or more files to open",
					".",
					"XML Files (*.xml)");

	for(int i=0; i<XMLfilesToOpen.size(); i++) 
		launchGLBox( XMLfilesToOpen.at(i), false );
}
/////////////////////////////////////////////////
// NEW SCORE SETTINGS
/////////////////////////////////////////////////
void NxView::launchGLBox( QString inFileName, bool isNew )
{	
	printf("---------------------------------------\n") ;
	if(noScoreOpened) {
		myTabs->removeTab(0) ;
		enableMenuToolsSlider(true) ;
	}
	
	NxGLScore* tmpBox = new NxGLScore( NULL, this, inFileName, partLength, partHeight, isNew );
	GLwidgets << tmpBox;
	
  	myTabs->addTab( GLwidgets.last(), "tempName" );
	myTabs->setCurrentIndex( index );
	
	GLwidgets.last()->fitToPageView() ;
	
	printf("- New Score Tab added\n") ;
	
	///////////////// TAB TEXT : GET name and dimensions //////////////////////
	NxEnv tmpEnv = GLwidgets.last()->getPartition()->getEnvironment() ;
	QString nStr = QString("[%1x%2] ").arg(tmpEnv.length).arg(tmpEnv.height) ;
	nStr.append( tmpEnv.name ) ;
	myTabs->setTabText(focusTab, nStr) ;
	
	index++ ;
	
	if(firstLaunch) {
		connectFilterDisplayC() ;
		firstLaunch = false ;
	}
	noScoreOpened = false ;
	
	giveKeyboard( GLwidgets.last() );
	
	printf("---------------------------------------\n") ;
}
void NxView::openSettings()
{
	if (index > 0)
		GLwidgets[index-1]->releaseKeyboard();
  	
	opener = new QDialog;
  	opener->setWindowTitle( "New Score Settings" );	
	opener->setWindowModality( Qt::ApplicationModal ) ;
	
  	QVBoxLayout *setL = new QVBoxLayout;
	
  	///////// Groupboxes with layouts
  	QGroupBox* groupBox1 	= new QGroupBox(opener);
  	QGroupBox* groupBox2 	= new QGroupBox(opener);
  	QHBoxLayout* lay1 		= new QHBoxLayout;
  	QHBoxLayout* lay2 		= new QHBoxLayout;
	QHBoxLayout* butts 		= new QHBoxLayout;
		
  	///////// Widgets
  	QLabel* nameL 		= new QLabel( "Name", groupBox1 );
  	QLabel* IDL 		= new QLabel( "ID", groupBox1 );
  	QLineEdit* nameF 	= new QLineEdit( groupBox1 );							
  	QLineEdit* IDF 		= new QLineEdit( groupBox1 );
  	QLabel* lengthL 	= new QLabel( "Length (in s)", groupBox2 );
  	QLabel* heightL 	= new QLabel( "Height (in s)", groupBox2 );
  	QLineEdit* lengthF 	= new QLineEdit( groupBox2 );							
  	QLineEdit* heightF 	= new QLineEdit( groupBox2 );
	
  	QPushButton* validBtn = new QPushButton( "Create Score", opener );
	QPushButton* cancelBtn = new QPushButton( "Cancel", opener );
	
  	///////// Adding widgets
  	lay1->addWidget( nameL );
  	lay1->addWidget( nameF );
  	lay1->addWidget( IDL );
  	lay1->addWidget( IDF );
  	lay2->addWidget( lengthL );
  	lay2->addWidget( lengthF );
  	lay2->addWidget( heightL );
  	lay2->addWidget( heightF );
	butts->addWidget( cancelBtn ) ;
	butts->addWidget( validBtn );
  	setL->addWidget( groupBox1 );
  	setL->addWidget( groupBox2 );
  	setL->addLayout( butts ) ;
	
	groupBox1->setLayout(lay1);
	groupBox2->setLayout(lay2);
	opener->setLayout(setL);
	
  	nameL->setMaximumSize(70, 30);
  	IDL->setMaximumSize(70, 30);
  	lengthL->setMaximumSize(100, 30);
  	heightL->setMaximumSize(100, 30);
	//validBtn->setMaximumSize( 150, 30 );
	//cancelBtn->setMaximumSize( 150, 30 );
	
	//cancelBtn->setShortcut(Qt::Key_Escape) ;
	
	connect( cancelBtn, SIGNAL( clicked() ),	opener, SLOT( reject() ) );
  	connect( validBtn, SIGNAL( clicked() ),		opener, SLOT( accept() ) );
  	
	if ( opener->exec() == QDialog::Accepted ) {
    	bool ok;
      	settingsOk = true;
      	partName = nameF->text();
		
      	partID = IDF->text().toInt( &ok, 10 );
      	partLength = lengthF->text().toInt( &ok, 10 );
      	partHeight = heightF->text().toInt( &ok, 10 );
		if(partLength == 0) partLength = 10 ;
		if(partHeight == 0) partHeight = 10 ;
    }
  	else {
		settingsOk = false;
	}
}
/////////////////////////////////////////////////
// QUIT DIALOGS
/////////////////////////////////////////////////
void NxView::quitDialog()
{
	if(!noScoreOpened) {
		txtL->setText("Do you want to save changes before exit ?") ;
		saveAndQuitB->setText("Save All and Quit") ; 
		if(!quitDialogMode) {
			////// DISCONNECT SLOTS
			disconnect( quitB,			SIGNAL( clicked() ), closeDialog,	SLOT( accept() ) ) ;
			disconnect( saveAndQuitB,	SIGNAL( clicked() ), this,			SLOT( saveAndAccept() ) ) ;
			////// CONNECT SLOTS
			connect( quitB,			SIGNAL( clicked() ), qApp,	SLOT( quit() ) ) ;
			connect( saveAndQuitB,	SIGNAL( clicked() ), this,	SLOT( saveAndQuit() ) ) ;
			quitDialogMode = true ;
		}
		closeDialog->show() ;
	}
	else qApp->quit() ;
}
void NxView::saveAndQuit()
{
	closeDialog->accept() ;
	saveAll() ;
	if(allClosedSaved) qApp->quit() ;
}
void NxView::save()
{
	if(!noScoreOpened) {
		bool tmp = GLwidgets.at(focusTab)->save();
	}
}
void NxView::saveAs()
{
	if(!noScoreOpened) {
		bool tmp = GLwidgets.at(focusTab)->saveAs();
	}
}
void NxView::saveAll()
{
	allClosedSaved = true ;
	if(!noScoreOpened) {
		NxGLScore *tmpGLScore ;
		bool tmpsav ;
		for( int i=index-1; i>=0;i--) {
			myTabs->setCurrentIndex(i) ;
			tmpGLScore = (NxGLScore*)GLwidgets.at(i) ;
			giveKeyboard( (QWidget*)tmpGLScore ) ;
			tmpsav = ((NxGLScore*)tmpGLScore)->save() ;
			if(!tmpsav) allClosedSaved = false ;
		}
	}
	else allClosedSaved = false ;
}
void NxView::closeScoreDialog()
{
	if(!noScoreOpened) {
		int currFoc = myTabs->currentIndex();
		QString str = myTabs->tabText(currFoc) ;
		txtL->setText( QString("%1\nDo you want to save changes before closing ?").arg(str) ) ;
		saveAndQuitB->setText("Save and Close") ; 
		if(quitDialogMode) {
			////// DISCONNECT SLOTS
			disconnect( quitB,			SIGNAL( clicked() ), qApp,	SLOT( quit() ) ) ;
			disconnect( saveAndQuitB,	SIGNAL( clicked() ), this,	SLOT( saveAndQuit() ) ) ;
			////// CONNECT SLOTS
			connect( quitB,			SIGNAL( clicked() ), closeDialog,	SLOT( accept() ) ) ;
			connect( saveAndQuitB,	SIGNAL( clicked() ), this,			SLOT( saveAndAccept() ) ) ;
			quitDialogMode = false ;
		}
		closeDialog->show() ;
	
		closeOk = saveOk = false ;
	
		if ( closeDialog->exec() == QDialog::Accepted ) {
			closeOk = true ;
		}
		else {
			closeOk = false ;
		}
	}
}
void NxView::saveAndAccept()
{
	saveOk = true ;
	closeDialog->accept() ;
}
void NxView::closeScore()
{	
	if(!noScoreOpened) {
		closeScoreDialog() ;
	
		if(closeOk) {
			int tFocus = myTabs->currentIndex();
			bool savedSuceeded = false ;
			if(saveOk) 
				savedSuceeded = GLwidgets.at(tFocus)->save() ;
			
			if( savedSuceeded || !saveOk ) {
				GLwidgets.at(tFocus)->updateUDPListener( false ) ; // Stop OSC Listening
				myTabs->removeTab( tFocus );
				GLwidgets.removeAt(tFocus);
				index--;
				if(index==0) { // NO MORE SCORE OPENED
					noScoreOpened = true ;
					showNoScoreText() ;
					enableMenuToolsSlider(false) ;
				}
				else { // RE GIVE KEYBOARD TO SCORES
					int currFoc = myTabs->currentIndex();
					giveKeyboard( (QWidget*)GLwidgets.at(currFoc) ) ;
				}
			}
		}
	}
}
void NxView::closeAll()
{	
	if(!noScoreOpened) {
		NxGLScore *tmpGLScore ;
		for( int i=index-1; i>=0 ; i-- ) {
			myTabs->setCurrentIndex(i) ;
			tmpGLScore = (NxGLScore*)GLwidgets.at(i) ;
			giveKeyboard( (QWidget*)tmpGLScore ) ;
			closeScore() ;
		}
	}
}
void NxView::closeAllButActive()
{
	if(!noScoreOpened) {
		int tFocus = myTabs->currentIndex();
	
		NxGLScore *tmpGLScore ;
		for( int i=index-1; i>tFocus ; i-- ) {
			myTabs->setCurrentIndex(i) ;
			tmpGLScore = (NxGLScore*)GLwidgets.at(i) ;
			giveKeyboard( (QWidget*)tmpGLScore ) ;
			closeScore() ;
		}
		for( int i=tFocus-1; i>=0 ; i-- ) {
			myTabs->setCurrentIndex(i) ;
			tmpGLScore = (NxGLScore*)GLwidgets.at(i) ;
			giveKeyboard( (QWidget*)tmpGLScore ) ;
			closeScore() ;
		}
		giveKeyboard( GLwidgets.last() );
	}
}
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
// CUT / COPY / PASTE / DUPLICATE - TOOLS
/////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////
void NxView::cut()
{
	if(!noScoreOpened) {
		int nbS = GLwidgets.at(focusTab)->getNumberSelected() ;
		
		////////////////// CLEAR
		if(cursCopied.size()!=0)	cursCopied.clear() ;
		if(copied.size()!=0)		copied.clear() ;
		
		if( GLwidgets.at(focusTab)->getHighlighted() != NULL && !GLwidgets.at(focusTab)->getHighlighted()->isSelected() ) { /////////////// ONE HIGHLIGHTED, NOT SELECTED
			pauseScore() ;
			
			NxObject *highlObj = GLwidgets.at(focusTab)->getHighlighted() ;
			copied.append( highlObj ) ;
			/////// If trajectory, we have to copy the cursors too ...
			if( highlObj->getObjectType() == Trajectory ) {
				QList<int> indcur = ((NxTraject*)highlObj)->getCursorIDs() ;
				for(int i=0; i<indcur.count(); i++) { // save the cursors
					cursCopied.append( (NxCursor*)GLwidgets.at(focusTab)->getPartition()->getCursor(indcur.at(i)) ) ;
				}
			}
			GLwidgets.at(focusTab)->destroyObject( highlObj ) ;
		}
		else if(nbS!=0) { /////////////////////////////////////////////////////////////// COPY THE LIST OF SELECTED OBJECTS
			pauseScore() ;
			
			QList<NxObject*> selected = GLwidgets.at(focusTab)->getSelected() ;
			for(int k=0;k<nbS;k++) {
				copied.append( selected.at(k) ) ;
				/////// If trajectory, we have to copy the cursors too ...
				if( selected.at(k)->getObjectType() == Trajectory ) {
					QList<int> indcur = ((NxTraject*)selected.at(k))->getCursorIDs() ;
					for(int i=0; i<indcur.count(); i++) { // save the cursors
						cursCopied.append( (NxCursor*)GLwidgets.at(focusTab)->getPartition()->getCursor(indcur.at(i)) ) ;
					}
				}
			}
			GLwidgets.at(focusTab)->removeSelected() ;
		}
		else
			QMessageBox::about(this, "Warning", "Nothing to cut") ;
	}
}
void NxView::copy()
{
	if(!noScoreOpened) {
		incObjectID = 1 ;
		int nbS = GLwidgets.at(focusTab)->getNumberSelected() ;
		
		////////////////// CLEAR
		if(cursCopied.size()!=0)	cursCopied.clear() ;
		if(copied.size()!=0)		copied.clear() ;
		
		if( GLwidgets.at(focusTab)->getHighlighted() != NULL && !GLwidgets.at(focusTab)->getHighlighted()->isSelected() ) { /////////////// ONE HIGHLIGHTED, NOT SELECTED
			NxObject *highlObj = GLwidgets.at(focusTab)->getHighlighted() ;
			copied.append( highlObj ) ;
			/////// If trajectory, we have to copy the cursors too ...
			if( highlObj->getObjectType() == Trajectory ) {
				QList<int> indcur = ((NxTraject*)highlObj)->getCursorIDs() ;
				for(int i=0; i<indcur.count(); i++) { // save the cursors
					cursCopied.append( (NxCursor*)GLwidgets.at(focusTab)->getPartition()->getCursor(indcur.at(i)) ) ;
				}
			}
		}
		else if(nbS!=0) { //////////////////////////////////////////////////////////////// COPY THE LIST OF SELECTED OBJECTS
			QList<NxObject*> selected = GLwidgets.at(focusTab)->getSelected() ;
			for(int k=0;k<nbS;k++) {
				copied.append( selected.at(k) ) ;
				/////// If trajectory, we have to copy the cursors too ...
				if( selected.at(k)->getObjectType() == Trajectory ) {
					QList<int> indcur = ((NxTraject*)selected.at(k))->getCursorIDs() ;
					for(int i=0; i<indcur.count(); i++) { // save the cursors
						cursCopied.append( (NxCursor*)GLwidgets.at(focusTab)->getPartition()->getCursor(indcur.at(i)) ) ;
					}
				}
			}
		}

		else
			QMessageBox::about(this, "Warning", "Nothing to copy") ;
	}
}

void NxView::basicPaste()
{
	paste(true, false) ;
}

void NxView::paste(bool incr, bool dec) // INCR : to increment the custom ID // DEC : to shift the object horizontaly
{
	if (!noScoreOpened && (copied.size()!=0) ) {
		
		GLwidgets.at(focusTab)->deselectAll() ;
		int indCursor = 0 ; // initialisation
		
		// Normal Parameters
		NxObject	*toPaste ;
		int			incID, inGroupID, layer, inPort, inID , inIntValue, drawt, inSpatialPeriod, trajectType ;
		QString		inName, inHost, inOscF ;
		Color		inColor1, inColor2 ;
		NxPoint		inPos, clicPos ;
		inPos.ID = clicPos.ID = 0 ;												// initialisation
		clicPos.x = clicPos.y = clicPos.z = inPos.x = inPos.y = inPos.z = 0 ;	// initialisation
		// Specific Parameters
		QVector<NxPoint>	editedCurve ;
		float				realMin, realMax, scale, angle ;
		QList<int>	indcur ;
		int			nbCursors ;
		
		NxScore *tmpScore = GLwidgets.at(focusTab)->getPartition() ;
	
		NxPoint	gravPos ;
		gravPos.ID = 0 ;						// initialisation
		gravPos.x = gravPos.y = gravPos.z = 0 ;	// initialisation
		/////////////////////////////////////////// Determine where to paste
		if(!dec) { // not duplicating step case
			clicPos = GLwidgets.at(focusTab)->getCursor() ;
			inPos = clicPos ;
			if(copied.size()>1) { //////// many objects -> group gravity center
				for(int u=0;u<copied.size();u++) {
					gravPos.x += copied.at(u)->getPosition().x ;
					gravPos.y += copied.at(u)->getPosition().y ;
				}
				gravPos.x /= copied.size() ;
				gravPos.y /= copied.size() ;
			}
		}
		
		for(int u=0;u<copied.size();u++) { // For all objects copied
			toPaste		= copied.at(u) ;
			incID		= toPaste->getCustomID() ;
			inGroupID	= toPaste->getGroupID() ;
			layer		= toPaste->getLayer() ;
			inName		= toPaste->getName() ;
			inColor1	= toPaste->getColor() ;
	
			if(incr) {
				incID += incObjectID ;
				inName.append( QString("_c%1").arg(incObjectID) ) ;
				// only one time for each paste :
				if(u==0) incObjectID += 1 ;
			}	
			if(dec) {
				// only one time for each paste :
				if(u==0) decObjectX += GLwidgets.at(focusTab)->gridStepInt / 100.0 ;
				inPos.x = toPaste->getPosition().x + decObjectX ;
				inPos.y = toPaste->getPosition().y ;
				inPos.z = toPaste->getPosition().z ;
			}
			else
			if(copied.size()>1) { // place objects around the gravity center
				inPos.x = toPaste->getPosition().x + clicPos.x - gravPos.x ;
				inPos.y = toPaste->getPosition().y + clicPos.y - gravPos.y ;
				inPos.z = toPaste->getPosition().z ;
			}
	
			switch ( toPaste->getObjectType() ) {
				case Curve: /////////////////////// curve
		
					pauseScore() ;
			
					inID		= tmpScore->getCptCurve() ;
					inColor2			= ((NxCurve*)toPaste)->getCircleColor() ;
					editedCurve			= ((NxCurve*)toPaste)->getCurve() ;
					realMin				= ((NxCurve*)toPaste)->getRealMin() ;
					realMax				= ((NxCurve*)toPaste)->getRealMax() ;
					scale				= ((NxCurve*)toPaste)->getScale() ;
					angle				= ((NxCurve*)toPaste)->getAngle() ;
					drawt				= ((NxCurve*)toPaste)->getDrawType() ;
					inSpatialPeriod		= ((NxCurve*)toPaste)->getSpatialPeriod() ;
					inOscF				= ((NxCurve*)toPaste)->getOscFormat() ;
					inHost				= ((NxCurve*)toPaste)->getHost() ;
					inPort				= ((NxCurve*)toPaste)->getPort() ;
					tmpScore->addCurve( inPos, inID, incID, inGroupID, layer, inName, inColor1, inColor2, editedCurve, editedCurve.count(), realMin, realMax, scale, angle, inSpatialPeriod, inOscF, inHost, inPort );
					GLwidgets.at(focusTab)->updateIndexes() ;
					((NxCurve*)tmpScore->getCurve( inID ))->setDrawType( drawt ) ;
					GLwidgets.at(focusTab)->getControlCenter()->updateItems( Curve, true, inID, false ) ;
					((NxCurve*)tmpScore->getCurve( inID ))->setPosition(inPos) ; // useful ?
					((NxCurve*)tmpScore->getCurve( inID ))->updateBB( GLwidgets.at(focusTab)->BBsize ) ;
					tmpScore->maxOfIndexes() ;
					GLwidgets.at(focusTab)->getScheduler()->addNewCurve() ;
					
					GLwidgets.at(focusTab)->addObjectToSelection( ((NxCurve*)tmpScore->getCurve( inID )) ) ;
				break;
				case Trajectory: /////////////////////// trajectory
			
					inID = tmpScore->getCptTraject() ;
					trajectType = ((NxTraject*)toPaste)->getType() ;
					switch (trajectType) {
						case 1 : ////// Circle
							tmpScore->addTraject( inPos, inID, incID, inGroupID, layer, inName, inColor1, 1 );
							tmpScore->getTraject( inID )->setCircleDef( ((NxTraject*)toPaste)->getCircle() );
						break;		
						case 2 : ////// Line
							tmpScore->addTraject( inPos, inID, incID, inGroupID, layer, inName, inColor1, 2 );
							tmpScore->getTraject( inID )->setOrientation( ((NxTraject*)toPaste)->getOrientation() );
							tmpScore->getTraject( inID )->setLineDef( ((NxTraject*)toPaste)->getInitialPoint(), ((NxTraject*)toPaste)->getFinalPoint() );
						break;
					}
					tmpScore->getTraject( inID )->updateBB( GLwidgets.at(focusTab)->BBsize ) ;
					GLwidgets.at(focusTab)->updateIndexes();
					GLwidgets.at(focusTab)->getControlCenter()->updateItems( Trajectory, true, inID, false );
					tmpScore->maxOfIndexes() ;
					
					indcur		= ((NxTraject*)toPaste)->getCursorIDs() ;
					nbCursors	= indcur.count() ;
					
					if(nbCursors!=0) pauseScore() ;
					
					////////// We now have to copy the correspondant cursors /////////
					for(int i=0; i<nbCursors; i++) {
						indCursor += i ; 
						NxCursor *toCop = cursCopied.at(indCursor) ;
						// Get cursor infos
						int newCursID	= tmpScore->getCptCursor() ;
						QString newNam	= toCop->getName().append("_c") ;
						float initValue		= toCop->getInitOffset() ;
						float finalValue	= toCop->getFinalOffset() ;
						if(trajectType==1) { // CIRCLE
							initValue	= toCop->getInitAngle() ;
							finalValue	= toCop->getFinalAngle() ;
						}
						// Add cursor
						tmpScore->addCursor( toCop->getPosition(), newCursID, toCop->getCustomID(), (NxTraject*)tmpScore->getTraject(inID), toCop->getGroupID(), toCop->getLayer(), newNam, toCop->getColor(), toCop->getForward(), toCop->getMode(), 0, initValue, finalValue );
						tmpScore->getCursor(newCursID)->setSpeed( toCop->getSpeed() );
						tmpScore->getCursor(newCursID)->setWidth( toCop->getWidth() );
						tmpScore->getCursor(newCursID)->setPlayFinished( toCop->getPlayFinished() ) ;
						if(trajectType==1) { // CIRCLE
							tmpScore->getCursor(newCursID)->setAngle(  toCop->getAngle(), true ) ;
							tmpScore->getCursor(newCursID)->reEvalAngles() ;
						}
						else { // LINE
							tmpScore->getCursor(newCursID)->setOffset(  toCop->getOffset() ) ;
							tmpScore->getCursor(newCursID)->reEvalOffsets() ;
						}
						tmpScore->getCursor(newCursID)->updateBB( GLwidgets.at(focusTab)->BBsize ) ;
						GLwidgets.at(focusTab)->updateIndexes();
						GLwidgets.at(focusTab)->assignCursors();
						GLwidgets.at(focusTab)->getControlCenter()->updateItems( 2, true, newCursID, false );	
						tmpScore->maxOfIndexes() ;
					}
					indCursor += 1 ; // for the next copied trajectory
					
					if(cursCopied.count()!=0) GLwidgets.at(focusTab)->getScheduler()->resetSchedule() ;
					
					GLwidgets.at(focusTab)->addObjectToSelection( tmpScore->getTraject( inID ) ) ;
					
				break;
				case 2: /////////////////////// cursor
					// DO NOTHING
				break;
				case Trigger: /////////////////////// trigger

					pauseScore() ;
					
					int triggID = tmpScore->getCptTrigger() ;
					
					inOscF		= ((NxTrigger*)toPaste)->getOscFormat() ;
					inHost		= ((NxTrigger*)toPaste)->getHost() ;
					inPort		= ((NxTrigger*)toPaste)->getPort() ;
					inIntValue	= ((NxTrigger*)toPaste)->getValue() ;

					tmpScore->addTrigger( inPos, triggID, incID, inGroupID, layer, inName, inColor1, inOscF, inHost, inPort, inIntValue );
					GLwidgets.at(focusTab)->updateIndexes();
					GLwidgets.at(focusTab)->getControlCenter()->updateItems( Trigger, true, triggID, false );
					tmpScore->getTrigger( triggID )->updateBB( GLwidgets.at(focusTab)->BBsize ) ;
					GLwidgets.at(focusTab)->getScheduler()->addNewTrigger() ;
					tmpScore->maxOfIndexes() ;
					
					GLwidgets.at(focusTab)->addObjectToSelection( tmpScore->getTrigger( triggID ) ) ;
				break;
				
			} // end switch(objectType)
			printf("%d objects copied\n",u+1) ;
		} // end for(all copied)
		GLwidgets.at(focusTab)->getControlCenter()->resizeTables() ;
	} // end if(somethingToPaste)
	else
		QMessageBox::about(this, "Warning", "Nothing to paste") ;
}

void NxView::duplicate()
{
	if (copied.size()!=0 && !noScoreOpened) {
		lastCopied = copied ;
	}
	copy() ;
	if (copied.size()!=0  && !noScoreOpened) {
		if( lastCopied != copied ) decObjectX = 0 ;
	}
	if (copied.size()!=0  && !noScoreOpened) paste(false, true) ;
}

// Temp func (DEBUG)
void NxView::tmp() {}

//////////////////////////////////////////////////////////////////////////////////////////////
// Keyboard handler
/////////////////////////////////////////////////////////////////////////////////////////////
void NxView::reGiveHand()
{
	if(!noScoreOpened) {
		//printf("hand\n") ;
		focusTab = myTabs->currentIndex();
		GLwidgets.at(focusTab)->setFocus() ;
	}
}
void NxView::giveKeyboard( QWidget* inW )
{
	if(!noScoreOpened) {
		//printf("giveKeyboard(widget)\n") ;
		for(int i=0 ; i<index ; i++) { ////// Release all keyboard focus and manage timers
			GLwidgets.at(i)->releaseKeyboard() ;
			GLwidgets.at(i)->releaseMouse() ;
			GLwidgets.at(i)->stopGLTimer() ;
			GLwidgets.at(i)->updateUDPListener( false ) ; // Stop OSC Listening
		}
	
		focusTab = myTabs->currentIndex();
		GLwidgets.at(focusTab)->startGLTimer() ;
		//////// TEMPO / ZOOM
		tbTempoF->setText( QString("%1").arg( (float)((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getTempo() ) ) ;
		tempoSlider->setValue(  int( ((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getTempo() * 100 )  );
		this->setWindowTitle( QString("IanniX Viewer - %1 %2").arg( GLwidgets.at(focusTab)->getZoomScale() ).arg('%') );
		
		GLwidgets.at(focusTab)->updateCheckedMenus() ;
		GLwidgets.at(focusTab)->updateGroupFilterList() ;
		GLwidgets.at(focusTab)->updateLayerFilterList() ;
		
		setToolBarPlayback() ;
		GLwidgets.at(focusTab)->updateAntialiasing() ;
		((NxGLScore*)inW)->updateUDPListener( ((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getEnvironment().oscInMode ) ; // Re-init OSC Listener if active
				
		inW->setMouseTracking( true );
		inW->setFocus() ; // (Grab Keyboard too)
	}
}
void NxView::setToolBarInfosTxt( QString inTxt )
{
	tbInfoTxt->setText( inTxt ) ;
}
void NxView::setToolBarTime(NxGLScore* score, float inTime ) // refresh time value
{
	if( score == GLwidgets.at(focusTab) )
	{
		tbMin	= (int)floor( inTime/60.0 ) ;
		tbSec	= (int)floor( (inTime - tbMin*60.0) ) ;
		tbMs	= (int)floor( (inTime - tbMin*60.0 - tbSec)*1000.0 );
		time = QString(" Time : %1:%2:%3 ")
					.arg(tbMin, 3, 10, QChar( QLatin1Char('0') ) )
					.arg(tbSec, 2, 10, QChar( QLatin1Char('0') ) )
					.arg(tbMs, 3, 10, QChar( QLatin1Char('0') ) ) ;
					
		tbTime->setText( time ) ;
	}
}
void NxView::setToolBarPlayback() ///////// set play / pause icons
{
	usleep(10000) ;
	NxEnv tmpEnv = GLwidgets[focusTab]->getPartition()->getEnvironment() ;
	QString nStr = QString("[%1x%2] ").arg(tmpEnv.length).arg(tmpEnv.height) ;
	nStr.append( tmpEnv.name ) ;
	
	if( GLwidgets.at(focusTab)->isPlaying() ) {
		playPause->setIcon( pauseIcon ) ;
		myTabBar->setTabTextColor( focusTab,  Qt::red ) ;
	}
	else {
		playPause->setIcon( playIcon ) ;
		myTabBar->setTabTextColor( focusTab,  Qt::black ) ;
	}
	myTabs->setTabText(focusTab, nStr) ;
}
void NxView::setTbTempo()
{
	// Due to the Bug : 1.050000*100 = 104.999992 (?)
	int tempTempo = (int)floor( (100.0 * tbTempoF->text().toFloat()) + 0.1 ) ;
	changeTempo( tempTempo ) ;
}
void NxView::changeTempo( int inTempo )
{
	if(!noScoreOpened) {
		( (NxGLScore*)myTabs->widget(focusTab) )->setTempo( (float) ( inTempo / 100.0) );

		GLwidgets.at(focusTab)->setFocus() ;
		tbTempoF->setText( QString("%1").arg( inTempo/100.0 ) ) ;
		tempoSlider->setValue( inTempo );
	}
}
void NxView::changeTempoTrigg( int inTempo )
{
	if(!noScoreOpened) {
		if(inTempo==3) {
			float newTempo = ((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getTempo() + 0.1 ;
			( (NxGLScore*)myTabs->widget(focusTab) )->setTempo( newTempo );
			tbTempoF->setText( QString("%1").arg( newTempo ) ) ;
		}
		if(inTempo==4) {
			float newTempo = ((NxGLScore*)myTabs->widget(focusTab))->getPartition()->getTempo() - 0.1 ;
			( (NxGLScore*)myTabs->widget(focusTab) )->setTempo( newTempo );
			tbTempoF->setText( QString("%1").arg( newTempo ) ) ;
		}
	
		GLwidgets.at(focusTab)->setFocus() ;
	}
}
void NxView::resetTempo()
{
	if(!noScoreOpened) {
		( (NxGLScore*)myTabs->widget(focusTab) )->setTempo( 1.0 );

		GLwidgets.at(focusTab)->setFocus() ;
		tbTempoF->setText( "1" ) ;
		tempoSlider->setValue( 100 );
	}
}
void NxView::hardOffsetsReset() 
{
	if(!noScoreOpened) ( (NxGLScore*)myTabs->widget(focusTab) )->hardOffsetsReset() ;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////// MENUS SLOTS
////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////
// SCORE Modules Display
//////////////////////////////////
void NxView::showObjectCreator() {
	if(!noScoreOpened) GLwidgets[focusTab]->openCreator();
}
void NxView::showGroupEditor() {
	if(!noScoreOpened) GLwidgets[focusTab]->editGroup();
}
void NxView::showControl() {
	if(!noScoreOpened) GLwidgets[focusTab]->openControl();
}
void NxView::showOscConsole() {
	if(!noScoreOpened) GLwidgets[focusTab]->openOscConsole();
}
void NxView::showAdvanced() {
	if(!noScoreOpened) GLwidgets[focusTab]->open3DCenter();
}
void NxView::showPreferences() {
	if(!noScoreOpened) GLwidgets[focusTab]->openPreferences() ;
}
//////////////////////////////////
// Externals Handle
//////////////////////////////////
void NxView::launchPd()
{
	/*external = new QProcess;
	
	external->start("sh", QStringList() << "OSCControl/launchPd");
	if ( !external->start() )
	{
		QMessageBox* errorBox = new QMessageBox(this);
  		errorBox->setIcon( QMessageBox::Critical );
		errorBox->setText( "Unable to launch PureData !! \nPlease install it first, or check your system paths..." );
		errorBox->show();
	}*/
}
void NxView::launchMaxMSP()
{
}
void NxView::launchExternal()
{
}
void NxView::showCPUinfo()
{
	QMessageBox::information( this, "CPU Infos" , 
		"The time value displayed (in ms) correspond\nto the scheduler step\n\n"
		"The CPU warning light is turned on when :\n\n"
		"- Computer is too slow (scheduler step has to be\nincreased)\n"
		"- At least one cursor is going too fast (loss of\ninformations / curve scan period)"
		) ;
}
void NxView::showShortcuts()
{
	QDialog* aboutBox = new QDialog(this);
	aboutBox->setWindowTitle("Shortcuts") ;
	//aboutBox->setFixedSize(490, 570) ;
	
	QPushButton* ok = new QPushButton("OK",aboutBox) ;
	QLabel* txt = new QLabel(aboutBox) ;
	
	QVBoxLayout* lay = new QVBoxLayout(aboutBox) ;
	lay->addWidget(txt) ;
	lay->addWidget(ok) ;
	
	txt->setText(
			"<b>Selection and Editing</b> <blockquote><br>"
			"<b>D</b>	-	Drag Score<br>"
			"<b>Backspace</b>	-	Delete an Object<br>"
			"<b>E</b>		-	Objects Edition<br>"
			"<b>A</b>		-	Curve Angle Edition<br>"
			"<b>A</b>		-	Add a Point (Curve Edition)<br>"
			"<b>K</b>		-	Remove a Point (Curve Edition)<br><hr></blockquote>"
			"<b>Display</b> <blockquote><br>"
			"<b>-/+</b>			Zoom - / +<br>"
			"<b>Z</b>		-	Custom Zoom Rectangle<br>"
			"<b>X</b>		-	Line Rule Mode<br>"
			"<b>V</b>		-	2D / 3D View<br>"
			"<b>'Less'</b>		-	3D View Mouse Rotation<br>"
			"<b>1/2</b>		-	3D View Y-Rotation<br>"
			"<b>3/4</b>		-	3D View X-Rotation<br>"
			"<b>F</b>		-	Show / Hide Cursors Areas<br>"
			"<b>C</b>		-	Show / Hide Curves Circles<br>"
			"<b>M</b>		-	Show / Hide On-Screen Informations<br>"
			"<b>N</b>		-	Show / Hide Objects Names<br>"
			"<b>P</b>		-	Show / Hide Score<br>"
			"<b>G</b>		-	Show / Hide Grid<br>"
			"<b>S</b>		-	Snap to Grid Mode<br><hr></blockquote>"
			"<b>Playback</b> <blockquote><br>"
			"<b>Space</b>	-	Play / Pause<br>"
			"<b>W</b>		-	Reset Cursors Positions and Scheduler<br></blockquote>"
		) ;
	
	aboutBox->setLayout(lay) ;
	aboutBox->setWindowModality( Qt::ApplicationModal ) ;
	connect( ok, SIGNAL( clicked() ), aboutBox, SLOT( close() ) );
	aboutBox->show();
}
void NxView::about() 
{	
	QPixmap logoKitch( ":/pixmaps/logoKitch.png" );
	QPixmap logoCult( ":/pixmaps/logoCult.png" );
  	QDialog* aboutBox = new QDialog(this);
	aboutBox->setWindowTitle("About IanniX") ;
	aboutBox->setFixedSize(520, 640) ;
	
	QLabel* iconKitch = new QLabel(aboutBox) ;
	iconKitch->setPixmap(logoKitch) ;
	QLabel* iconCult = new QLabel(aboutBox) ;
	iconCult->setPixmap(logoCult) ;
	QLabel* icontxt = new QLabel(aboutBox) ;
	icontxt->setAlignment( Qt::AlignCenter );
	QLabel* txt = new QLabel(aboutBox) ;
	txt->setAlignment( Qt::AlignHCenter );
	QPushButton* ok = new QPushButton("OK",aboutBox) ;
	icontxt->setText(
		    "<b>IanniX v0.650 Beta</b><br>"
		    "<b>2007 (c) la kitchen</b><br>"
		) ;		
	txt->setText(
			"<b>Project Manager</b><br>"
			"Thierry Coduys<br><br>"
			"<b>Contributors</b><br>"
			"Adrien Lefevre<br>"
			"Gerard Pape (CCMIX)<br><br>"
			"<b>Developpers</b><br>"
			"Adrien Lefevre - v0.54<br>"
			"Guillaume Ferry - v0.60<br>"
			"Cyrill Duneau - v0.6xx<br>"
			"Pierre Jullian de la Fuente - v0.6xx<br><br><br>"
			"<b>Thanks to</b><br>"
			"Jean Bresson, Paul Crabbe, Cyrille Henry, Joachim Montessuis,<br>Thomas Thiery, Romain Vuillet,<br>"
			"and all users for their observations and suggestions.<br><br><br>"
			"see<br>" 
			"<b>http://iannix.la-kitchen.fr/iannix.html</b><br>"
			"for more informations<br><br>"
		) ;
	//txt->setOpenExternalLinks( true ) ;
	
	QVBoxLayout* lay = new QVBoxLayout(aboutBox) ;
	QHBoxLayout* hlay = new QHBoxLayout(aboutBox) ;
	hlay->addWidget(iconKitch) ;
	hlay->addWidget(icontxt) ;
	hlay->addWidget(iconCult) ;

	lay->addLayout(hlay) ;
	lay->addWidget(txt) ;
	lay->addWidget(ok) ;
	
	aboutBox->setLayout(lay) ;
	aboutBox->setWindowModality( Qt::ApplicationModal ) ;
	
	//connect( txt, SIGNAL( linkActivated ( const QString & link) ), this, SLOT( launchWebLink( const QString & link ) ) );
	connect( ok, SIGNAL( clicked() ), aboutBox, SLOT( close() ) );
	aboutBox->show();
}
void NxView::launchWebLink( const QString & link )
{
	//bool tmp = QDesktopServices::openUrl( QUrl( link ) ) ;
}
//////////////////////////////////////////////////////////////////
// TUTORIALS
//////////////////////////////////////////////////////////////////
void NxView::launchTutorial( QAction* action )
{
	QString name = (action->text()).remove(0,1) ;
	QString inScore = QString("./Tutorials/").append(name).append(".xml") ;
	printf("- Tutorial to open : %s\n", inScore.toAscii().constData() ) ;
	QFile file(inScore) ;
	if( file.exists() )
		launchGLBox( inScore, false );
	else
		QMessageBox::about(this, "Warning !", "Tutorial file was not found.") ;
}
////////////////////////
// EDIT MENU
////////////////////////
void NxView::selectAll()
{
	if(!noScoreOpened) GLwidgets[focusTab]->selectAll() ;
}
void NxView::deselectAll()
{
	if(!noScoreOpened) GLwidgets[focusTab]->deselectAll() ;
}
void NxView::font()
{
	bool ok;
  	QApplication::setFont( QFontDialog::getFont( &ok ) );	
}

void NxView::languageChange()
{
}

///////////////////////////////////////////////
///////////////////////////////////////////////
void NxView::setCurrentTabText( QString inText )
{
	int tFocus = myTabs->currentIndex();
	myTabs->setTabText( tFocus, inText ) ;
}
/////////////// CPU Light Warning /////////////
void NxView::setCPUWarning( bool act )
{
	if(!cpuWarning && act) {
		tbCpu->setIcon( cpuOnIcon ) ;
		cpuWarning = true ;
	}
	if(cpuWarning && !act) {
		tbCpu->setIcon( cpuOffIcon ) ;
		cpuWarning = false ;
	}
}
void NxView::setCPUTimeStep( int val )
{
	tbCpuValue->setText( QString("%1 ms").arg( val ) ) ;
}
//////////////////////////////////////////////////////////////////////////////
//////// Settings Shortcuts
//////////////////////////////////////////////////////////////////////////////
bool NxView::getAntialiasing()
{
	return antialiasing ;
}
void NxView::updateCheckedMenus( bool algo, bool moveC, bool broadcM, bool visuM, bool oscifM, bool oscOutM, bool oscInM, bool snap, bool gridS, bool autostep, bool trajS, bool cursS, bool curvS, bool trigS, bool crossesS, bool circlesS, bool rectS, bool cursAS, bool infosS, bool namesS, bool workspaceS, bool viewtype )
{	
	////////////////////////////////////////// RENDERING
	rTrajectsC->setChecked( trajS ) ;
	rCursorsC->setChecked( cursS ) ;
	rCurvesC->setChecked( curvS ) ;
	rTriggersC->setChecked( trigS ) ;
	////////////////////////////////////////// TIME / OSC
	algoChoose->setChecked( algo ) ;
	oscSendIFChoose->setChecked( oscifM ) ;
	moveCursorsChoose->setChecked( moveC ) ;
	autoManualStep->setChecked( autostep ) ;
	broadcastChoose->setChecked( broadcM ) ;
	////////////////////////////////////////// MODES
	visualModeChoose->setChecked( visuM ) ;
	oscOutModeChoose->setChecked( oscOutM ) ;
	oscInModeChoose->setChecked( oscInM ) ;
	////////////////////////////////////////// DISPLAY
	snapGrid->setChecked( snap ) ;
	gridShow->setChecked( gridS ) ;
	crossesShow->setChecked( crossesS ) ;
	circlesShow->setChecked( circlesS ) ;
	curvesRectShow->setChecked( rectS ) ;
	cursArea->setChecked( cursAS ) ;
	infosShow->setChecked( infosS ) ;
	namesShow->setChecked( namesS ) ;
	workspaceShow->setChecked( workspaceS ) ;
	view3d->setChecked( viewtype ) ;
}
void NxView::switchCirclesAlgo() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchCirclesAlgo() ;
}
void NxView::switchSendIFmessages() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchSendIFmessages() ;
}
void NxView::switchMoveCursors() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchMoveCursors() ;
}
void NxView::switchBroadcastMode() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchBroadcastMode() ;
}
void NxView::switchVisualMode() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchVisualMode() ;
}
void NxView::switchOSCoutMode() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchOSCoutMode() ;
}
void NxView::switchOSCinMode() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchOSCinMode() ;
}
void NxView::setPlaybackTime() {
	if(!noScoreOpened) GLwidgets[focusTab]->changePlaybackTime() ;
}
void NxView::change2D3DView() {
	if(!noScoreOpened) GLwidgets[focusTab]->change2D3DView() ;
}
void NxView::switchAntialiasing() {
	if(!noScoreOpened) {
		antialiasing = !antialiasing ;
		GLwidgets[focusTab]->updateAntialiasing() ;
	}
}
////////////////////////////////////////// Switch Fullscreen Mode
void NxView::exitFullscreen() {
	if(fullscreen) switchFullscreen() ;
}
void NxView::switchFullscreen() {
	fullscreen = !fullscreen ;
	if(fullscreen) {
		tBar->close() ;
		statusBar()->close() ;
		myTabBar->close() ;
		showFullScreen() ;
	}
	else {
		tBar->show() ;
		statusBar()->show() ;
		myTabBar->show() ;
		showNormal() ;
		for(int i=0;i<index;i++) GLwidgets.at(i)->showNormal() ;
	}
}
//////////////////////////////////////////////////////////////////////////////////////
// DISPLAY FILTERS by Object Type
//////////////////////////////////////////////////////////////////////////////////////
void NxView::showHideTrajects() {
	if(!noScoreOpened) {
		GLwidgets[focusTab]->showHideTrajects() ;
		GLwidgets[focusTab]->setFocus() ;
	}
}
void NxView::showHideCursors() {
	if(!noScoreOpened) {
		GLwidgets[focusTab]->showHideCursors() ;
		GLwidgets[focusTab]->setFocus() ;
	}
}
void NxView::showHideCurves() {
	if(!noScoreOpened) {
		GLwidgets[focusTab]->showHideCurves() ;
		GLwidgets[focusTab]->setFocus() ;
	}
}
void NxView::showHideTriggers() {
	if(!noScoreOpened) {
		GLwidgets[focusTab]->showHideTriggers() ;
		GLwidgets[focusTab]->setFocus() ;
	}
}
//////////////////////////////////////////////////////////////////////////////////////
// DISPLAY FILTERS by Group/Layer
//////////////////////////////////////////////////////////////////////////////////////
void NxView::updateGroupFilter() 
{
	if(!noScoreOpened) {
		//printf("updateGroupFilterList (retrieving from NxGL List)\n") ;
		// Clear List
		groupFilterC->clear() ;
		QList<int> gpList = GLwidgets[focusTab]->getGroupFilterList() ;
		groupFilterC->addItem("All Groups") ;
		for(int u=0;u<gpList.count();u++) {
			groupFilterC->addItem( QString("Group %1").arg(gpList.at(u)) ) ;
		}
		groupFilterC->setCurrentIndex(0) ;
	}
}
void NxView::updateLayerFilter() 
{
	if(!noScoreOpened) {
		// Clear List
		layerFilterC->clear() ;
		QList<int> layList = GLwidgets[focusTab]->getLayerFilterList() ;
		layerFilterC->addItem("All Layers") ;
		for(int u=0;u<layList.count();u++) {
			layerFilterC->addItem( QString("Layer %1").arg(layList.at(u)) ) ;
		}
		layerFilterC->setCurrentIndex(0) ;
	}
}
void NxView::connectFilterDisplayC()
{
	//printf("Connecting Filters Comboboxes\n") ;
	connect(groupFilterC, SIGNAL( currentIndexChanged(QString) ),	this, SLOT( showHideGroups(QString) ) ) ;
	connect(layerFilterC, SIGNAL( currentIndexChanged(QString) ),	this, SLOT( showHideLayers(QString) ) ) ;
}
void NxView::showHideGroups( QString inStr ) {
	if(!noScoreOpened) {
		//printf("Index Changed : update GLBox bool and int\n") ;
		if(inStr.startsWith("All"))
			GLwidgets[focusTab]->showHideGroups( 0, false ) ;
		else {
			int gp = inStr.remove(0,5).toInt() ;
			GLwidgets[focusTab]->showHideGroups( gp, true ) ;
		}
		GLwidgets[focusTab]->setFocus() ;
	}
}
void NxView::showHideLayers( QString inStr ) {
	if(!noScoreOpened) {
		if(inStr.startsWith("All"))
			GLwidgets[focusTab]->showHideLayers( 0, false ) ;
		else {
			int lay = inStr.remove(0,5).toInt() ;
			GLwidgets[focusTab]->showHideLayers( lay, true ) ;
		}
		GLwidgets[focusTab]->setFocus() ;
	}
}
//////////////////////////////////////////////////////////////////////////////////////
// DISPLAY OPTIONS
//////////////////////////////////////////////////////////////////////////////////////
void NxView::showHideCrosses() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideCrosses() ;
}
void NxView::showHideGrid() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideGrid() ;
}
void NxView::showHideScore() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideScore() ;
}
void NxView::showHideCurvesCircles() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideCurvesCircles() ;
}
void NxView::showHideCurvesRect() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideCurvesRect() ;
}
void NxView::showHideCursorsArea() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideCursorsArea() ;
}
void NxView::showHideScreenInfos() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideScreenInfos() ;
}
void NxView::showHideObjectsNames() {
	if(!noScoreOpened) GLwidgets[focusTab]->showHideObjectsNames() ;
}
void NxView::switchSnapToGrid() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchSnapToGrid() ;
}
void NxView::switchAutoGridStep() {
	if(!noScoreOpened) GLwidgets[focusTab]->switchAutoGridStep() ;
}
void NxView::setGridStep() {
	if(!noScoreOpened) GLwidgets[focusTab]->changeGridStep() ;
}
void NxView::resetView() {
	if(!noScoreOpened) GLwidgets[focusTab]->fitToPageView() ;
}




