///////////////
// IanniX
///////////////

////////////////////////////
// NxGLScore.cpp
////////////////////////////

// C standard libs
#include <cstdlib>
#include <iostream>

// Class Header
#include "NxGLScore.h"

////////////////////////////////
// Warning disabling
///////////////////////////////
#if defined(Q_CC_MSVC)
#pragma warning(disable:4305) 
#endif

//////////////////////
// Constructor
//////////////////////
NxGLScore::NxGLScore( QWidget* parent, NxView* daddy, QString inFile, int inL, int inH, bool isNew )
  : QGLWidget( parent )
{
	mParent = (NxView*)daddy;
	
	doubleClickOk = false ;
			
	neverSaved = firstClick = lineRuleSetFirstPt = true ;
	dragging = false ;
	frustum = onDrag = onClick = firstPoint = zoomMode = scoreDragMode = false ;
	cursorMode = cursorMoving = mouseZoom = false ;
  	curveEdition = curveCreation = curveAngleEdition = false ;
	trajectCreation = lineEdition = circleEdition = cursorEdition = false ;
	curvEdPtChoosed = false ;	// pt choosed to be moved (on a curve)
	curvEdOnlyClic = true ;		// to avoid 'only clic' bug
	lockedSelection = false ;	// for selection lock by contextual menu
	lineRuleMode = gridMove = false ;
	gridDecX = 0 ;		// Grid X translation
	gridDecY = 0 ;		// Grid Y translation
		
	///////////////////////////////////////////// SELECTION
	objectDragging = selectionMode = scoreDragging = false ;
	shiftPressed = dPressed = false ;
	selected.clear() ;
	highlighted = NULL ;
	numberSelected = 0 ;
	
	UDPListenerCreated = false ;
	
	mouseXPixPos = mouseYPixPos = 0 ;
	timePosPix.x = timePosPix.y = timePosPix.z = 0 ;
	posX = posY = transX = transY = diffX = diffY = 0;
  	mouseX = mouseY = 0;
	currentPoints = 1 ;			// Counter for the curve creation
  	pointID = -1 ;				// curves points edition
	selTrajectPoint = -1 ;		// traject lines edition
	selCursorPoint = -1 ;		// cursor width edition
	drawMode = LineStrip ;
	curveXMinStep = 0.001 ;		// x-step between values on a curve
	
  	//////////////////// COLORS /////////////////////////////
	selectedColor.ID = BBColor.ID = inactiveColor.ID = hiddenColor.ID = 0 ;
	selectedColor.c		= QColor(255,255,255,255) ;
	BBColor.c			= QColor(255,127,0,255) ;
	inactiveColor.c		= QColor(90, 90, 90, 150) ;	
	hiddenColor.c		= QColor(90, 90, 90, 60) ;
	/////////////////////////////////////////////////////////
	
  	setMouseTracking( true ) ;
	
  	/////////// Opening score
  	if (inFile.contains(".xml")) {
		score = new NxScore(inFile);
		neverSaved = false ;
	}
  	else score = new NxScore(inFile, 0, inL, inH);	
	
	////// XML VERSION TEST //////
	QString xmlVersion = score->getEnvironment().xmlVersion ;
	if(xmlVersion=="0.631b" || xmlVersion=="0.635b" || xmlVersion=="0.636b" || xmlVersion=="0.637b" || xmlVersion=="0.638b" || xmlVersion=="0.639b" || xmlVersion=="0.640b" || xmlVersion=="0.650b") printf("- Opening Score - XML Version(v%s) OK\n", xmlVersion.toAscii().constData()) ;
	else QMessageBox::about(this, "Warning !", "The XML score seems to be from an older\nversion of IanniX (before 0.631b).\nPlease check the XML and re-open the score.\n") ;
		
	int numb = inFile.lastIndexOf("/") ;
	inFile.truncate(numb+1) ;
	NxEnv tmpEnv = score->getEnvironment() ;
  	tmpEnv.path = inFile ;
	score->setEnvData( tmpEnv ) ;
	
  	////////// Retrieving global indexes
  	nbCurves 	= score->getEnvironment().nbCurves;
  	nbTrajects 	= score->getEnvironment().nbTrajects;
  	nbTriggers 	= score->getEnvironment().nbTriggers;
  	nbCursors 	= score->getEnvironment().nbCursors;
  	
	if (isNew) {
		length = inL ;
  		height = inH ;
	}
	else {
		length = score->getEnvironment().length ;
		height = score->getEnvironment().height ;
	}
	
	/////////////////////////////////////// RENDERING FILTER by layers/groups
	showAllG = showAllL = true ;
	showOnlyG = showOnlyL = 0 ;
	
	///////////////////////////////////////////////////////////////////////////
	///////////////// ENVIRONMENT VARIABLES INITIALISATION ////////////////////
	updateScoreColors() ;
	////////////////////////////// MODES
	visualMode		= score->getEnvironment().visualMode ;
	oscOutMode		= score->getEnvironment().oscOutMode ;
	oscInMode		= score->getEnvironment().oscInMode ;
	showTrajects	= score->getEnvironment().showTrajects ;
	showCursors		= score->getEnvironment().showCursors ;
	showCurves		= score->getEnvironment().showCurves ;
	showTriggers	= score->getEnvironment().showTriggers ;
	////////////////////////////// DISPLAY
	updateLocalDisplayParameters() ; // Put environment variables in the correspondant local variables
	antialiasing	= true ;
	// OPACITIES
	gridOpacity			= score->getEnvironment().gridOp ;
	cursZonesOpacity	= score->getEnvironment().zonesOp ;
	namesOpacity		= score->getEnvironment().namesOp ;
	// Grid (showGrid variable included in 'updateLocalDisplayParameters()')
	snapGrid		= score->getEnvironment().snapGrid ;		
	autoGridStep	= score->getEnvironment().autoGridStep ;
	gridStep		= score->getEnvironment().gridStep ;
	if(gridStep > 0.01) gridStepInt = int(floor( gridStep*100 )) ;
	else gridStepInt = 1 ;
	////////////////////////////// TIME
	useCirclesAlgo	= score->getEnvironment().useCirclesAlgo ;
	broadcastMode	= score->getEnvironment().broadcastMode ;
	sendIFmessages	= score->getEnvironment().sendIFmessages ;
	// non Env
	moveCursors		= true ;

	//////////////////////// Objects Initialization ////////////////////////
	printf("- Creating IanniX Object Creator\n") ;
  	objectCreator		= new NxObjectCreator( this );
	printf("- Creating IanniX Control Center\n") ;
  	controlCenter		= new NxControlCenter( this );
	printf("- Creating IanniX Object Editor\n") ;
	objectEditor		= new NxObjectEditor( this );

	preferencesDialog	= new NxPreferences( this ) ;
	
	timeSpeedEditor		= new NxTime( this, 1 ) ; 
	timeWidthEditor		= new NxTime( this, 0 ) ;
	fileChooser			= new QFileDialog;
	printf("- Creating IanniX Scheduler\n") ;
	timeScheduler		= new NxScheduler( this );
	
	timeScheduler->moveCursors( moveCursors ) ;
	timeScheduler->useAutoSchedulerStep( true ) ;
	timeScheduler->useBroadcastMode( broadcastMode ) ;
	timeScheduler->setOSCOutActive( oscOutMode ) ;
	timeScheduler->setInitialTime( score->getEnvironment().initialTime ) ;
	timeScheduler->setFinalTime( score->getEnvironment().finalTime ) ;
	timeScheduler->useCirclesAlgorithm( useCirclesAlgo, false ) ;
	timeScheduler->updateSendingOptions() ;

	timeScheduler->initScheduler();	
	
	printf("- Creating IanniX OSC Console\n") ;
	oscConsole			= new NxOscConsole( this, timeScheduler );
	//////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////////////////

	
  	/////////////////////////////////////////////////////
  	////////////// OpenGL Section
  	////////////////////////////////////////////////////
	//////// Retrieving GL Widget size
	
	initSelfW = selfW = 796 ;
	initSelfH = selfH = 483 ;
	
	scaleX = scaleY = scaleZ = 0.17 ;
	RScaleX = RScaleY = 1.0 ;
  	nHeight = (float)length*initSelfH/initSelfW ; // depends of initial selfH and selfW (Qmainwindow)
	NbSLICES = 60 ; // number of segments for 360 degrees
  	nbCercles = 0 ;
	
	angleX = angleY = oldAngleX = oldAngleY = 0.0 ;
	mouseRotation = false ;
  	bX = length / (RScaleX*scaleX*10.0) ;
  	bY = nHeight / (RScaleY*scaleY*10.0) ;
		
	BBsize = (float)length/(600*scaleX) ; // init first time -> initial cursor selection
		
  	////// Initializing randomizer seed with current time
  	randomizer = static_cast<unsigned>(time(NULL)) ;

	glTimer = new QTimer( this );
	connect(glTimer, SIGNAL(timeout()), this, SLOT(refreshView()));
	
	//////////////////////////////////////////////////////////////////
	////////////////////////// UDP RECEIVER //////////////////////////
	oscCurveCreation = false ;
	oscCurveListening = false ;
	listener = new NxListener( this, oscConsole ) ;
	if(oscInMode) updateUDPListener( true ) ; // Listener Creation
	//////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////////
	
	updateCheckedMenus() ;
	
	updateGroupFilterList() ;
	updateLayerFilterList() ;
	
	assignCursors() ;
	printf("- Opening Done\n") ;

}
void NxGLScore::updateScoreColors()
{
	bColor = score->getEnvironment().backColor ;
	sColor = score->getEnvironment().scoreColor ;
	gColor = score->getEnvironment().gridColor ;
}
void NxGLScore::updateLocalDisplayParameters()
{
	crossesDraw		= score->getEnvironment().crossesDraw ;	
	scoreDraw		= score->getEnvironment().scoreDraw ;	
	displayOn		= score->getEnvironment().displayOn ;
	showNames		= score->getEnvironment().namesDisplay ;
	circlesDraw		= score->getEnvironment().circlesDraw ;	
	rectDraw		= score->getEnvironment().rectDraw ;
	showCursorsArea = score->getEnvironment().showCursorsArea ;
	showGrid		= score->getEnvironment().showGrid ;
}
////////////////////
// Destructor
///////////////////
NxGLScore::~NxGLScore()
{
	//delete objectCreator;
  	//delete controlCenter;
  	//delete fileChooser;
}
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
//////////////////// UDP OSC SERVER
///////////////////////////////////////////////////////////////////////////////////////////////
void NxGLScore::updateUDPListener( bool chooser ) {
	if( chooser && !UDPListenerCreated ) {
		printf("Create and Connect OSC UDP Listener Slot (port=%d) \n", score->getEnvironment().UDPin ) ;
		udpReceiver = new QUdpSocket( this );
		udpReceiver->bind( score->getEnvironment().UDPin );
		connect(udpReceiver, SIGNAL(readyRead()), this, SLOT(processUDPMessage()) ) ;
		UDPListenerCreated = true ;
	}
	if( !chooser && UDPListenerCreated ) {
		printf("Disconnect and Delete OSC UDP Listener\n") ;
		disconnect(udpReceiver, SIGNAL(readyRead()), this, SLOT(processUDPMessage()) ) ;
		delete( udpReceiver ) ;
		UDPListenerCreated = false ;
	}
}
void NxGLScore::processUDPMessage()
{
	//printf("Message Received OK !! - Process receiver...\n") ;	
	while ( udpReceiver->hasPendingDatagrams() ) {
		QByteArray		datagram ;
		datagram.resize( udpReceiver->pendingDatagramSize() ) ;
		QHostAddress	sender ;
		quint16			senderPort ;
		udpReceiver->readDatagram( datagram.data(), datagram.size(), &sender, &senderPort ) ;

		//printf("MESSAGE UDP Datagram : %s \n", datagram.data() ) ;
		
		osc::ReceivedPacket packetOSC( datagram.data(), datagram.size() ) ;
		if( !packetOSC.IsBundle() ) {
			//printf("OSC - Message non Bundle\n") ;
			osc::ReceivedMessage messageOSC( packetOSC ) ;
			listener->ProcessMessage( messageOSC, 0 ) ;
		}
		else {
			//printf("OSC - Message Bundle\n") ;
			listener->ProcessBundle( osc::ReceivedBundle(packetOSC), 0 ) ;
		}
	}
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// SCORE
void NxGLScore::oscSendInfos()
{
	timeScheduler->sendOSCConfigMessage() ;
}
void NxGLScore::oscSetTempo( float inTempo )
{
	mParent->changeTempo( (int)floor(100 * inTempo) ) ;
}
void NxGLScore::oscMoveCursors( int chooser )
{	
	moveCursors = (chooser==1) ;
	timeScheduler->moveCursors( moveCursors ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::oscBroadcastMode( int chooser )
{
	broadcastMode = (chooser==1) ;
	timeScheduler->useBroadcastMode( broadcastMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::oscOSCMode( int chooser ) 
{
	oscOutMode = (chooser==1) ;
	timeScheduler->setOSCOutActive( oscOutMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// TRIGGER CREATION
void NxGLScore::oscNewTrigger( float inX, float inY, float inZ, int inCID, int inGID, int inIntValue, int colR, int colG, int colB  )
{	
	//printf("OSC Creation : TRIGGER ADDED !\n") ;
	// Position
	NxPoint inPos ;
	inPos.ID = 0 ;
	inPos.x = inX ;
	inPos.y = inY ;
	inPos.z = inZ ;
	// Color
	QColor col = QColor(colR, colG, colB) ;
	Color inColor ;
	inColor.ID = 0 ;
	inColor.c = col ;
	// Others
	int inLay = 0 ;
	QString inOscF = score->getEnvironment().OSCTriggersDefaultMFormat ;
	QString inName = QString("oscTrigger") ;
	QString inHost = score->getEnvironment().UDPhost ;
	int inID = score->getCptTrigger() ;
	int inPort = score->getEnvironment().UDPout ;
	
	addNewTrigger( inPos, inID, inCID, inGID, inLay, inName, inColor, inOscF, inHost, inPort, inIntValue ) ;
	score->maxOfIndexes() ;
	
	updateGroupFilterList() ;
	updateLayerFilterList() ;
}
void NxGLScore::oscTriggerValue( int inID, int inValue )
{
	if( score->isThereTrigger( inID ) ) { // Test of existence
		((NxTrigger*)score->getTrigger( inID ))->setValue( inValue ) ;
		controlCenter->updateParameters( Trigger, inID, true ) ;
	}
	else printf("OSC Trigger Value - Cursor doesn't exist\n") ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// CURVES CREATION
void NxGLScore::oscCurveCreationBegin( float inPx, float inPy, float inPz, int inCID, int inGID, int c1R, int c1G, int c1B )
{
	if(!oscCurveCreation) {
		printf("Curve Creation\n") ;
		// Initialisation
		oscCurveRealMin = oscCurveRealMax = 0 ;
		oscNbPoints = 0 ;
		
		// Position
		oscCurve1R = c1R ;
		oscCurve1G = c1G ;
		oscCurve1B = c1B ;
		oscCurvePos.ID = 0 ;
		oscCurvePos.x = inPx ;
		oscCurvePos.y = inPy ;
		oscCurvePos.z = inPz ;
		// Parameters
		oscCurveCID = inCID ;
		oscCurveGID = inGID ;
		
		oscCurveCreation = true ;
		oscCurveListening = true ;
	}
}
void NxGLScore::oscCurveCreationAddPt( float inPx, float inPy )
{
	if(oscCurveListening) {
		NxPoint oscNewPt ;
		oscNewPt.z = 0 ;
		oscNewPt.ID = oscNbPoints ;
		oscNewPt.x = inPx ;
		oscNewPt.y = inPy ;
		// Force inPx to advance
		float lastx = 0 ;
		if(oscNbPoints != 0) {
			lastx = ( oscEditedCurve.last() ).x ;
			if(inPx<lastx+curveXMinStep) oscNewPt.x = lastx + curveXMinStep ;
		}
		else oscNewPt.x = 0 ;
		
		//printf("Add point %d // x: %f y : %f\n", oscNbPoints, oscNewPt.x, oscNewPt.y ) ;
		
		oscEditedCurve.append( oscNewPt ) ;
		
		// Curve Real Bounds update
		if(inPy>oscCurveRealMax) oscCurveRealMax = inPy ;
		if(inPy<oscCurveRealMin) oscCurveRealMin = inPy ;
		
		oscNbPoints++ ;
	}
}
void NxGLScore::oscCurveCreationEnd()
{
	if(oscNbPoints>=2) {
		
		//pause() ; // Stop Playback

		// Fixed Parameters
		int realID = score->getCptCurve() ;
		int inLay = 0 ;
		float inScale = 1 ;
		float inAngle = 0 ;
		int inSpatialPeriod = 20 ;
		int inPort = score->getEnvironment().UDPout ;
		QString inOscF = score->getEnvironment().OSCCurvesDefaultMFormat  ;
		QString inHost = score->getEnvironment().UDPhost ;
		QString savName = "oscCurve" ;
		Color inColor1, inColor2 ;
		QColor col1 = QColor(oscCurve1R, oscCurve1G, oscCurve1B) ;
		QColor col2 = QColor(50, 200, 50) ;
		inColor1.ID = inColor2.ID =  0 ;
		inColor1.c = col1 ;
		inColor2.c = col2 ;
	
		// ADD CURVE
		printf("End : add curve\n") ;
	
		score->addCurve( oscCurvePos, realID, oscCurveCID, oscCurveGID, inLay, savName, inColor1, inColor2, oscEditedCurve, oscEditedCurve.count(), oscCurveRealMin, oscCurveRealMax, inScale, inAngle, inSpatialPeriod, inOscF, inHost, inPort ) ;
		score->getCurve(realID)->setDrawType( LineStrip ) ;
		score->getCurve(realID)->updateBB( BBsize ) ;
		score->maxOfIndexes() ;
		updateIndexes() ;
		controlCenter->updateItems( Curve, true, realID, true ) ;
		timeScheduler->addNewCurve() ;
		
		updateGroupFilterList() ;
		updateLayerFilterList() ;
		
		oscCurveListening = oscCurveCreation = false ;
		oscEditedCurve.clear() ;
	}
	else {
		oscCurveListening = oscCurveCreation = false ;
		if(oscNbPoints!=0) oscEditedCurve.clear() ;
	}
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// CURVE POINT MODIFICATION
void NxGLScore::oscCurveModifPt( int inID, int nb, float inX, float inY )
{
	if( score->isThereCurve( inID ) ) { // Test Curve existence
		printf("Curve Point Modif\n") ;
		NxCurve *curv = score->getCurve( inID ) ;
		int initialSize = curv->size() ;
		if( nb < initialSize && nb>0 ) { // Test Point existence
			QVector<NxPoint> oscSelCurve = curv->getCurve() ;
			NxPoint cvP		= curv->getPosition() ;
			float cvMin		= curv->getRealMin() ;
			float cvMax		= curv->getRealMax() ;
			NxPoint newP = oscSelCurve[nb] ;
			newP.x = inX ;
			newP.y = inY ;
			
			///// Constrain the position
			if( nb != 0 )
				if (newP.x < oscSelCurve[nb-1].x + curveXMinStep )	newP.x = oscSelCurve[nb-1].x + curveXMinStep ;
			if( nb!=initialSize-1 )
				if (newP.x > oscSelCurve[nb+1].x - curveXMinStep )	newP.x = oscSelCurve[nb+1].x - curveXMinStep ;
			if( newP.y > cvMax )	curv->setRealMax( newP.y ) ;
			if( newP.y < cvMin )	curv->setRealMin( newP.y ) ;

			oscSelCurve[nb] = newP ;
			curv->setCurve( oscSelCurve ) ;
			curv->updateBB( BBsize ) ;
			controlCenter->updateParameters( Curve, inID, true ) ;
		}
	}
	else printf("Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveAddPt( int inID, float inX, float inY )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		printf("Curve Point Add\n") ;
		if( inX>0 ) { // we don't do anything if x < 0
			NxCurve *curv = score->getCurve( inID ) ;
			QVector<NxPoint> oscSelCurve = curv->getCurve() ;
		
			NxPoint newP ;
			newP.z = 0 ;
			int initialSize = curv->size() ;
			int newID ;
			int localID = -1 ;
			bool IDfound = false ;
			float cvMin		= curv->getRealMin() ;
			float cvMax		= curv->getRealMax() ;
			NxPoint cvP		= curv->getPosition() ;
		
			while(!IDfound) { /////// Search new point location
				localID++ ;
				if( (inX < oscSelCurve[localID].x ) || (localID == initialSize) ) IDfound = true ;
			}
			
			////// we don't do anything if there isn't any x-space left
			if( fabs( oscSelCurve[localID-1].x-oscSelCurve[localID].x )>=curveXMinStep*2 ) 
			{
				oscSelCurve.append(newP) ; // enlarge the array size
			
				newID = localID ;
		
				for(int l=initialSize-1 ; l>=newID ; l--) {
					oscSelCurve[l+1] = oscSelCurve[l];
					oscSelCurve[l+1].ID = l+1 ;
				}

				newP.x = inX ;
				newP.y = inY ;
		
				//////////////////// Constrain the position
				if( newID != 0 )
					if (newP.x < oscSelCurve[newID-1].x + curveXMinStep )	newP.x = oscSelCurve[newID-1].x + curveXMinStep ;
				if( (newID!=initialSize-1) && (newID!=initialSize) )
					if (newP.x > oscSelCurve[newID+1].x - curveXMinStep )	newP.x = oscSelCurve[newID+1].x - curveXMinStep ;
				if( newP.y > cvMax )	curv->setRealMax( newP.y ) ;
				if( newP.y < cvMin )	curv->setRealMin( newP.y ) ;
		
				newP.ID = newID ;
				oscSelCurve[newID] = newP ;
		
				curv->setCurve( oscSelCurve );
				curv->updateBB( BBsize ) ;
				controlCenter->updateParameters( Curve, inID, true ) ;
			}
			else printf("No space left for that x - point wasn't added on the curve\n") ;
		}
		else printf("The point has a negative abscisse - it wasn't added on the curve\n") ;
	}
	else printf("Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveRmPt( int inID, int inN )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		printf("Curve Point Removing\n") ;
		NxCurve *curv = score->getCurve( inID ) ;
		
		if ( curv->size() > 2 && inN>0 ) { // do not remove if nb < 2 or first point
			QVector<NxPoint> oscSelCurve = curv->getCurve() ;
			NxPoint newPt	= curv->getPosition() ; // curve position
			int initialSize = curv->size() ;
			
			for(int l=inN ; l<initialSize-1 ; l++) {
				oscSelCurve[l] = oscSelCurve[l+1];
				oscSelCurve[l].ID = l ;
			}
			
			oscSelCurve.remove(initialSize-1) ;
			curv->setCurve( oscSelCurve );
			curv->updateBB( BBsize ) ;
			controlCenter->updateParameters( Curve, inID, true ) ;
		}
	}
	else printf("Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveAngle( int inID, float inAngle )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		((NxCurve*)score->getCurve( inID ))->setAngle( inAngle ) ;
		controlCenter->updateParameters( Curve, inID, true ) ;
	}
	else printf("OSC Curve Angle - Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveScale( int inID, float inScale )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		NxCurve *scCurv = score->getCurve( inID ) ;
		scCurv->setScale( inScale ) ;
		scCurv->updateBB( BBsize ) ;
		controlCenter->updateParameters( Curve, inID, true ) ;
	}
	else printf("OSC Curve Scale - Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveStyle( int inID, int inStyle )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		NxCurve *scCurv = score->getCurve( inID ) ;
		if(inStyle==1)	scCurv->setDrawType( SplineStrip ) ;
		else			scCurv->setDrawType( LineStrip ) ;
		controlCenter->updateParameters( Curve, inID, true ) ;
	}
	else printf("OSC Curve Drawing Style - Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveMin( int inID, float inMin )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		((NxCurve*)score->getCurve( inID ))->setRealMin( inMin ) ;
		controlCenter->updateParameters( Curve, inID, true ) ;
	}
	else printf("OSC Curve Min - Curve doesn't exist\n") ;
}
void NxGLScore::oscCurveMax( int inID, float inMax )
{
	if( score->isThereCurve( inID ) ) { // Test of existence
		((NxCurve*)score->getCurve( inID ))->setRealMax( inMax ) ;
		controlCenter->updateParameters( Curve, inID, true ) ;
	}
	else printf("OSC Curve Max - Curve doesn't exist\n") ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// TRAJECTS PARAMETERS
void NxGLScore::oscNewCircle( float inX, float inY, float inZ, float inRadius, int inCID, int inGID, int inR, int inG, int inB )
{
	NxPoint inPos ;
	inPos.ID = 0 ;
	inPos.x = inX ;
	inPos.y = inY ;
	inPos.z = inZ ;
	Color inColor ;
	inColor.ID = 0 ;
	inColor.c = QColor(inR,inG,inB) ;
	
	int inID = score->getCptTraject() ;
	int inLay = 0 ;
	QString inName = QString("oscCircle") ;
	
	addNewTraject( inPos, inID, inCID, inGID, inLay, inName, inColor, 1, 0, inRadius ) ;
	score->maxOfIndexes() ;
	
	updateGroupFilterList() ;
	updateLayerFilterList() ;
}
void NxGLScore::oscNewLine( float inX1, float inY1, float inZ1, float inX2, float inY2, float inZ2, int inCID, int inGID, int inR, int inG, int inB )
{
	NxPoint inPos1, inPos2 ;
	inPos1.ID = inPos2.ID = 0 ;
	inPos1.x = inX1 ;
	inPos1.y = inY1 ;
	inPos1.z = inZ1 ;
	inPos2.x = inX2 ;
	inPos2.y = inY2 ;
	inPos2.z = inZ2 ;
	Color inColor ;
	inColor.ID = 0 ;
	inColor.c = QColor(inR,inG,inB) ;

	int inID = score->getCptTraject() ;
	int inLay = 0 ;
	QString inName = QString("oscLine") ;
	
	score->addTraject( inPos1, inID, inCID, inGID, inLay, inName, inColor, 2 );
	score->getTraject( inID )->setOrientation( 4 ) ; // Free Line
	score->getTraject( inID )->setLineDef( inPos1, inPos2 ) ;
	updateIndexes();
	controlCenter->updateItems( Trajectory, true, inID, true );
	
	score->maxOfIndexes() ;
	
	updateGroupFilterList() ;
	updateLayerFilterList() ;
}
void NxGLScore::oscTrajectRadius( int inID, float inRadius )
{
	if( score->isThereTraject( inID ) && inRadius!=0 ) { // Test of existence
		if( ((NxTraject*)score->getTraject( inID ))->getType() == 1 ) { // Circle only
			NxTraject* radTraj = score->getTraject( inID ) ;
			Circle newC ;
			newC.radius = inRadius ;
			newC.p = radTraj->getPosition() ;
			radTraj->setCircleDef( newC ) ; 
			radTraj->updateBB( BBsize ) ;

			updateCursorsAnglePosition( radTraj ) ;
			controlCenter->updateParameters( Trajectory, inID, true ) ;
		}
		else printf("OSC Radius - Lines don't have radius\n") ;
	}
	else printf("OSC Radius - Circle doesn't exist (or = 0)\n") ;
}
void NxGLScore::oscTrajectLength( int inID, float inLength )
{
	if( score->isThereTraject( inID ) && inLength!=0 ) { // Test of existence
		NxTraject* lenTraj = score->getTraject( inID ) ;
		lenTraj->setLength( fabs(inLength) ) ;
		lenTraj->updateBB( BBsize ) ;

		updateCursorsAnglePosition( lenTraj ) ;
		controlCenter->updateParameters( Trajectory, inID, true ) ;
		assignCursors() ;
	}
	else printf("OSC Length - Traject doesn't exist (or = 0)\n") ;
}
void NxGLScore::oscTrajectPoint( int inID, int firstPt, float inX, float inY, float inZ )
{
	if( score->isThereTraject( inID ) ) { // Test of existence
		NxTraject *pointTraj = (NxTraject*)score->getTraject( inID ) ;
		if( pointTraj->getType() == 2 ) { // Line only
			NxPoint inPos ;
			inPos.ID = 0 ;
			inPos.x = inX ;
			inPos.y = inY ;
			inPos.z = inZ ;
		
			switch(firstPt) {
				case 0: // first point
					pointTraj->setInitialPoint( inPos ) ;
				break ;
				case 1: // 2nd point
					pointTraj->setFinalPoint( inPos ) ;
				break ;
			}
			updateCursorsAnglePosition( pointTraj ) ;
		}
		else printf("OSC Line Pts - Circle ?? \n") ;
	}
	else printf("OSC Line Pts - Line doesn't exist\n") ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// CURSORS PARAMETERS
void NxGLScore::oscNewCursor( int inID, int inCID, int inGID, float inSpeed, float inWidth, int inForward, int inMode, int inR, int inG, int inB )
{
	if( score->isThereTraject( inID ) ) { // Test of existence
		NxTraject *pTraj = score->getTraject( inID ) ; // Parent
		
		NxPoint inPos ;
		inPos.ID = 0 ;
		inPos.x = inPos.y = inPos.z = 0.0 ; // Init only
		Color inColor ;
		inColor.ID = 0 ;
		inColor.c = QColor(inR,inG,inB) ;
		
		int inID = score->getCptCursor() ;
		int inLay = pTraj->getLayer() ;
		QString inName = QString("oscCursor") ;
		bool forw = (inForward==1) ;
		
		float inOff1 = 0.0 ;
		float inOff2 ;
		if(pTraj->getType()==1) // CIRCLE
			inOff2 = 360 ;
		else // LINE
			inOff2 = pTraj->getLength() ;
			
		pause() ; // Stop playback
		
		score->addCursor( inPos, inID, inCID, pTraj, inGID, inLay, inName, inColor, forw, inMode, inWidth, inOff1, inOff2 );
		score->getCursor(inID)->reEvalOffsets() ;
		score->getCursor(inID)->setSpeed( inSpeed );
		score->getCursor(inID)->setWidth( inWidth );
		score->getCursor(inID)->updateBB( BBsize ) ;
		updateIndexes();
		assignCursors();
		controlCenter->updateItems( 2, true, inID, true );	
		timeScheduler->addNewCursor() ;		
		score->maxOfIndexes() ;
		
		updateGroupFilterList() ;
		updateLayerFilterList() ;
	}
	else printf("OSC New Cursor - Traject doesn't exist\n") ;
}
void NxGLScore::oscCursorOffset( int inID, float inOffsetPC )
{
	if( score->isThereCursor( inID ) ) { // Test of existence
		NxCursor *cur = (NxCursor*)score->getCursor( inID ) ;
		if(cur->getType()==1) { // CIRCLE
			cur->setAngle( inOffsetPC, true ) ;
			cur->reEvalAngles() ;
		}
		else { // LINE
			cur->setOffset( inOffsetPC ) ;
			cur->reEvalOffsets() ;
		}
	}
	else printf("OSC Offset - Cursor doesn't exist\n") ;
}
void NxGLScore::oscCursorInitOffset( int inID, float inOffsetPC )
{
	if( score->isThereCursor( inID ) ) { // Test of existence
		NxCursor *cur = (NxCursor*)score->getCursor( inID ) ;
		if(cur->getType()==1) { // CIRCLE
			cur->setInitAngle( inOffsetPC ) ;
			cur->reEvalAngles() ;
		}
		else { // LINE
			cur->setInitOffset( inOffsetPC ) ;
			cur->reEvalOffsets() ;
		}
	}
	else printf("OSC Init Offset - Cursor doesn't exist\n") ;
}
void NxGLScore::oscCursorFinalOffset( int inID, float inOffsetPC )
{
	if( score->isThereCursor( inID ) ) { // Test of existence
		NxCursor *cur = (NxCursor*)score->getCursor( inID ) ;
		if(cur->getType()==1) { // CIRCLE
			cur->setFinalAngle( inOffsetPC ) ;
			cur->reEvalAngles() ;
		}
		else { // LINE
			cur->setFinalOffset( inOffsetPC ) ;
			cur->reEvalOffsets() ;
		}
	}
	else printf("OSC Final Offset - Cursor doesn't exist\n") ;
}
void NxGLScore::oscCursorForward( int inID, int playb )
{
	if( score->isThereCursor( inID ) && (playb==0||playb==1) ) { // Test of existence
		((NxCursor*)score->getCursor( inID ))->setForward( (playb==1) ) ;
		controlCenter->updateParameters( 2, inID, true ) ;
	}
	else printf("OSC Forward - Cursor doesn't exist (or wrong value)\n") ;
}
void NxGLScore::oscCursorMode( int inID, int mode )
{
	if( score->isThereCursor( inID ) && (mode==0||mode==1||mode==2) ) { // Test of existence
		((NxCursor*)score->getCursor( inID ))->setMode( mode ) ;
		controlCenter->updateParameters( 2, inID, true ) ;
	}
	else printf("OSC Mode - Cursor doesn't exist (or wrong value)\n") ;
}
void NxGLScore::oscCursorWidth( int inID, float wid )
{
	if( score->isThereCursor( inID ) ) { // Test of existence
		((NxCursor*)score->getCursor( inID ))->setWidth( wid ) ;
		controlCenter->updateParameters( 2, inID, true ) ;
	}
	else printf("OSC Width - Cursor doesn't exist\n") ;
}
void NxGLScore::oscCursorSpeed( int inID, float speed )
{
	if( score->isThereCursor( inID ) ) { // Test of existence
		((NxCursor*)score->getCursor( inID ))->setSpeed( speed ) ;
		controlCenter->updateParameters( 2, inID, true ) ;
	}
	else printf("OSC Speed - Cursor doesn't exist\n") ;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////// OBJET GENERAL PARAMETERS
void NxGLScore::oscDeleteObject( int type, int inID ) 
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) )
				destroyObject( (NxTraject*)score->getTraject( inID ) ) ;
			else printf("OSC rm - Traject doesn't exist\n") ;
		break ;
		case 2:
			if( score->isThereCursor( inID ) )
				destroyObject( (NxCursor*)score->getCursor( inID ) ) ;
			else printf("OSC rm - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) )
				destroyObject( (NxCurve*)score->getCurve( inID ) ) ;
			else printf("OSC rm - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) )
				destroyObject( (NxTrigger*)score->getTrigger( inID ) ) ;
			else printf("OSC rm - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectLayer( int type, int inID, int inLay )
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setLayer( inLay ) ;
				controlCenter->updateParameters( Trajectory, inID, true ) ;
			}
			else printf("OSC Layer - Trajectory doesn't exist\n") ;
		break ;
		case 2:
			if( score->isThereCursor( inID ) ) {
				score->getCursor( inID )->setLayer( inLay ) ;
				controlCenter->updateParameters( 2, inID, true ) ;
			}
			else printf("OSC Layer - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setLayer( inLay ) ;
				controlCenter->updateParameters( Curve, inID, true ) ;
			}
			else printf("OSC Layer - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setLayer( inLay ) ;
				controlCenter->updateParameters( Trigger, inID, true ) ;
			}
			else printf("OSC Layer - Trigger doesn't inLay\n") ;
		break ;
	}
	updateLayerFilterList() ;
}
void NxGLScore::oscObjectName( int type, int inID, const char *inName )
{
	QString inNam = QString( inName ) ;
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setName( inNam ) ;
				controlCenter->updateParameters( Trajectory, inID, true ) ;
			}
			else printf("OSC Name - Trajectory doesn't exist\n") ;
		break ;
		case 2:
			if( score->isThereCursor( inID ) ) {
				score->getCursor( inID )->setName( inNam ) ;
				controlCenter->updateParameters( 2, inID, true ) ;
			}
			else printf("OSC Name - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setName( inNam ) ;
				controlCenter->updateParameters( Curve, inID, true ) ;
			}
			else printf("OSC Name - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setName( inNam ) ;
				controlCenter->updateParameters( Trigger, inID, true ) ;
			}
			else printf("OSC Name - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectCID( int type, int inID, int inCID )
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setCustomID( inCID ) ;
				controlCenter->updateParameters( Trajectory, inID, true ) ;
			}
			else printf("OSC CID - Trajectory doesn't exist\n") ;
		break ;
		case 2:
			if( score->isThereCursor( inID ) ) {
				score->getCursor( inID )->setCustomID( inCID ) ;
				controlCenter->updateParameters( 2, inID, true ) ;
			}
			else printf("OSC CID - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setCustomID( inCID ) ;
				controlCenter->updateParameters( Curve, inID, true ) ;
			}
			else printf("OSC CID - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setCustomID( inCID ) ;
				controlCenter->updateParameters( Trigger, inID, true ) ;
			}
			else printf("OSC CID - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectGID( int type, int inID, int inGID )
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setGroupID( inGID ) ;
				controlCenter->updateParameters( Trajectory, inID, true ) ;
			}
			else printf("OSC GID - Trajectory doesn't exist\n") ;
		break ;
		case 2:
			if( score->isThereCursor( inID ) ) {
				score->getCursor( inID )->setGroupID( inGID ) ;
				controlCenter->updateParameters( 2, inID, true ) ;
			}
			else printf("OSC GID - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setGroupID( inGID ) ;
				controlCenter->updateParameters( Curve, inID, true ) ;
			}
			else printf("OSC GID - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setGroupID( inGID ) ;
				controlCenter->updateParameters( Trigger, inID, true ) ;
			}
			else printf("OSC GID - Trigger doesn't exist\n") ;
		break ;
	}
	updateGroupFilterList() ;
}
void NxGLScore::oscObjectPosition( int type, int inID, float inx, float iny, float inz )
{
	NxPoint newPos ;
	newPos.ID = 0 ;
	newPos.x = inx ;
	newPos.y = iny ;
	newPos.z = inz ;
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setPosition( newPos ) ;
				score->getTraject( inID )->updateBB( BBsize ) ; 
				updateCursorsAnglePosition( score->getTraject(inID) ) ; }
			else printf("OSC pos - Trajectory doesn't exist\n") ;
		break ;
		case 2:
			printf("OSC pos - Cursor's position cannot be modified\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setPosition( newPos ) ;
				score->getCurve( inID )->updateBB( BBsize ) ; }
			else printf("OSC pos - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setPosition( newPos ) ;
				score->getTrigger( inID )->updateBB( BBsize ) ; }
			else printf("OSC pos - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectColor( int type, int inID, int inR, int inG, int inB )
{
	Color newCol ;
	newCol.ID = 0 ;
	newCol.c = QColor( inR, inG, inB ) ;
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) )
				score->getTraject( inID )->setColor( newCol ) ;
			else printf("OSC Color - Trajectory doesn't exist\n") ;
		break ;
		case 2:
			if( score->isThereCursor( inID ) )
				score->getCursor( inID )->setColor( newCol ) ;
			else printf("OSC Color - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) )
				score->getCurve( inID )->setColor( newCol ) ;
			else printf("OSC Color - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) )
				score->getTrigger( inID )->setColor( newCol ) ;
			else printf("OSC Color - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectActivity( int type, int inID, int chooser )
{
	switch(type) {
		case Trajectory:
			if( score->isThereTraject( inID ) ) {
				score->getTraject( inID )->setActive( (chooser==1) ) ;
				setCursorsActive( score->getTraject( inID ), (chooser==1) ) ;
			}
			else printf("OSC Act - Trajectory doesn't exist\n") ;
		break ;
		case 2:
			if( score->isThereCursor( inID ) )
				score->getCursor( inID )->setActive( (chooser==1) ) ;
			else printf("OSC Act - Cursor doesn't exist\n") ;
		break ;
		case Curve:
			if( score->isThereCurve( inID ) )
				score->getCurve( inID )->setActive( (chooser==1) ) ;
			else printf("OSC Act - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) )
				score->getTrigger( inID )->setActive( (chooser==1) ) ;
			else printf("OSC Act - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectHost( int type, int inID, const char *inHost )
{
	QString inHos = QString( inHost ) ;
	switch(type) {
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setHost( inHos ) ;
				controlCenter->updateParameters( Curve, inID, true ) ;
			}
			else printf("OSC Host - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setHost( inHos ) ;
				controlCenter->updateParameters( Trigger, inID, true ) ;
			}
			else printf("OSC Host - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscObjectPort( int type, int inID, int inPort )
{
	switch(type) {
		case Curve:
			if( score->isThereCurve( inID ) ) {
				score->getCurve( inID )->setPort( inPort ) ;
				controlCenter->updateParameters( Curve, inID, true ) ;
			}
			else printf("OSC Port - Curve doesn't exist\n") ;
		break ;
		case Trigger:
			if( score->isThereTrigger( inID ) ) {
				score->getTrigger( inID )->setPort( inPort ) ;
				controlCenter->updateParameters( Trigger, inID, true ) ;
			}
			else printf("OSC Port - Trigger doesn't exist\n") ;
		break ;
	}
}
void NxGLScore::oscShowHideObject( int type, int chooser)
{
	switch(type) {
		case Trajectory:
			showTrajects = (chooser==1) ;
		break ;
		case 2:
			showCursors = (chooser==1) ;
		break ;
		case Curve:
			showCurves = (chooser==1) ;
		break ;
		case Trigger:
			showTriggers = (chooser==1) ;
		break ;
	}
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::osc3DRotateX( float ang )
{
	rotationSetX( ang ) ;
}
void NxGLScore::osc3DRotateY( float ang )
{
	rotationSetY( ang ) ;
}
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////
// OpenGL Render Methods
////////////////////////////////////
void NxGLScore::refreshView()
{ 
	if(!frustum)	BBsize = (float)length/(600.0*scaleX) ;
	else			BBsize = 1.0/(30.0*scaleX) ;
	if(visualMode) updateGL();
	mParent->setToolBarTime( this,  timeScheduler->getCurrentTime() ) ;
}
void NxGLScore::repaint()
{
}
void NxGLScore::setToolBarInfosTxt( QString inTxt )
{
	mParent->setToolBarInfosTxt( inTxt ) ;
}
void NxGLScore::paintGL()
{
	QFont textFont("Verdana") ;
	
	//////// Mouse Position in Pixel Coordinates
	mouseXPixPos = (int)floor(timePosPix.x) ;
	mouseYPixPos = (int)floor(timePosPix.y) ;
	
	glClearColor( bColor.c.red()/255.0, bColor.c.green()/255.0, bColor.c.blue()/255.0, bColor.c.alpha()/255.0 ) ;
  	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  	glLoadIdentity( );
  	glTranslatef( transX, transY, -10.0 );
	
	/*if(shiftPressed) { ////////////////////////////////////////// DEBUG PRINTS
		printf("\n\n///////////////////// DEBUG ////// PRINT //////////////////////////\n") ;
		//bX = length / (RScaleX*scaleX*10.0) ;
		//bY = nHeight / (RScaleY*scaleY*10.0) ;
		printf("Taille fenetre en reel : bx by : %f et %f\n", 2*bX, 2*bY) ;
		// nHeight = (float)length*initSelfH/initSelfW
		printf("nHeight : %f\n", nHeight) ;
		printf("trans : %f et %f\n", transX, transY ) ;
		printf("real trans : %f et %f\n", transX/(RScaleX*scaleX), transY/(RScaleY*scaleY) ) ;
		printf("--------\n") ;
		printf("p : %f et %f\n", p.x, p.y) ;
		printf("timePos : %f et %f\n", timePos.x, timePos.y) ;
		//printf("timePosPix : %f et %f\n", timePosPix.x, timePosPix.y) ;
		printf("mousePixPos : %d et %d\n", mouseXPixPos, mouseYPixPos) ;
		printf("------------------------\n") ;
		printf("scale : %f et %f\n", scaleX, scaleY) ;
		//selfW = (GLint)wf;
		//selfH = (GLint)hf;
		printf("TAILLE : %d et %d\n", selfW, selfH) ;
		printf("TAILLE INITIALE : %d et %d\n", initSelfW, initSelfH) ;
		//RScaleX = (float)initSelfW/selfW ;
		//RScaleY = (float)initSelfH/selfH ;
		printf("Rscale : %f et %f\n", RScaleX, RScaleY) ;
	}*/
	
	glRotatef( angleX, 1, 0, 0 ) ; 	
  	glRotatef( angleY, 0, 1, 0 ) ;
  	glScalef( (float)RScaleX*scaleX, (float)RScaleY*scaleY, scaleZ );
	glTranslatef( -(float)length/2, -(float)height/2, 0 );
	
  	////////////////////////////////////////////////////////////
  	////////////////// SCORE RECTANGLE
  	////////////////////////////////////////////////////////////
	if (scoreDraw)
	{
		//Rectangle
		glColor4fv( colorConv( sColor ) );
		glBegin( GL_QUADS );
		glVertex3f( 0, 0, 0.0 );
		glVertex3f( 0, height, 0.0 );
		glVertex3f( length, height, 0.0 );
		glVertex3f( length, 0, 0.0 );
		glEnd();
		//Contour
		GLfloat colorFond2[] 	= {1.0, 1.0, 1.0, 0.4};	
		glColor4fv( colorFond2 );
		glBegin( GL_LINE_LOOP );
		glVertex3f( 0, 0, 0.0 );
		glVertex3f( 0, height, 0.0 );
		glVertex3f( length, height, 0.0 );
		glVertex3f( length, 0, 0.0 );
		glEnd();
		//Red (0,0) cross
		GLfloat colorFond3[] 	= {1.0, 0, 0, 1.0};	
		glColor4fv( colorFond3 );
		float crossDim = length/(160*scaleX) ;
		if(frustum) crossDim = 0.5 ;
		glBegin( GL_LINES );
		glVertex3f( 0, 0 - crossDim, 0.0 );
		glVertex3f( 0, 0 + crossDim, 0.0 );
		glVertex3f( 0 - crossDim, 0, 0.0 );
		glVertex3f( 0 + crossDim, 0, 0.0 );
		glEnd();
	}
	////////////////////////////////////////////////////////
	//////////////////// SCORE GRID
	////////////////////////////////////////////////////////
	if(showGrid) { // TODO must be better (because of bad performances while big zoom -> big for()-loop...)
		if(autoGridStep) {
			if(gridStep > 0.01) gridStepInt = (int)floor(gridStep*100) ;
			else gridStepInt = 1 ;
		}
		//TODO glTranslatef( gridDecX, gridDecY, 0 );
		glColor4f((GLfloat)gColor.c.red()/255, (GLfloat)gColor.c.green()/255, (GLfloat)gColor.c.blue()/255, gridOpacity);
		glBegin(GL_LINES);
		for(int i=0; i<=height*100.0; i+= (int)gridStepInt) {
			glVertex2f(0, i/100.0);
			glVertex2f(length, i/100.0);
		}
		for(int j=0; j<=length*100.0; j+= (int)gridStepInt) {
			glVertex2f(j/100.0, 0);
			glVertex2f(j/100.0, height);
		}
		glEnd() ;
		if(snapGrid)
		{
			glColor4f(1.0, 1.0, 1.0, gridOpacity*4);
			glBegin(GL_POINTS);
			for (int i=0; i<=height*100.0; i+= (int)gridStepInt) {
				for (int j=0; j<=length*100.0; j+= (int)gridStepInt) {
					glVertex2f(j/100.0, i/100.0);
				}	
			}
			glEnd() ;
		}
		//TODO glTranslatef( -gridDecX, -gridDecY, 0 );
	}
	
  	////////////////////////////////////////////////////
  	////////////////// SELECTION RECTANGLES
  	////////////////////////////////////////////////////
  	if( !frustum && mousePressed && onDrag && (selectionMode || zoomMode) )
	{
		if(zoomMode) ///////////////////////// ZOOM box
			glColor4f(1.0, 0.7, 0.0, 0.8);
		else ///////////////////////////////// SELECTION box
			glColor4f(1.0, 1.0, 1.0, 0.5);
		glEnable(GL_LINE_STIPPLE);
		glLineStipple(3, 0xAAAA);
		glBegin(GL_LINE_LOOP);
		glVertex3f( clickPos.x, clickPos.y, 0.0 );
		glVertex3f( clickPos.x, p.y, 0.0 );
		glVertex3f( p.x, p.y, 0.0 );
		glVertex3f( p.x, clickPos.y, 0.0 );
		glVertex3f( clickPos.x, clickPos.y, 0.0 );
		glEnd();
		glDisable(GL_LINE_STIPPLE);
	}
	
  	/////////////////////////////////////////////////////
  	////////// OBJECTS RENDERING
  	/////////////////////////////////////////////////////
  	if(showTriggers) renderTriggers();
  	renderTrajects();
  	if(showCurves) renderCurves();

  	////////////////////////////////////////////////////////////////	
  	////////// SPECIAL CASE : CURVE EDITION - 1st point
  	/////////////////////////////////////////////////////////////// 
  	if (firstPoint) {
		drawPointBox(firstCurvPos, QColor(255,255,255)) ;
	}
  	////////////////////////////////////////////////////////////////	
  	////////// MOVE A POINT ON A CURVE
  	/////////////////////////////////////////////////////////////// 	
	if (curveEdition && curvEdPtChoosed && pointID!=-1 && !curvEdOnlyClic) { 
		glColor4f(1.0, 1.0, 1.0, 1.0);
		float dimPt = (float)length/(1500*scaleX) ;
		glBegin( GL_LINES );
		if( !curvEdfirst ) {
			glVertex3f( curvEd1x, curvEd1y, 0.0 ) ;
			glVertex3f( tempP.x + curvEdPosX, timePos.y, 0.0 ) ;
		}
		if( !curvEdlast ) {
			glVertex3f( tempP.x + curvEdPosX, timePos.y, 0.0 ) ;
			glVertex3f( curvEd2x, curvEd2y, 0.0 ) ;
			}
		glEnd();
		glBegin( GL_QUADS );
		glVertex3f( tempP.x + curvEdPosX-dimPt, timePos.y-dimPt, 0.0 );
		glVertex3f( tempP.x + curvEdPosX+dimPt, timePos.y-dimPt, 0.0 );
		glVertex3f( tempP.x + curvEdPosX+dimPt, timePos.y+dimPt, 0.0 );
		glVertex3f( tempP.x + curvEdPosX-dimPt, timePos.y+dimPt, 0.0 );
		glEnd();
	}
  	////////////////////////////////////////////////////////////////	
  	/////////// LINE RULE RENDERING
  	///////////////////////////////////////////////////////////////
  	if (lineRuleMode)
	{
		glColor4f( 1.0, 0.5, 0.0, 1.0 );
		glBegin( GL_LINES);
		glVertex3f( lineRulePt1.x, lineRulePt1.y, 0.0 );
		glVertex3f( timePos.x, timePos.y, 0.0 );
		glEnd();
	}
  	////////////////////////////////////////////////////////////////	
  	//////////// TEXT RENDERING
  	///////////////////////////////////////////////////////////////
  	glColor4f( 1.0, 1.0, 1.0, 1.0 );
	if ( displayOn )
	{	
		renderText( selfW - 210, selfH - 10, QString("Grid Step : %1").arg( gridStepInt/100.0 ), textFont ) ;
		if(!frustum) { //// ONLY 2D
			renderText( selfW - 100, selfH - 70, QString("Selected : %1").arg(numberSelected), textFont );
			renderText( selfW - 100, selfH - 50, QString("X = %1 s").arg(timePos.x), textFont );
			renderText( selfW - 100, selfH - 40, QString("Y = %1 s").arg(timePos.y), textFont );
			renderText( selfW - 100, selfH - 30, QString("Z = %1 s").arg(0), textFont );
			renderText( selfW - 100, selfH - 10, QString("2D Mode"), textFont ); 
		}
		else { //////////// ONLY 3D
			renderText( selfW - 100, selfH - 40, QString("Angle (X) : %1").arg(floor(angleX)), textFont ); 
			renderText( selfW - 100, selfH - 30, QString("Angle (Y) : %1").arg(floor(angleY)), textFont ); 	
			renderText( selfW - 100, selfH - 10, QString("3D Mode"), textFont );  
		}			
		
		////////////////////////////////////////////////////////////////////
		////////// SELECTION INFORMATIONS DISPLAY
		////////////////////////////////////////////////////////////////////
		if (highlighted != NULL)
		{	
			switch ( highlighted->getObjectType() ) {
				case 0: ////// CURVE
					renderText( 10, selfH - 180, QString("CURVE : %1").arg( highlighted->getName() ), textFont ) ; 	
					renderText( 10, selfH - 160, QString("Real ID (unique) : %1").arg( highlighted->getID() ), textFont ) ;
					renderText( 10, selfH - 150, QString("Custom ID : %1").arg( highlighted->getCustomID() ), textFont ) ;
					renderText( 10, selfH - 140, QString("Group ID : %1").arg( highlighted->getGroupID() ), textFont ) ;
					renderText( 10, selfH - 130, QString("Layer : %1").arg( highlighted->getLayer() ), textFont ) ;
					renderText( 10, selfH - 110, QString("OSC : %1:%2 %3").arg( ((NxCurve*)highlighted)->getHost() ).arg( ((NxCurve*)highlighted)->getPort() ).arg( ((NxCurve*)highlighted)->getOscFormat() ), textFont );
					renderText( 10, selfH - 90, QString("Scan Period : %1 ms").arg( ( (NxCurve*)highlighted)->getSpatialPeriod() ), textFont ) ;
					renderText( 10, selfH - 80, QString("Control Points : %1").arg( ((NxCurve*)highlighted)->size()), textFont ) ;
					renderText( 10, selfH - 70, QString("Values in [ %1 , %2 ]").arg( ((NxCurve*)highlighted)->getRealMin() ).arg( ((NxCurve*)highlighted)->getRealMax() ), textFont ) ;
					renderText( 10, selfH - 60, QString("Duration : %1 s").arg( ( (NxCurve*)highlighted)->getXRange() ), textFont ) ;
					renderText( 10, selfH - 50, QString("Angle : %1 ").arg( ((NxCurve*)highlighted)->getAngle()), textFont ) ;
					renderText( 10, selfH - 30, QString("X: %1").arg( ((NxCurve*)highlighted)->getPosition().x), textFont );
					renderText( 10, selfH - 20, QString("Y: %1").arg( ((NxCurve*)highlighted)->getPosition().y), textFont );
					renderText( 10, selfH - 10, QString("Z: %1").arg( ((NxCurve*)highlighted)->getPosition().z), textFont );	
				break;
				case 1: ////// TRAJECTORY
					renderText( 10, selfH - 140, QString("TRAJECTORY : %1").arg( highlighted->getName() ), textFont ) ; 	
					renderText( 10, selfH - 120, QString("Real ID (unique) : %1").arg( highlighted->getID() ), textFont ) ;
					renderText( 10, selfH - 110, QString("Custom ID : %1").arg( highlighted->getCustomID() ), textFont ) ;
					renderText( 10, selfH - 100, QString("Group ID : %1").arg( highlighted->getGroupID() ), textFont ) ;
					renderText( 10, selfH - 90, QString("Layer : %1").arg( highlighted->getLayer() ), textFont ) ;
					renderText( 10, selfH - 70, QString("Child Cursors : %1").arg( ((NxTraject*)highlighted)->getNbCursors() ), textFont );
					renderText( 10, selfH - 60, QString("Length: %1").arg( ((NxTraject*)highlighted)->getLength()), textFont ); 
					if ( ((NxTraject*)highlighted)->getType() == 1) // Circle
						renderText( 10, selfH - 50, QString("Radius: %1 s").arg( ((NxTraject*)highlighted)->getRadius()), textFont ); 
					else
						renderText( 10, selfH - 50, QString("Angle: %1 ").arg( ((NxTraject*)highlighted)->getAngle()), textFont ); 
					renderText( 10, selfH - 30, QString("X: %1").arg( ((NxTraject*)highlighted)->getPosition().x), textFont );
					renderText( 10, selfH - 20, QString("Y: %1").arg( ((NxTraject*)highlighted)->getPosition().y), textFont );
					renderText( 10, selfH - 10, QString("Z: %1").arg( ((NxTraject*)highlighted)->getPosition().z), textFont );
				break;	
				case 2: ////// CURSOR
					renderText( 10, selfH - 140, QString("CURSOR : %1").arg( highlighted->getName() ) , textFont ) ; 	
					renderText( 10, selfH - 120, QString("Real ID (unique) : %1").arg( highlighted->getID() ), textFont ) ;
					renderText( 10, selfH - 110, QString("Custom ID : %1").arg( highlighted->getCustomID() ), textFont ) ;
					renderText( 10, selfH - 100, QString("Group ID : %1").arg( highlighted->getGroupID() ), textFont ) ;
					renderText( 10, selfH - 90, QString("Layer : %1").arg( highlighted->getLayer() ), textFont ) ;
					renderText( 10, selfH - 70, QString("Parent ID : %1").arg( ((NxCursor*)highlighted)->getParentID() ), textFont ); 
					renderText( 10, selfH - 50, QString("Width : %1 s").arg( ((NxCursor*)highlighted)->getWidth() ), textFont ); 
					renderText( 10, selfH - 40, QString("Speed : %1").arg( ((NxCursor*)highlighted)->getSpeed() ), textFont ); 
					if( ((NxCursor*)highlighted)->getType() == 1 ) {
						renderText( 10, selfH - 20, QString("Angle = %1 ").arg( ((NxCursor*)highlighted)->getNormalizedAngle() ), textFont ); 
						renderText( 10, selfH - 10, QString("[ %1 - %2 ]").arg( ((NxCursor*)highlighted)->getInitAngle() ).arg( ((NxCursor*)highlighted)->getFinalAngle() ), textFont );
					}
					else {
						renderText( 10, selfH - 20, QString("Offset = %1 s").arg( ((NxCursor*)highlighted)->getOffset() ), textFont ); 
						renderText( 10, selfH - 10, QString("[ %1 - %2 ]").arg( ((NxCursor*)highlighted)->getInitOffset() ).arg( ((NxCursor*)highlighted)->getFinalOffset() ), textFont );   
					}
				break;	
				case 3: ////// TRIGGER
					renderText( 10, selfH - 140, QString("TRIGGER : %1").arg( highlighted->getName() ), textFont ) ;
					renderText( 10, selfH - 120, QString("Real ID (unique) : %1").arg( highlighted->getID() ), textFont ) ;
					renderText( 10, selfH - 110, QString("Custom ID : %1").arg( highlighted->getCustomID() ), textFont ) ;
					renderText( 10, selfH - 100, QString("Group ID : %1").arg( highlighted->getGroupID() ), textFont ) ;
					renderText( 10, selfH - 90, QString("Layer : %1").arg( highlighted->getLayer() ), textFont ) ;
					renderText( 10, selfH - 70, QString("Int Value : %1").arg( ((NxTrigger*)highlighted)->getValue() ), textFont ) ;
					renderText( 10, selfH - 50, QString("OSC : %1:%2 %3").arg( ((NxTrigger*)highlighted)->getHost() ).arg( ((NxTrigger*)highlighted)->getPort() ).arg( ((NxTrigger*)highlighted)->getOscFormat() ), textFont );
					renderText( 10, selfH - 30, QString("X: %1").arg( ((NxTrigger*)highlighted)->getPosition().x), textFont );
					renderText( 10, selfH - 20, QString("Y: %1").arg( ((NxTrigger*)highlighted)->getPosition().y), textFont );
					renderText( 10, selfH - 10, QString("Z: %1").arg( ((NxTrigger*)highlighted)->getPosition().z), textFont ); 
				break;
			}	
		}
	}	
	///////////////////////////////////////////////////////////////////////////
	// MANUAL RULE INFOS DISPLAY - angle and length
	///////////////////////////////////////////////////////////////////////////
	if(lineRuleMode) {
		renderText( mouseXPixPos + 10,  mouseYPixPos , QString("Length : %1 s").arg( lineRuleLength ), textFont ) ; 
		renderText( mouseXPixPos + 10,  mouseYPixPos-15 , QString("Angle : %1 ").arg( lineRuleAngle ), textFont ) ; 
	}
	///////////////////////////////////////////////////////////////////////////
	// TRAJECT EDITION - angle display
	///////////////////////////////////////////////////////////////////////////
	if( (lineEdition && selTrajectPoint != -1) || trajectCreation ) {
		if( trajectCreation )  editedLine = createdTraject ;
		renderText( mouseXPixPos + 10,  mouseYPixPos , QString("Length : %1 s").arg(  editedLine->getLength() ), textFont ) ;
		if( editedLine->getOrientation() == 4) // Free line -> display angle too
			renderText( mouseXPixPos + 10,  mouseYPixPos-15 , QString("Angle : %1 ").arg( editedTrajectAngle ), textFont ) ; 
	}
	///////////////////////////////////////////////////////////////////////////
	// CIRCLE RADIUS
	///////////////////////////////////////////////////////////////////////////
	if( circleEdition && selTrajectPoint != -1 )
		renderText( mouseXPixPos + 10,  mouseYPixPos , QString("Radius : %1 s").arg(  editedCircle->getRadius() ), textFont ) ;
	///////////////////////////////////////////////////////////////////////////
	// CURVE EDITION - local coordinates
	///////////////////////////////////////////////////////////////////////////
	if(curveEdition && curvEdPtChoosed && pointID!=-1 && !curvEdOnlyClic) { ///// Local Coordinates of Edited Curve Point 
		renderText( mouseXPixPos + 10, mouseYPixPos-15 , QString("X = %1").arg( curvEdInfoX ), textFont );
		renderText( mouseXPixPos + 10, mouseYPixPos , QString("Y = %1").arg( curvEdInfoY ), textFont );
	}
}

////////////////////////////////////
// OpenGL initializer
////////////////////////////////////
void NxGLScore::initializeGL()
{
  	qglClearColor( Qt::black );
	glLineWidth( 1.5 );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE );	
  	glEnable( GL_BLEND );
  	setAntialiasing( antialiasing );
}
////////////////////////////////////////////////
// Antialiasing
////////////////////////////////////////////////
void NxGLScore::updateAntialiasing()
{
	makeCurrent() ;
	setAntialiasing( mParent->getAntialiasing() );
}
void NxGLScore::setAntialiasing( bool state )
{
  	if ( state )  {
		glEnable( GL_POLYGON_SMOOTH );
		glEnable( GL_POINT_SMOOTH );
		glEnable( GL_LINE_SMOOTH );
		glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	}
	else  {
		glDisable( GL_LINE_SMOOTH );
		glDisable( GL_POLYGON_SMOOTH );
		glDisable( GL_POLYGON_SMOOTH );
		glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
	}
}
//////////////////////////////////////////////////
// OpenGL Modelview setup
//////////////////////////////////////////////////
void NxGLScore::resizeGL( int w, int h )
{
	float hf = (float ) h;
	float wf = (float ) w;
	
	//printf("RESIZE ::: W %f H %f \n", wf,  hf );
	glViewport( 0, 0, (GLint)wf, (GLint)hf );
  	selfW = (GLint)wf;
	selfH = (GLint)hf;
	//printf("selfh, slefW, %d, %d \n", selfW,  selfH );
	/////////// Depends on initial window size
	RScaleX = (float)initSelfW/(float)selfW ;
	RScaleY = (float)initSelfH/(float)selfH ;
	//printf("Sizes ratio : %f, %f\n\n", RScaleX, RScaleY) ;
	
	frustumView();
}
///////////////////////////////////
// Model view switcher
///////////////////////////////////
void NxGLScore::frustumView()
{
  	glMatrixMode( GL_PROJECTION );
  	glLoadIdentity();
	
  	if ( frustum )
		//////////// 3D MODE - Frustum view
    	glFrustum( -1.0, 1.0, -nHeight/length, nHeight/length, 5.0, 200.0 );
		
	else
		//////////// 2D MODE - Orthographic view	
		glOrtho( -length/10.0, length/10.0, -nHeight/10.0, nHeight/10.0, 5.0, 200.0 );	
			
  	glMatrixMode( GL_MODELVIEW );
  	glClear( 1 );
}

///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
//////////////////////// Reset view
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
void NxGLScore::fitToPageView()
{	
	posX = posY = p.x = p.y = mouseX = mouseY = clickPos.x = clickPos.y = 0 ;
	transX = transY = diffX = diffY = 0 ;
	oldAngleX = oldAngleY = angleX = angleY = 0.0 ;
	pointID = -1 ;
	////////////////////////////// Grid
	if(autoGridStep) gridStep = length/10.0 ;
	gridDecX = 0 ;
	gridDecY = 0 ;
	////////////////////////////// Line Rule Mode
	lineRuleMode = false ;
	lineRuleSetFirstPt = true ;	

	if(frustum) { // 3D
		scaleX = scaleY = scaleZ = 2.0/(float)length ;
		bX = length / (RScaleX*scaleX*10.0);
		bY = nHeight / (RScaleY*scaleY*10.0);
		mParent->setWindowTitle( QString("IanniX Viewer - 3D Mode") );
	}
	else { // 2D
	
		/*
		1)) want  bX = length
		donc scaleX =  1/RSCaleX 
		2)) want by = height
		donc scaleX = nHeight/(RScaleY*height  )
		donc setzomm(oldscaleX/saclex)
		scaleX/= factor;
		scaleY/= factor;
		scaleZ/= factor;
		bX = length / (RScaleX*scaleX*10.0);
		bY = nHeight / (RScaleY*scaleX*10.0);
		*/
	
		scaleX = scaleY = scaleZ = 0.17;
		
		float oldScaleX = scaleX ;
	
		if(  (float)(length/height) > (float)(selfW/selfH)  )
			setZoom(oldScaleX*RScaleX*5) ;
		else
			setZoom(oldScaleX*RScaleY*height*5/nHeight) ;
		
		setZoom( 1.05 ) ; // a little less to see the borders
	
		updateTrigg_LinesBB() ;
		
		mParent->setWindowTitle( QString("IanniX Viewer - %1 %2").arg( getZoomScale() ).arg('%') );
	}
}
void NxGLScore::setZoom( float factor )
{
  	scaleX /= factor;
  	scaleY /= factor;
  	scaleZ /= factor;
	
  	bX = length / (RScaleX*scaleX*10.0);
  	bY = nHeight / (RScaleY*scaleX*10.0);
	
	if(autoGridStep) gridStep *= factor ;
	 	
  	timePos.x *= factor;
  	timePos.y *= factor;
	
  	// Test Slices
  	NbSLICES = (int) ( NbSLICES / factor );
  	if (NbSLICES < 60 ) NbSLICES = 60; // Threshold
  	if (NbSLICES > 100 ) NbSLICES = 100;
	
	if(frustum)		mParent->setWindowTitle( QString("IanniX Viewer - 3D Mode") );
	else			mParent->setWindowTitle( QString("IanniX Viewer - %1 %2").arg( getZoomScale() ).arg('%') );
}
void NxGLScore::zoomBy( float inZoom )
{	
	if(!onClick) {
		NxPoint initCenter ; // center of view before zoom
		float oldScaleX ;
	
		initCenter.x = (float)transX/(RScaleX*scaleX) ;
		initCenter.y = (float)transY/(RScaleY*scaleY) ;
	
		//printf("avant centre : %f et %f\n",initCenter.x,initCenter.y) ;
	
		oldScaleX = scaleX ;
		scaleX = scaleY = scaleZ = 0.17 ;
		
		setZoom(scaleX*inZoom/oldScaleX) ;
		
		transX = initCenter.x*(RScaleX*scaleX) ;
		transY = initCenter.y*(RScaleY*scaleY) ;
	
		//printf("apres : %f et %f\n",transX/(RScaleX*scaleX),transY/(RScaleY*scaleY)) ;
	
		updateTrigg_LinesBB() ;
	}
}
void NxGLScore::zoomRect( NxPoint inP1, NxPoint inP2 )
{
	float rLen = fabs(inP2.x - inP1.x) ;
	float rHei = fabs(inP2.y - inP1.y) ;
	float oldScaleX ;
	NxPoint initCenter ; // center of view before zoom
	NxPoint wantedCenter ;
	
	initCenter.x = (float)transX/(RScaleX*scaleX) ;
	initCenter.y = (float)transY/(RScaleY*scaleY) ;
	wantedCenter.x = (inP2.x + inP1.x)/2 - length/2 ;
	wantedCenter.y = (inP2.y + inP1.y)/2 - height/2 ;
	
	//printf("avant centre : %f et %f\n",initCenter.x,initCenter.y) ;
	//printf("wanted : %f et %f\n",wantedCenter.x,wantedCenter.y) ;

	scaleX = scaleY = scaleZ = 0.17;
	
	oldScaleX = scaleX ;
	
	if( (float)(rLen/rHei) > (float)(selfW/selfH) ) {
		setZoom(oldScaleX*RScaleX*5*rLen/length) ;
		//printf("FIT longueur\n") ;
		}
	else {
		setZoom(oldScaleX*RScaleY*rHei*5/nHeight) ;
		//printf("FIT largeur\n") ;
		}
	
	transX = - wantedCenter.x*(RScaleX*scaleX) ;
	transY = - wantedCenter.y*(RScaleY*scaleY) ;	

	//printf("aprestrans : %f et %f\n",transX/(RScaleX*scaleX),transY/(RScaleY*scaleY)) ;

	updateTrigg_LinesBB() ;
}
void NxGLScore::hardResetView() // when score's length or height change
{
	scaleX = scaleY = scaleZ = 0.17 ;
	RScaleX = RScaleY = 1 ;
	selfW = initSelfW = frameSize().width() ;
	selfH = initSelfH = frameSize().height() ;
	nHeight = (float)length*initSelfH/initSelfW ;
	bX = length / (RScaleX*scaleX*10.0) ;
  	bY = nHeight / (RScaleY*scaleY*10.0) ;
	
	frustumView() ;
	fitToPageView() ;
}

///////////////////////////////////////////
// UPDATES
///////////////////////////////////////////
void NxGLScore::updateTrigg_LinesBB()
{
	NxTrigger *trig ;
	NxTraject *traj ;
	NxCursor *curs ;
	
	QList<int> ind1 = score->triggersIndexes();
	QList<int> ind2 = score->trajectsIndexes();
	QList<int> ind3 = score->cursorsIndexes();
		
	if(!frustum)	BBsize = (float)length/(600*scaleX) ;
	else			BBsize = 1/(30*scaleX) ;
	
	for(int im=0 ; im<ind1.count() ; im++) {
		trig = score->getTrigger( ind1.at(im) ) ;
		trig->updateBB( BBsize ) ;
	}
	for(int in=0 ; in<ind2.count() ; in++) {
		traj = score->getTraject( ind2.at(in) ) ;
		if( ((NxTraject*)traj)->getType() == 2 ) // Lines
			traj->updateBB( BBsize ) ;
	}
	for(int in=0 ; in<ind3.count() ; in++) {
		curs = score->getCursor( ind3.at(in) ) ;
		curs->updateAnglePosition() ;
		curs->updateBB( BBsize ) ;
	}
}
void NxGLScore::updateCursorsAnglePosition( NxTraject* inTraject )
{
	NxCursor* cur ;
	QList<int> indexes = score->cursorsIndexes();
	for( int i=0; i< indexes.count(); i++ ) {	
		cur = score->getCursor( indexes.at(i) ) ;
		if ( inTraject->getID() == cur->getParentTrajectory()->getID() ) {
			if(cur->getType()==1) // CIRCLE
				cur->reEvalAngles() ;
			else // LINE
				cur->reEvalOffsets() ;
			cur->updateAnglePosition() ;
			cur->updateBB( BBsize ) ;
		}
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////
// Object Management Section
///////////////////////////////////////////////////////
/////////////////////////
// New Traject
/////////////////////////
void NxGLScore::addNewTraject( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, int inChx, int inAxis, float inRadius )
{
	switch (inChx) {
		case 1 : ////// Circle
	  		newC.p = inPos;
	  		newC.radius = inRadius;
	 		score->addTraject( inPos, inID, incID, inGroupID, inLay, inName, inColor, 1 );
	  		score->getTraject( inID )->setCircleDef( newC );
	  		score->getTraject( inID )->updateBB( BBsize );
			
			updateIndexes();
			controlCenter->updateItems( Trajectory, true, inID, true );
		break;		
		case 2 : ////// Line
	    	trajID = inID;
			score->addTraject( inPos, inID, incID, inGroupID, inLay, inName, inColor, 2 );
			score->getTraject( inID )->setOrientation( inAxis );
	    	score->getTraject( inID )->setLineDef( inPos, inPos ); // ??? - 2nd point later...
			createdTraject = score->getTraject( inID ) ;
			trajectCreation = true;
			lineEdPt1 = inPos ;
			
			updateIndexes();
		break;
	}
}
///////////////////////
// New Trigger
///////////////////////
void NxGLScore::addNewTrigger( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, QString inOscF, QString inHost, int inPort, int inIntValue )
{
	//pause() ;
	
  	score->addTrigger( inPos, inID, incID, inGroupID, inLay, inName, inColor, inOscF, inHost, inPort, inIntValue );
  	updateIndexes();
  	controlCenter->updateItems( Trigger, true, inID, true );
	score->getTrigger( inID )->updateBB( BBsize ) ;
	timeScheduler->addNewTrigger() ;
}
//////////////////////
// New Cursor
//////////////////////
void NxGLScore::addNewCursor( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor, bool inForward, int inMode, float inFactor, float inWidth, float inInitOffset, float inFinalOffset )
{
  	if (cursorMode) {
		pause() ;
		score->addCursor( inPos, inID, incID, (NxTraject*)edited, inGroupID, inLay, inName, inColor, inForward, inMode, inWidth, inInitOffset, inFinalOffset );
		if( ((NxTraject*)edited)->getType()==1 )
			score->getCursor(inID)->reEvalAngles() ;
		else 
			score->getCursor(inID)->reEvalOffsets() ;
		score->getCursor(inID)->setSpeed( inFactor );
		score->getCursor(inID)->setWidth( inWidth );
		score->getCursor(inID)->updateAnglePosition() ;
		score->getCursor(inID)->updateBB( BBsize ) ;
		updateIndexes();
		controlCenter->updateItems( 2, true, inID, true );	
		cursorMode = false;
		timeScheduler->addNewCursor() ;
	}
}
//////////////////////////////////////////////////
// ADD CURVES
//////////////////////////////////////////////////
void NxGLScore::setDrawMode( int inMode )
{
	drawMode = inMode;
}
void NxGLScore::addRandomCurve( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor1, Color inColor2, int nbCtrlPoints, float rangeMin, float rangeMax, int newDuration, float rMin, float rMax, float inScale, int inSpatialPeriod, QString inOscF, QString inHost, int inPort )
{	
	//pause() ;
	
	QVector<NxPoint> newDef;
	// Creating random curve		
	newDef = randomCurve( nbCtrlPoints, rangeMin, rangeMax, newDuration ) ;
	float inAngle = 0.0 ;
	score->addCurve( inPos, inID, incID, inGroupID, inLay, inName, inColor1, inColor2, newDef, nbCtrlPoints, rMin, rMax, inScale, inAngle, inSpatialPeriod, inOscF, inHost, inPort );
	score->getCurve(inID)->setDrawType(drawMode);
	  
	// Updating Indexes
	updateIndexes();
	controlCenter->updateItems( Curve, true, inID, true );
	timeScheduler->addNewCurve() ;
}
void NxGLScore::addEditCurve( NxPoint inPos, int inID, int incID, int inGroupID, int inLay, QString inName, Color inColor1, Color inColor2, float rMin, float rMax, float inScale, int inSpatialPeriod, QString inOscF, QString inHost, int inPort )
{
	//pause() ;
	
	// Manual Edition -> saving 1st point & wait extra points with mouse clicks....
	curveCreation = true ;
	savID = inID;
	firstCurvPos = inPos ;
	savGroupID = inGroupID;
	savName = inName;
	realMin = rMin;
	realMax = rMax;

	// Allocating first point
	NxPoint fPt ;
	inPos.ID = 0;
	fPt.ID	 = 0 ;
	fPt.x	 = 0.0 ;
	fPt.y	 = fabs(rMax-rMin)/2 ;
	fPt.z	 = 0.0 ;
	
	editedCurve.append( fPt );
	firstPoint = true;
	
	float inAngle = 0.0 ;
	score->addCurve( inPos, savID, incID, savGroupID, inLay, savName, inColor1, inColor2, editedCurve, editedCurve.count(), realMin, realMax, inScale, inAngle, inSpatialPeriod, inOscF, inHost, inPort ) ;
	score->getCurve(inID)->setDrawType(drawMode) ;
	timeScheduler->addNewCurve() ;
}

////////////////////////////////////////////////////////////////////////////////
// RENDERING SECTION
////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////
// CIRCLES & DISKS
/////////////////////////////////////
// mode = 0 : CIRCLE
// mode = 1 : DISK
void NxGLScore::drawCircle( GLfloat R, GLfloat cX, GLfloat cY, GLfloat cZ, int mode, QColor inColor )
{
	GLfloat a1, a2 ;
	int _r = inColor.red() ;
	int _g = inColor.green() ;
	int _b = inColor.blue() ;
	int _a = inColor.alpha() ;
	glColor4f( (GLfloat)_r/255, (GLfloat)_g/255, (GLfloat)_b/255, (GLfloat)_a/255 );
	
  	glPushMatrix();
  		glTranslatef( cX, cY, cZ );
		if ( mode == 1 ) {
			glBegin( GL_LINE_LOOP );
			for (GLfloat i=0.0; i<NbSLICES; i++) {
				a1 = i * 2 * PI / NbSLICES ;
				a2 = (i + 1) * 2 * PI / NbSLICES ;
				glVertex3f( R * cos( a1 ), R * sin( a1 ), 0.0 );
				glVertex3f( R * cos( a2 ), R * sin( a2 ), 0.0 );
			}
			glEnd();
		}
  		else {
			glBegin( GL_TRIANGLES );
			for (GLfloat i=0.0; i<NbSLICES; i++) {
				a1 = i * 2 * PI / NbSLICES ;
				a2 = (i + 1) * 2 * PI / NbSLICES ;
				glVertex3f( 0.0, 0.0, 0.0 );
				glVertex3f( R * cos( a1 ), R * sin( a1 ), 0.0 );
				glVertex3f( R * cos( a2 ), R * sin( a2 ), 0.0 );
			}
			glEnd(); 
		}
	glPopMatrix();
}
void NxGLScore::drawDashedArc( Circle inC, float inA1, float inA2, QColor inColor )
{
	GLfloat a1 = 0 ; // Initialisation 
	GLfloat a2 = 0 ; // Initialisation
	GLfloat radius = inC.radius ;
	int _r = inColor.red() ;
	int _g = inColor.green() ;
	int _b = inColor.blue() ;
	int _a = inColor.alpha() ;
	glColor4f( (GLfloat)_r/255, (GLfloat)_g/255, (GLfloat)_b/255, (GLfloat)_a/255 );
  	glPushMatrix();
	glTranslatef( inC.p.x, inC.p.y, inC.p.z );
	glEnable(GL_LINE_STIPPLE);
	glLineStipple(3, 0xAAAA);
	if(inA1==inA2) { ////////////////////////// ALL CIRCLE
		glBegin(GL_LINE_STRIP);
		for (GLfloat i=0.0; i<NbSLICES; i++) {
			a1 = i*2*PI/NbSLICES ;
			a2 = (i+1)*2*PI/NbSLICES ;
			glVertex3f( radius * cos( a1 ),	radius * sin( a1 ), 0.0 );
			glVertex3f( radius * cos( a2 ),	radius * sin( a2 ), 0.0 );
		}
	}
	else { ////////////////////////////////// in1A != inA2 -> ARC
		GLfloat tmpNbSLICES ;
		if(inA2>inA1)
			tmpNbSLICES = (inA2-inA1)*NbSLICES/360 ;
		else
			tmpNbSLICES = (360-(inA1-inA2))*NbSLICES/360 ;
		if(tmpNbSLICES<=1)
			a2 = inA1*PI/180 ;
		glBegin(GL_LINE_STRIP);
		for (GLfloat i=0.0; i<tmpNbSLICES-1; i++) {
			a1 = inA1*PI/180 + i*2*PI/NbSLICES ;
			a2 = inA1*PI/180 + (i+1)*2*PI/NbSLICES ;
			glVertex3f( radius * cos( a1 ),	radius * sin( a1 ), 0.0 );
			glVertex3f( radius * cos( a2 ),	radius * sin( a2 ), 0.0 );
		}
		glVertex3f( radius * cos( a2 ),				radius * sin( a2 ), 0.0 );
		glVertex3f( radius * cos( inA2*PI/180 ),	radius * sin( inA2*PI/180 ), 0.0 );
	}
	glEnd();
	glPopMatrix();
	glDisable(GL_LINE_STIPPLE);
}
////////////////////////
// CURSORS
////////////////////////
void NxGLScore::drawCursor( NxCursor *inCursor, Color inColor )
{
	float Csize = BBsize*2 ;
	
	NxPoint inP		= inCursor->getPosition() ;
	float inWidth	= inCursor->getWidth() ;
	float inAngle	= inCursor->getAngle() ;

	if(inCursor->getType()==1) // CIRCLE
		inAngle += 90 ; 
	else // LINE
		inAngle -= 90 ;
	if(!inCursor->getForward())
		inAngle += 180 ;
	
	glColor4fv( colorConv(inColor) );
  	glPushMatrix();
  		glTranslatef( inP.x, inP.y, inP.z );
		glRotatef( inAngle, 0, 0, 1 ) ;
		////// Triangle
  		glBegin(GL_TRIANGLES);
  			glVertex3f(0, -0.4*Csize, 0.0);
  			glVertex3f(0,  0.4*Csize, 0.0);
  			glVertex3f((float)Csize, 0.0, 0.0);		
  		glEnd();
  		////// Cursor's width
  		glBegin(GL_LINES);
  			glVertex3f( 0.0, -inWidth/2, 0.0);
  			glVertex3f( 0.0, inWidth/2, 0.0); 
  		glEnd();
  	glPopMatrix();	
}
void NxGLScore::drawCursorZone( NxCursor *inCursor )
{
	NxTraject *parentTraj	= inCursor->getParentTrajectory() ;
	NxPoint	trajPos			= parentTraj->getPosition() ;
	float cursWid			= inCursor->getWidth() ;
	
	glColor4f( 1.0, 1.0, 1.0, cursZonesOpacity ) ; /// Cursor Area Color
	
	if(parentTraj->getType()==1) { //////////////////////////// CIRCLE
		float trajRadius = parentTraj->getCircle().radius ;
		GLfloat initA	= inCursor->getInitAngle()*PI/180 ;
		GLfloat finalA	= inCursor->getFinalAngle()*PI/180 ;
		if(!inCursor->getForward()) {
			initA	= inCursor->getFinalAngle()*PI/180 ;
			finalA	= inCursor->getInitAngle()*PI/180 ;
		}
		GLfloat R1 = trajRadius - cursWid/2 ;
		GLfloat R2 = trajRadius + cursWid/2 ;
		GLfloat a1 = 0 ; 
		GLfloat a2 = 0 ;
		
		glPushMatrix() ;
		glTranslatef( trajPos.x, trajPos.y, trajPos.z ) ;			

		if(cursWid/2<=trajRadius) { //////////////////// NORMAL
			GLfloat tmpNbSLICES = (finalA*180/PI - initA*180/PI)*NbSLICES/360 ;
			if(initA>finalA)
				tmpNbSLICES += NbSLICES ;
			glBegin( GL_QUADS );
			if(tmpNbSLICES<=1)
				a2 = initA ;
			if(initA==finalA)
				tmpNbSLICES = NbSLICES ;
			for (GLfloat i=0.0; i<tmpNbSLICES-1; i++) {
				a1 = initA + i*2*PI/NbSLICES ;
				a2 = initA + (i+1)*2*PI/NbSLICES ;
				glVertex3f( R1 * cos( a1 ),	R1 * sin( a1 ), 0.0 );
				glVertex3f( R1 * cos( a2 ),	R1 * sin( a2 ), 0.0 );
				glVertex3f( R2 * cos( a2 ), R2 * sin( a2 ), 0.0 );
				glVertex3f( R2 * cos( a1 ),	R2 * sin( a1 ), 0.0 );
			}
			glVertex3f( R1 * cos( a2 ),			R1 * sin( a2 ), 0.0 );
			glVertex3f( R1 * cos( finalA ),		R1 * sin( finalA ), 0.0 );
			glVertex3f( R2 * cos( finalA ),		R2 * sin( finalA ), 0.0 );
			glVertex3f( R2 * cos( a2 ),			R2 * sin( a2 ), 0.0 );
		}
		else { /////////////////////////////////////////// NORMAL + small parts
			if(initA==finalA) { // simple circle draw
				glBegin( GL_TRIANGLES );
				for (GLfloat i=0.0; i<NbSLICES; i++) {
					a1 = i*2*PI/NbSLICES ;
					a2 = (i+1)*2*PI/NbSLICES ;
					glVertex3f( 0.0, 0.0, 0.0 );
					glVertex3f( R2 * cos( a1 ), R2 * sin( a1 ), 0.0 );
					glVertex3f( R2 * cos( a2 ),	R2 * sin( a2 ), 0.0 );
				}
			}
			else { // COMPLEX draw
				GLfloat tmpNbSLICES = (finalA*180/PI-initA*180/PI)*NbSLICES/360 ;
				if(initA>finalA)
					tmpNbSLICES += NbSLICES ;
				glBegin( GL_TRIANGLES );
				if(tmpNbSLICES<=1)
					a2 = initA ;
				for (GLfloat i=0.0; i<tmpNbSLICES-1; i++) {
					a1 = initA + i*2*PI/NbSLICES ;
					a2 = initA + (i+1)*2*PI/NbSLICES ;
					glVertex3f( 0.0, 0.0, 0.0 );
					glVertex3f( R2 * cos( a1 ), R2 * sin( a1 ), 0.0 );
					glVertex3f( R2 * cos( a2 ),	R2 * sin( a2 ), 0.0 );
				}
				glVertex3f( 0.0, 0.0, 0.0 );
				glVertex3f( R2 * cos( a2 ),		R2 * sin( a2 ), 0.0 );
				glVertex3f( R2 * cos( finalA ),	R2 * sin( finalA ), 0.0 );
			
				if(initA>finalA) {
					if( initA-finalA>PI ) {
						float tmpA = initA ;
						initA = finalA + PI ;
						finalA = tmpA + PI ;
					}
					else {
						tmpNbSLICES = (initA*180/PI-finalA*180/PI)*NbSLICES/360 ;
					}
				}
				else {
					if( finalA-initA>PI ) {
						tmpNbSLICES = (360 - (finalA*180/PI-initA*180/PI) )*NbSLICES/360 ;
					}
					else {
						float tmpA = initA ;
						initA = finalA + PI ;
						finalA = tmpA + PI ;
					}
				}
				if(tmpNbSLICES<=1)
					a2 = finalA ;
				for (GLfloat i=0.0; i<tmpNbSLICES-1; i++) {
					a1 = finalA + i*2*PI/NbSLICES ;
					a2 = finalA + (i+1)*2*PI/NbSLICES ;
					glVertex3f( 0.0, 0.0, 0.0 );
					glVertex3f( -R1 * cos( a1 ),	-R1 * sin( a1 ), 0.0 );
					glVertex3f( -R1 * cos( a2 ),	-R1 * sin( a2 ), 0.0 );
				}
				glVertex3f( 0.0, 0.0, 0.0 );
				glVertex3f( -R1 * cos( a2 ),	-R1 * sin( a2 ), 0.0 );
				glVertex3f( -R1 * cos( initA ),	-R1 * sin( initA ), 0.0 );
			}
		}
		glEnd();
		glPopMatrix() ;
	}
	else { //////////////////////////////////////////////////// LINE
		float lineAng		= parentTraj->getAngle() ;
		float trajLen		= parentTraj->getLength() ;
		float initOff	= inCursor->getInitOffset() ;
		float finalOff	= inCursor->getFinalOffset() ;
		if(!inCursor->getForward()) {
			finalOff	= trajLen - inCursor->getInitOffset() ;
			initOff		= trajLen - inCursor->getFinalOffset() ;
		}
		glPushMatrix() ;
		glTranslatef( trajPos.x, trajPos.y, trajPos.z ) ;
		glRotatef( lineAng, 0, 0, 1 ) ;
		glBegin(GL_QUADS);
		glVertex3f( initOff, cursWid/2, 0.0 ) ;
		glVertex3f( initOff, -cursWid/2, 0.0 ) ;
		glVertex3f( finalOff, -cursWid/2, 0.0 ) ;
		glVertex3f( finalOff, cursWid/2, 0.0 ) ;
		glEnd();
		glPopMatrix() ;
	}
}
////////////////////////
// TRIGGERS
////////////////////////
void NxGLScore::drawTrigger( NxPoint inP, Color inColor )
{
	float Tsize = BBsize ;
	
	glColor4fv( colorConv(inColor) );
  	glPushMatrix();
  		glTranslatef( inP.x, inP.y, inP.z );
  		glBegin(GL_QUADS);
  			glVertex3f( 0.0, (float)Tsize, 0.0 );
  			glVertex3f( -0.5*Tsize, 0.0, 0.0 );
  			glVertex3f( 0.0, -(float)Tsize, 0.0 );
  			glVertex3f( 0.5*Tsize, 0.0, 0.0 ); 		
  		glEnd();
  	glPopMatrix();
}
//////////////////////////////////////////////////
// Objects Names Display
//////////////////////////////////////////////////
void NxGLScore::drawObjectName( NxObject *inObj )
{
	glColor4f( 1.0, 1.0, 1.0, namesOpacity );
	QFont textFont("Verdana") ;
	
	int oTyp = inObj->getObjectType() ;
	int xpos = realToPixX( inObj->getPosition().x ) ;
	int ypos = realToPixY( inObj->getPosition().y ) ;
	switch( oTyp ) {
		case 0: // Curve
			xpos += 5 ;
			ypos += 5 ;
		break;
		case 1: // Trajectory
			if(((NxTraject*)inObj)->getType()==1) { // Circles
				xpos = realToPixX( inObj->getPosition().x + ((NxTraject*)inObj)->getCircle().radius ) + 5 ; 
				ypos += 5 ; 
			}
			else { // Lines
				xpos += 5 ; 
				ypos -= 5 ; 
			}
		break;
		case 2: // Cursor
			xpos += 5 ; 
			ypos += 5 ;  
		break;
		case 3: // Trigger
			xpos += 9 ;
			ypos += 6 ; 
		break;
	}
	renderText( xpos,  ypos , inObj->getName(), textFont ) ;
}
//////////////////////////////////////////////////
// Point Text
//////////////////////////////////////////////////
void NxGLScore::drawPointText( QString inText, NxPoint inP, int decx, int decy )
{
	glColor4f( 1.0, 1.0, 1.0, 1.0 );
	QFont textFont("Verdana") ;
	
	int xpos = realToPixX( inP.x ) ;
	int ypos = realToPixY( inP.y ) ;
	
	renderText( xpos + decx,  ypos + decy , inText, textFont ) ;
}
/////////////////////////////////////////////////////////////////////////////
// RENDERING UTILITIES
/////////////////////////////////////////////////////////////////////////////
void NxGLScore::drawBoundingBox( BBox inBox )
{
  	glColor4fv( colorConv( BBColor )); // BoundingBox Color
  	glBegin( GL_LINE_LOOP );
  		glVertex3f( inBox.p1.x, inBox.p1.y, inBox.p1.z );
  		glVertex3f( inBox.p1.x, inBox.p2.y, inBox.p1.z );
  		glVertex3f( inBox.p2.x, inBox.p2.y, inBox.p1.z );
  		glVertex3f( inBox.p2.x, inBox.p1.y, inBox.p1.z );
  	glEnd();
}
void NxGLScore::drawCursorBoundingBox( NxCursor* inCurs )
{
  	glColor4fv( colorConv( BBColor )); // BoundingBox Color
	glPushMatrix() ;
	glTranslatef( inCurs->getPosition().x, inCurs->getPosition().y, 0) ;
	glRotatef( inCurs->getAngle(), 0, 0, 1 ) ;
  	glBegin( GL_LINE_LOOP );
  		glVertex3f( -1.5*BBsize , -3*BBsize, 0 );
  		glVertex3f( -1.5*BBsize, 3*BBsize, 0 );
  		glVertex3f( 1.5*BBsize, 3*BBsize, 0 );
  		glVertex3f( 1.5*BBsize, -3*BBsize, 0 );
  	glEnd();
	glPopMatrix() ;
}
void NxGLScore::drawPointBox( NxPoint inP, QColor inColor )
{
	float dimPt = 0.0007*length/scaleX ;
	
	int _r = inColor.red() ;
	int _g = inColor.green() ;
	int _b = inColor.blue() ;
	int _a = inColor.alpha() ;
	glColor4f( (GLfloat)_r/255, (GLfloat)_g/255, (GLfloat)_b/255, (GLfloat)_a/255 );
	glBegin( GL_QUADS );
	glVertex3f( inP.x-dimPt, inP.y-dimPt, 0.0 );
	glVertex3f( inP.x+dimPt, inP.y-dimPt, 0.0 );
	glVertex3f( inP.x+dimPt, inP.y+dimPt, 0.0 );
	glVertex3f( inP.x-dimPt, inP.y+dimPt, 0.0 );
	glEnd();
}
void NxGLScore::drawRect( NxPoint p1, NxPoint p2 )
{
  	glBegin( GL_LINE_LOOP );
  		glVertex3f( p1.x, p1.y, p1.z );
  		glVertex3f( p1.x, p2.y, p1.z );
  		glVertex3f( p2.x, p2.y, p1.z );
  		glVertex3f( p2.x, p1.y, p1.z );
  	glEnd();
}
void NxGLScore::drawCross( NxPoint p1, float inSize ) ///// circle center crosse
{	
	glColor4f( 1.0, 1.0, 1.0, 0.15 ); //// crosses color
  	glBegin( GL_LINES );
  		glVertex3f( p1.x - inSize, p1.y, p1.z );
  		glVertex3f( p1.x + inSize, p1.y, p1.z );
  		glVertex3f( p1.x, p1.y - inSize, p1.z );
  		glVertex3f( p1.x, p1.y + inSize, p1.z );
  	glEnd();
}
void NxGLScore::drawLine( NxPoint inP1, NxPoint inP2, QColor inColor )
{
	int _r = inColor.red() ;
	int _g = inColor.green() ;
	int _b = inColor.blue() ;
	int _a = inColor.alpha() ;
	glColor4f( (GLfloat)_r/255, (GLfloat)_g/255, (GLfloat)_b/255, (GLfloat)_a/255 );
  	glBegin(GL_LINES);
  		glVertex3f( inP1.x, inP1.y, inP1.z );
  		glVertex3f( inP2.x, inP2.y, inP2.z );
  	glEnd();
}
void NxGLScore::drawDashedLine( NxPoint inP1, NxPoint inP2, QColor inColor )
{
	int _r = inColor.red() ;
	int _g = inColor.green() ;
	int _b = inColor.blue() ;
	int _a = inColor.alpha() ;
	glColor4f( (GLfloat)_r/255, (GLfloat)_g/255, (GLfloat)_b/255, (GLfloat)_a/255 );
	glEnable(GL_LINE_STIPPLE);
	glLineStipple(3, 0xAAAA);
  	glBegin(GL_LINES);
  		glVertex3f( inP1.x, inP1.y, inP1.z );
  		glVertex3f( inP2.x, inP2.y, inP2.z );
  	glEnd();
	glDisable(GL_LINE_STIPPLE);
}
//////////////////////////////////////
// Triggers Rendering
//////////////////////////////////////
void NxGLScore::renderTriggers() 
{
	Color triggColor ;
	NxPoint pt ;
  	QList<int> trigindexes 	= score->triggersIndexes();
  	
	for (int i=0; i<trigindexes.count(); i++) {
		renderedTrig	= score->getTrigger( trigindexes.at(i) );
		int objGp	= renderedTrig->getGroupID() ;
		int objLay	= renderedTrig->getLayer() ;
		bool hidden = true ;
		if( (showAllG||objGp==showOnlyG) && (showAllL||objLay==showOnlyL) ) ///// GROUP / LAYER RENDERING FILTER
			hidden = false ;
			
		pt = renderedTrig->getPosition();
		
		///////////// Normal / Inactive / Highlighted Color /////////////
		triggColor = renderedTrig->getColor() ;
		if(!renderedTrig->isActive())		triggColor = inactiveColor ;
		if (renderedTrig->isHighlighted())	triggColor = selectedColor ;
		if( hidden )						triggColor = hiddenColor ;
		
		///////////// Drawing Bounding Box /////////////
		if ( renderedTrig->isSelected() ) {
			inBox = renderedTrig->getBoundingBox() ;
			drawBoundingBox( inBox ) ; 
		}
			
		drawTrigger( pt, triggColor );
		if(showNames && !frustum && !hidden) drawObjectName( renderedTrig ) ;
	} 
}

/////////////////////////////////////////////
// Trajectories Rendering
/////////////////////////////////////////////
void NxGLScore::renderTrajects()
{
  	QList<int> indexes 	= score->trajectsIndexes();

  	///////////// Trajects rendering
  	for (int i=0; i<indexes.count(); i++) {	
		int idx = indexes.at(i);
		renderedTraj = score->getTraject(idx) ;
		int objGp	= renderedTraj->getGroupID() ;
		int objLay	= renderedTraj->getLayer() ;
		bool hidden = true ;
		if( (showAllG||objGp==showOnlyG) && (showAllL||objLay==showOnlyL) ) ///// GROUP / LAYER RENDERING FILTER
			hidden = false ;
		
		/////////////// INACTIVE / HIGHLIGHTED / NORMAL COLORS
		Color coull = score->getTraject(idx)->getColor() ;
		if( !renderedTraj->isActive() )		coull = inactiveColor ;
		if( renderedTraj->isHighlighted() ) coull = selectedColor ;
		if( hidden )						coull = hiddenColor ;
		
		trajShape = renderedTraj->getType();
			
		switch ( trajShape ) {
			case 1:  ////////// Circle //////////
				if ( renderedTraj->isSelected() ) { // BBox
					bb = renderedTraj->getBoundingBox();
					drawBoundingBox( bb );
				}
				if (showTrajects) {
					circ = renderedTraj->getCircle();
					drawCircle( circ.radius, circ.p.x, circ.p.y, circ.p.z, 1, coull.c );	
					if (crossesDraw) drawCross( circ.p, BBsize ) ;
				}
				if( circleEdition ) { // circle control point
					if( renderedTraj == editedCircle ) {
						drawPointBox( circleEdPt, QColor(255,255,255) ) ;					
					}
				}
			break;
			case 2:  ////////// Line //////////
				////////////////// Draw BBox if selected ///////////////
				if ( renderedTraj->isSelected() ) {
					float trajAng = renderedTraj->getAngle() ;
					float trajLen = renderedTraj->getLength() ;
					if(trajLen<BBsize) trajLen = BBsize ;
					NxPoint initPt = renderedTraj->getInitialPoint() ;
					NxPoint finalPt = renderedTraj->getFinalPoint() ;
					glColor4fv( colorConv( BBColor )); // BoundingBox Color
					glPushMatrix() ;
					glTranslatef( initPt.x, initPt.y, initPt.z ) ;
					glRotatef( trajAng, 0, 0, 1 ) ;
					glBegin( GL_LINE_LOOP );
					glVertex3f( 0, BBsize, 0 );
					glVertex3f( 0, -BBsize, 0 );
					glVertex3f( trajLen, -BBsize, 0 );
					glVertex3f( trajLen, BBsize, 0 );
					glEnd();
					glPopMatrix();
				}
				////////// LINE rendering
				if (showTrajects) drawLine( renderedTraj->getInitialPoint(), renderedTraj->getFinalPoint(), coull.c );
				if( lineEdition ) { // traject control points
					if( renderedTraj == editedLine ) {
						drawPointBox( renderedTraj->getInitialPoint(), QColor(255,255,255) ) ;
						drawPointBox( renderedTraj->getFinalPoint(), QColor(255,255,255) ) ;
					}
				}
			break;
		} // end switch	
	
		if(showTrajects && showNames && !frustum && !hidden) drawObjectName( renderedTraj ) ; // Render Object Name
		if(showCursors) renderCursors( renderedTraj );
	}
}

/////////////////////////////////////////////////////////////////////////////
// Curves Rendering with custom B-splines
/////////////////////////////////////////////////////////////////////////////
void NxGLScore::renderCurves()
{
  	GLfloat controlPtsColor[] = {0.8, 0.8, 0.8, 1.0} ;
	GLfloat curvesRectColor[] = {1.0, 1.0, 1.0, 0.4} ;
	GLfloat curvesZeroColor[] = {1.0, 1.0, 1.0, 0.2} ;
	GLfloat curvesLineColor[] = {0.8, 0.7, 0.2, 0.4} ;
	
  	//////////////////// Rendering Curves ////////////////////
  	QList<int> indexes = score->curvesIndexes();
	
	for (int i=0; i<indexes.count(); i++) {
		NxPoint curvPos ;
		NxPoint mMpt ; // for the min/max rectangle
		int idx = indexes.at(i);
		renderedCurv = score->getCurve(idx) ;
		int objGp	= renderedCurv->getGroupID() ;
		int objLay	= renderedCurv->getLayer() ;
		bool hidden = true ;
		if( (showAllG||objGp==showOnlyG) && (showAllL||objLay==showOnlyL) ) ///// GROUP / LAYER RENDERING FILTER
			hidden = false ;
			
		tmpCurves 	= renderedCurv->getCurve();
		lengthC 	= renderedCurv->size();	
		radius 		= renderedCurv->getRadius();
		tmpPos 		= renderedCurv->getCenter();
		curvPos		= renderedCurv->getPosition() ;
		float cvSc	= renderedCurv->getScale() ;
		float cvMin = renderedCurv->getRealMin() ;
		float cvMax = renderedCurv->getRealMax() ;
		float ang	= renderedCurv->getAngle() ;
		float xRange = renderedCurv->getXRange() ;
		float yRange = renderedCurv->getYRange() ;
		Color coulCir = renderedCurv->getCircleColor() ;
		Color coulGraph = renderedCurv->getColor() ;
		bool beingEdited = (curveEdition && (renderedCurv == highlighted) ) ;
	
		/////////////////////////// INACTIVITY ///////////////////////////
		if( !renderedCurv->isActive() ) {
			coulCir = inactiveColor ;	
			coulGraph = inactiveColor ;	
		}
		if( hidden ) {
			coulCir = hiddenColor ;
			coulGraph = hiddenColor ;
		}
	
		if (circlesDraw) ////////////////////// Circle (circle algorithm)
			drawCircle( radius, tmpPos.x, tmpPos.y, tmpPos.z, 1, coulCir.c );
				
		if (!beingEdited) { ////////////////////// If edited
			glTranslatef( tmpPos.x, tmpPos.y, tmpPos.z) ;	
			glRotatef( ang, 0, 0, 1 ) ;
			glTranslatef( -tmpPos.x, -tmpPos.y, -tmpPos.z) ;
		}
		
		/////////////////////////// BOUNDING BOX ///////////////////////////
		if ( renderedCurv->isSelected() ) {
			if( ang==0 ) {
				bb = renderedCurv->getBoundingBox();
				drawBoundingBox( bb ) ;
			}
			else {
				float ll = xRange ;
				float hh = yRange*cvSc ;
				if(ll<BBsize) ll = BBsize ;
				if(hh<BBsize) hh = BBsize ;
				glColor4fv( colorConv( BBColor )); ///// BoundingBox Color
				glPushMatrix() ;
				glTranslatef( tmpPos.x, tmpPos.y, tmpPos.z ) ;
				//glRotatef( ang, 0, 0, 1 ) ;
				glBegin( GL_LINE_LOOP );
				glVertex3f( -ll/2.0,	hh/2.0,		0 ) ;
				glVertex3f( -ll/2.0,	-hh/2.0,	0 ) ;
				glVertex3f( ll/2.0,		-hh/2.0,	0 ) ;
				glVertex3f( ll/2.0,		hh/2.0,		0 ) ;
				glEnd();
				glPopMatrix();
			}
		}
		
		if (crossesDraw) ////////////////////// Center crosses
			drawCross( renderedCurv->getCenter(), BBsize ) ;
	
		if (rectDraw) {
			if(lengthC>1) { //// Normal CASE
				//////////////////// Draws the min & max (rect)
				if ( !renderedCurv->isSelected() ) {
					glColor4fv( curvesRectColor ) ;
					if ( renderedCurv->isHighlighted() ) 
						glColor4fv( colorConv( selectedColor ) ) ;
					if( !renderedCurv->isActive() ) glColor4fv( colorConv( inactiveColor ) ) ;
					if( hidden )					glColor4fv( colorConv( hiddenColor ) ) ;
					BBox box = renderedCurv->getBoundingBox() ;
					mMpt = box.p1 ;
					mMpt.z = tmpPos.z ;
					drawRect( mMpt, box.p2 ) ;
				}
				//////////////////// Draws the 0 line horizontally (same color) (only if the 0 is in the rect...)
				/*glColor4fv( curvesZeroColor );
				if( !renderedCurv->isActive() ) glColor4fv( colorConv( inactiveColor ) ) ;
				if( hidden )					glColor4fv( colorConv( hiddenColor ) ) ;
				if( cvMax>=0.0 && cvMin<=0.0 ) {
					glPushMatrix();
					glTranslatef( curvPos.x, curvPos.y + (cvMin + yRange/2 )*cvSc, curvPos.z ) ;
					glBegin( GL_LINE_STRIP );	
					glVertex3f( 0.0, 0.0, 0.0 );
					glVertex3f( 2*radius, 0.0, 0.0 );	
					glEnd();
					glPopMatrix();	
				}*/		
				//////////////////// Draws a simple line to see the segment (collisions)
				glColor4fv( curvesLineColor );
				if( !renderedCurv->isActive() ) glColor4fv( colorConv( inactiveColor ) ) ;
				if( hidden )					glColor4fv( colorConv( hiddenColor ) ) ;
				glPushMatrix();
				glTranslatef( tmpPos.x, tmpPos.y, tmpPos.z ); // tmpPos is the curve Center
				//glRotatef( ang, 0, 0, 1 );
				glBegin( GL_LINE_STRIP );		
				//glLineStipple(6, 0xAAAA);  			
				glVertex3f( -radius, 0.0, 0.0 );
				glVertex3f( radius, 0.0, 0.0 );	
				glEnd();
				glPopMatrix();
			}
			else { //// Curve Creation CASE
				NxPoint opp1, opp2 ;
				glColor4fv( curvesRectColor );
				opp1.ID = opp2.ID = 0 ;	// initialisation
				opp1.z = opp2.z = 0 ;	// initialisation
				opp1.x = curvPos.x ; 
				opp1.y = curvPos.y + yRange*cvSc/2 ;
				opp2.x = curvPos.x + yRange*cvSc/2 ; 
				opp2.y = curvPos.y - yRange*cvSc/2 ;
				drawRect( opp1, opp2 ) ;
			}
		}
			
		/////////////////////// RENDERING CURVE POINTS ////////////////////////////////
		glColor4fv( colorConv(coulGraph) );		
		////////////// Drawing Curve or Lines
		glBegin( GL_LINE_STRIP );
		switch ( renderedCurv->getDrawType() ) {
			case LineStrip :
				for ( int j=0; j<lengthC;j++ )
					glVertex3f( curvPos.x + tmpCurves[j].x, curvPos.y + cvSc*((tmpCurves[j].y - cvMin) - yRange/2 ), curvPos.z );
			break;	
			case SplineStrip :
				glTranslatef( 0, 0, curvPos.z) ;
				renderedCurv->getSpline()->draw() ;
				glTranslatef( 0, 0, -curvPos.z) ;
			break;
		}
		glEnd();	
		
		///////////// Re-rotate if not being edited
		if (!beingEdited) {
			glTranslatef( tmpPos.x, tmpPos.y, tmpPos.z) ;
			glRotatef( -ang, 0, 0, 1 ) ;
			glTranslatef( -tmpPos.x, -tmpPos.y, -tmpPos.z) ;
		}	
		
		///////////// CURVE EDITION (Drawing controlPoints)
		if ( renderedCurv->displayPoints() ) {
			glColor4fv( controlPtsColor );
			for( int j=0; j<lengthC; j++) {
				float dimPt = 0.0007*length/scaleX ;
				float dimPt2 = 2*0.0007*length/scaleX ;
				glBegin( GL_QUADS );
				glVertex3f( curvPos.x + tmpCurves[j].x-dimPt, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) - dimPt, 0.0 );
				glVertex3f( curvPos.x + tmpCurves[j].x+dimPt, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) - dimPt, 0.0 );
				glVertex3f( curvPos.x + tmpCurves[j].x+dimPt, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) + dimPt, 0.0 );
				glVertex3f( curvPos.x + tmpCurves[j].x-dimPt, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) + dimPt, 0.0 );
								
				///////////// Display selected one (if any)
				if (tmpCurves[j].ID == pointID) {	
					GLfloat ccc[] 	= {1.0, 0, 0, 1.0};	
					glColor4fv( ccc );
					glVertex3f( curvPos.x + tmpCurves[j].x-dimPt2, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) - dimPt2, 0.0 );
					glVertex3f( curvPos.x + tmpCurves[j].x+dimPt2, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) - dimPt2, 0.0 );
					glVertex3f( curvPos.x + tmpCurves[j].x+dimPt2, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) + dimPt2, 0.0 );
					glVertex3f( curvPos.x + tmpCurves[j].x-dimPt2, curvPos.y + cvSc*((tmpCurves[j].y-cvMin) - yRange/2 ) + dimPt2, 0.0 );
					glColor4fv( controlPtsColor );
				}
				glEnd();
			}
		} // end if (display points )
		
		///////////// CURVE ANGLE EDITION (Drawing controlPoint)
		if(curveAngleEdition && renderedCurv==editedAngleCurve) drawPointBox(curveAngleEdPt, QColor(255,255,255) ) ;				
		
		if(showNames && !frustum && !hidden) drawObjectName( renderedCurv ) ; // Render Object Name
	} // end for (curves )
}
//////////////////////////////////////
// Cursors Rendering
//////////////////////////////////////
void NxGLScore::renderCursors( NxTraject *inTraject )
{
	float cursAngle ;
  	QList<int> IDs 		= inTraject->getCursorIDs();
  	NxPoint center 		= inTraject->getCircle().p;
	NxPoint l1			= inTraject->getInitialPoint();
  	NxPoint l2			= inTraject->getFinalPoint();
  	NxPoint orig;
	Color coul ;
	orig.ID = 0 ;
  	orig.x = orig.y = orig.z = 0.0 ; 
	
  	for ( int i=0; i<IDs.count(); i++ ) {
		renderedCurs	= score->getCursor( IDs.at(i) );
		int objGp	= renderedCurs->getGroupID() ;
		int objLay	= renderedCurs->getLayer() ;
		bool hidden = true ;
		if( (showAllG||objGp==showOnlyG) && (showAllL||objLay==showOnlyL) ) ///// GROUP / LAYER RENDERING FILTER
			hidden = false ;
			
		cursAngle = renderedCurs->getAngle();

		NxPoint cBox ;
		BBox newBox ;
		cBox.ID = 0 ;
		cBox.x = cBox.y = cBox.z = 0 ; // initialisation
		newBox.p1 = newBox.p2 = cBox ; // initialisation

		if ( renderedCurs != NULL ) {	
			///////////////////// COLORS
			coul = renderedCurs->getColor() ;
			if( !renderedCurs->isActive() )		coul = inactiveColor ;
			if( renderedCurs->isHighlighted() )	coul = selectedColor ;
			if( hidden )						coul = hiddenColor ;

			if(showCursorsArea && !hidden)	drawCursorZone( renderedCurs ) ;
			
			switch (inTraject->getType()) {	
				case 1 : //////////// CIRCLE ////////////
					if ( renderedCurs->isSelected() ) { // Render BB
						if( dragging && onClick && numberSelected<2) {// Cursor dragged -> change offset
							float newAngle = getLineAngle(center, timePos) ;
							if(shiftPressed)
								newAngle = shiftAngleRound( newAngle ) ;
							renderedCurs->setAngle( newAngle, true ) ;
							renderedCurs->reEvalAngles() ;
							renderedCurs->updateAnglePosition() ;
						}
						renderedCurs->updateBB( BBsize ) ;
						drawCursorBoundingBox( renderedCurs );
					}
					drawCursor( renderedCurs, coul );
				break;
				case 2 : //////////// LINE ////////////
					////////// Cursors need specific bounding boxes, as they can be rotated!
					if ( renderedCurs->isSelected() ) { // Render BB
						if( dragging && onClick && numberSelected<2) { // Cursor dragged -> change Offset
							float newOff = (p.x-l1.x)*sin(PI*cursAngle/180) - (p.y-l1.y)*cos(PI*cursAngle/180) ;
							if(!renderedCurs->getForward())
								newOff = inTraject->getLength() - newOff ;
							
							renderedCurs->setOffset( newOff );
							renderedCurs->reEvalOffsets() ;
							renderedCurs->updateAnglePosition() ;
						}
						renderedCurs->updateBB( BBsize ) ;
						drawCursorBoundingBox( renderedCurs );
					}
					drawCursor( renderedCurs, coul );
				break;
			}
		}
		/////////////////////////////////////////// CURSOR EDITION MODE
		if(cursorEdition) {
			if( renderedCurs == editedCursor ) {
				drawPointBox(cursorEdPt1, QColor(255,255,255) ) ;
				drawPointBox(cursorEdPt2, QColor(255,255,255) ) ;
				if(inTraject->getType()==1) { // CIRCLE
					if(renderedCurs->getForward())
						drawDashedArc(inTraject->getCircle(), renderedCurs->getInitAngle(), renderedCurs->getFinalAngle(), QColor(255,255,255) ) ;
					else
						drawDashedArc(inTraject->getCircle(), renderedCurs->getFinalAngle(), renderedCurs->getInitAngle(), QColor(255,255,255) ) ;
					if(renderedCurs->hasCustomFinalAngle()) { // render 'I' and 'F'
						drawPointText( QString("I = %1").arg(renderedCurs->getInitAngle()), initOffPt, 6, 0 ) ;
						drawPointText( QString("F = %1").arg(renderedCurs->getFinalAngle()), finalOffPt, 6, 0 ) ;
					}
				}
				else { // LINE
					drawDashedLine(initOffPt, finalOffPt, QColor(255,255,255) ) ;
				}
				drawPointBox(initOffPt, QColor(255,255,255) ) ;
				drawPointBox(finalOffPt, QColor(255,255,255) ) ;
			}
		}
	if(showNames && !frustum && !hidden) drawObjectName( renderedCurs ) ; // Render Object Name
	}
}
/////////////////////////////////////////////
// Rotation around X and Y Axis (3D View)
/////////////////////////////////////////////
void NxGLScore::rotationSetX( float angX )
{
  	angleX = ((int)floor(angX))%((int)360) ;
}
void NxGLScore::rotationSetY( float angY )
{
  	angleY = ((int)floor(angY))%((int)360) ;
}
void NxGLScore::rotationAddX( float offset )
{
  	angleX += offset;
  	if (angleX == 360 || angleX == -360 ) angleX = 0 ;
	oldAngleX = angleX ;
}
void NxGLScore::rotationAddY( float offset )
{
  	angleY += offset;
  	if (angleY == 360 || angleY == -360 ) angleY = 0 ;
	oldAngleY = angleY ;
}
///////////////////////////////////////////////////
// Random curves generator
///////////////////////////////////////////////////
QVector<NxPoint> NxGLScore::randomCurve( int nbpoints, float inRangeMin, float inRangeMax, int duration )
{
	// Initializing randomizer
  	srand( randomizer );
  	QVector<NxPoint> newCurve;
  	NxPoint newPt ;
  	duration /= 1000 ;
  	// Debug prints
  	printf( "Generating random curve : RMIN = %f, RMAX = %f, Controls = %i, Duration = %i\n", inRangeMin, inRangeMax, nbpoints, duration );
  	for (int i=0;i<nbpoints;i++) {	
		newPt.ID = i;
		newPt.x = ( (float)( i*duration ) / (float) nbpoints);
		newPt.y = rand()%( (int)(inRangeMax-inRangeMin)*100 );
		newPt.y = newPt.y / 100 + inRangeMin ;
		newPt.z = 0.0 ;
		newCurve.append(newPt);
	}
  	// Incrementing seed (TO UPDATE : check size)
  	randomizer++;
	
  	return newCurve;
}

//////////////////////////////////////////
// EVENTS HANDLERS
//////////////////////////////////////////
///////////////////////////////  
// Keyboard Events
//////////////////////////////
void NxGLScore::keyPressEvent( QKeyEvent* e )
{
  	switch ( e->key() ) {		
		case Qt::Key_A :
			if(!onClick && !onDrag) {
				if(!frustum && curveEdition) addCurvePoint() ;
				else if (!frustum && highlighted!= NULL ) { // CURVE ANGLE EDITION
					editedAngleCurve = (NxCurve*)highlighted ;
					curveAngleEdPt = editedAngleCurve->getCenter() ;
					float rad = editedAngleCurve->getXRange()/2 ;
					float ang = editedAngleCurve->getAngle() ;
					curveAngleEdPt.x += rad*cos(PI*ang/180) ;
					curveAngleEdPt.y += rad*sin(PI*ang/180) ;
					curveAngleEdition = !curveAngleEdition ;
					if(curveAngleEdition) {
						deselectAll() ;
						highlighted = editedAngleCurve ;
						addObjectToSelection(editedAngleCurve) ;						
					}
				}
			}
		break;		
		case Qt::Key_C :
			showHideCurvesCircles() ;
		break;
		case Qt::Key_D :
			if(!dPressed) setCursorOpenHand() ; // (only the firstTime)
			dPressed = true ;
		break;		
    	case Qt::Key_E :
			if(!onClick && !onDrag) checkEditionModes() ;
		break;
		case Qt::Key_F :
			showHideCursorsArea() ;
      	break;
		case Qt::Key_G :
			showHideGrid() ;
      	break;
		case Qt::Key_K :
			if(!frustum) removeCurvePoint() ;
		break ;		
		case Qt::Key_M :
			showHideScreenInfos() ;
		break;	
		case Qt::Key_N :
			showHideObjectsNames() ;
		break;	
		case Qt::Key_P :
			showHideScore() ;
      	break;					
		case Qt::Key_S :
      		if(!frustum) switchSnapToGrid() ;
      	break;		
    	case Qt::Key_V :	// 2D / 3D VIEW	
			exitEditionModes() ;
			change2D3DView() ;
      	break;			
	 	case Qt::Key_W :	// Stop Playback and Reset Cursors	
			exitEditionModes() ;
			stopAndReset() ;
		break;		
		case Qt::Key_X :
			if(!frustum) {
				lineRuleMode = true;
				if(lineRuleSetFirstPt) {
					lineRulePt1.x = p.x ;
					lineRulePt1.y = p.y ;
					lineRuleSetFirstPt = false ;
				}
			}
		break;		    	
		case Qt::Key_Z :
      		if(!frustum) zoomMode = true;
      	break;		
    	case Qt::Key_Space :
			exitEditionModes() ;
			playPause() ;
   		break;
    	case Qt::Key_Plus :	// Zoom In / Out
      		zoomBy( 0.95 ) ;
      	break;		
		case Qt::Key_Minus :
      		zoomBy( 1.05 ) ;
      	break;	
		case Qt::Key_Shift :
      		if (!curveEdition && !lineEdition && !circleEdition && !frustum) {
				shiftPressed = true ;
	  			if(!lockedSelection && !cursorEdition && !curveAngleEdition) checkHighlighted( mouseX, mouseY );
	  			//printf( "Selection Mode : On\n" );
			}
		break;
		case Qt::Key_Escape : // Exit Fullscreen Mode
			mParent->exitFullscreen() ; // The function does nothing if fullscreenMode is Off
		break;
    	case Qt::Key_Left : // Moving with keypad arrows
			transX += 0.05 ;
      	break;		
    	case Qt::Key_Right :
      		transX -= 0.05 ;
      	break;		
    	case Qt::Key_Up :
      		transY -= 0.05 ;
      	break;
    	case Qt::Key_Down :
      		transY += 0.05 ;
      	break;
		case Qt::Key_Less : // Mouse rotation for 3D view
      		if (frustum) mouseRotation = true ;
      	break;
		case Qt::Key_1: //	Y rotation for 3D view
      		if (frustum) rotationAddY(2.0);
      	break;		
    	case Qt::Key_2 :
      		if (frustum) rotationAddY(-2.0);
      	break;
		case Qt::Key_3 : //	X rotation for 3D view
      		if (frustum) rotationAddX(2.0);
      	break;		
    	case Qt::Key_4 :
      		if (frustum) rotationAddX(-2.0);
      	break;

	}
}
void NxGLScore::keyReleaseEvent( QKeyEvent* e )
{
	if(!e->isAutoRepeat()) { // to avoid automatic release on some systems
		switch ( e->key() ) {
			case Qt::Key_D :
				dPressed = false ;
				scoreDragMode = false ;
				if(!frustum) setCursorNormal() ;
			break;	
			case Qt::Key_X :
				lineRuleSetFirstPt = true ;
				lineRuleMode = false;
			break;		
			case Qt::Key_Z :	
				zoomMode = false;
			break;
			case Qt::Key_Shift :
				if(!curveEdition && !lineEdition && !circleEdition && !frustum) {
					shiftPressed = false;
					lockedSelection = false ;
				}
			break;		
			case Qt::Key_Backspace :
				if( (highlighted!=NULL || numberSelected!=0) && !frustum)
					removeSelected() ;
			break;
			case Qt::Key_Less : //	Mouse rotation for 3D view
				mouseRotation = false ;
			break;	
		}
	}
}
void NxGLScore::exitEditionModes()
{
	curveAngleEdition = curveEdition = lineEdition = circleEdition = cursorEdition = false ;
}
void NxGLScore::checkEditionModes()
{
	if(curveAngleEdition)
		curveAngleEdition = false ;
	if (highlighted!= NULL && !frustum) {
		if( highlighted->getObjectType() == Curve ) { // CURVE POINTS
			editedPointsCurve = (NxCurve*)highlighted ;
			curveEdition = !curveEdition ;
			editedPointsCurve->setPoints( curveEdition ) ;
			if(curveEdition) {
				deselectAll() ;
				highlighted = editedPointsCurve ;
				addObjectToSelection(editedPointsCurve) ;						
			}
		}
		if( highlighted->getObjectType() == Trajectory ) {
			if( ((NxTraject*)highlighted)->getType() == 2 ) { // LINE
				editedLine = (NxTraject*)highlighted ;
				lineEdition = !lineEdition ;
				if(lineEdition) {
					deselectAll() ;
					highlighted = editedLine ;
					addObjectToSelection(editedLine) ;						
				}
			}
			else { // CIRCLE
				editedCircle = (NxTraject*)highlighted ;
				circleEdPt = editedCircle->getPosition() ;
				circleEdPt.x += editedCircle->getRadius() ;
				circleEdition = !circleEdition ;
				if(circleEdition) {
					deselectAll() ;
					highlighted = editedCircle ;
					addObjectToSelection(editedCircle) ;
				}
			}
		}
		if( highlighted->getObjectType() == 2 ) { // CURSOR
			editedCursor = (NxCursor*)highlighted ;
			NxPoint cursPos = editedCursor->getPosition() ;
			float cursAngle = editedCursor->getAngle()*PI/180 ;
			float cursWidth = editedCursor->getWidth() ;
			cursorEdPt1.x = cursPos.x + cursWidth*cos(cursAngle)/2 ;
			cursorEdPt1.y = cursPos.y + cursWidth*sin(cursAngle)/2 ;
			cursorEdPt2.x = cursPos.x - cursWidth*cos(cursAngle)/2 ;
			cursorEdPt2.y = cursPos.y - cursWidth*sin(cursAngle)/2 ;
			initOffPt	= editedCursor->getInitOffsetPoint() ;
			finalOffPt	= editedCursor->getFinalOffsetPoint() ;
			cursorEdition = !cursorEdition ;
			if(cursorEdition) {
				deselectAll() ;
				highlighted = editedCursor ;
				addObjectToSelection(editedCursor) ;					
			}
		}
	}
}
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////
void NxGLScore::updateIndexes() 
{
  	nbCurves 	= score->getEnvironment().nbCurves;
  	nbTrajects 	= score->getEnvironment().nbTrajects;
  	nbTriggers 	= score->getEnvironment().nbTriggers;
  	nbCursors 	= score->getEnvironment().nbCursors;
  	length 		= score->getEnvironment().length;
  	height 		= score->getEnvironment().height;
	////////////////////////////////////////////
	bX = length / (RScaleX*scaleX*10.0);
  	bY = nHeight / (RScaleY*scaleY*10.0);
}
void NxGLScore::updateScoreEnvironmentVariables()
{
	NxEnv newEnv = score->getEnvironment() ;
	////// MODES
	newEnv.visualMode		= visualMode ;
	newEnv.oscOutMode		= oscOutMode ;
	newEnv.oscInMode		= oscInMode ;
	newEnv.showTrajects		= showTrajects ;
	newEnv.showCursors		= showCursors ;
	newEnv.showCurves		= showCurves ;
	newEnv.showTriggers		= showTriggers ;
	////// DISPLAY
	newEnv.showGrid			= showGrid ;
	newEnv.crossesDraw		= crossesDraw ;
	newEnv.scoreDraw		= scoreDraw ;
	newEnv.displayOn		= displayOn ;
	newEnv.namesDisplay		= showNames ;
	newEnv.circlesDraw		= circlesDraw ;
	newEnv.rectDraw			= rectDraw ;
	newEnv.snapGrid			= snapGrid ;
	newEnv.autoGridStep		= autoGridStep ;
	newEnv.gridStep			= gridStep ;
	newEnv.showCursorsArea	= showCursorsArea ;
	////// TIME / OSC
	newEnv.useCirclesAlgo	= useCirclesAlgo ;
	newEnv.sendIFmessages	= sendIFmessages ;
	newEnv.moveCursors		= moveCursors ;
	newEnv.broadcastMode	= broadcastMode ;
	newEnv.initialTime		= timeScheduler->getInitialTime() ;
	newEnv.finalTime		= timeScheduler->getFinalTime() ;
	
	score->setEnvData( newEnv ) ;
}
void NxGLScore::updateCheckedMenus() 
{	
	mParent->updateCheckedMenus( useCirclesAlgo, moveCursors, broadcastMode, visualMode, sendIFmessages, oscOutMode, oscInMode, snapGrid, showGrid, autoGridStep, showTrajects, showCursors, showCurves, showTriggers, crossesDraw, circlesDraw, rectDraw, showCursorsArea, displayOn, showNames, scoreDraw, frustum ) ;
}
////////////////////////////////////////////////////////////////////////
// GROUP / LAYER NxView Filters
////////////////////////////////////////////////////////////////////////
void NxGLScore::updateGroupFilterList()
{
	//printf("Update Group Comboboxe List of NxView (from NXGL)\n") ;
	mParent->updateGroupFilter() ;
}
void NxGLScore::updateLayerFilterList()
{
	mParent->updateLayerFilter() ;
}
QList<int> NxGLScore::getGroupFilterList()
{
	//printf("GetGroupFilterList\n") ;
	QList<int> indCurv = score->curvesIndexes() ;
	QList<int> indTrig = score->triggersIndexes() ;
	QList<int> indCurs = score->cursorsIndexes() ;
	QList<int> indTraj = score->trajectsIndexes() ;
	QSet<int> gpList ;
	for (int i=0; i<indCurv.count(); i++)
		gpList += score->getCurve( indCurv.at(i) )->getGroupID() ;
	for	(int i=0; i<indTrig.count(); i++)
		gpList += score->getTrigger( indTrig.at(i) )->getGroupID() ;
	for (int i=0; i<indCurs.count(); i++)
		gpList += score->getCursor( indCurs.at(i) )->getGroupID() ;
	for (int i=0; i<indTraj.count(); i++)
		gpList += score->getTraject( indTraj.at(i) )->getGroupID() ;
	QList<int> res = gpList.toList() ;
	qSort(res) ;
	return res;
}
QList<int> NxGLScore::getLayerFilterList()
{
	QList<int> indCurv = score->curvesIndexes() ;
	QList<int> indTrig = score->triggersIndexes() ;
	QList<int> indCurs = score->cursorsIndexes() ;
	QList<int> indTraj = score->trajectsIndexes() ;
	QSet<int> layList ;
	for (int i=0; i<indCurv.count(); i++)
		layList += score->getCurve( indCurv.at(i) )->getLayer() ;
	for (int i=0; i<indTrig.count(); i++)
		layList += score->getTrigger( indTrig.at(i) )->getLayer() ;
	for (int i=0; i<indCurs.count(); i++)
		layList += score->getCursor( indCurs.at(i) )->getLayer() ;
	for (int i=0; i<indTraj.count(); i++)
		layList += score->getTraject( indTraj.at(i) )->getLayer() ;
		
	QList<int> res = layList.toList() ;
	qSort(res) ;
	return res;
}
void NxGLScore::showHideGroups( int gp, bool filter )
{
	//if(numberSelected!=0) deselectAll() ;
	exitEditionModes() ;
	if(!filter)
		showAllG = true ;
	else {
		showAllG = false ;
		showOnlyG = gp ;
	}
}
void NxGLScore::showHideLayers( int lay, bool filter )
{
	//if(numberSelected!=0) deselectAll() ;
	exitEditionModes() ;
	if(!filter)
		showAllL = true ;
	else {
		showAllL = false ;
		showOnlyL = lay ;
	}
}
////////////////////////////////////////////////////////////////////////
// SWITCHES
////////////////////////////////////////////////////////////////////////
void NxGLScore::switchCirclesAlgo() {	
	stopAndReset() ;
	useCirclesAlgo	= !useCirclesAlgo ;
	circlesDraw		= useCirclesAlgo ;
	timeScheduler->useCirclesAlgorithm( useCirclesAlgo, true ) ; // (with reset)
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchSendIFmessages() {	
	sendIFmessages = !sendIFmessages ;
	timeScheduler->sendIFmessages( sendIFmessages ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchMoveCursors() {
	moveCursors = !moveCursors ;
	timeScheduler->moveCursors( moveCursors ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchBroadcastMode() {	
	broadcastMode = !broadcastMode ;
	timeScheduler->useBroadcastMode( broadcastMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchOSCoutMode() {	
	oscOutMode = !oscOutMode ;
	timeScheduler->setOSCOutActive( oscOutMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchOSCinMode() {	
	oscInMode = !oscInMode ;
	updateUDPListener( oscInMode ) ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchVisualMode() {	
	visualMode = !visualMode ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::change2D3DView() {
	deselectAll() ;
	
	frustum = !frustum;
	if(frustum) { ////////////////// 3D
		setCursorOpenHand() ;
		snapGrid = false ;
		frustumView();
		fitToPageView() ;
	}
	else { ///////////////////////// 2D
		setCursorNormal() ;
		frustumView();
		fitToPageView() ;
	}

	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
/////////////////// OBJECT RENDER //////////////////////////
void NxGLScore::showHideTrajects() {
	showTrajects = !showTrajects ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCursors() {
	showCursors = !showCursors ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCurves() {
	showCurves = !showCurves ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideTriggers() {
	showTriggers = !showTriggers ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
//////////////////////////////////////////////////////////
void NxGLScore::showHideCursorsArea() {
	showCursorsArea = !showCursorsArea ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideGrid() {
	showGrid = !showGrid ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchAutoGridStep()
{
	autoGridStep = !autoGridStep ;
	if(autoGridStep) {
		gridStep = length/10.0 ;
		if(gridStep > 0.01) gridStepInt = int(floor( gridStep*100 )) ;
		else gridStepInt = 1 ;
	}
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideScore() {
	scoreDraw = !scoreDraw;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCurvesCircles() {
	circlesDraw = !circlesDraw;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCurvesRect() {
	rectDraw = !rectDraw;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideCrosses() {
	crossesDraw = !crossesDraw;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideScreenInfos() {
	displayOn = !displayOn;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::showHideObjectsNames() {
	showNames = !showNames;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}
void NxGLScore::switchSnapToGrid() {
	snapGrid = !snapGrid ;
	updateScoreEnvironmentVariables() ;
	updateCheckedMenus() ;
}

///////////////////////////////////  
// MOUSE EVENTS
///////////////////////////////////
//////////////////////////////////
// Mouse Press Event
//////////////////////////////////
void NxGLScore::mousePressEvent( QMouseEvent *e )
{
	// Converting click coordinates
	clickPixPos.x = e->x() ;
	clickPixPos.y = e->y() ;
	clickPos.x = pixToRealX(e->x()) ;
	clickPos.y = pixToRealY(e->y()) ;
	clickPos.z = 0 ;
	
	if(snapGrid) {
		clickPos.x = snapXY( clickPos.x ) ;
		clickPos.y = snapXY( clickPos.y ) ;
	}
	
	bool editingMode = trajectCreation || curveCreation || curveEdition || curveAngleEdition || lineEdition || circleEdition || cursorEdition ;
	
	if ( e->button()==Qt::LeftButton && !editingMode && dPressed ) {
		scoreDragMode = true ;
		onClick = true ;
	}
		
	if ( e->button()==Qt::LeftButton && highlighted==NULL && !editingMode && !scoreDragMode && !frustum ) 
		selectionMode = true ;
		
	//////////////////////////////////////////////////////
	////// immediate SELECTION->DRAG
	//////////////////////////////////////////////////////
	if ( e->button()==Qt::LeftButton && !editingMode && !scoreDragMode && highlighted!=NULL && !highlighted->isSelected() ) { 
		if(!shiftPressed) {
			NxObject* tmpHighlighted = highlighted ;
			deselectAll() ;
			highlighted = tmpHighlighted ;
			addObjectToSelection(tmpHighlighted) ;
		}
	}	
	
  	// Switches to drag mode if left button is pressed
  	if ( e->button()==Qt::LeftButton && !editingMode && !scoreDragMode && !zoomMode )  {
		onClick = true ;
		if(numberSelected!=0 && highlighted!=NULL) {
			selectedPositions.clear() ;
			for(int u=0;u<numberSelected;u++) selectedPositions.append( selected.at(u)->getPosition() ) ;
		}
	}	
  	if ( editingMode || zoomMode || (selectionMode && !scoreDragMode)) {	
		onDrag = true;
		onClick = false;
	}
	if (curveEdition) curvEdPtChoosed = true ;
	
	doubleClickOk = true ;
	mousePressed = true ;
}	
	
//////////////////////////////////
// Mouse Move Event
//////////////////////////////////
void NxGLScore::mouseMoveEvent ( QMouseEvent * e )
{	
	doubleClickOk = false ;
	
	p.x = pixToRealX( e->x() ) ;
	p.y = pixToRealY( e->y() ) ;
	p.z = 0 ;
	
	if(snapGrid) {
		p.x = snapXY( p.x ) ;
		p.y = snapXY( p.y ) ;
	}		
	
	timePosPix.x = e->x() ;
	timePosPix.y = e->y() ;
	
	if(!scoreDragging) timePos = p ;
	bool editingMode = trajectCreation || curveCreation || curveEdition || curveAngleEdition || lineEdition || circleEdition || cursorEdition ;
	
	/*
	// Grid Translation : TO DO ...
	if (gridMove) {
		gridDecX = clickPos.x + p.x ;
		gridDecY = clickPos.y + p.y ;
	}*/
	
  	mouseX = e->x();
	mouseY = e->y();
	
	///////////////////// On Screen Values Display
	if(lineRuleMode) {
		lineRuleLength	= sqrt( pow(timePos.x-lineRulePt1.x,2) + pow(timePos.y-lineRulePt1.y,2)  ) ;
		lineRuleAngle = getLineAngle( lineRulePt1, timePos ) ;
	}
	editedTrajectAngle = getLineAngle( lineEdPt1, lineEdPt2) ;
	
	// Flags checking
  	if (onDrag) {
		///////////////////////////////////
		// CURSOR EDITION
		///////////////////////////////////
		if ( cursorEdition && selCursorPoint!=-1 ) {
			float newWidth ; 
			NxTraject *parentTraj	= editedCursor->getParentTrajectory() ;
			float cursAngle			= editedCursor->getAngle()*PI/180 ;
			NxPoint cursPos			= editedCursor->getPosition() ;
			
			if(selCursorPoint == 1) { ///////////////////////////////////////// CURSOR WIDTH
				newWidth = fabs((p.x-cursPos.x)*cos(cursAngle) + (p.y-cursPos.y)*sin(cursAngle)) ;
				editedCursor->setWidth( 2*newWidth ) ;
			}
			else { /// OFFSETS/ANGLES
				if(parentTraj->getType()==2) { ////////////////////////////////////////////////// LINE
					float newInitOff, newFinalOff ;
					NxPoint trajInitP		= parentTraj->getInitialPoint() ;
					NxPoint trajFinalP		= parentTraj->getFinalPoint() ;
					float trajLen			= parentTraj->getLength() ;
					float trajAngle			= parentTraj->getAngle()*PI/180 ;
					float pInitOff			= editedCursor->getInitOffset() ;
					float pFinalOff			= editedCursor->getFinalOffset() ;
					float trajProjection	= (p.x-trajInitP.x)*sin(cursAngle) - (p.y-trajInitP.y)*cos(cursAngle) ;
					float newOffset			= trajProjection ;
					if(!editedCursor->getForward())
						newOffset = trajLen - newOffset ;
					if(selCursorPoint == 2) { // CURSOR INIT OFFSET
						newInitOff = newOffset ;
						if(newInitOff<=0)			newInitOff = 0 ;
						if(newInitOff>=pFinalOff)	newInitOff = pFinalOff ;
						editedCursor->setInitOffset( newInitOff ) ;
					}
					else if(selCursorPoint == 3) { // CURSOR FINAL OFFSET
						newFinalOff = newOffset ;
						if(newFinalOff<=pInitOff)	newFinalOff = pInitOff ;
						if(newFinalOff>=trajLen)	newFinalOff = trajLen ;
						editedCursor->setFinalOffset( newFinalOff ) ;
					}
					editedCursor->reEvalOffsets() ;
					float tpInitO	= editedCursor->getInitOffset() ;
					float tpFinalO	= editedCursor->getFinalOffset() ;
					if(editedCursor->getForward()) {
						initOffPt.x = trajInitP.x + tpInitO*cos(trajAngle) ;
						initOffPt.y = trajInitP.y + tpInitO*sin(trajAngle) ;
						finalOffPt.x = trajInitP.x + tpFinalO*cos(trajAngle) ;
						finalOffPt.y = trajInitP.y + tpFinalO*sin(trajAngle) ;
					}
					else {
						initOffPt.x = trajFinalP.x - tpInitO*cos(trajAngle) ;
						initOffPt.y = trajFinalP.y - tpInitO*sin(trajAngle) ;
						finalOffPt.x = trajFinalP.x - tpFinalO*cos(trajAngle) ;
						finalOffPt.y = trajFinalP.y - tpFinalO*sin(trajAngle) ;
					}
				} // end(Line)
				else { /////////////////////////////////////////////////////////////////////////// CIRCLE
					float newInitAngle, newFinalAngle ;
					float trajRad		= parentTraj->getCircle().radius ;
					NxPoint trajP		= parentTraj->getCircle().p ;
					float newAngle		= getLineAngle(trajP, p);
					if(shiftPressed)	newAngle = shiftAngleRound(newAngle) ;
					if(selCursorPoint == 2) { // CURSOR INIT OFFSET
						newInitAngle = newAngle ;
						if( fabs(newAngle-editedCursor->getFinalAngle())<0.05*length/scaleX )
							newInitAngle = editedCursor->getFinalAngle() ;
						editedCursor->setInitAngle( newInitAngle ) ;
					}
					else if(selCursorPoint == 3) { // CURSOR FINAL OFFSET
						newFinalAngle = newAngle ;
						if( fabs(newAngle-editedCursor->getInitAngle())<0.05*length/scaleX )
							newFinalAngle = editedCursor->getInitAngle() ;
						editedCursor->setFinalAngle( newFinalAngle ) ;
					}
					editedCursor->reEvalAngles() ;
					float tpInitA	= editedCursor->getInitAngle()*PI/180 ;
					float tpFinalA	= editedCursor->getFinalAngle()*PI/180 ;
					initOffPt.x = trajP.x + trajRad*cos( tpInitA ) ;
					initOffPt.y = trajP.y + trajRad*sin( tpInitA ) ;
					finalOffPt.x = trajP.x + trajRad*cos( tpFinalA ) ;
					finalOffPt.y = trajP.y + trajRad*sin( tpFinalA ) ;
				} // end(Circle)
				editedCursor->updateAnglePosition() ;
				editedCursor->updateBB( BBsize ) ;
			} // end(OFFSETS/ANGLES)
			///////////////// Update Cursor Position and Width Control Points
			NxPoint tpPos	= editedCursor->getPosition() ;
			float tpWid		= editedCursor->getWidth() ;
			float tpAng		= editedCursor->getAngle()*PI/180 ;
			cursorEdPt1.x = tpPos.x + tpWid*cos(tpAng)/2 ;
			cursorEdPt1.y = tpPos.y + tpWid*sin(tpAng)/2 ;
			cursorEdPt2.x = tpPos.x - tpWid*cos(tpAng)/2 ;
			cursorEdPt2.y = tpPos.y - tpWid*sin(tpAng)/2 ;
		} // end(cursorEdition)
		///////////////////////////////////
		// CURVE POINTS EDITION
		///////////////////////////////////
		if ( curveEdition && pointID!=-1 ) {
			
			curvEdInfoX = timePos.x - curvEdPosX ;
			curvEdInfoY = (timePos.y - curvEdPosY)/curvEdScale + curvEdMin + curvEdYRange/2 ;
			
			tempP.x = p.x - curvEdPosX ;
			tempP.y = (p.y - curvEdPosY)/curvEdScale + curvEdMin + curvEdYRange/2 ;
			// Force the edited point to stay between its neighbors and in [min, max]
			// anything simpler with a 'constrain' like fonction ?
			if( !curvEdfirst ) { // normal curve edition
				if (p.x < curvEdPosX + curvEdXmin ) {
					tempP.x = curvEdXmin ;
					curvEdInfoX = curvEdXmin ;
				}
				if( !curvEdlast && (p.x >curvEdPosX + curvEdXmax) ) {
					tempP.x = curvEdXmax ;
					curvEdInfoX = curvEdXmax ;
				}
			} 
			else { // first point curve edition
				tempP.x = 0 ;
				curvEdInfoX = 0 ;
			}
			
			if( p.y > curvEdYmax )	tempP.y = curvEdInfoY = curvEdMax ;
			if( p.y < curvEdYmin )	tempP.y = curvEdInfoY = curvEdMin ;
			
			curvEdOnlyClic = false ;
			// Coords are updated when mouse is released
		} // end(curveEdition)
		
		///////////////////////////////////
		// CURVE ANGLE EDITION
		///////////////////////////////////
		if ( curveAngleEdition && pointID!=-1 ) {
			NxPoint cP = editedAngleCurve->getCenter() ;
			float newAng = getLineAngle(cP, p) ;
			if(shiftPressed) 
				newAng = shiftAngleRound(newAng) ;
			float rad = editedAngleCurve->getXRange()/2 ;
			curveAngleEdPt = cP ;
			curveAngleEdPt.x += rad*cos(newAng*PI/180) ;
			curveAngleEdPt.y += rad*sin(newAng*PI/180) ;
			editedAngleCurve->setAngle(newAng) ;
		} // end( curveAngelEdition)
		///////////////////////////////////////////////
		// TRAJECT EDITION (LINES)
		///////////////////////////////////////////////
		if ( lineEdition && selTrajectPoint!=-1 ) 
		{
			tempP.x = p.x ;
			tempP.y = p.y ;
			tempP.z = p.z ; 
			
			switch ( editedLine->getOrientation() ) {
				case Horizontal :
					tempP.y = editedLine->getInitialPoint().y ;
				break;
				case Vertical :
					tempP.x = editedLine->getInitialPoint().x ;
				break;
			}
			tempP.ID = 0 ;
			lineEdPt2 = tempP ;
			if(selTrajectPoint == 0) { 
				editedLine->setInitialPoint( tempP );
				editedLine->setPosition(tempP) ;
			}
			if(selTrajectPoint == 1) editedLine->setFinalPoint( tempP );
			
			editedLine->updateBB( BBsize );
			updateCursorsAnglePosition( editedLine ) ;
			
			////////////////////////////////////////////
			controlCenter->updateParameters( Trajectory, editedLine->getID(), true ) ;
		} // end(TrajectEdition)
		///////////////////////////////////////////////
		// TRAJECT EDITION (CIRCLES)
		///////////////////////////////////////////////
		if ( circleEdition && selTrajectPoint!=-1 ) 
		{
			NxPoint cP ;
			cP = editedCircle->getPosition() ;
			tempP.x = p.x ;
			tempP.y = p.y ;
			tempP.z = p.z ;
			tempP.ID = 0 ;
			circleEdPt = tempP ;
			float newRad = sqrt( (tempP.x - cP.x)*(tempP.x - cP.x)+(tempP.y - cP.y)*(tempP.y - cP.y) ) ;
			editedCircle->setRadius( newRad ) ;
			editedCircle->updateBB( BBsize ) ;
			
			updateCursorsAnglePosition( editedCircle ) ;
			
			controlCenter->updateParameters( Trajectory, editedCircle->getID(), true ) ;
		} // end(circleEdition)
	} // end(onDrag)
  	else
		if (onClick) {
			if (firstClick) {
				posX = e->x();
				posY = e->y();
				firstClick = false;
				dragging = true ;
			}
			
			if ( frustum && mouseRotation ) { // Mouse Rotation for the 3D View
				angleY = oldAngleY + (e->x() - clickPixPos.x) ;
				angleX = oldAngleX + (e->y() - clickPixPos.y) ;
			}
			
			////////////////////////////////////////////////////////////////////////
			// Drag & Drop selected Objects
			////////////////////////////////////////////////////////////////////////
			if( !scoreDragMode && !editingMode && numberSelected!=0 && highlighted!=NULL && highlighted->isSelected() ) {
				objectDragging = true ;
				// Updating position (1st control point)
				diffX = p.x - clickPos.x ;
				diffY = p.y - clickPos.y ;
								
				for(int k=0;k<numberSelected;k++) { // for all selected objects
					NxObject* tmpSelected = selected.at(k) ;
					if (tmpSelected->getObjectType() != 2 ) {
						if(snapGrid) {
							NxPoint tmp = selectedPositions.at(k) ;
							tmp.x = snapXY( tmp.x ) ;
							tmp.y = snapXY( tmp.y ) ;
							selectedPositions.replace( k, tmp ) ;
						}
						newPos.x = selectedPositions.at(k).x + diffX ;
						newPos.y = selectedPositions.at(k).y + diffY ;
						newPos.z = selectedPositions.at(k).z ;
						if(shiftPressed) {
							if(isShiftXLocked(clickPos,p))	newPos.y = selectedPositions.at(k).y ; // Y locked
							else							newPos.x = selectedPositions.at(k).x ; // X locked
						}
			
						///////////// Updating Bounding Box;
						tmpSelected->setPosition( newPos );
						tmpSelected->updateBB( BBsize );
						if( tmpSelected->getObjectType() == Trajectory )
							updateCursorsAnglePosition( (NxTraject*)tmpSelected ) ;
					} // end(ifNotCursor)
				} // end(for(selectedObjects))	    	
			} // end(dragAndDrop)
		else {
			/////////////////////////////////////////////////
			// SCORE DRAGGING
			/////////////////////////////////////////////////
			if ( scoreDragMode || (!mouseRotation && frustum) ) {
				scoreDragging = true ;
				//setCursorClosedHand() ;
				float diffTransX = ( ( e->x() )-posX )*(float)(2*length/10.0) /selfW ;
				float diffTransY = ( posY-( e->y() ) )*(float)(2*nHeight/10.0) /selfH ;
				if(frustum) { // if 3D Mode
					diffTransX = ( ( e->x() )-posX )*4.0 /selfW ;
					diffTransY = ( posY-( e->y() ) )*(float)(4.0*nHeight/length) /selfH ;
				} 
				transX += diffTransX ;
				transY += diffTransY ;
			}
	  	}
		//////////// Updating coordinates
		posX = e->x();
		posY = e->y();
	}
	else
		if (trajectCreation) { ////////// Lines creation
		switch ( (score->getTraject( trajID )->getOrientation() ) ) {
			case Horizontal :
				p.y = score->getTraject( trajID )->getInitialPoint().y;
			break;
			
			case Vertical :
				p.x = score->getTraject( trajID )->getInitialPoint().x;
			break;
		}
		lineEdPt2 = p ;
		score->getTraject( trajID )->setFinalPoint( p );
		score->getTraject( trajID )->updateBB( BBsize ) ;
	}
	else
	if (curveCreation) 
	{
		savPos = p;
	}
	else
	if ((editedPointsCurve != NULL) && curveEdition && !curvEdPtChoosed) {
		selectCurvePoints(editedPointsCurve, e->x(), e->y() ) ;
	}
	else
	if ((editedAngleCurve != NULL) && curveAngleEdition) {
		selectCurveAnglePoint( e->x(), e->y() ) ;
	}
	else
	if ((editedLine != NULL) && lineEdition) {
		selectTrajectPoints(editedLine, e->x(), e->y() ) ;
	}
	else
	if ((editedCircle != NULL) && circleEdition) {
		selectTrajectPoints(editedCircle, e->x(), e->y() ) ;
	}
	else
	if ((editedCursor != NULL) && cursorEdition) {
		selectCursorPoints( e->x(), e->y() ) ;
	}
	else
	if( !dPressed && !editingMode && !lockedSelection && !frustum) {
		checkHighlighted( e->x(), e->y() );
	}
}

///////////////////////////////////////////////////
// Mouse DoubleClick Event
///////////////////////////////////////////////////
void NxGLScore::mouseDoubleClickEvent ( QMouseEvent* e )
{
	if(!frustum && !dPressed) {
		if(curveEdition) {
			curveEdition = false ;
			( (NxCurve*) highlighted )->setPoints( false ) ;
		}
	
		lineEdition = circleEdition = cursorEdition = curveAngleEdition = false ;
	
		releaseKeyboard();
		float clX = pixToRealX( e->x() ) ;
		float clY = pixToRealY( e->y() ) ;
		if(snapGrid)  {
			clX = snapXY( clX ) ;
			clY = snapXY( clY ) ;
		}
	
		if (curveCreation)  {
			curveCreation = false;
			controlCenter->updateItems( Curve, true, savID, true );
			score->getCurve(savID)->setPoints( false );
			currentPoints = 1;
			editedCurve.clear() ;
		}
		else {
			if(!shiftPressed && doubleClickOk) {
				doubleClickOk = false ;
				checkHighlighted( e->x(), e->y() ) ;
				edited = highlighted ;
				if( highlighted != NULL) {
					if( highlighted->getObjectType()==Trajectory ) {
						cursorMode = true;
						objectCreator->raiseEditor( clX, clY, edited ) ;
					}
					else
						objectEditor->showEditObject( edited ) ;
				}
				else
					objectCreator->raiseEditor( clX, clY, edited ) ;
			}
		}
	} // end(notFrustum)
}

////////////////////////////////////////////
// Mouse Release Event
////////////////////////////////////////////
void NxGLScore::mouseReleaseEvent ( QMouseEvent*e ) 
{	
	oldAngleX = angleX ;
	oldAngleY = angleY ;
	
	firstClick		= true ;
	
	gridMove		= false ;
	dragging		= false ;
	mousePressed	= false ;
	curvEdPtChoosed = false ;
	
	bool editingMode = trajectCreation || curveCreation || curveEdition || curveAngleEdition || lineEdition || circleEdition || cursorEdition ;
	
	///////////////////////// Zoom Rect Mode
  	if (onDrag) {
		if(zoomMode)  
			zoomRect( clickPos, p );
		else 
		if( !editingMode && !dPressed && selectionMode )
			checkSelection( clickPos, p );
	}		
	
  	onClick = false;
  	onDrag = false;	
	
  	/////////////////////////////
  	// Curve Edition
  	/////////////////////////////
	if(curveEdition && !curvEdOnlyClic) {
		tempP.z = 0.0 ; 
		tempP.ID = IDindex ;
	  	selCurve[IDindex] = tempP;
		( (NxCurve*) highlighted )->setCurve( selCurve );
		controlCenter->updateParameters( Curve, highlighted->getID(), true ) ;
	
		NxPoint curveCenterP = ((NxCurve*)highlighted)->getPosition() ;
		((NxCurve*)highlighted)->setPosition( curveCenterP );
		((NxCurve*)highlighted)->updateBB( BBsize );
		curvEdOnlyClic = true ;
	}
	
  	/////////////////////////////
  	// Curve Creation
  	/////////////////////////////
  	if (curveCreation) {
		currentPoints++;
		savPos.ID = currentPoints - 1 ; // The ID begin at 0
		float lastx = ((NxPoint)editedCurve.last()).x ;
		float rMin = score->getCurve(savID)->getRealMin() ;
		float rMax = score->getCurve(savID)->getRealMax() ;
		float cScale = score->getCurve(savID)->getScale() ;
		NxPoint ccP = score->getCurve(savID)->getPosition() ;
		
		NxPoint newCurvPt ;
		newCurvPt.ID = savPos.ID ;
		newCurvPt.x = savPos.x - ccP.x ;
		newCurvPt.y = (savPos.y - ccP.y)/cScale + rMin + fabs(rMax-rMin)/2 ;
		newCurvPt.z = 0.0 ;
		
		if( newCurvPt.x < (lastx+curveXMinStep) )		newCurvPt.x = lastx + curveXMinStep ;
		if( newCurvPt.y < rMin )						newCurvPt.y = rMin ;
		if( newCurvPt.y > rMax )						newCurvPt.y = rMax ;
		editedCurve.append( newCurvPt );
		
		//////// Creating or updating curve
		if (currentPoints == 2) {	// Creation
			score->getCurve(savID)->setDrawType( drawMode );
			firstPoint = false ;
	  		score->getCurve( savID )->setCurve( editedCurve );
			updateIndexes();
			score->getCurve( savID )->setPoints(true);
		}
		if ( currentPoints > 2 ) {	///// Update created curve
	  		score->getCurve( savID )->setCurve( editedCurve );
			updateIndexes();
		}
	}

  	///////////////////////////////
  	// Editing trajectory
  	///////////////////////////////
  	if ( trajectCreation ) {
		NxPoint p1;

		p.x = pixToRealX( e->x() ) ;
		p.y = pixToRealY( e->y() ) ;
		p.z = 0 ;
		if(snapGrid)  {
			p.x = snapXY( p.x ) ;
			p.y = snapXY( p.y ) ;
		}
		
		p1 = score->getTraject( trajID )->getInitialPoint();
		
		switch ( score->getTraject( trajID )->getOrientation() ) {
			case Horizontal :
				p.y = p1.y;
			break;
			case Vertical :
				p.x = p1.x;
			break;
		}
		p.ID = 0 ;
		score->getTraject( trajID )->setFinalPoint( p );
		controlCenter->updateItems( Trajectory, true, trajID, true );
		score->getTraject( trajID )->updateBB( BBsize );
		trajectCreation = false;
	}
	
	////////////////////////////////////////////////////////////////////////////////
	// SELECTIONS
	////////////////////////////////////////////////////////////////////////////////
	if( !editingMode && !selectionMode && !dPressed )
	{
		if( highlighted==NULL ) { // outside clic -> deselection
			if(!scoreDragging && !shiftPressed) 
				deselectAll() ;
		}
		else { // clic on a object
			if(shiftPressed) { // add or remove an object from selection
				if( highlighted->isSelected() ) { // remove
					int indx = selected.indexOf(highlighted) ;
					selected.removeAt(indx) ;
					highlighted->setSelected(false) ;
					numberSelected-- ;
					if(numberSelected==0) selected.clear() ;
				}
				else { // add the object to the selection
					addObjectToSelection( highlighted ) ;
				}
			} // end(shiftPressed)
			else { // shift not pressed -> new selection
				if(!objectDragging) { // to avoid deselection after dragging objects
					NxObject* tmpHighlighted = highlighted ;
					deselectAll() ;
					selected.append(tmpHighlighted) ;
					tmpHighlighted->setSelected(true) ;
					numberSelected = 1 ;
				}
				else {
					objectDragging = false ;
				}
			}
		} // end(selectionManagement)
	} // end(notSelection/EditionMode)
	
	scoreDragging = false ;
	selectionMode = false ;
}

/////////////////////////////////////////////////////////////////////////
// CHECK HIGHLIGHTED ALGORITHM ( 2D )
/////////////////////////////////////////////////////////////////////////
void NxGLScore::checkHighlighted( int curX, int curY )
{
	if( selectedTrajectList.size()!=0 )		selectedTrajectList.clear() ;
	if( selectedCursorList.size()!=0 )		selectedCursorList.clear() ;
	if( selectedCurveList.size()!=0 )		selectedCurveList.clear() ;
	if( selectedTriggerList.size()!=0 )		selectedTriggerList.clear() ;
	
	highlighted = NULL ;
	
	selX = pixToRealX( curX ) ;
	selY = pixToRealY( curY ) ;
	
	if(showCursors) { ///////////////////// CURSORS TEST // cursors curves and triggers can be better (see trajects...)
		QList<int> curind = score->cursorsIndexes();
		for (int i=0; i<curind.count(); i++) {
			int idx = curind.at(i);
			curs = score->getCursor(idx);
			if( (showAllG||curs->getGroupID()==showOnlyG) && (showAllL||curs->getLayer()==showOnlyL) ) {
				if ( curs->isInside(selX,selY) ) {
					selectedCursorList.append( curs ) ;
					highlighted = curs ;
					//printf("ANGLE : %f, POS : \n%f\n%f\n", curs->getAngle(), curs->getPosition().x, curs->getPosition().y) ;
					curs->setHighlighted(true) ;
				}
				else {
					curs->setHighlighted(false) ;
				}
			}
		}
	}	
	if(showTriggers) { ///////////////////// TRIGGERS TEST
		QList<int> trigind = score->triggersIndexes();
		for (int i=0; i<trigind.count(); i++) {
			int idx = trigind.at(i);
			trig = score->getTrigger(idx);
			if( (showAllG||trig->getGroupID()==showOnlyG) && (showAllL||trig->getLayer()==showOnlyL) ) {
				if ( trig->isInside(selX,selY) ) {
					selectedTriggerList.append( trig ) ;
					if (highlighted == NULL) {
						highlighted = trig;
						trig->setHighlighted(true) ;
					}
				}	
				else {
					trig->setHighlighted(false) ;
				}
			}
		}
	}	
	if(showCurves) { ///////////////////// CURVES TEST
		QList<int> curvind = score->curvesIndexes();
		for (int i=0; i<curvind.count(); i++) {
			int idx = curvind.at(i);
			curve = score->getCurve(idx);
			if( (showAllG||curve->getGroupID()==showOnlyG) && (showAllL||curve->getLayer()==showOnlyL) ) {
				if ( curve->isInside(selX,selY) ) {
					selectedCurveList.append( curve ) ;
					if (highlighted == NULL) {
						highlighted = curve;
						curve->setHighlighted(true) ;
					}
				}
				else {
					curve->setHighlighted(false) ;
				}
			}
		}
	}		
	if(showTrajects) { ///////////////////// TRAJECTS TEST
		QList<int> trajind = score->trajectsIndexes();
		for (int i=0; i<trajind.count(); i++) {
			int idx = trajind.at(i);
			traj = score->getTraject(idx);
			if( (showAllG||traj->getGroupID()==showOnlyG) && (showAllL||traj->getLayer()==showOnlyL) ) {
				if ( traj->isInside(selX, selY, BBsize) ) {
					selectedTrajectList.append( traj ) ;
					if (highlighted == NULL) {
						highlighted = traj;
						traj->setHighlighted(true) ;
					}
				}
				else {
					traj->setHighlighted(false) ;
				}
			}
		}
 	}
}

/////////////////////////////////////////////////////////
// CURVE EDITION
/////////////////////////////////////////////////////////
void NxGLScore::selectCurvePoints( NxCurve* inCurve, int inX, int inY )
{
  	pointID = -1;
	float rS = 0.002*length/scaleX ; ///// size of the selection zone

	selX = pixToRealX( inX ) ;
	selY = pixToRealY( inY ) ;

	NxPoint editedCenter = inCurve->getPosition() ;
	float cScal = inCurve->getScale() ;
	float cMin = inCurve->getRealMin() ;
	float cMax = inCurve->getRealMax() ;
  	selCurve = inCurve->getCurve();
	
  	for (int i=0; i<inCurve->size(); i++) {
		if (	(selX >= ( editedCenter.x + selCurve[i].x - rS ) ) && 
				(selX <= ( editedCenter.x + selCurve[i].x + rS ) ) && 
				(selY >= ( editedCenter.y + cScal*(selCurve[i].y-cMin-fabs(cMax-cMin)/2) - rS ) ) && 
				(selY <= ( editedCenter.y + cScal*(selCurve[i].y-cMin-fabs(cMax-cMin)/2) + rS ) )		)
		{
			pointID = selCurve[i].ID ;
			IDindex = i ;
		}
	}
	
	////////////// if a point is selected, we pre-calculate the borders ...
	if( curveEdition && pointID != -1 ) {
		curvEdYRange = ((NxCurve*)highlighted)->getYRange() ;
		curvEdPosX	= highlighted->getPosition().x ;
		curvEdPosY	= highlighted->getPosition().y ;
		curvEdScale = ((NxCurve*)highlighted)->getScale() ;
		curvEdMax	= ((NxCurve*)highlighted)->getRealMax() ;
		curvEdMin	= ((NxCurve*)highlighted)->getRealMin() ;
		curvEdYmin	= highlighted->getPosition().y - curvEdScale*curvEdYRange/2 ;
		curvEdYmax	= highlighted->getPosition().y + curvEdScale*curvEdYRange/2 ;
		curvEdfirst	= curvEdlast = true ;
		if(IDindex!= 0 ) {	
			curvEdfirst	= false ;
			curvEdXmin	= selCurve[IDindex-1].x + curveXMinStep ;
			curvEd1x	= selCurve[IDindex-1].x + curvEdPosX ;
			curvEd1y	= curvEdPosY + curvEdScale*(selCurve[IDindex-1].y - curvEdMin - curvEdYRange/2 ) ;
		}
		if(IDindex!= ((NxCurve*)highlighted)->size()-1 ) {
			curvEdlast	= false ;
			curvEdXmax	= selCurve[IDindex+1].x - curveXMinStep ;
			curvEd2x	= selCurve[IDindex+1].x + curvEdPosX ;
			curvEd2y	= curvEdPosY + curvEdScale*(selCurve[IDindex+1].y - curvEdMin - curvEdYRange/2 ) ;
		}
	}
}
/////////////////////////////////////////////////////////
// CURVE ANGLE EDITION
/////////////////////////////////////////////////////////
void NxGLScore::selectCurveAnglePoint( int inX, int inY )
{
	pointID = -1;
	float rS = 0.0007*length/scaleX ; //// size of the selection zone
	selX = pixToRealX( inX ) ;
	selY = pixToRealY( inY ) ;
	NxPoint pt = curveAngleEdPt ;
	if (	(selX >= ( pt.x - rS )) && 
			(selX <= ( pt.x + rS )) && 
			(selY >= ( pt.y - rS )) && 
			(selY <= ( pt.y + rS ))		) 
	{
		pointID = 1 ;
	}
}
/////////////////////////////////////////////////////////
// CURSOR EDITION
/////////////////////////////////////////////////////////
void NxGLScore::selectCursorPoints( int inX, int inY )
{
	selCursorPoint = -1 ;
	float rS = 0.0007*length/scaleX ; //// size of the selection zone
	selX = pixToRealX( inX ) ;
	selY = pixToRealY( inY ) ;

	if ( (	(selX >= ( cursorEdPt1.x - rS )) && 
			(selX <= ( cursorEdPt1.x + rS )) && 
			(selY >= ( cursorEdPt1.y - rS )) && 
			(selY <= ( cursorEdPt1.y + rS )) ) ||
		 (	(selX >= ( cursorEdPt2.x - rS )) && 
			(selX <= ( cursorEdPt2.x + rS )) && 
			(selY >= ( cursorEdPt2.y - rS )) && 
			(selY <= ( cursorEdPt2.y + rS )) )     )
	{
		selCursorPoint = 1 ; // CURSOR WIDTH
	}
	else if (	(selX >= ( initOffPt.x - rS )) && 
				(selX <= ( initOffPt.x + rS )) && 
				(selY >= ( initOffPt.y - rS )) && 
				(selY <= ( initOffPt.y + rS )) )
	{
		selCursorPoint = 2 ; // INIT OFFSET/ANGLE POINT
	}
	else if (	(selX >= ( finalOffPt.x - rS )) && 
				(selX <= ( finalOffPt.x + rS )) && 
				(selY >= ( finalOffPt.y - rS )) && 
				(selY <= ( finalOffPt.y + rS )) )
	{
		selCursorPoint = 3 ; // FINAL OFFSET/ANGLE POINT
	}
}
/////////////////////////////////////////////////////////
// TRAJECT EDITION
/////////////////////////////////////////////////////////
void NxGLScore::selectTrajectPoints( NxTraject* inTraject, int inX, int inY )
{
	selTrajectPoint = -1 ;
	float rS = 0.0007*length/scaleX ; //// size of the selection zone
	selX = pixToRealX( inX ) ;
	selY = pixToRealY( inY ) ;
	
	if ( ((NxTraject*)inTraject)->getType() == 2 ) // LINES
	{
		NxPoint pt1 = inTraject->getInitialPoint() ;
		NxPoint pt2 = inTraject->getFinalPoint() ;
		if (	(selX >= ( pt1.x - rS )) && 
				(selX <= ( pt1.x + rS )) && 
				(selY >= ( pt1.y - rS )) && 
				(selY <= ( pt1.y + rS ))		) 
		{
			selTrajectPoint = 0 ;
			lineEdPt1 = pt2 ; // For length /angle display calculation
		}
		if (	(selX >= ( pt2.x - rS )) && 
				(selX <= ( pt2.x + rS )) && 
				(selY >= ( pt2.y - rS )) && 
				(selY <= ( pt2.y + rS ))		) 
		{
			selTrajectPoint = 1 ;
			lineEdPt1 = pt1 ; // For length /angle display calculation
		}
	}
	else { // CIRCLES
		NxPoint pt1 = circleEdPt ;
		if (	(selX >= ( pt1.x - rS )) && 
				(selX <= ( pt1.x + rS )) && 
				(selY >= ( pt1.y - rS )) && 
				(selY <= ( pt1.y + rS ))		) 
		{
			selTrajectPoint = 1 ;
		}
	
	}
}
/////////////////////////////
// REMOVE A CURVE POINT
/////////////////////////////
void NxGLScore::removeCurvePoint()
{
	if ( curveEdition && pointID != -1 && ( ((NxCurve*)highlighted)->size() )>2 ) {
		NxPoint newPosi = highlighted->getPosition() ;
		int initialSize = ((NxCurve*)highlighted)->size() ;
		
		if( IDindex == 0 ) { // we have to translate all the points
			float xDec = selCurve[1].x - selCurve[0].x ;
			for(int l=0 ; l<initialSize ; l++) 
				selCurve[l].x = selCurve[l].x - xDec ;
			newPosi.x += xDec ; 
			highlighted->setPosition( newPosi ) ;
			}
			
		for(int l=IDindex ; l<initialSize-1 ; l++) {
			selCurve[l] = selCurve[l+1];
			selCurve[l].ID = l ;
			}
			
		selCurve.remove(initialSize-1) ;
		((NxCurve*)highlighted)->setCurve( selCurve );
		highlighted->updateBB( BBsize ) ;
		controlCenter->updateParameters( Curve, highlighted->getID(), true ) ;	
	}
}
/////////////////////////////
// ADD A CURVE POINT
/////////////////////////////
void NxGLScore::addCurvePoint()
{
	if ( curveEdition ) {	
		NxPoint newP ;
		newP.z = 0 ;
		int initialSize = ((NxCurve*)highlighted)->size() ;
		int newID ;
		int localID=-1 ;
		bool IDfound = false ;
		float cvMin		= ((NxCurve*)highlighted)->getRealMin() ;
		float cvMax		= ((NxCurve*)highlighted)->getRealMax() ;
		float cvScale	= ((NxCurve*)highlighted)->getScale() ;
		float yRange	= ((NxCurve*)highlighted)->getYRange() ;
		NxPoint cvP		= highlighted->getPosition() ;
		
		bool noSpaceLeft = false ; // to avoid adding a point if there isn't any space left
		
		while(!IDfound) { /////// Search new point location
			localID++ ;
			if( (timePos.x < (cvP.x + selCurve[localID].x) ) || (localID == initialSize) ) IDfound = true ;
		}
		
		if( localID!=0 ) noSpaceLeft = ( fabs(selCurve[localID-1].x-selCurve[localID].x)<=curveXMinStep*2 ) ;
		////// we don't do anything if there isn't any x-space left
		if( !noSpaceLeft ) 
		{		
			selCurve.append(newP) ; //// enlarge the array size
			newID = localID ;
		
			for(int l=initialSize-1 ; l>=newID ; l--) {
				selCurve[l+1] = selCurve[l];
				selCurve[l+1].ID = l+1 ;
			}

			newP.x = timePos.x - cvP.x ;
			newP.y = (timePos.y - cvP.y)/cvScale + cvMin + yRange/2 ;
		
			///// Constrain the position
			if( newID != 0 )
				if (timePos.x < (cvP.x + selCurve[newID-1].x + curveXMinStep) )	newP.x = selCurve[newID-1].x + curveXMinStep ;
			if( (newID!=initialSize-1) && (newID!=initialSize) )
				if (timePos.x > (cvP.x + selCurve[newID+1].x - curveXMinStep) )	newP.x = selCurve[newID+1].x - curveXMinStep ;
			if( timePos.y > cvP.y + cvScale*yRange/2 )	newP.y = cvMax ;
			if( timePos.y < cvP.y - cvScale*yRange/2 )	newP.y = cvMin ;
		
			if( newID == 0 ) { // we have to translate all the points
				newP.x = 0.0 ;
				for(int l=1 ; l<initialSize+1 ; l++) {
					selCurve[l].x = selCurve[l].x + fabs(timePos.x - cvP.x) ;
				}
				cvP.x = timePos.x ;
				( (NxCurve*) highlighted )->setPosition( cvP ) ;
			}
			newP.ID = newID ;
			selCurve[newID] = newP ;
		
			( (NxCurve*) highlighted )->setCurve( selCurve );
			highlighted->updateBB( BBsize ) ;
			controlCenter->updateParameters( Curve, highlighted->getID(), true ) ;
		}
		else printf("No space left for that x - point wasn't added on the curve\n") ;
	}
}
/////////////////////////////////////////////////
// OBJECT DESTRUCTION
/////////////////////////////////////////////////
void NxGLScore::removeSelected()
{	
	if(numberSelected==0 && highlighted!=NULL) { ///// DELETE HIGHLIGHTED
		destroyObject( highlighted ); 
		highlighted = NULL ;
	}
	else { //////////////////////////////////////////// DELETE SELECTION
		for(int s=0;s<numberSelected;s++) {
			destroyObject( selected.at(s) ); 
		}
		selected.clear() ;
		numberSelected = 0 ;
	}
}
void NxGLScore::destroyObject( NxObject* inSelected )
{
  	printf( "Stopping score and Removing Object - ID : %i Type %i \n", inSelected->getID(), inSelected->getObjectType() );
	pause() ; ///// STOP PLAYBACK
	
  	// Must also remove child cursors in case of trajectories...
  	if (inSelected->getObjectType() == Trajectory ) {
		NxCursor *cur ;
		QList<int> indexes = score->cursorsIndexes();
		int nbC = nbCursors ;
		for( int i=0; i< nbC; i++ )
		{	
			cur = score->getCursor( indexes.at(i) ) ;
			if ( inSelected->getID() == cur->getParentTrajectory()->getID() )	
			{	
				controlCenter->updateItems( 2, false, cur->getID(), false );
				score->removeObject( cur->getID(), 2 );
			}
		}
		updateIndexes();
		controlCenter->updateIndexes();
	}
	if (inSelected->getObjectType() == 2 ) { // we have to remove the cursor in the parent trajectory
		NxTraject *pTraj = ((NxCursor*)inSelected)->getParentTrajectory() ;
		pTraj->removeChild( inSelected->getID() ) ;
	}
  	controlCenter->updateItems( inSelected->getObjectType(), false, inSelected->getID(), false );	
  	score->removeObject( inSelected->getID(), inSelected->getObjectType() );
  	updateIndexes();
  	controlCenter->updateIndexes();
	if (nbCursors > 0)
		timeScheduler->resetSchedule();
		
	updateGroupFilterList() ;
	updateLayerFilterList() ;
}

////////////////////////
// GUI SLOTS
////////////////////////
bool NxGLScore::save()
{
	bool savedOK = false ;
	if(neverSaved) { 
		savedOK = saveAs() ;
	}
	else {
		score->saveToXML( (score->getEnvironment().path).append(score->getEnvironment().name) ); 
		savedOK = true ;
	}
	return savedOK ;
}
bool NxGLScore::saveAs()
{
	bool savedOK = false ;
	QString XMLfileToSave ;
	fileChooser->selectFile( QString("%1.xml").arg(score->getEnvironment().name) ) ;
	XMLfileToSave = fileChooser->getSaveFileName(
					0,
					score->getEnvironment().name,
					".",
					"XML Files (*.xml)" ) ;

	if ( XMLfileToSave.length() > 1 ) {
		score->saveToXML( XMLfileToSave ) ;	
		neverSaved = false ;
		savedOK = true ;
	}
		
	/////// Set Score Tab Text
	int tFocus = mParent->myTabs->currentIndex();
	QString nStr = QString("[%1x%2] ").arg(length).arg(height) ;
	nStr.append( score->getEnvironment().name ) ;
	mParent->myTabs->setTabText(tFocus, nStr) ;
	
	return savedOK ;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Right-click contextual menu
//////////////////////////////////////////
void NxGLScore::contextMenuEvent( QContextMenuEvent* )
{
	// If right-clic during specific events -> deactivate events
	shiftPressed = zoomMode = scoreDragMode = lockedSelection = lineRuleMode = mouseRotation = false ;
	lineRuleSetFirstPt = true ;
	
  	// Context Menu Init
  	contextMenu = new QMenu;
	contextMenu->setSeparatorsCollapsible(false) ;
  	QLabel *caption = new QLabel( "<font color=darkblue><b>IanniX Menu</b></font>");
  	caption->setAlignment( Qt::AlignCenter );
	
  	// Dynamic popup with objects list
	QMenu *subSelectMenu	= new QMenu("Select Object...") ;
	subUnderMenu			= new QMenu("Select Highlighted Object...") ;
	QMenu *subPasteMenu		= new QMenu("Paste Object...") ;
	//--------------------------------------------------------------------------// CURSORS
	QAction *setCurrentOffs = new QAction(tr("&Set Current Offset..."), this);
	connect(setCurrentOffs, SIGNAL(triggered()), this, SLOT(changeCurrentOffset()) ) ; 
	QAction *editWidth = new QAction(tr("&Edit Width Timeline"), this);
	connect(editWidth, SIGNAL(triggered()), this, SLOT(editWidth()) ) ; 
	QAction *editSpeed = new QAction(tr("&Edit Speed Timeline"), this);
	connect(editSpeed, SIGNAL(triggered()), this, SLOT(editSpeed()) ) ; 
	//--------------------------------------------------------------------------// ALL OBJECTS
	QAction *setActivity = new QAction(tr("&Active"), this);
	connect(setActivity, SIGNAL(triggered()), this, SLOT(switchActivity()) ) ;	
	setActivity->setCheckable(true) ;	
	QAction *removeObject = new QAction(tr("&Delete"), this);
	connect(removeObject, SIGNAL(triggered()), this, SLOT(removeSelected()) ) ;	
	QAction *editObj = new QAction(tr("&Edit Object..."), this);
	connect(editObj, SIGNAL(triggered()), this, SLOT(editObject()) ) ;
	QAction *editSelec = new QAction(tr("&Edit Objects..."), this);
	connect(editSelec, SIGNAL(triggered()), this, SLOT(editSelection()) ) ;
	QAction *editGp = new QAction(tr("&Edit by Group..."), this);
	connect(editGp, SIGNAL(triggered()), this, SLOT(editGroup()) ) ;
	QAction *deleteGp = new QAction(tr("&Delete by Group..."), this);
	connect(deleteGp, SIGNAL(triggered()), this, SLOT(deleteGroup()) ) ;
	//------------------------ Add MENUS -------------------------------------// 
	contextMenu->addMenu(subUnderMenu);
  	displayToViewUnderObjects(subUnderMenu);
	contextMenu->addMenu(subSelectMenu);
  	displayToSelectObjects(subSelectMenu);
	contextMenu->addSeparator();
	contextMenu->addMenu(subPasteMenu);
  	displayToPasteObjects(subPasteMenu);	
	///////////////////////////
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	if (highlighted!=NULL) {	
		QString str = highlighted->getName() ;
		str.append( QString(" CID:%1 GID:%2").arg( highlighted->getCustomID() ).arg( highlighted->getGroupID() ) ) ;
		QAction *objInfos = new QAction( str , this );
		contextMenu->addAction( objInfos ) ;
		objInfos->setEnabled( false ) ;
	} else {
		QAction *objEmptyInfos = new QAction( QString("Nothing Selected") , this );
		contextMenu->addAction( objEmptyInfos ) ;
		objEmptyInfos->setEnabled( false ) ;
		subUnderMenu->setEnabled( false ) ; // Under Cursor Selection
	}
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	contextMenu->addSeparator();
	contextMenu->addAction( setCurrentOffs );
	contextMenu->addSeparator();
	//cursorMenu->addAction( editWidth );
	//cursorMenu->addAction( editSpeed );
	contextMenu->addAction( setActivity );
	if(numberSelected>1) contextMenu->addAction( editSelec );	
	else contextMenu->addAction( editObj );
	contextMenu->addAction( removeObject );	
	contextMenu->addSeparator();
	contextMenu->addAction( editGp ) ;
	contextMenu->addAction( deleteGp ) ;
	//--------------------------------------------------------------------------//
	setCurrentOffs->setEnabled( false ) ;
	editWidth->setEnabled( false ) ;
	editSpeed->setEnabled( false ) ;
	setActivity->setEnabled( false ) ;
	editSelec->setEnabled( false ) ;
	editObj->setEnabled( false ) ;
	removeObject->setEnabled( false ) ;
	//--------------------------------------------------------------------------//
  	if (highlighted!=NULL) {	
		editSelec->setEnabled( true ) ;
		editObj->setEnabled( true ) ;
		setActivity->setEnabled( true ) ;
		removeObject->setEnabled( true ) ;
		if (highlighted->isActive()) setActivity->setChecked(true) ;
		else setActivity->setChecked(false) ;
		if(highlighted->getObjectType() == 2) {
			//editWidth->setEnabled( true ) ;
			//editSpeed->setEnabled( true ) ;
			setCurrentOffs->setEnabled( true ) ;
		}
	}
  	///// Opening menu at cursor's position
  	if(!frustum) contextMenu->exec( QCursor::pos() );
  	//delete contextMenu;
}
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
/////////////// CONTEXTUALS
/////////////// - The first fonction makes the Dialog
/////////////// - The second applies the changes
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
void NxGLScore::changePlaybackTime() 
{
	cPlaybackTimeDialog = new QDialog(this) ;
	QString str = QString("Set Playback") ;
	cPlaybackTimeDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup = new QGroupBox(cPlaybackTimeDialog) ;
	QGroupBox* gridgroup = new QGroupBox(cPlaybackTimeDialog) ;
	gridgroup->setFlat(true) ;
		
	QGridLayout* poslay = new QGridLayout(gridgroup) ;
	QHBoxLayout* blay	= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cPlaybackTimeDialog) ;
	
	QLabel* timiL = new QLabel("Initial Time (in s)", gridgroup) ;
	timiF = new QLineEdit("0", gridgroup) ;
	QLabel* timfL = new QLabel("Final Time (in s)", gridgroup) ;
	timfF = new QLineEdit("0", gridgroup) ;
	timiF->setText( QString().setNum(timeScheduler->getInitialTime()) ) ;
	timfF->setText( QString().setNum(timeScheduler->getFinalTime()) ) ;

	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(timiL ,  0, 0 ) ;
	poslay->addWidget(timiF ,  0, 1 ) ;
	poslay->addWidget(timfL ,  1, 0 ) ;
	poslay->addWidget(timfF ,  1, 1 ) ;
	
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	gridgroup->setLayout(poslay) ;
	
	genlay->addWidget(gridgroup) ;
	genlay->addWidget(buttgroup) ;
	cPlaybackTimeDialog->setLayout(genlay) ;

	timiF->selectAll() ;
	
	cPlaybackTimeDialog->show() ;
	connect( cancel, SIGNAL( clicked() ), cPlaybackTimeDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setPlaybackTime() ) );
}
void NxGLScore::setPlaybackTime()
{
	float initTime	= timiF->text().toFloat() ;
	float finalTime = timfF->text().toFloat() ;
	if (initTime<0) initTime = 0 ;
	if (finalTime<0) finalTime = 0 ;
	if (finalTime!=0 && initTime>finalTime) {
		float temp = initTime ;
		initTime = finalTime ;
		finalTime = temp ;
	}
	
	timeScheduler->setInitialTime( initTime ) ;
	timeScheduler->setFinalTime( finalTime ) ;
	
	updateScoreEnvironmentVariables() ;
	
	timeScheduler->initScheduler() ;
	cPlaybackTimeDialog->accept() ;
}
void NxGLScore::setGridOpacity( float inOp )
{
	gridOpacity = inOp ;
	NxEnv tmpEnv = score->getEnvironment() ;
	tmpEnv.gridOp = inOp ;
	score->setEnvData( tmpEnv ) ;
}
void NxGLScore::setCursorZonesOpacity( float inOp )
{
	cursZonesOpacity = inOp ;
	NxEnv tmpEnv = score->getEnvironment() ;
	tmpEnv.zonesOp = inOp ;
	score->setEnvData( tmpEnv ) ;
}
void NxGLScore::setNamesOpacity( float inOp )
{
	namesOpacity = inOp ;
	NxEnv tmpEnv = score->getEnvironment() ;
	tmpEnv.namesOp = inOp ;
	score->setEnvData( tmpEnv ) ;
}
void NxGLScore::changeGridStep()
{
	cGridDialog = new QDialog(this) ;
	QString str = QString("Change Grid Step") ;
	cGridDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(cGridDialog) ;
	QGroupBox* gridgroup	= new QGroupBox(cGridDialog) ;
	QGroupBox* slidergroup	= new QGroupBox(cGridDialog) ;
	gridgroup->setFlat(true) ;
	slidergroup->setFlat(true) ;
		
	QHBoxLayout* poslay		= new QHBoxLayout(gridgroup) ;
	QHBoxLayout* slidlay	= new QHBoxLayout(slidergroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay = new QVBoxLayout(cGridDialog) ;
	
	gridStepF			= new QLineEdit(gridgroup) ;
	QLabel* gridStepL	= new QLabel("Grid Step", gridgroup) ;
	gStepSlider		= new QSlider(slidergroup) ;
	gStepSlider->setOrientation( Qt::Horizontal );
	gStepSlider->setRange(1, 100*length); // 100*gridstep
	gStepSlider->setTickPosition(QSlider::NoTicks);
	gStepSlider->setValue( gridStepInt ) ;
	
	QPushButton* apply = new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(gridStepL) ;
	poslay->addWidget(gridStepF) ;
	slidlay->addWidget(gStepSlider) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	gridgroup->setLayout(poslay) ;
	slidergroup->setLayout(slidlay) ;
	
	genlay->addWidget(gridgroup) ;
	genlay->addWidget(slidergroup) ;
	genlay->addWidget(buttgroup) ;
	cGridDialog->setLayout(genlay) ;

	gridStepF->setText( QString("%1").arg( gridStepInt/100.0 ) ) ;
	gridStepF->selectAll() ;
	
	cGridDialog->show() ;
	connect( gStepSlider, SIGNAL( sliderMoved(int) ), this,	SLOT( setDynamicGridStep(int) ) );
	connect( cancel, SIGNAL( clicked() ), cGridDialog,		SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this,				SLOT( setGridStep() ) );
}
void NxGLScore::setDynamicGridStep( int val )
{
	autoGridStep = false ;
	gridStep = val/100.0 ;
	if(gridStep > 0.01) gridStepInt = (int)floor(gridStep*100.0) ;
	else gridStepInt = 1 ;
	gridStepF->setText(QString("%1").arg( val/100.0 )) ;
	updateScoreEnvironmentVariables() ;
}
void NxGLScore::setGridStep()
{
	autoGridStep = false ;

	gridStep = floor( ( fabs(gridStepF->text().toFloat()) )*100.0 ) / 100.0 ;
	if(gridStep > 0.01) gridStepInt = (int)floor(gridStep*100.0) ;
	else gridStepInt = 1 ;
	
	updateScoreEnvironmentVariables() ;
	cGridDialog->accept() ;
}
////////////////////////////////////////////////////////////////////////
// CHANGE CURVE BOUNDS/SCALE/FLATTEN
////////////////////////////////////////////////////////////////////////
void NxGLScore::changeCurveBounds( NxCurve* inCurve, float newMin, float newMax, bool resc, bool chgescale, bool transObj )
{
	bool pbZero		= false ; // points are an horizontal line
	bool pbRange	= false ; // min choosed = max choosed
	float oldRealMin = inCurve->getRealMin() ;
	float oldRealMax = inCurve->getRealMax() ;
	float oldSca	= inCurve->getScale() ;
	float localMin, localMax, oldYR, newSca, decal ;
	
	/////////// inverse Min and Max if Min > Max
	if( newMax < newMin ) {
		float newTemp = newMin ;
		newMin = newMax ;
		newMax = newTemp ;
	}
	if( newMax == newMin ) pbRange = true ;
		
	if(!resc) { // Just set the new bounds...
		inCurve->setRealMin( newMin ) ;
		inCurve->setRealMax( newMax ) ; 
	}
	else { // Move all curve points
		QVector<NxPoint> points = ((NxCurve*)inCurve)->getCurve();
		int cSize = ((NxCurve*)inCurve)->size() ;
	
		////// Get the local Min and Max
		localMin = points[0].y ;
		localMax = points[0].y ;
		for(int i=1; i<cSize; i++) {
			if (localMin>points[i].y) localMin = points[i].y ;
			if (localMax<points[i].y) localMax = points[i].y ;
		}
		if( localMin == localMax ) pbZero = true ;
		oldYR = fabs(localMax-localMin) * oldSca ;
		
		////// Rescale all points
		if(!pbZero) for(int i=0; i<cSize; i++) points[i].y = newMin + (points[i].y - localMin)*(newMax-newMin)/(localMax-localMin) ;
		else for(int i=0; i<cSize; i++) points[i].y = newMin ;
		////// Set the curve
		inCurve->setCurve( points ) ;
		inCurve->setRealMin( newMin ) ;
		inCurve->setRealMax( newMax ) ; 
	
		if(chgescale) {	
			newSca = oldYR / inCurve->getYRange() ;
			inCurve->setScale( newSca ) ;
			if(transObj) {
				decal = ( (localMax+localMin)/2.0 - (oldRealMax+oldRealMin)/2.0 )*oldSca ;
				NxPoint oldPos = inCurve->getPosition() ;
				oldPos.y += decal ;
				inCurve->setPosition( oldPos ) ;
			}
		}
	}
	inCurve->updateBB( BBsize ) ;
	controlCenter->updateParameters( Curve, inCurve->getID(), true ) ;
}
void NxGLScore::flattenCurve( NxCurve *toFlattenCurve)
{
	NxPoint curvPos = toFlattenCurve->getPosition() ;
	float curvMin	= toFlattenCurve->getRealMin() ;
	float curvYRge	= toFlattenCurve->getYRange() ;
	float curvScale = toFlattenCurve->getScale() ;
	QVector<NxPoint> points = toFlattenCurve->getCurve();
	int cSize = toFlattenCurve->size() ;
	float newMax, newMin ;
	
	newMin = curvPos.y - curvYRge*curvScale/2 ;
	newMax = curvPos.y + curvYRge*curvScale/2 ;
	
	////// Rescale all points
	for(int i=0; i<cSize; i++) 
		points[i].y = curvPos.y + curvScale*(points[i].y - curvMin - curvYRge/2.0) ;

	toFlattenCurve->setCurve( points ) ;
	toFlattenCurve->setRealMin( newMin ) ;
	toFlattenCurve->setRealMax( newMax ) ; 
	toFlattenCurve->setScale( 1.0 ) ;
	controlCenter->updateParameters( Curve, toFlattenCurve->getID(), true ) ;
}

////////////////////////////////////////////////////////////
// GROUP DELETE
////////////////////////////////////////////////////////////
void NxGLScore::deleteGroup() 
{	
	gpDeleteDialog = new QDialog(this) ;
	QString str = QString("Delete Group") ;
	gpDeleteDialog->setWindowTitle( str );
	
	QGroupBox* buttgroup	= new QGroupBox(gpDeleteDialog) ;
	QGroupBox* typegroup	= new QGroupBox(gpDeleteDialog) ;
	QGroupBox* gpgroup		= new QGroupBox(gpDeleteDialog) ;
	typegroup->setFlat(true) ;
		
	QGridLayout* poslay		= new QGridLayout(gpgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QGridLayout* typeLay	= new QGridLayout(typegroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(gpDeleteDialog) ;
	
	gpDelF = new QLineEdit(gpgroup) ;
	QLabel* gpDelL = new QLabel("Group ID to delete", gpgroup) ;
	trajdC	= new QCheckBox("Trajectories", typegroup) ;
	cursdC	= new QCheckBox("Cursors", typegroup) ;
	curvdC	= new QCheckBox("Curves", typegroup) ; 
	triggdC	= new QCheckBox("Triggers", typegroup) ; 
	trajdC->setChecked( true ) ;
	cursdC->setChecked( true ) ;
	curvdC->setChecked( true ) ;
	triggdC->setChecked( true ) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
	
	poslay->addWidget(gpDelL ,  0, 0 ) ;
	poslay->addWidget(gpDelF ,  0, 1 ) ;
	typeLay->addWidget( trajdC,		0, 0 ) ;
	typeLay->addWidget( cursdC,		1, 0 ) ;
	typeLay->addWidget( curvdC,		0, 1 ) ;
	typeLay->addWidget( triggdC,	1, 1 ) ;
	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	gpgroup->setLayout(typeLay) ;
	buttgroup->setLayout(blay) ;
	gpgroup->setLayout(poslay) ;
	genlay->addWidget(gpgroup) ;
	genlay->addWidget(typegroup) ;
	genlay->addWidget(buttgroup) ;
	gpDeleteDialog->setLayout(genlay) ;

	gpDelF->setText("0") ;
	gpDelF->selectAll() ;

	gpDeleteDialog->show() ;
	connect( cancel, SIGNAL( clicked() ), gpDeleteDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setDeletedGroup() ) );
}
void NxGLScore::setDeletedGroup() 
{
	gpDeleteDialog->accept() ;
	int gpeID = gpDelF->text().toInt() ;
	NxObject *obj ;
	
	if(curvdC->isChecked()) { ////// Curves
		QList<int> indcurv = score->curvesIndexes();
		for(int i=nbCurves-1 ; i>=0; i--) {
			obj = score->getCurve( indcurv.at(i) ) ;
			if( obj->getGroupID() == gpeID ) destroyObject( obj ) ;
		}
	}
	if(cursdC->isChecked()) { ////// Cursors
		QList<int> indcurs = score->cursorsIndexes();
		for(int i=nbCursors-1 ; i>=0; i--) {
			obj = score->getCursor( indcurs.at(i) ) ;
			if( obj->getGroupID() == gpeID ) destroyObject( obj ) ;
		}
	}
	if(trajdC->isChecked()) { ////// Trajects
		QList<int> indtraj = score->trajectsIndexes();
		for(int i=nbTrajects-1 ; i>=0; i--) {
			obj = score->getTraject( indtraj.at(i) ) ;
			if( obj->getGroupID() == gpeID ) destroyObject( obj ) ;
		}
	}
	if(triggdC->isChecked()) { ////// Triggers
		QList<int> indtrig = score->triggersIndexes();
		for(int i=nbTriggers-1 ; i>=0; i--) {
			obj = score->getTrigger( indtrig.at(i) ) ;
			if( obj->getGroupID() == gpeID ) destroyObject( obj ) ;
		}
	}
}
//////////////////////////////////////////////////////////////////
/////////// OBJECTS
//////////////////////////////////////////////////////////////////
void NxGLScore::switchActivity()
{
	bool newAct = !highlighted->isActive() ;
	if(!highlighted->isSelected()) {
		highlighted->setActive( newAct ) ;
		if( highlighted->getObjectType()==Trajectory )
			setCursorsActive( (NxTraject*)highlighted, newAct ) ;
	}
	else
	for(int s=0;s<numberSelected;s++) {
		selected.at(s)->setActive( newAct ) ;
		if( selected.at(s)->getObjectType()==Trajectory )
			setCursorsActive( (NxTraject*)selected.at(s), newAct ) ;
	}
}
void NxGLScore::setCursorsActive( NxTraject *inTraject, bool act )
{
	NxCursor *cur ;
	QList<int> indexes = score->cursorsIndexes();
	for( int i=0; i< indexes.count(); i++ ) {	
		cur = score->getCursor( indexes.at(i) ) ;
		if ( inTraject->getID() == cur->getParentTrajectory()->getID() ) {
			cur->setActive( act ) ;
		}
	}
}
void NxGLScore::changeCurrentOffset()
{
	toChgOffsetsCursor = (NxCursor*)highlighted ;
	
	curOffsDialog = new QDialog(this) ;
	QString str = QString("Set Current Offset") ;
	curOffsDialog->setWindowTitle( str );
			
	QGroupBox* buttgroup	= new QGroupBox(curOffsDialog) ;
	QGroupBox* posgroup		= new QGroupBox(curOffsDialog) ;
	posgroup->setFlat(true) ;
	
	QGridLayout* poslay		= new QGridLayout(posgroup) ;
	QHBoxLayout* blay		= new QHBoxLayout(buttgroup) ;
	QVBoxLayout* genlay		= new QVBoxLayout(curOffsDialog) ;
	
	curOffF			= new QLineEdit(posgroup) ;
	QLabel* curOffL = new QLabel("Offset", posgroup) ;
	QPushButton* apply	= new QPushButton("Apply", buttgroup) ;
	QPushButton* cancel = new QPushButton("Cancel", buttgroup) ;
		
	poslay->addWidget(curOffL ,	0, 0 ) ;
	poslay->addWidget(curOffF ,	1, 0 ) ;

	blay->addWidget(cancel) ;
	blay->addWidget(apply) ;
	
	buttgroup->setLayout(blay) ;
	posgroup->setLayout(poslay) ;
	
	genlay->addWidget(posgroup) ;
	genlay->addWidget(buttgroup) ;
	curOffsDialog->setLayout(genlay) ;
	
	curOffF->setText( QString("%1").arg(toChgOffsetsCursor->getOffset() ) ) ;
	curOffF->selectAll() ;
	
	curOffsDialog->show() ;
	
	connect( cancel, SIGNAL( clicked() ), curOffsDialog, SLOT( close() ) );
	connect( apply, SIGNAL( clicked() ), this, SLOT( setChangedCurrentOffset() ) );
}
void NxGLScore::setChangedCurrentOffset()
{
	curOffsDialog->accept() ;
	float newOffs = curOffF->text().toFloat()  ;
	toChgOffsetsCursor->setOffset( newOffs ) ;
	toChgOffsetsCursor->reEvalOffsets() ; // reajust offsets / length
	
	toChgOffsetsCursor->updateBB( BBsize ) ;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////
// IANNIX MODULES OPENER
///////////////////////////////////////////////////////
void NxGLScore::openControl()
{
	controlCenter->show();
	controlCenter->activateWindow() ;
	controlCenter->setFocus();
}
void NxGLScore::openCreator()
{
	if(!frustum) objectCreator->raiseEditor( timePos.x, timePos.y, highlighted );	
}
void NxGLScore::openOscConsole()
{
	oscConsole->start() ;
	oscConsole->openWindow();
}
void NxGLScore::closeOscConsole()
{
	oscConsole->stopOutListening() ;
}
////////////////////////////////////////////////////////////
// OBJECT EDITION
////////////////////////////////////////////////////////////
void NxGLScore::editObject()
{
	objectEditor->showEditObject( highlighted ) ;
}
void NxGLScore::editSelection()
{
	objectEditor->showEditSelection( selected ) ;
}
void NxGLScore::editGroup()
{
	objectEditor->showEditGroup() ;
}
void NxGLScore::openPreferences()
{
	preferencesDialog->openPreferences() ;
}
void NxGLScore::open3DCenter()
{
	// Nothing
}
void NxGLScore::editWidth()
{
	this->releaseKeyboard();
	timeWidthEditor->show();
}
void NxGLScore::editSpeed()
{
	this->releaseKeyboard();
	timeSpeedEditor->show();
}
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
// Dynamic object display in context menu + PASTE
///////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
//////// SELECT MENU
////////////////////////////////////////////////////////
void NxGLScore::displayToSelectObjects( QMenu* inMenu )
{
  	QList<int> trajectIndexes 	= score->trajectsIndexes();
	QList<int> cursorIndexes 	= score->cursorsIndexes();
	QList<int> curveIndexes 	= score->curvesIndexes();
  	QList<int> triggerIndexes 	= score->triggersIndexes();
	
  	QMenu *trajMenu 	= new QMenu( QString("Trajects (%1)").arg( trajectIndexes.count() )) ;
	QMenu *cursorMenu	= new QMenu( QString("Cursors (%1)").arg( cursorIndexes.count() )) ;
	QMenu *curveMenu 	= new QMenu( QString("Curves (%1)").arg( curveIndexes.count() )) ;
  	QMenu *trigMenu 	= new QMenu( QString("Triggers (%1)").arg( triggerIndexes.count() )) ;
	
	QString str ;
	
	for(int i=0; i<trajectIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getTraject(trajectIndexes.at(i))->getID() ).arg( score->getTraject(trajectIndexes.at(i))->getName() ) ;
		trajMenu->addAction( str );
	}
	for(int i=0; i<cursorIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getCursor(cursorIndexes.at(i))->getID() ).arg( score->getCursor(cursorIndexes.at(i))->getName() ) ;
    	cursorMenu->addAction( str );
	}
	for(int i=0; i<curveIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getCurve(curveIndexes.at(i))->getID() ).arg( score->getCurve(curveIndexes.at(i))->getName() ) ;
		curveMenu->addAction( str );
	}
	for(int i=0; i<triggerIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getTrigger(triggerIndexes.at(i))->getID() ).arg( score->getTrigger(triggerIndexes.at(i))->getName() ) ;
    	trigMenu->addAction( str );
	}
	
	if(trajectIndexes.count()==0)	trajMenu->setEnabled(false) ;
	if(cursorIndexes.count()==0)	cursorMenu->setEnabled(false) ;
	if(curveIndexes.count()==0)		curveMenu->setEnabled(false) ;
	if(triggerIndexes.count()==0)	trigMenu->setEnabled(false) ;
	
	connect(trajMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectTraject( QAction* )) ) ;
	connect(cursorMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectCursor( QAction* )) ) ;
	connect(curveMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectCurve( QAction* )) ) ;
	connect(trigMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectTrigger( QAction* )) ) ;
	
	inMenu->addMenu(trajMenu);
	inMenu->addMenu(cursorMenu);
  	inMenu->addMenu(curveMenu);
  	inMenu->addMenu(trigMenu);
}
void NxGLScore::selectTraject( QAction *action ) 
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;
	highlighted = score->getTraject( id ) ;
	highlighted->setSelected(true);
	highlighted->setHighlighted(true) ;
	lockedSelection = true ;
}
void NxGLScore::selectCursor( QAction *action )
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;
	highlighted = score->getCursor( id ) ;
	highlighted->setSelected(true);
	highlighted->setHighlighted(true) ;
	lockedSelection = true ;
}
void NxGLScore::selectCurve( QAction *action )
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;
	highlighted = score->getCurve( id ) ;
	highlighted->setSelected(true);
	highlighted->setHighlighted(true) ;
	lockedSelection = true ;
}
void NxGLScore::selectTrigger( QAction *action )
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;
	highlighted = score->getTrigger( id ) ;
	highlighted->setSelected(true);
	highlighted->setHighlighted(true) ;
	lockedSelection = true ;
}
////////////////////////////////////////////////////////
//////// PASTE MENU
////////////////////////////////////////////////////////
void NxGLScore::displayToPasteObjects( QMenu* inMenu )
{
	QList<int> trajectIndexes 	= score->trajectsIndexes();
	QList<int> curveIndexes 	= score->curvesIndexes();
  	QList<int> triggerIndexes 	= score->triggersIndexes();
	
	QMenu *trajMenu 	= new QMenu( QString("Trajects (%1)").arg( trajectIndexes.count() )) ;
	QMenu *curveMenu 	= new QMenu( QString("Curves (%1)").arg( curveIndexes.count() )) ;
  	QMenu *trigMenu 	= new QMenu( QString("Triggers (%1)").arg( triggerIndexes.count() )) ;

	for(int i=0; i<trajectIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getTraject(trajectIndexes.at(i))->getID() ).arg( score->getTraject(trajectIndexes.at(i))->getName() ) ;
    	trajMenu->addAction( str );
	}
	for(int i=0; i<curveIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getCurve(curveIndexes.at(i))->getID() ).arg( score->getCurve(curveIndexes.at(i))->getName() ) ;
		curveMenu->addAction( str );
	}
	for(int i=0; i<triggerIndexes.count(); i++) {
		str = QString("%1 - %2").arg( score->getTrigger(triggerIndexes.at(i))->getID() ).arg( score->getTrigger(triggerIndexes.at(i))->getName() ) ;
    	trigMenu->addAction( str );
	}
	
	if(trajectIndexes.count()==0)	trajMenu->setEnabled(false) ;
	if(curveIndexes.count()==0)		curveMenu->setEnabled(false) ;
	if(triggerIndexes.count()==0)	trigMenu->setEnabled(false) ;
	
	connect(trajMenu, SIGNAL(triggered( QAction* )),	this, SLOT(pasteTraject( QAction* )) ) ;
	connect(curveMenu, SIGNAL(triggered( QAction* )),	this, SLOT(pasteCurve( QAction* )) ) ;
	connect(trigMenu, SIGNAL(triggered( QAction* )),	this, SLOT(pasteTrigger( QAction* )) ) ;
	
	inMenu->addMenu(trajMenu);
  	inMenu->addMenu(curveMenu);
  	inMenu->addMenu(trigMenu);
}
void NxGLScore::pasteTraject(QAction *action)
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;

	NxTraject* copiedTraject = score->getTraject( id ) ;
	int incID		= copiedTraject->getCustomID() ;
	int inGroupID	= copiedTraject->getGroupID() ;
	int layer		= copiedTraject->getLayer() ;
	QString inName	= copiedTraject->getName() ;
	Color inColor	= copiedTraject->getColor() ;
	int inID		= score->getCptTraject() ; 
	NxPoint inPos	= getCursor() ; 
	
	switch ( copiedTraject->getType() ) {
		case 1 : ////// Circle
			score->addTraject( inPos, inID, incID, inGroupID, layer, inName, inColor, 1 );
			score->getTraject( inID )->setCircleDef( copiedTraject->getCircle() );
		break;		
		case 2 : ////// Line
			score->addTraject( inPos, inID, incID, inGroupID, layer, inName, inColor, 2 );
			score->getTraject( inID )->setOrientation( copiedTraject->getOrientation() );
			score->getTraject( inID )->setLineDef( copiedTraject->getInitialPoint(), copiedTraject->getFinalPoint() );
		break;
	}
	score->getTraject( inID )->updateBB( BBsize ) ;
	updateIndexes();
	controlCenter->updateItems( Trajectory, true, inID, true );
	score->maxOfIndexes() ;
	
	pause() ; // Stop playback
	
	////////// We now have to copy the correspondant cursors /////////
	QList<int> indCopCur = copiedTraject->getCursorIDs() ;
	for(int i=0; i<indCopCur.count(); i++) {
		NxCursor *toCop = score->getCursor( indCopCur.at(i) ) ;
		// Get cursor infos
		int newCursID = score->getCptCursor() ;
		QString newNam = toCop->getName().append("_c") ;
		// Add cursor
		score->addCursor( toCop->getPosition(), newCursID, toCop->getCustomID(), (NxTraject*)score->getTraject(inID), toCop->getGroupID(), toCop->getLayer(), newNam, toCop->getColor(), toCop->getForward(), toCop->getMode(), 0, toCop->getInitOffset(), toCop->getFinalOffset() );
		score->getCursor(newCursID)->reEvalOffsets() ;
		score->getCursor(newCursID)->setSpeed( toCop->getSpeed() );
		score->getCursor(newCursID)->setWidth( toCop->getWidth() );
		score->getCursor(newCursID)->setPlayFinished( toCop->getPlayFinished() ) ;
		score->getCursor(newCursID)->setOffset(  toCop->getOffset() ) ;
		score->getCursor(newCursID)->updateBB( BBsize ) ;
		updateIndexes();
		assignCursors();
		controlCenter->updateItems( 2, true, newCursID, true );	
		score->maxOfIndexes() ;
	}
	timeScheduler->resetSchedule() ;
}
void NxGLScore::pasteCurve(QAction *action)
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;

	NxCurve* copiedCurve = score->getCurve( id ) ;
	int incID		= copiedCurve->getCustomID() ;
	int inGroupID	= copiedCurve->getGroupID() ;
	int layer		= copiedCurve->getLayer() ;
	QString inName	= copiedCurve->getName() ;
	int inID		= score->getCptCurve() ;
	Color inColor1	= copiedCurve->getColor() ;
	Color inColor2	= copiedCurve->getCircleColor() ;
	QVector<NxPoint> editedCurve = copiedCurve->getCurve() ;
	float realMin	= copiedCurve->getRealMin() ;
	float realMax	= copiedCurve->getRealMax() ;
	float scale		= copiedCurve->getScale() ;
	float angle		= copiedCurve->getAngle() ;
	int drawt		= copiedCurve->getDrawType() ;
	QString inOscF	= copiedCurve->getOscFormat() ;
	QString inHost	= copiedCurve->getHost() ;
	int inPort		= copiedCurve->getPort() ;
	int inPeriod	= copiedCurve->getSpatialPeriod() ;
	NxPoint inPos = getCursor() ; 
	
	score->addCurve( inPos, inID, incID, inGroupID, layer, inName, inColor1, inColor2, editedCurve, editedCurve.count(), realMin, realMax, scale, angle, inPeriod, inOscF, inHost, inPort );
	updateIndexes() ;
	((NxCurve*)score->getCurve( inID ))->setDrawType( drawt ) ;
	controlCenter->updateItems( Curve, true, inID, true ) ;
	((NxCurve*)score->getCurve( inID ))->setPosition(inPos) ;
	((NxCurve*)score->getCurve( inID ))->updateBB( BBsize ) ;
	
	score->maxOfIndexes() ;
	timeScheduler->addNewCurve() ;
}
void NxGLScore::pasteTrigger(QAction *action)
{
	QString str = action->text() ;
	int tmp = str.indexOf("-") ;
	int id = str.left(tmp-1).toInt() ;

	NxTrigger* copiedTrigger = score->getTrigger( id ) ;
	int incID		= copiedTrigger->getCustomID() ;
	int inGroupID	= copiedTrigger->getGroupID() ;
	int layer		= copiedTrigger->getLayer() ;
	QString inName	= copiedTrigger->getName() ;
	Color inColor1	= copiedTrigger->getColor() ;
	int inID		= score->getCptTrigger() ; 
	int inIntValue	= copiedTrigger->getValue() ;
	QString inOscF	= copiedTrigger->getOscFormat() ;
	QString inHost	= copiedTrigger->getHost() ;
	int inPort		= copiedTrigger->getPort() ;
	NxPoint inPos	= getCursor() ; 
	
	addNewTrigger( inPos, inID, incID, inGroupID, layer, inName, inColor1, inOscF, inHost, inPort, inIntValue ) ;

	score->maxOfIndexes() ;
	timeScheduler->addNewTrigger() ;
}
////////////////////////////////////////////////////////
//////// UNDER MENU
////////////////////////////////////////////////////////
void NxGLScore::displayToViewUnderObjects( QMenu* inMenu )
{
	QMenu *trajMenu 	= new QMenu( QString("Trajects (%1)").arg( selectedTrajectList.count() )) ;
	QMenu *cursorMenu	= new QMenu( QString("Cursors (%1)").arg( selectedCursorList.count() )) ;
	QMenu *curveMenu 	= new QMenu( QString("Curves (%1)").arg( selectedCurveList.count() )) ;
  	QMenu *trigMenu 	= new QMenu( QString("Triggers (%1)").arg( selectedTriggerList.count() )) ;
	
	QString str ;
	
	for(int i=0; i<selectedTrajectList.count(); i++) {
		str = QString("%1 - %2").arg( ((NxTraject*)selectedTrajectList.at(i))->getID() )
								.arg( ((NxTraject*)selectedTrajectList.at(i))->getName() ) ;
		trajMenu->addAction( str );
	}
	for(int i=0; i<selectedCursorList.count(); i++) {
		str = QString("%1 - %2").arg( ((NxCursor*)selectedCursorList.at(i))->getID() )
								.arg( ((NxCursor*)selectedCursorList.at(i))->getName() ) ;
		cursorMenu->addAction( str );
	}
	for(int i=0; i<selectedCurveList.count(); i++) {
		str = QString("%1 - %2").arg( ((NxCurve*)selectedCurveList.at(i))->getID() )
								.arg( ((NxCurve*)selectedCurveList.at(i))->getName() ) ;
		curveMenu->addAction( str );
	}
	for(int i=0; i<selectedTriggerList.count(); i++) {
		str = QString("%1 - %2").arg( ((NxTrigger*)selectedTriggerList.at(i))->getID() )
								.arg( ((NxTrigger*)selectedTriggerList.at(i))->getName() ) ;
		trigMenu->addAction( str );
	}
	
	if(selectedTrajectList.count()==0)	trajMenu->setEnabled(false) ;
	if(selectedCursorList.count()==0)	cursorMenu->setEnabled(false) ;
	if(selectedCurveList.count()==0)	curveMenu->setEnabled(false) ;
	if(selectedTriggerList.count()==0)	trigMenu->setEnabled(false) ;
	
	connect(trajMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectTraject( QAction* )) ) ;
	connect(cursorMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectCursor( QAction* )) ) ;
	connect(curveMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectCurve( QAction* )) ) ;
	connect(trigMenu, SIGNAL(triggered( QAction* )),	this, SLOT(selectTrigger( QAction* )) ) ;
	
	inMenu->addMenu(trajMenu);
	inMenu->addMenu(cursorMenu);
  	inMenu->addMenu(curveMenu);
  	inMenu->addMenu(trigMenu);
}
////////////////////////////////////////////////////////
// ACCESS METHODS
////////////////////////////////////////////////////////
NxView* NxGLScore::getMainWindow()
{
	return mParent ;
}
NxControlCenter* NxGLScore::getControlCenter()
{
	return controlCenter ;
}
NxScore* NxGLScore::getPartition()
{
	return score;
}
NxScheduler* NxGLScore::getScheduler()
{
	return timeScheduler ;
}
NxPoint NxGLScore::getCursor()
{
	return timePos ;
}
float NxGLScore::getBBsize()
{
	return BBsize ;
}
QList<NxObject*> NxGLScore::getSelected()
{
	return selected ;
}
NxObject* NxGLScore::getHighlighted()
{
	if (highlighted != NULL) return highlighted ;
	else return NULL ;
}
int NxGLScore::getNumberSelected()
{
	return numberSelected ;
}
////////////////////////////////////////////////////////
// SELECTION TOOLS
////////////////////////////////////////////////////////
void NxGLScore::selectAll()
{
	deselectAll() ;
	NxCurve		*tmpCurve ;
	NxTrigger	*tmpTrigger ;
	NxCursor	*tmpCursor ;
	NxTraject	*tmpTraject ;
	QList<int> indCurv = score->curvesIndexes() ;
	QList<int> indTrig = score->triggersIndexes() ;
	QList<int> indCurs = score->cursorsIndexes() ;
	QList<int> indTraj = score->trajectsIndexes() ;
	
	if(showCurves)
	for (int i=0; i<indCurv.count(); i++) {
		tmpCurve = score->getCurve( indCurv.at(i) ) ;
		if( (showAllG||tmpCurve->getGroupID()==showOnlyG) && (showAllL||tmpCurve->getLayer()==showOnlyL) )
			addObjectToSelection( tmpCurve ) ;
	}
	if(showTriggers)
	for (int i=0; i<indTrig.count(); i++) {
		tmpTrigger = score->getTrigger( indTrig.at(i) )  ;
		if( (showAllG||tmpTrigger->getGroupID()==showOnlyG) && (showAllL||tmpTrigger->getLayer()==showOnlyL) )
			addObjectToSelection(tmpTrigger ) ;
	}
	if(showCursors)
	for (int i=0; i<indCurs.count(); i++) {
		tmpCursor = score->getCursor( indCurs.at(i) ) ;
		if( (showAllG||tmpCursor->getGroupID()==showOnlyG) && (showAllL||tmpCursor->getLayer()==showOnlyL) )
			addObjectToSelection( tmpCursor ) ;
	}
	if(showTrajects)
	for (int i=0; i<indTraj.count(); i++) {
		tmpTraject = score->getTraject( indTraj.at(i) ) ;
		if( (showAllG||tmpTraject->getGroupID()==showOnlyG) && (showAllL||tmpTraject->getLayer()==showOnlyL) )
			addObjectToSelection( tmpTraject ) ;
	}
}
void NxGLScore::deselectAll()
{
	//printf("Deselect All\n") ;
	QList<int> cursind = score->cursorsIndexes();
	QList<int> curvind = score->curvesIndexes();
	QList<int> trigind = score->triggersIndexes();
	QList<int> trajind = score->trajectsIndexes();
	for (int i=0; i<cursind.count(); i++)
		score->getCursor(cursind.at(i)) ->setHighlighted(false) ;
	for (int i=0; i<curvind.count(); i++)
		score->getCurve(curvind.at(i)) ->setHighlighted(false) ;
	for (int i=0; i<trigind.count(); i++)
		score->getTrigger(trigind.at(i)) ->setHighlighted(false) ;
	for (int i=0; i<trajind.count(); i++)
		score->getTraject(trajind.at(i)) ->setHighlighted(false) ;
		
	for(int k=0;k<numberSelected;k++) {
		selected.at(k)->setSelected(false) ;
	}
	selected.clear() ;
	numberSelected = 0 ;
	highlighted = NULL ;
}
void NxGLScore::addObjectToSelection( NxObject* inObj )
{
	selected.append( inObj ) ;
	inObj->setSelected( true ) ;
	numberSelected++ ;
}
void NxGLScore::checkSelection( NxPoint inP1, NxPoint inP2 )
{
	if(!shiftPressed) deselectAll() ;
	
	NxPoint p1, p2 ; // leftUp and rightBottom corners
	p1.ID = p2.ID = 0 ;
	p1.z = p2.z = 0 ;
	p1.x = fmin(inP1.x,inP2.x) ;
	p1.y = fmax(inP1.y,inP2.y) ;
	p2.x = fmax(inP1.x,inP2.x) ;
	p2.y = fmin(inP1.y,inP2.y) ;
	NxTraject	*traj ;
	NxCursor	*curs ;
	NxTrigger	*trig ;
	NxCurve		*curv ;
	
	int objGp, objLay ;
	
	QList<int> indCurv = score->curvesIndexes() ;
	QList<int> indTrig = score->triggersIndexes() ;
	QList<int> indCurs = score->cursorsIndexes() ;
	QList<int> indTraj = score->trajectsIndexes() ;
	////////////////////////////////////////////////////////////////// Curves
	if(showCurves)
	for (int i=0; i<indCurv.count(); i++) {
		curv = score->getCurve(indCurv.at(i)) ;
		objGp	= curv->getGroupID() ;
		objLay	= curv->getLayer() ;
		if( (showAllG||objGp==showOnlyG) && (showAllL||objLay==showOnlyL) ) {
			NxPoint cp1 = p1 ;
			NxPoint cp2 = p2 ;
			cp1.x -= curv->getXRange()/2 ;
			cp2.x += curv->getXRange()/2 ;
			cp1.y += curv->getYRealRange()/2 ;
			cp2.y -= curv->getYRealRange()/2 ;
			if( isInside( curv->getCenter(), cp1, cp2 ) )
				addObjectToSelection( curv ) ;
		}
	}
	////////////////////////////////////////////////////////////////// Triggers
	if(showTriggers)
	for (int i=0; i<indTrig.count(); i++) {
		trig = score->getTrigger(indTrig.at(i)) ;
		objGp	= trig->getGroupID() ;
		objLay	= trig->getLayer() ;
		if( (showAllG||objGp==showOnlyG) && (showAllL||objLay==showOnlyL) ) {
			if( isInside( trig->getPosition(), p1, p2 ) )
				addObjectToSelection( trig ) ;
		}
	}
	////////////////////////////////////////////////////////////////// Cursors
	if(showCursors)
	for (int i=0; i<indCurs.count(); i++) {
		curs = score->getCursor(indCurs.at(i)) ;
		objGp	= curs->getGroupID() ;
		objLay	= curs->getLayer() ;
		if( (showAllG||objGp==showOnlyG) && (showAllL||objLay==showOnlyL) ) {
			if( isInside( curs->getPosition(), p1, p2 ) )
				addObjectToSelection( curs ) ;
		}
	}
	////////////////////////////////////////////////////////////////// Trajects
	if(showTrajects)
	for (int i=0; i<indTraj.count(); i++) {
		traj = score->getTraject(indTraj.at(i)) ;
		objGp	= traj->getGroupID() ;
		objLay	= traj->getLayer() ;
		if( (showAllG||objGp==showOnlyG) && (showAllL||objLay==showOnlyL) ) {
			switch(traj->getType()) {
				case 1: { // Circle
					NxPoint tp1 = p1 ;
					NxPoint tp2 = p2 ;
					float rad = traj->getCircle().radius ;
					tp1.x -= rad ;
					tp2.x += rad ;
					tp1.y += rad ;
					tp2.y -= rad ;
					if( isInside( traj->getCircle().p, tp1, tp2 ) )
						addObjectToSelection( traj ) ;
				}
				break;
				case 2: // Lines
					NxPoint iP = traj->getInitialPoint() ;
					NxPoint fP = traj->getFinalPoint() ;
					if( isInside( iP, p1, p2 ) ||
						isInside( fP, p1, p2 ) )
						addObjectToSelection( traj ) ;
					else {
						if( isLineInBox( iP, fP, p1, p2 ) )
							addObjectToSelection( traj ) ;
					}
				break ;
			}
		}
	}
}
//////////////////////////////////////////////////////////////////////////
// Return true if line (AB) cross the box p1|p2
//////////////////////////////////////////////////////////////////////////
bool NxGLScore::isLineInBox( NxPoint A, NxPoint B, NxPoint p1, NxPoint p2 )
{
	bool res = false ;

	if( !(	(A.x<p1.x && B.x<p1.x) || 
			(A.x>p2.x && B.x>p2.x) ||
			(A.y<p2.y && B.y<p2.y) || 
			(A.y>p1.y && B.y>p1.y)  ) )
	{
		bool horizontal = false ;
		bool vertical	= false ;
		float X = A.x ;
		float Y = A.y ;
		if(A.y==B.y) horizontal = true ;
		if(A.x==B.x) vertical = true ;
	
		if(horizontal) {
			if(A.y<=p1.y && A.y>=p2.y) res = true ;
		}
		else if(vertical) {
			if(A.x>=p1.x && A.x<=p2.x) res = true ;
		}
		else { // Normal case
			NxPoint p3 = p1 ;
			NxPoint p4 = p2 ;
			p3.y = p2.y ;
			p4.y = p1.y ;
			float a = (B.y-A.y)/(B.x-A.x) ;
			float b = Y - a*X ;
			bool sign = ( p1.y - a*p1.x - b > 0 ) ;
			bool calcSign = ( p2.y - a*p2.x - b > 0 ) ;
		
			if(calcSign==sign) {
				calcSign = ( p3.y - a*p3.x - b > 0 ) ;
				if(calcSign==sign) {
					calcSign = ( p4.y - a*p4.x - b > 0 ) ;
					if(calcSign==sign)
						res = false ;
					else res = true ;
				}
				else res = true ;
			}
			else res = true ;
		}
	}
	
	return res ;
}
////////////////////////////////////////////////////////
////////////////////////////////////////////////////////
////// Timelines for cursors
void NxGLScore::setTimeline( State* inStates, int chooser )
{
	if (chooser == 0)
		editCursor->setWidthDef( inStates );
	else
		editCursor->setSpeedDef( inStates );
}

//////////////////////////
// Deactivate flags
//////////////////////////
void NxGLScore::flagsOff()
{
  	cursorMode = false;
}

////////////////////////////////////////////////////////
// Cursors initial assignment
////////////////////////////////////////////////////////
void NxGLScore::assignCursors()
{
	QList<int> indexes = score->cursorsIndexes() ;
	for (int i=0; i<indexes.count(); i++) {
		score->getCursor( indexes.at(i) )->updateAnglePosition() ;
		score->getCursor( indexes.at(i) )->updateBB( BBsize ) ;
	}
}
void NxGLScore::hardOffsetsReset()
{
	NxCursor *cur ;
	QList<int> indexes = score->cursorsIndexes();
	for( int i=0; i<indexes.count(); i++) {
		cur	= score->getCursor(indexes.at(i)) ;
		cur->hardOffsetsReset() ;
	}
	assignCursors();
	timeScheduler->initScheduler() ;
}
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
//////////////////////// PLAYBACK // Timing functions
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
void NxGLScore::resetSchedule()
{
	NxCursor *cur ;
	
	//pause() ; // Stopping playback already done
	
	////// Resetting Cursors Positions
	QList<int> indexes = score->cursorsIndexes();
	for( int i=0; i<indexes.count(); i++) {
		cur	= score->getCursor(indexes.at(i)) ;
		if(cur->getType()==1) // CIRCLE
			cur->setAngle( cur->getInitAngle(), true );
		else // LINE
			cur->setOffset( cur->getInitOffset() );
	}
	
	timeScheduler->initScheduler() ;
	timeScheduler->resetTime() ;
	assignCursors() ;
}
bool NxGLScore::isPlaying()
{
	return ( timeScheduler->isPlaying() ) ;
}
void NxGLScore::playPause()
{
	if (timeScheduler->isReadyToPlay()) {
		if (!timeScheduler->isPlaying() && nbCursors > 0)
		{
			timeScheduler->setPlaying( true ) ;
			timeScheduler->start();
		}
		else {
			pause() ;
		}
		mParent->setToolBarPlayback() ; // play / pause icon refresh
	}
}
void NxGLScore::pause()
{
	if (timeScheduler->isPlaying()) timeScheduler->stop() ;
	
	assignCursors() ;
	mParent->setToolBarPlayback() ; // play / pause icon refresh
}
void NxGLScore::stopAndReset()
{
	timeScheduler->setReadyToPlay( false ) ;
	pause() ;
	usleep(100000) ;
	resetSchedule() ;
}
void NxGLScore::sendConfigOSC()
{
	timeScheduler->sendOSCConfigMessage() ;
}
void NxGLScore::stopGLTimer()
{
	glTimer->stop();
}
void NxGLScore::startGLTimer()
{
	glTimer->start( int(score->getEnvironment().displayStep) );
}
void NxGLScore::setTempo( float inTempo )
{
	score->setTempo( inTempo ) ;
	timeScheduler->changeTempo( (float)inTempo ) ;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////// UTILITIES /////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void NxGLScore::setCPUWarning( bool act )
{
	mParent->setCPUWarning( act ) ;
}
void NxGLScore::setCPUTimeStep( int val )
{
	mParent->setCPUTimeStep( val ) ;
}
float NxGLScore::snapXY(float _a)
{
  	return ( gridStepInt/100.0 * floor( ( _a / (gridStepInt/100.0) ) + 0.5 ) ) ;
}
////////////////////////////////////////
//// To convert Coordinates
////////////////////////////////////////
float NxGLScore::pixToRealX(float _x)
{
	float _r = (float)length/2.0 + ( _x*(float)length/(5.0*selfW) - (float)length/10.0 - transX )/(float)(RScaleX*scaleX) ;
  	return _r ;
}
float NxGLScore::pixToRealY(float _y)
{
	float _r = (float)height/2.0 + ( - _y*(float)nHeight/(5.0*selfH) + (float)nHeight/10.0 - transY )/(float)(RScaleY*scaleY) ;	
	return _r ;
}
int NxGLScore::realToPixX(float _x)
{
	float _r = (float)selfW/2.0 + ( (RScaleX*scaleX)*( _x - (float)length/2.0 ) + (float)transX )*5.0*(float)selfW/(float)length ;
  	return floor((float)_r) ;
}
int NxGLScore::realToPixY(float _y)
{
	float _r = (float)selfH/2.0 - ( (RScaleY*scaleY)*( _y - (float)height/2.0 ) + (float)transY )*5.0*(float)selfH/(float)nHeight ;
	return floor((float)_r) ;
}
float NxGLScore::getZoomScale() 
{
	return floor(1000000.0*scaleX/length)/100.0 ;
}
bool NxGLScore::isInside( NxPoint inPoint, NxPoint inP1, NxPoint inP2 )
{
	// p1=leftUp and p2=rightBottom corners
	if( inPoint.x >= inP1.x &&
		inPoint.x <= inP2.x &&
		inPoint.y >= inP2.y &&
		inPoint.y <= inP1.y )
		return true ;
	else
		return false ;
}
float NxGLScore::getLineAngle( NxPoint inP1, NxPoint inP2 )
{
	float angle ;
	if(inP2.x!=inP1.x) {
		if(inP2.x>=inP1.x && inP2.y>=inP1.y)
			angle = 180*atan((inP2.y-inP1.y)/(inP2.x-inP1.x))/PI ;
		else
		if(inP2.x>=inP1.x && inP2.y<=inP1.y)
			angle = 360 + 180*atan((inP2.y-inP1.y)/(inP2.x-inP1.x))/PI ;
		else
			angle = 180 + 180*atan((inP2.y-inP1.y)/(inP2.x-inP1.x))/PI ;
	}
	else {
		if(inP2.y>inP1.y)
			angle = 90 ;
		else
			angle = 270 ;
	}
	return angle ;
}
float NxGLScore::shiftAngleRound( float inAngle )
{
	if(inAngle>=22.5 && inAngle<=67.5)
		return 45 ;
	else if(inAngle>=67.5 && inAngle<=112.5)
		return 90 ;
	else if(inAngle>=112.5 && inAngle<=157.5)
		return 135 ;
	else if(inAngle>=157.5 && inAngle<=202.5)
		return 180 ;
	else if(inAngle>=202.5 && inAngle<=247.5)
		return 225 ;
	else if(inAngle>=247.5 && inAngle<=292.5)
		return 270 ;
	else if(inAngle>=292.5 && inAngle<=337.5)
		return 315 ;
	else return 0 ;
}
bool NxGLScore::isShiftXLocked( NxPoint inP1, NxPoint inP2 )
{
	float angle = getLineAngle(inP1,inP2) ;
	if(angle>=135 && angle<=225)
		return true ;
	else if(angle>=315 || angle<=45)
		return true ;
	else
		return false ;
}
GLfloat* NxGLScore::colorConv(Color inColor)
{
	GLfloat *col = new GLfloat[4] ;
	int _r = inColor.c.red() ;
	int _g = inColor.c.green() ;
	int _b = inColor.c.blue() ;
	int _a = inColor.c.alpha() ;
	col[0] = (GLfloat)_r/255 ;
	col[1] = (GLfloat)_g/255 ;
	col[2] = (GLfloat)_b/255 ;
	col[3] = (GLfloat)_a/255 ;
	return col ;
}
void NxGLScore::setCursorOpenHand()
{
	QCursor curs( Qt::OpenHandCursor ) ;
	setCursor(curs) ;
}
void NxGLScore::setCursorClosedHand()
{
	QCursor curs( Qt::ClosedHandCursor ) ;
	setCursor(curs) ;
}
void NxGLScore::setCursorNormal()
{
	unsetCursor() ;
}


